(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    981715,      23404]
NotebookOptionsPosition[    958301,      22723]
NotebookOutlinePosition[    958751,      22742]
CellTagsIndexPosition[    958708,      22739]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Forces between a thick coil and a coaxial magnet\
\>", "Title",
 CellChangeTimes->{{3.5327666304530373`*^9, 3.532766647506733*^9}}],

Cell[CellGroupData[{

Cell["Setup", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{"SetDirectory", "@", 
   RowBox[{"NotebookDirectory", "[", "]"}]}],
  InitializationCell->True], "\[IndentingNewLine]", 
 RowBox[{"<<", 
  StyleBox["\"\<MagnetCode`\>\"",
   InitializationCell->True]}], "\[IndentingNewLine]", 
 RowBox[{"<<", "\"\<MathPSfrag`\>\""}], "\[IndentingNewLine]", 
 RowBox[{"Needs", "[", "\"\<CustomTicks`\>\"", "]"}]}], "Input",
 CellChangeTimes->{{3.53040742751167*^9, 3.5304074375405903`*^9}, {
   3.531447642189475*^9, 3.531447643893031*^9}, 3.532767009932138*^9}],

Cell[BoxData["\<\"/Users/will/magcode/examples\"\>"], "Output",
 CellChangeTimes->{
  3.529272943877357*^9, 3.52972163514178*^9, 3.530348520309431*^9, 
   3.530407438226385*^9, 3.530485206769328*^9, 3.5310144942887697`*^9, 
   3.5310146498618727`*^9, {3.531447634048451*^9, 3.53144764470409*^9}, 
   3.531447795433221*^9, 3.531447870247406*^9, 3.531447902279379*^9, 
   3.531448054773741*^9, 3.5314481749718523`*^9, 3.531701510709565*^9, 
   3.531701571814033*^9, 3.5317016200311327`*^9, 3.531701749443104*^9, 
   3.5317018332735853`*^9, 3.531704505103334*^9, 3.5317904724396887`*^9}]
}, Closed]],

Cell[BoxData[{
 RowBox[{"mm", ":=", "0.001"}], "\[IndentingNewLine]", 
 RowBox[{"tesla", ":=", "1"}], "\[IndentingNewLine]", 
 RowBox[{"ohms", ":=", "1"}], "\[IndentingNewLine]", 
 RowBox[{"amps", ":=", "1"}]}], "Input",
 CellChangeTimes->{{3.529272891368222*^9, 3.529272938335629*^9}}],

Cell[CellGroupData[{

Cell["Params", "Subsection",
 CellChangeTimes->{{3.529272954637397*^9, 3.529272956091848*^9}}],

Cell[BoxData[{
 RowBox[{"magnet", "=", 
  RowBox[{"{", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"MagnetRadius", "\[Rule]", 
     RowBox[{"5", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"MagnetLength", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"Magnetisation", "\[Rule]", 
     RowBox[{"1", " ", "tesla"}]}]}], "\[IndentingNewLine]", 
   "}"}]}], "\[IndentingNewLine]", 
 RowBox[{"coil", "=", 
  RowBox[{"{", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoilRadius", "\[Rule]", 
     RowBox[{"7", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilLength", "\[Rule]", 
     RowBox[{"20", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"Current", "\[Rule]", 
     RowBox[{"1", " ", "amps"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"WireRadius", "\[Rule]", 
     RowBox[{"0.1", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"WireCoating", "\[Rule]", 
     RowBox[{"0.01", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilResistance", "\[Rule]", 
     RowBox[{"8", "ohms"}]}]}], "\[IndentingNewLine]", "}"}]}]}], "Input",
 CellChangeTimes->{{3.5327665838200502`*^9, 3.53276658443891*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Comparison between different methods", "Section"],

Cell["\<\
Here we hope that a zero-thickness coil and a thin-thickness coil gives \
similar results.
Eccentricity should result in only small changes in force.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"thinvthickparams", "=", 
   RowBox[{"{", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"MagnetRadius", "\[Rule]", 
      RowBox[{"9", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"MagnetLength", "\[Rule]", 
      RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Magnetisation", "\[Rule]", 
      RowBox[{"1", " ", "tesla"}]}], ",", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"CoilRadius", "\[Rule]", 
      RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"CoilLength", "\[Rule]", 
      RowBox[{"20", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"CoilTurns", "\[Rule]", "100"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Current", "\[Rule]", "1"}], ",", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Displacement", "\[Rule]", 
      RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"IntegrationPrecision", "\[Rule]", "3"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Verbose", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "}"}]}],
   ";"}]], "Input",
 CellChangeTimes->{{3.523854768359768*^9, 3.523854772774658*^9}, {
   3.52393152227109*^9, 3.523931531931178*^9}, {3.5239328491385593`*^9, 
   3.523932858026626*^9}, {3.5239336988481503`*^9, 3.5239337505682297`*^9}, 
   3.5292730173798237`*^9, {3.5292731148736753`*^9, 3.529273115841112*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"{", "thinvthickparams", "}"}]}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{"thinvthickparams", ",", "\[IndentingNewLine]", 
      RowBox[{"CoilThickness", "\[Rule]", 
       RowBox[{"0.2", "mm"}]}]}], "\[IndentingNewLine]", "}"}]}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{"thinvthickparams", ",", "\[IndentingNewLine]", 
      RowBox[{"Method", "\[Rule]", "\"\<Babic\>\""}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"CoilThickness", "\[Rule]", 
       RowBox[{"0.2", "mm"}]}]}], "\[IndentingNewLine]", "}"}]}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"Timing", "@", 
  RowBox[{"MagnetCoilForce", "@", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"thinvthickparams", ",", "\[IndentingNewLine]", 
     RowBox[{"CoilThickness", "\[Rule]", 
      RowBox[{"0.2", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Eccentricity", "\[Rule]", 
      RowBox[{"0.1", "mm"}]}]}], "\[IndentingNewLine]", 
    "}"}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.52385483875845*^9, 3.5238548950350103`*^9}, {
  3.5238549567496758`*^9, 3.5238549572199583`*^9}, {3.523855462469131*^9, 
  3.523855513793501*^9}, {3.523855561141726*^9, 3.523855563991454*^9}, {
  3.523932769546707*^9, 3.523932771433309*^9}, {3.523933341070142*^9, 
  3.523933341394802*^9}, {3.526266294119831*^9, 3.526266297818021*^9}, {
  3.529273034308737*^9, 3.529273069377644*^9}, {3.529273119289298*^9, 
  3.5292731235958347`*^9}}],

Cell[BoxData["\<\"Coaxial thin coil:\"\>"], "Print",
 CellChangeTimes->{
  3.525484108551716*^9, {3.525484184114688*^9, 3.525484199947837*^9}, 
   3.525484230694221*^9, {3.529273020339876*^9, 3.529273038038802*^9}, 
   3.529273070350425*^9, 3.529273126822496*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.0007090000000005148`", ",", 
   RowBox[{"-", "0.6503208565335336`"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.523852027751328*^9, 3.523852063759798*^9, 3.523854684433894*^9, 
   3.52385473832205*^9, 3.523854775661187*^9, 3.5238548980254383`*^9, 
   3.52385495776099*^9, 3.5238554342753897`*^9, 3.523855518276782*^9, 
   3.523855564562868*^9, 3.523856165377019*^9, {3.523856197980447*^9, 
   3.523856227414467*^9}, 3.523856406421115*^9, 3.523856448101622*^9, {
   3.52385668091588*^9, 3.523856744971036*^9}, {3.5239315250005817`*^9, 
   3.5239315343341713`*^9}, 3.523931603895668*^9, 3.523931801164318*^9, 
   3.523931892141079*^9, 3.523932476516691*^9, 3.523932771968796*^9, {
   3.5239328094943523`*^9, 3.523932860292911*^9}, 3.5239329314422693`*^9, {
   3.523933123498343*^9, 3.523933150407197*^9}, {3.523933191876821*^9, 
   3.523933206573092*^9}, {3.523933701490283*^9, 3.523933751943412*^9}, 
   3.525484020055337*^9, {3.5254840792480803`*^9, 3.525484108555332*^9}, {
   3.5254841841176157`*^9, 3.525484199951354*^9}, 3.525484230698022*^9, {
   3.529273020344078*^9, 3.5292730380425653`*^9}, 3.529273070354384*^9, 
   3.529273126825592*^9}],

Cell[BoxData["\<\"Coaxial thick coil (plain integral):\"\>"], "Print",
 CellChangeTimes->{
  3.525484108551716*^9, {3.525484184114688*^9, 3.525484199947837*^9}, 
   3.525484230694221*^9, {3.529273020339876*^9, 3.529273038038802*^9}, 
   3.529273070350425*^9, 3.529273126827903*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.018128999999998285`", ",", 
   RowBox[{"-", "0.6427656225597573`"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.523852027751328*^9, 3.523852063759798*^9, 3.523854684433894*^9, 
   3.52385473832205*^9, 3.523854775661187*^9, 3.5238548980254383`*^9, 
   3.52385495776099*^9, 3.5238554342753897`*^9, 3.523855518276782*^9, 
   3.523855564562868*^9, 3.523856165377019*^9, {3.523856197980447*^9, 
   3.523856227414467*^9}, 3.523856406421115*^9, 3.523856448101622*^9, {
   3.52385668091588*^9, 3.523856744971036*^9}, {3.5239315250005817`*^9, 
   3.5239315343341713`*^9}, 3.523931603895668*^9, 3.523931801164318*^9, 
   3.523931892141079*^9, 3.523932476516691*^9, 3.523932771968796*^9, {
   3.5239328094943523`*^9, 3.523932860292911*^9}, 3.5239329314422693`*^9, {
   3.523933123498343*^9, 3.523933150407197*^9}, {3.523933191876821*^9, 
   3.523933206573092*^9}, {3.523933701490283*^9, 3.523933751943412*^9}, 
   3.525484020055337*^9, {3.5254840792480803`*^9, 3.525484108555332*^9}, {
   3.5254841841176157`*^9, 3.525484199951354*^9}, 3.525484230698022*^9, {
   3.529273020344078*^9, 3.5292730380425653`*^9}, 3.529273070354384*^9, 
   3.529273126863618*^9}],

Cell[BoxData["\<\"Coaxial thick coil (Babic):\"\>"], "Print",
 CellChangeTimes->{
  3.525484108551716*^9, {3.525484184114688*^9, 3.525484199947837*^9}, 
   3.525484230694221*^9, {3.529273020339876*^9, 3.529273038038802*^9}, 
   3.529273070350425*^9, 3.52927312686646*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.04139799999999916`", ",", 
   RowBox[{"-", "0.6427600455043444`"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.523852027751328*^9, 3.523852063759798*^9, 3.523854684433894*^9, 
   3.52385473832205*^9, 3.523854775661187*^9, 3.5238548980254383`*^9, 
   3.52385495776099*^9, 3.5238554342753897`*^9, 3.523855518276782*^9, 
   3.523855564562868*^9, 3.523856165377019*^9, {3.523856197980447*^9, 
   3.523856227414467*^9}, 3.523856406421115*^9, 3.523856448101622*^9, {
   3.52385668091588*^9, 3.523856744971036*^9}, {3.5239315250005817`*^9, 
   3.5239315343341713`*^9}, 3.523931603895668*^9, 3.523931801164318*^9, 
   3.523931892141079*^9, 3.523932476516691*^9, 3.523932771968796*^9, {
   3.5239328094943523`*^9, 3.523932860292911*^9}, 3.5239329314422693`*^9, {
   3.523933123498343*^9, 3.523933150407197*^9}, {3.523933191876821*^9, 
   3.523933206573092*^9}, {3.523933701490283*^9, 3.523933751943412*^9}, 
   3.525484020055337*^9, {3.5254840792480803`*^9, 3.525484108555332*^9}, {
   3.5254841841176157`*^9, 3.525484199951354*^9}, 3.525484230698022*^9, {
   3.529273020344078*^9, 3.5292730380425653`*^9}, 3.529273070354384*^9, 
   3.529273126895874*^9}],

Cell[BoxData["\<\"Eccentric thick coil:\"\>"], "Print",
 CellChangeTimes->{
  3.525484108551716*^9, {3.525484184114688*^9, 3.525484199947837*^9}, 
   3.525484230694221*^9, {3.529273020339876*^9, 3.529273038038802*^9}, 
   3.529273070350425*^9, 3.529273126898243*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.03206900000000168`", ",", 
   RowBox[{"-", "0.642950858103505`"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.523852027751328*^9, 3.523852063759798*^9, 3.523854684433894*^9, 
   3.52385473832205*^9, 3.523854775661187*^9, 3.5238548980254383`*^9, 
   3.52385495776099*^9, 3.5238554342753897`*^9, 3.523855518276782*^9, 
   3.523855564562868*^9, 3.523856165377019*^9, {3.523856197980447*^9, 
   3.523856227414467*^9}, 3.523856406421115*^9, 3.523856448101622*^9, {
   3.52385668091588*^9, 3.523856744971036*^9}, {3.5239315250005817`*^9, 
   3.5239315343341713`*^9}, 3.523931603895668*^9, 3.523931801164318*^9, 
   3.523931892141079*^9, 3.523932476516691*^9, 3.523932771968796*^9, {
   3.5239328094943523`*^9, 3.523932860292911*^9}, 3.5239329314422693`*^9, {
   3.523933123498343*^9, 3.523933150407197*^9}, {3.523933191876821*^9, 
   3.523933206573092*^9}, {3.523933701490283*^9, 3.523933751943412*^9}, 
   3.525484020055337*^9, {3.5254840792480803`*^9, 3.525484108555332*^9}, {
   3.5254841841176157`*^9, 3.525484199951354*^9}, 3.525484230698022*^9, {
   3.529273020344078*^9, 3.5292730380425653`*^9}, 3.529273070354384*^9, 
   3.529273126946439*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"thickfilamentparams", ":=", 
  RowBox[{"{", "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Displacement", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"Eccentricity", "\[Rule]", 
     RowBox[{"0", "mm"}]}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{"MagnetRadius", "\[Rule]", 
     RowBox[{"9", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"MagnetLength", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"Magnetisation", "\[Rule]", "1"}], ",", "\[IndentingNewLine]", 
    "\[IndentingNewLine]", 
    RowBox[{"CoilRadius", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilLength", "\[Rule]", 
     RowBox[{"20", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilThickness", "\[Rule]", 
     RowBox[{"5", "mm"}]}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{"CoilTurns", "\[Rule]", "100"}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilTurnsR", "\[Rule]", "5"}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilTurnsZ", "\[Rule]", "20"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Current", "\[Rule]", "1"}], ",", "\[IndentingNewLine]", 
    "\[IndentingNewLine]", 
    RowBox[{"Verbose", "\[Rule]", "True"}]}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", "}"}]}]], "Input",
 CellChangeTimes->{{3.5255655574528017`*^9, 3.5255655731587543`*^9}, {
  3.526269603414728*^9, 3.526269608893355*^9}, {3.526270548461328*^9, 
  3.5262705487411203`*^9}, {3.527287062863903*^9, 3.5272870729977837`*^9}, {
  3.5272903700476713`*^9, 3.527290374199184*^9}, {3.5292731493376493`*^9, 
  3.529273188574204*^9}, {3.529273257842905*^9, 3.529273260868804*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Timing", "@", 
  RowBox[{"MagnetCoilForce", "@", 
   RowBox[{"Flatten", "@", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"CoilThickness", "\[Rule]", 
       RowBox[{"0", "mm"}]}], ",", 
      RowBox[{"Method", "\[Rule]", "\"\<Filament\>\""}], ",", 
      "thickfilamentparams"}], "}"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"CoilThickness", "\[Rule]", 
        RowBox[{"0", "mm"}]}], ",", "thickfilamentparams"}], "}"}]}]}]}], "//.",
   "thickfilamentparams"}]}], "Input",
 CellChangeTimes->{{3.523852129909368*^9, 3.523852175464472*^9}, {
   3.523852608458832*^9, 3.5238526156278067`*^9}, {3.525484300444316*^9, 
   3.52548430732273*^9}, {3.52548438263474*^9, 3.525484411260295*^9}, {
   3.525484864907833*^9, 3.525484865229104*^9}, {3.526266262415948*^9, 
   3.5262662729495087`*^9}, {3.5262696869347258`*^9, 3.526269695916193*^9}, {
   3.526270368217671*^9, 3.5262703744558887`*^9}, {3.526270477507333*^9, 
   3.5262705170888577`*^9}, 3.526270552154606*^9, {3.527287146917223*^9, 
   3.527287148204864*^9}, {3.527287822938772*^9, 3.527287870749827*^9}, {
   3.52728835800705*^9, 3.527288361793992*^9}, {3.527288505461522*^9, 
   3.527288539099778*^9}, {3.5272903824869757`*^9, 3.527290385466888*^9}, {
   3.527290478978107*^9, 3.527290485273223*^9}}],

Cell[BoxData["\<\"Coaxial thin coil: (filament)\"\>"], "Print",
 CellChangeTimes->{
  3.523852547483646*^9, {3.523852603905986*^9, 3.52385261173208*^9}, 
   3.523855617935769*^9, 3.523856111589815*^9, 3.5241952881550426`*^9, 
   3.5254842667734947`*^9, {3.525484301903129*^9, 3.525484307848443*^9}, {
   3.525484384638681*^9, 3.52548441234166*^9}, {3.525484499043209*^9, 
   3.525484505385235*^9}, {3.525484630012857*^9, 3.5254846426429462`*^9}, {
   3.5254847216790524`*^9, 3.525484734124528*^9}, 3.525484818489867*^9, 
   3.525484865718192*^9, 3.525484933506158*^9, {3.525565094335869*^9, 
   3.525565123040349*^9}, {3.52556515318436*^9, 3.525565182245829*^9}, 
   3.526265961723193*^9, 3.526265998620324*^9, 3.5262662646125507`*^9, 
   3.526269636885107*^9, 3.526269687616173*^9, 3.5262698350766172`*^9, 
   3.5262700593287687`*^9, 3.526270375282317*^9, 3.526270427025936*^9, {
   3.526270481891268*^9, 3.526270517879434*^9}, 3.5262705530382233`*^9, 
   3.526270590688048*^9, 3.5262706513507757`*^9, 3.5262710058271017`*^9, 
   3.526271148349676*^9, 3.526271222682184*^9, 3.5262712713895607`*^9, 
   3.5272871807322206`*^9, 3.527287910644376*^9, 3.527288364931965*^9, 
   3.52728846541965*^9, 3.527288541651824*^9, 3.527288891864675*^9, {
   3.527290365117353*^9, 3.5272903863609533`*^9}, 3.527290485817597*^9, 
   3.529273193891515*^9, 3.529273264276649*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.6161559999999966`", ",", 
   RowBox[{"-", "0.6412067309461899`"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5238525478355227`*^9, {3.523852604265719*^9, 3.523852612102681*^9}, 
   3.523855618304336*^9, 3.5238561119390373`*^9, 3.5241952885215073`*^9, 
   3.525484266779669*^9, {3.525484301906219*^9, 3.525484307852613*^9}, {
   3.52548438464172*^9, 3.525484412344819*^9}, {3.5254844990462713`*^9, 
   3.5254845053884153`*^9}, {3.525484630389093*^9, 3.525484643008451*^9}, {
   3.52548472204315*^9, 3.5254847344794817`*^9}, 3.52548481884594*^9, 
   3.52548486608364*^9, 3.52548493386012*^9, {3.5255650947105627`*^9, 
   3.525565123420583*^9}, {3.525565153548499*^9, 3.525565182614749*^9}, 
   3.526265961727632*^9, 3.526265998990752*^9, 3.526266265125485*^9, 
   3.5262696374061213`*^9, 3.526269688159833*^9, 3.526269835601342*^9, 
   3.52627005987431*^9, 3.526270375807373*^9, 3.526270427546919*^9, {
   3.526270482448186*^9, 3.5262705184294977`*^9}, 3.526270553602921*^9, 
   3.526270591242494*^9, 3.526270651887429*^9, 3.526271006366365*^9, 
   3.526271148878829*^9, 3.526271223222638*^9, 3.526271271922318*^9, 
   3.5272871807367783`*^9, 3.527287911259639*^9, 3.527288365530987*^9, 
   3.527288466020885*^9, 3.5272885422535677`*^9, 3.527288892476809*^9, {
   3.527290365719096*^9, 3.527290386972736*^9}, 3.5272904864334517`*^9, 
   3.529273227683714*^9, 3.529273264883089*^9}],

Cell[BoxData["\<\"Coaxial thin coil:\"\>"], "Print",
 CellChangeTimes->{
  3.523852547483646*^9, {3.523852603905986*^9, 3.52385261173208*^9}, 
   3.523855617935769*^9, 3.523856111589815*^9, 3.5241952881550426`*^9, 
   3.5254842667734947`*^9, {3.525484301903129*^9, 3.525484307848443*^9}, {
   3.525484384638681*^9, 3.52548441234166*^9}, {3.525484499043209*^9, 
   3.525484505385235*^9}, {3.525484630012857*^9, 3.5254846426429462`*^9}, {
   3.5254847216790524`*^9, 3.525484734124528*^9}, 3.525484818489867*^9, 
   3.525484865718192*^9, 3.525484933506158*^9, {3.525565094335869*^9, 
   3.525565123040349*^9}, {3.52556515318436*^9, 3.525565182245829*^9}, 
   3.526265961723193*^9, 3.526265998620324*^9, 3.5262662646125507`*^9, 
   3.526269636885107*^9, 3.526269687616173*^9, 3.5262698350766172`*^9, 
   3.5262700593287687`*^9, 3.526270375282317*^9, 3.526270427025936*^9, {
   3.526270481891268*^9, 3.526270517879434*^9}, 3.5262705530382233`*^9, 
   3.526270590688048*^9, 3.5262706513507757`*^9, 3.5262710058271017`*^9, 
   3.526271148349676*^9, 3.526271222682184*^9, 3.5262712713895607`*^9, 
   3.5272871807322206`*^9, 3.527287910644376*^9, 3.527288364931965*^9, 
   3.52728846541965*^9, 3.527288541651824*^9, 3.527288891864675*^9, {
   3.527290365117353*^9, 3.5272903863609533`*^9}, 3.527290485817597*^9, 
   3.529273193891515*^9, 3.529273264886038*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.0006609999999938054`", ",", 
   RowBox[{"-", "0.6503208565335336`"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5238525478355227`*^9, {3.523852604265719*^9, 3.523852612102681*^9}, 
   3.523855618304336*^9, 3.5238561119390373`*^9, 3.5241952885215073`*^9, 
   3.525484266779669*^9, {3.525484301906219*^9, 3.525484307852613*^9}, {
   3.52548438464172*^9, 3.525484412344819*^9}, {3.5254844990462713`*^9, 
   3.5254845053884153`*^9}, {3.525484630389093*^9, 3.525484643008451*^9}, {
   3.52548472204315*^9, 3.5254847344794817`*^9}, 3.52548481884594*^9, 
   3.52548486608364*^9, 3.52548493386012*^9, {3.5255650947105627`*^9, 
   3.525565123420583*^9}, {3.525565153548499*^9, 3.525565182614749*^9}, 
   3.526265961727632*^9, 3.526265998990752*^9, 3.526266265125485*^9, 
   3.5262696374061213`*^9, 3.526269688159833*^9, 3.526269835601342*^9, 
   3.52627005987431*^9, 3.526270375807373*^9, 3.526270427546919*^9, {
   3.526270482448186*^9, 3.5262705184294977`*^9}, 3.526270553602921*^9, 
   3.526270591242494*^9, 3.526270651887429*^9, 3.526271006366365*^9, 
   3.526271148878829*^9, 3.526271223222638*^9, 3.526271271922318*^9, 
   3.5272871807367783`*^9, 3.527287911259639*^9, 3.527288365530987*^9, 
   3.527288466020885*^9, 3.5272885422535677`*^9, 3.527288892476809*^9, {
   3.527290365719096*^9, 3.527290386972736*^9}, 3.5272904864334517`*^9, 
   3.529273227683714*^9, 3.529273264888836*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"{", 
      RowBox[{"thickfilamentparams", ",", 
       RowBox[{"Method", "\[Rule]", "\"\<Filament\>\""}]}], "}"}]}]}]}], "//.",
   "thickfilamentparams"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"{", 
      RowBox[{"thickfilamentparams", ",", 
       RowBox[{"Method", "\[Rule]", "\"\<Shell\>\""}]}], "}"}]}]}]}], "//.", 
  "thickfilamentparams"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"{", 
      RowBox[{"thickfilamentparams", ",", 
       RowBox[{"Method", "\[Rule]", "Automatic"}]}], "}"}]}]}]}], "//.", 
  "thickfilamentparams"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"{", 
      RowBox[{"thickfilamentparams", ",", 
       RowBox[{"Method", "\[Rule]", "\"\<Babic\>\""}]}], "}"}]}]}]}], "//.", 
  "thickfilamentparams"}]}], "Input",
 CellChangeTimes->{3.527290489377556*^9}],

Cell[BoxData["\<\"Coaxial thick coil (filament):\"\>"], "Print",
 CellChangeTimes->{3.5272904923735237`*^9, 3.529273293732924*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.5412589999999966`", ",", 
   RowBox[{"-", "0.47364450499161553`"}]}], "}"}]], "Output",
 CellChangeTimes->{3.527290493753304*^9, 3.529273295270402*^9}],

Cell[BoxData["\<\"Coaxial thick coil (shell):\"\>"], "Print",
 CellChangeTimes->{3.5272904923735237`*^9, 3.529273295273435*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.0016180000000005634`", ",", 
   RowBox[{"-", "0.49766947025856023`"}]}], "}"}]], "Output",
 CellChangeTimes->{3.527290493753304*^9, 3.529273295276308*^9}],

Cell[BoxData["\<\"Coaxial thick coil (plain integral):\"\>"], "Print",
 CellChangeTimes->{3.5272904923735237`*^9, 3.5292732952792997`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.012566999999990003`", ",", 
   RowBox[{"-", "0.4994396037183639`"}]}], "}"}]], "Output",
 CellChangeTimes->{3.527290493753304*^9, 3.529273295316861*^9}],

Cell[BoxData["\<\"Coaxial thick coil (Babic):\"\>"], "Print",
 CellChangeTimes->{3.5272904923735237`*^9, 3.529273295319785*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.041352999999993756`", ",", 
   RowBox[{"-", "0.4937842017924463`"}]}], "}"}]], "Output",
 CellChangeTimes->{3.527290493753304*^9, 3.529273295366543*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Eccentricity", "\[Rule]", 
        RowBox[{"0.5", "mm"}]}], ",", "thickfilamentparams", ",", 
       RowBox[{"Method", "\[Rule]", "\"\<Filament\>\""}]}], "}"}]}]}]}], "//.",
   "thickfilamentparams"}]], "Input",
 CellChangeTimes->{{3.5272904972979116`*^9, 3.5272905320863447`*^9}, 
   3.527292290809004*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"445.860204`", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.47436450086071996`"}], ",", "0.002842605982187522`"}], 
    "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5272909788632717`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Timing", "@", 
   RowBox[{"MagnetCoilForce", "@", 
    RowBox[{"Flatten", "@", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Eccentricity", "\[Rule]", 
        RowBox[{"0.5", "mm"}]}], ",", "thickfilamentparams", ",", 
       RowBox[{"Method", "\[Rule]", "Automatic"}]}], "}"}]}]}]}], "//.", 
  "thickfilamentparams"}]], "Input",
 CellChangeTimes->{
  3.527292295661722*^9, {3.527292360566718*^9, 3.527292374619615*^9}}],

Cell[BoxData["\<\"Eccentric thick coil:\"\>"], "Print",
 CellChangeTimes->{
  3.527290978866765*^9, 3.5272922962335567`*^9, {3.527292345509901*^9, 
   3.527292375906945*^9}, 3.527292443133279*^9, 3.529273311681015*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.029163999999994417`", ",", 
   RowBox[{"-", "0.49970984038137034`"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5272909788700333`*^9, 3.527292296237958*^9, {3.527292345717753*^9, 
   3.5272923760983973`*^9}, 3.5272924433427343`*^9, 3.5292733117107353`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Thick coil speed tests", "Section",
 CellChangeTimes->{{3.523933758993113*^9, 3.523933772415608*^9}, {
  3.5254842405564203`*^9, 3.525484240812368*^9}, {3.527289029191101*^9, 
  3.5272890325828753`*^9}}],

Cell[BoxData[
 RowBox[{"babicparams", ":=", 
  RowBox[{"{", "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"MagnetRadius", "\[Rule]", 
     RowBox[{"9", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"MagnetLength", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"Magnetisation", "\[Rule]", 
     RowBox[{"1", " ", "tesla"}]}], ",", "\[IndentingNewLine]", 
    "\[IndentingNewLine]", 
    RowBox[{"CoilRadius", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilThickness", "\[Rule]", 
     RowBox[{"1", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilLength", "\[Rule]", 
     RowBox[{"20", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilTurnsR", "\[Rule]", "10"}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilTurnsZ", "\[Rule]", "40"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Current", "\[Rule]", "1"}], ",", "\[IndentingNewLine]", 
    "\[IndentingNewLine]", 
    RowBox[{"Displacement", "\[Rule]", 
     RowBox[{"10", "mm"}]}]}], "\[IndentingNewLine]", "}"}]}]], "Input",
 CellChangeTimes->{{3.523933798909704*^9, 3.5239338332892857`*^9}, {
   3.5239339167324343`*^9, 3.5239339171070347`*^9}, {3.527289085762452*^9, 
   3.527289097502639*^9}, 3.529273344380887*^9, {3.5292734102088223`*^9, 
   3.5292734104735126`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Timing", "@", 
  RowBox[{"MagnetCoilForce", "@", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"babicparams", ",", "\[IndentingNewLine]", 
     RowBox[{"Method", "\[Rule]", "\"\<Babic\>\""}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Verbose", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"IntegrationPrecision", "\[Rule]", "3"}]}], 
    "\[IndentingNewLine]", "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Timing", "@", 
  RowBox[{"MagnetCoilForce", "@", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"babicparams", ",", "\[IndentingNewLine]", 
     RowBox[{"Method", "\[Rule]", "Automatic"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Verbose", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"IntegrationPrecision", "\[Rule]", "3"}]}], 
    "\[IndentingNewLine]", "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Timing", "@", 
  RowBox[{"MagnetCoilForce", "@", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"babicparams", ",", "\[IndentingNewLine]", 
     RowBox[{"Method", "\[Rule]", "\"\<Shell\>\""}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Verbose", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"IntegrationPrecision", "\[Rule]", "3"}]}], 
    "\[IndentingNewLine]", "}"}]}]}]}], "Input",
 CellChangeTimes->{{3.523933797080801*^9, 3.523933863235026*^9}, {
   3.5239339190704737`*^9, 3.5239339218989563`*^9}, {3.526266305760298*^9, 
   3.5262663127284184`*^9}, 3.526267340659425*^9, {3.526267467485949*^9, 
   3.52626747101441*^9}, {3.527289047575961*^9, 3.52728905054307*^9}}],

Cell[BoxData["\<\"Coaxial thick coil (Babic):\"\>"], "Print",
 CellChangeTimes->{{3.5262663983912582`*^9, 3.526266424603766*^9}, 
   3.526266497806695*^9, 3.526266528176551*^9, {3.526266560383522*^9, 
   3.5262665899375963`*^9}, 3.5262668086322737`*^9, 3.52626720390213*^9, {
   3.5262673352518578`*^9, 3.526267341530731*^9}, {3.5262674644452744`*^9, 
   3.5262674717121077`*^9}, 3.5272813188686447`*^9, {3.527289040169148*^9, 
   3.527289051222824*^9}, {3.527289102169074*^9, 3.527289113351136*^9}, 
   3.529273349312129*^9, 3.529273412409546*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.041029999999999234`", ",", 
   RowBox[{"-", "2.4544407879895993`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.523933835257119*^9, 3.523933863823681*^9}, 
   3.523933922529394*^9, 3.525483544717436*^9, 3.5254840000233173`*^9, 
   3.5254842469597197`*^9, 3.5262663318983383`*^9, {3.5262663984591017`*^9, 
   3.526266424666884*^9}, 3.52626649785734*^9, 3.5262665282406473`*^9, {
   3.5262665604312067`*^9, 3.526266589989442*^9}, 3.5262668086276484`*^9, 
   3.526267203962393*^9, {3.526267335300452*^9, 3.526267341578497*^9}, {
   3.5262674644932013`*^9, 3.5262674717656593`*^9}, 3.527281319027768*^9, {
   3.527289040207909*^9, 3.527289051261503*^9}, {3.527289102199111*^9, 
   3.52728911340541*^9}, 3.529273349343074*^9, 3.5292734124468517`*^9}],

Cell[BoxData["\<\"Coaxial thick coil (plain integral):\"\>"], "Print",
 CellChangeTimes->{{3.5262663983912582`*^9, 3.526266424603766*^9}, 
   3.526266497806695*^9, 3.526266528176551*^9, {3.526266560383522*^9, 
   3.5262665899375963`*^9}, 3.5262668086322737`*^9, 3.52626720390213*^9, {
   3.5262673352518578`*^9, 3.526267341530731*^9}, {3.5262674644452744`*^9, 
   3.5262674717121077`*^9}, 3.5272813188686447`*^9, {3.527289040169148*^9, 
   3.527289051222824*^9}, {3.527289102169074*^9, 3.527289113351136*^9}, 
   3.529273349312129*^9, 3.529273412449684*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.014642999999999518`", ",", 
   RowBox[{"-", "2.4548594892044457`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.523933835257119*^9, 3.523933863823681*^9}, 
   3.523933922529394*^9, 3.525483544717436*^9, 3.5254840000233173`*^9, 
   3.5254842469597197`*^9, 3.5262663318983383`*^9, {3.5262663984591017`*^9, 
   3.526266424666884*^9}, 3.52626649785734*^9, 3.5262665282406473`*^9, {
   3.5262665604312067`*^9, 3.526266589989442*^9}, 3.5262668086276484`*^9, 
   3.526267203962393*^9, {3.526267335300452*^9, 3.526267341578497*^9}, {
   3.5262674644932013`*^9, 3.5262674717656593`*^9}, 3.527281319027768*^9, {
   3.527289040207909*^9, 3.527289051261503*^9}, {3.527289102199111*^9, 
   3.52728911340541*^9}, 3.529273349343074*^9, 3.529273412485214*^9}],

Cell[BoxData["\<\"Coaxial thick coil (shell):\"\>"], "Print",
 CellChangeTimes->{{3.5262663983912582`*^9, 3.526266424603766*^9}, 
   3.526266497806695*^9, 3.526266528176551*^9, {3.526266560383522*^9, 
   3.5262665899375963`*^9}, 3.5262668086322737`*^9, 3.52626720390213*^9, {
   3.5262673352518578`*^9, 3.526267341530731*^9}, {3.5262674644452744`*^9, 
   3.5262674717121077`*^9}, 3.5272813188686447`*^9, {3.527289040169148*^9, 
   3.527289051222824*^9}, {3.527289102169074*^9, 3.527289113351136*^9}, 
   3.529273349312129*^9, 3.5292734124875402`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.0028319999999979473`", ",", 
   RowBox[{"-", "2.454751529717687`"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.523933835257119*^9, 3.523933863823681*^9}, 
   3.523933922529394*^9, 3.525483544717436*^9, 3.5254840000233173`*^9, 
   3.5254842469597197`*^9, 3.5262663318983383`*^9, {3.5262663984591017`*^9, 
   3.526266424666884*^9}, 3.52626649785734*^9, 3.5262665282406473`*^9, {
   3.5262665604312067`*^9, 3.526266589989442*^9}, 3.5262668086276484`*^9, 
   3.526267203962393*^9, {3.526267335300452*^9, 3.526267341578497*^9}, {
   3.5262674644932013`*^9, 3.5262674717656593`*^9}, 3.527281319027768*^9, {
   3.527289040207909*^9, 3.527289051261503*^9}, {3.527289102199111*^9, 
   3.52728911340541*^9}, 3.529273349343074*^9, 3.529273412490273*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"babicAccuracy", " ", "=", " ", 
  RowBox[{"Table", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Flatten", "@", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Timing", "@", 
        RowBox[{"MagnetCoilForce", "@", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{"babicparams", ",", "\[IndentingNewLine]", 
           RowBox[{"Method", "\[Rule]", "\"\<Babic\>\""}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"IntegrationPrecision", "\[Rule]", "ii"}]}], 
          "\[IndentingNewLine]", "}"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"Timing", "@", 
        RowBox[{"MagnetCoilForce", "@", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{"babicparams", ",", "\[IndentingNewLine]", 
           RowBox[{"IntegrationPrecision", "\[Rule]", "ii"}]}], 
          "\[IndentingNewLine]", "}"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"Timing", "@", 
        RowBox[{"MagnetCoilForce", "@", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{"babicparams", ",", "\[IndentingNewLine]", 
           RowBox[{"Method", "\[Rule]", "\"\<Shell\>\""}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"IntegrationPrecision", "\[Rule]", "ii"}]}], 
          "\[IndentingNewLine]", "}"}]}]}]}], "}"}]}], "\[IndentingNewLine]", 
    ",", 
    RowBox[{"{", 
     RowBox[{"ii", ",", "1", ",", "8"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.523933877176345*^9, 3.523933908455961*^9}, {
   3.523933944865303*^9, 3.523933992620533*^9}, {3.523936989457852*^9, 
   3.523936991028719*^9}, {3.523937042403*^9, 3.523937064864964*^9}, {
   3.524192827509233*^9, 3.52419282756211*^9}, {3.524192860010481*^9, 
   3.524192860138134*^9}, 3.5255761125068417`*^9, 3.525576197698316*^9, {
   3.525576233005891*^9, 3.525576246914956*^9}, 3.5255778106500893`*^9, {
   3.526266317654663*^9, 3.526266321517255*^9}, 3.52626734702938*^9, {
   3.5272891735615473`*^9, 3.527289209991702*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.0392879999999991`", ",", 
     RowBox[{"-", "2.4544407879895993`"}], ",", "0.011755999999998323`", ",", 
     
     RowBox[{"-", "2.4744006907978187`"}], ",", "0.0025880000000100267`", ",", 
     RowBox[{"-", "2.454751529717687`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.037703000000000486`", ",", 
     RowBox[{"-", "2.4544407879895993`"}], ",", "0.013560000000005346`", ",", 
     
     RowBox[{"-", "2.4548594892044457`"}], ",", "0.002605000000002633`", ",", 
     
     RowBox[{"-", "2.454751529717687`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.03750699999999796`", ",", 
     RowBox[{"-", "2.4544407879895993`"}], ",", "0.013380000000005055`", ",", 
     
     RowBox[{"-", "2.4548594892044457`"}], ",", "0.002628000000001407`", ",", 
     
     RowBox[{"-", "2.454751529717687`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.03787799999999919`", ",", 
     RowBox[{"-", "2.4544438306124783`"}], ",", "0.020196000000005654`", ",", 
     
     RowBox[{"-", "2.4544392729491915`"}], ",", "0.002609000000006745`", ",", 
     
     RowBox[{"-", "2.454751529717687`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.037814000000004455`", ",", 
     RowBox[{"-", "2.4544438306124783`"}], ",", "0.023744000000000653`", ",", 
     
     RowBox[{"-", "2.454441045827852`"}], ",", "0.00263599999999542`", ",", 
     RowBox[{"-", "2.454751529717687`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.03873899999999253`", ",", 
     RowBox[{"-", "2.4544438296675`"}], ",", "0.03556199999999876`", ",", 
     RowBox[{"-", "2.454443786446628`"}], ",", "0.002631000000000938`", ",", 
     RowBox[{"-", "2.454751529717687`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.03991200000000106`", ",", 
     RowBox[{"-", "2.454443830093919`"}], ",", "0.05817499999999853`", ",", 
     RowBox[{"-", "2.4544438175568843`"}], ",", "0.002668000000006998`", ",", 
     
     RowBox[{"-", "2.454751529717687`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.04130100000000425`", ",", 
     RowBox[{"-", "2.4544438300903315`"}], ",", "0.14190800000000792`", ",", 
     RowBox[{"-", "2.4544438299997147`"}], ",", "0.0028160000000028163`", ",", 
     RowBox[{"-", "2.454751529717687`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5239339108876543`*^9, 3.523933956747155*^9}, 
   3.5239339971975317`*^9, 3.523936993372311*^9, {3.523937048540194*^9, 
   3.523937067005578*^9}, 3.524192830655697*^9, 3.5241928624155083`*^9, 
   3.52419345966777*^9, 3.525576115678053*^9, 3.525576200480741*^9, {
   3.525576235134819*^9, 3.5255762490634527`*^9}, 3.525577811643468*^9, 
   3.526267349128537*^9, 3.5272813221084023`*^9, 3.52728921781538*^9, 
   3.529273377553515*^9, 3.529273415674164*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"Map", "[", 
   RowBox[{"InputForm", ",", "babicAccuracy", ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.523937069892033*^9, 3.5239370803654423`*^9}, {
   3.5241916804722548`*^9, 3.524191719416551*^9}, 3.524191752537075*^9, {
   3.524191826212634*^9, 3.524191839379574*^9}, {3.524191941035555*^9, 
   3.524192041677874*^9}}],

Cell[BoxData[
 TagBox[GridBox[{
    {
     InterpretationBox[
      StyleBox["0.0392879999999991",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.0392879999999991],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544407879895993"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544407879895993`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.011755999999998323",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.011755999999998323`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4744006907978187"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4744006907978187`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.0025880000000100267",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.0025880000000100267`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454751529717687"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454751529717687],
      AutoDelete->True,
      Editable->True]},
    {
     InterpretationBox[
      StyleBox["0.037703000000000486",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.037703000000000486`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544407879895993"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544407879895993`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.013560000000005346",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.013560000000005346`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4548594892044457"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4548594892044457`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.002605000000002633",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.002605000000002633],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454751529717687"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454751529717687],
      AutoDelete->True,
      Editable->True]},
    {
     InterpretationBox[
      StyleBox["0.03750699999999796",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.03750699999999796],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544407879895993"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544407879895993`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.013380000000005055",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.013380000000005055`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4548594892044457"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4548594892044457`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.002628000000001407",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.002628000000001407],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454751529717687"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454751529717687],
      AutoDelete->True,
      Editable->True]},
    {
     InterpretationBox[
      StyleBox["0.03787799999999919",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.03787799999999919],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544438306124783"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544438306124783`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.020196000000005654",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.020196000000005654`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544392729491915"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544392729491915`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.002609000000006745",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.002609000000006745],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454751529717687"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454751529717687],
      AutoDelete->True,
      Editable->True]},
    {
     InterpretationBox[
      StyleBox["0.037814000000004455",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.037814000000004455`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544438306124783"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544438306124783`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.023744000000000653",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.023744000000000653`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454441045827852"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454441045827852],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.00263599999999542",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.00263599999999542],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454751529717687"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454751529717687],
      AutoDelete->True,
      Editable->True]},
    {
     InterpretationBox[
      StyleBox["0.03873899999999253",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.03873899999999253],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544438296675"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544438296675],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.03556199999999876",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.03556199999999876],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454443786446628"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454443786446628],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.002631000000000938",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.002631000000000938],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454751529717687"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454751529717687],
      AutoDelete->True,
      Editable->True]},
    {
     InterpretationBox[
      StyleBox["0.03991200000000106",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.03991200000000106],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454443830093919"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454443830093919],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.05817499999999853",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.05817499999999853],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544438175568843"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544438175568843`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.002668000000006998",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.002668000000006998],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454751529717687"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454751529717687],
      AutoDelete->True,
      Editable->True]},
    {
     InterpretationBox[
      StyleBox["0.04130100000000425",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.04130100000000425],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544438300903315"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544438300903315`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.14190800000000792",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.14190800000000792`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.4544438299997147"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.4544438299997147`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox["0.0028160000000028163",
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[0.0028160000000028163`],
      AutoDelete->True,
      Editable->True], 
     InterpretationBox[
      StyleBox[
       RowBox[{"-", "2.454751529717687"}],
       ShowStringCharacters->True,
       NumberMarks->True],
      InputForm[-2.454751529717687],
      AutoDelete->True,
      Editable->True]}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.523937080791597*^9, {3.524191686137579*^9, 3.5241917199402122`*^9}, 
   3.524191752886895*^9, {3.524191827279922*^9, 3.524191839765527*^9}, {
   3.524191944243614*^9, 3.524192042102148*^9}, 3.525576118544961*^9, 
   3.525576348755513*^9, 3.5255778132000237`*^9, 3.527289222422037*^9, 
   3.529273380957552*^9, 3.529273418216312*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"100", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"2.454751529717687", "-", 
     StyleBox["2.4544438300903315",
      ShowStringCharacters->True,
      NumberMarks->True]}], 
    StyleBox[")",
     ShowStringCharacters->True,
     NumberMarks->True]}], 
   StyleBox["/",
    ShowStringCharacters->True,
    NumberMarks->True], 
   StyleBox["2.4544438300903315",
    ShowStringCharacters->True,
    NumberMarks->True]}]}]], "Input",
 CellChangeTimes->{{3.527289373358665*^9, 3.527289387038608*^9}, {
  3.527289449455986*^9, 3.5272894608118563`*^9}}],

Cell[BoxData["0.012536429784344081`"], "Output",
 CellChangeTimes->{
  3.5272893878937073`*^9, {3.527289457440477*^9, 3.527289461293065*^9}, 
   3.529273422133131*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"1000000", 
      RowBox[{"Abs", "@", 
       RowBox[{"{", 
        RowBox[{"1", "-", 
         RowBox[{
          RowBox[{"babicAccuracy", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "2"}], "]"}], "]"}], "/", 
          RowBox[{"babicAccuracy", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "2"}], "]"}], "]"}]}]}], "}"}]}]}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "0.8"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", "Blue", "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "11"}], "}"}], ",", "Automatic"}], "}"}]}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"Frame", "\[Rule]", "True"}], ",", 
     RowBox[{"Axes", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<Integration precision\>\"", ",", 
        "\"\<Relative difference per million\>\""}], "}"}]}]}], "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"1000000", 
      RowBox[{"Abs", "@", 
       RowBox[{"{", 
        RowBox[{"1", "-", 
         RowBox[{
          RowBox[{"babicAccuracy", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "4"}], "]"}], "]"}], "/", 
          RowBox[{"babicAccuracy", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "4"}], "]"}], "]"}]}]}], "}"}]}]}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "0.8"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", "Orange", "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "11"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "300"}], "}"}]}], "}"}]}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"Frame", "\[Rule]", "True"}], ",", 
     RowBox[{"Axes", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<Integration precision\>\"", ",", 
        "\"\<Relative difference per million\>\""}], "}"}]}]}], "]"}]}], 
  "}"}]], "Input",
 CellChangeTimes->{{3.524192700994379*^9, 3.524192812640924*^9}, {
  3.524192870148778*^9, 3.524192982329918*^9}, {3.524193325748184*^9, 
  3.524193328858032*^9}, {3.524193375868857*^9, 3.5241933973278713`*^9}, {
  3.524193439508972*^9, 3.524193442298009*^9}, {3.5255762671086063`*^9, 
  3.525576315273584*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    GraphicsComplexBox[{{1., 1.239425687860063}, {2., 1.239425687860063}, {3.,
      1.239425687860063}, {4., 0.00021273516281894445`}, {5., 
     0.00021273516281894445`}, {6., 0.00017227186344115353`}, {7., 
     1.461497589616556*^-6}, {8., 0.}}, {{{}, {}, 
       {RGBColor[0, 0, 1], LineBox[{1, 2, 3, 4, 5, 6, 7, 8}]}}, {
       {RGBColor[0, 0, 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 8]}, {}}}],
    AspectRatio->0.8,
    AxesOrigin->{0, 0},
    Frame->True,
    FrameLabel->{
      FormBox["\"Integration precision\"", TraditionalForm], 
      FormBox["\"Relative difference per million\"", TraditionalForm]},
    PlotRange->{{0, 11}, {0., 1.239425687860063}},
    PlotRangeClipping->True,
    PlotRangePadding->{Automatic, 
      Scaled[0.02]}], ",", 
   GraphicsBox[
    GraphicsComplexBox[{{1., 300.}, {2., 169.34965047910032`}, {3., 
     169.34965047910032`}, {4., 1.8566530093355738`}, {5., 
     1.134339205011159}, {6., 0.01774458469050444}, {7., 
     0.005069511144206729}, {8., 0.}, {1.9835898553762616`, 
     300.}}, {{{}, {}, 
       {RGBColor[1, 0.5, 0], LineBox[{9, 2, 3, 4, 5, 6, 7, 8}]}}, {
       {RGBColor[1, 0.5, 0], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->8.96], 9]}, {}}}],
    AspectRatio->0.8,
    AxesOrigin->{0, 0},
    Frame->True,
    FrameLabel->{
      FormBox["\"Integration precision\"", TraditionalForm], 
      FormBox["\"Relative difference per million\"", TraditionalForm]},
    PlotRange->{{0, 11}, {0., 300}},
    PlotRangeClipping->True,
    PlotRangePadding->{Automatic, {
       Scaled[0.02], Automatic}}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5241927092678003`*^9, 3.524192833545555*^9}, {
   3.524192864189823*^9, 3.52419298384903*^9}, 3.524193329519252*^9, {
   3.524193388242403*^9, 3.524193397856143*^9}, {3.5241934429462852`*^9, 
   3.524193463849765*^9}, 3.525576121478661*^9, 3.5255762028434134`*^9, {
   3.525576236327098*^9, 3.525576316402954*^9}, 3.525577815051529*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"lx", ":=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "1.5", ",", "2", ",", "2.5"}], 
   "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ly", "=", 
   RowBox[{"{", 
    RowBox[{"120", ",", "100"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"prectiming", "=", 
  RowBox[{"Show", "@", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"1000", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"babicAccuracy", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "1"}], "]"}], "]"}], ",", 
          RowBox[{"babicAccuracy", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "3"}], "]"}], "]"}]}], "}"}]}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Blue", ",", "Orange"}], "}"}]}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "9"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "150"}], "}"}]}], "}"}]}], ",", 
       RowBox[{"Frame", "\[Rule]", "True"}], ",", 
       RowBox[{"FrameTicks", "->", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Range", "[", 
           RowBox[{"1", ",", "8"}], "]"}], ",", "Automatic", ",", "None", ",",
           "None"}], "}"}]}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", 
       RowBox[{"AspectRatio", "\[Rule]", "0.8"}], ",", 
       RowBox[{"ImageSize", "\[Rule]", 
        RowBox[{"6", " ", 
         RowBox[{"72", "/", "2.54"}]}]}], ",", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<Integration precision\>\"", ",", "\"\<Time, ms\>\""}], 
         "}"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], "}"}]}], "}"}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Blue", ",", "Orange"}], "}"}]}], ",", 
       RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<Babic\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"lx", "[", 
           RowBox[{"[", "4", "]"}], "]"}], ",", 
          RowBox[{"ly", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\<Integral\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"lx", "[", 
           RowBox[{"[", "4", "]"}], "]"}], ",", 
          RowBox[{"ly", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "}"}]}]}]}], "Input",
 CellChangeTimes->{{3.5241920963135977`*^9, 3.524192205718938*^9}, {
   3.524192289192164*^9, 3.5241926696659184`*^9}, {3.524193358172811*^9, 
   3.52419336114475*^9}, {3.525576693874839*^9, 3.525576695999165*^9}, {
   3.5255767269824247`*^9, 3.525576738416523*^9}, {3.525576846268918*^9, 
   3.525576849701084*^9}, {3.525576904996525*^9, 3.525576959427841*^9}, {
   3.525577006516293*^9, 3.5255770608969603`*^9}, {3.525577093506308*^9, 
   3.525577093805023*^9}, {3.5255771264364567`*^9, 3.525577137422752*^9}, {
   3.525577250352023*^9, 3.5255774020662518`*^9}, 3.5255777012196712`*^9, {
   3.5255777891442623`*^9, 3.525577825694035*^9}, {3.525577858378323*^9, 
   3.525577867871244*^9}, {3.525577945249748*^9, 3.525577959186408*^9}, 
   3.527281291349991*^9, {3.527281500572044*^9, 3.527281523325819*^9}, {
   3.527281606249009*^9, 3.527281682278696*^9}, {3.527281717272607*^9, 
   3.527281834479135*^9}, {3.52728223032327*^9, 3.527282234561103*^9}, {
   3.527282286386251*^9, 3.5272822958089123`*^9}}],

Cell[BoxData[
 GraphicsBox[{
   GraphicsComplexBox[{{1., 37.666999999984796`}, {2., 36.82399999999575}, {
    3., 36.999000000008664`}, {4., 37.67100000000312}, {5., 
    37.07799999999395}, {6., 38.22200000001885}, {7., 38.63100000000941}, {8.,
     40.55200000000525}, {1., 11.778999999989992`}, {2., 
    13.348000000007687`}, {3., 13.334000000000401`}, {4., 
    20.187999999990325`}, {5., 23.61099999998828}, {6., 
    35.481000000004315`}, {7., 57.6640000000026}, {8., 
    140.67099999999755`}}, {{{}, {}, 
      {RGBColor[0, 0, 1], LineBox[{1, 2, 3, 4, 5, 6, 7, 8}]}, 
      {RGBColor[1, 0.5, 0], LineBox[{9, 10, 11, 12, 13, 14, 15, 16}]}}, {
      {RGBColor[0, 0, 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 2], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 3], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 4], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 5], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 7], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 8]}, 
      {RGBColor[1, 0.5, 0], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 9], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 10], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 11], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 12], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 13], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 14], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 15], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 16]}, {}}}], 
   GraphicsComplexBox[{{1., 120.}, {1.5, 120.}, {2., 120.}, {1., 100.}, {1.5, 
    100.}, {2., 100.}}, {{{}, {}, 
      {RGBColor[0, 0, 1], LineBox[{1, 2, 3}]}, 
      {RGBColor[1, 0.5, 0], LineBox[{4, 5, 6}]}}, {
      {RGBColor[0, 0, 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 2], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 3]}, 
      {RGBColor[1, 0.5, 0], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 4], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 5], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 6]}, {}}}], 
   InsetBox["\<\"Babic\"\>", {2.5, 120}, {-1, 0}], 
   InsetBox["\<\"Integral\"\>", {2.5, 100}, {-1, 0}]},
  AspectRatio->0.8,
  AxesOrigin->{0, 0},
  Frame->True,
  FrameLabel->{
    FormBox["\"Integration precision\"", TraditionalForm], 
    FormBox["\"Time, ms\"", TraditionalForm]},
  FrameTicks->{{{1, 
      FormBox["1", TraditionalForm]}, {2, 
      FormBox["2", TraditionalForm]}, {3, 
      FormBox["3", TraditionalForm]}, {4, 
      FormBox["4", TraditionalForm]}, {5, 
      FormBox["5", TraditionalForm]}, {6, 
      FormBox["6", TraditionalForm]}, {7, 
      FormBox["7", TraditionalForm]}, {8, 
      FormBox["8", TraditionalForm]}}, Automatic, None, None},
  ImageSize->170.0787401574803,
  PlotRange->{{0, 9}, {0, 150}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.5241921212743597`*^9, 3.524192206070608*^9}, {
   3.5241922905275803`*^9, 3.524192356788471*^9}, {3.524192388874138*^9, 
   3.5241926624367228`*^9}, 3.524193361796977*^9, 3.524193465710586*^9, {
   3.525576675485058*^9, 3.525576739122881*^9}, 3.525576851538769*^9, {
   3.5255769229843082`*^9, 3.525576960262088*^9}, 3.525577016976837*^9, {
   3.525577047815624*^9, 3.525577061734036*^9}, 3.525577094404725*^9, {
   3.525577129018033*^9, 3.525577138184057*^9}, 3.525577329811857*^9, {
   3.525577379665477*^9, 3.5255774024045353`*^9}, 3.5255777040793324`*^9, {
   3.525577791340868*^9, 3.5255778263655157`*^9}, 3.525577868851832*^9, {
   3.525577949016912*^9, 3.525577959543396*^9}, 3.5272813273363256`*^9, 
   3.5272815250066023`*^9, 3.5272815685408*^9, {3.5272816098893547`*^9, 
   3.527281634059168*^9}, 3.527281683140765*^9, {3.527281726601551*^9, 
   3.527281779395158*^9}, {3.527281815293522*^9, 3.527281835142164*^9}, 
   3.527282235173607*^9, 3.527282296637597*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PSfragExport", "[", 
  RowBox[{"\"\<fig/prec-timing\>\"", ",", "prectiming", ",", 
   RowBox[{"RenumberTags", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.525576701334702*^9, 3.5255767055081997`*^9}, {
  3.525577910000146*^9, 3.525577930272477*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"fig/prec-timing\"\>", 
   ",", "\<\"fig/prec-timing-psfrag.eps\"\>", 
   ",", "\<\"fig/prec-timing-psfrag.tex\"\>"}], "}"}]], "Output",
 CellChangeTimes->{3.5255767115722227`*^9, 3.5255771102843313`*^9, 
  3.525577150485243*^9, 3.525577414912546*^9, 3.525577715993247*^9, 
  3.525577837993949*^9, 3.52557788405346*^9, 3.525577927601746*^9, 
  3.525577973564766*^9, 3.527281852492577*^9, 3.5272822453973827`*^9, 
  3.527282307435767*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Thin coil comparison", "Section",
 CellChangeTimes->{{3.529631448257457*^9, 3.529631449960298*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"filamentparams", "=", 
   RowBox[{"{", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"MagnetRadius", "\[Rule]", 
      RowBox[{"9", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"MagnetLength", "\[Rule]", 
      RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Magnetisation", "\[Rule]", "1"}], ",", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"CoilRadius", "\[Rule]", 
      RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"CoilLength", "\[Rule]", 
      RowBox[{"20", "mm"}]}], ",", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"CoilTurns", "\[Rule]", "40"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Current", "\[Rule]", "1"}]}], "\[IndentingNewLine]", 
    "\[IndentingNewLine]", "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5272870221200867`*^9, 3.5272870438481703`*^9}, {
  3.529272815888968*^9, 3.529272835962307*^9}}],

Cell["Single value test:", "Text",
 CellChangeTimes->{{3.525575684746497*^9, 3.5255756887692223`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"MagnetCoilForce", "@", 
   RowBox[{"Flatten", "@", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Displacement", "\[Rule]", 
       RowBox[{"10", "mm"}]}], ",", 
      RowBox[{"Method", "\[Rule]", "\"\<Filament\>\""}], ",", 
      "filamentparams"}], "}"}]}]}], "//.", 
  "filamentparams"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForce", "@", 
   RowBox[{"Flatten", "@", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Displacement", "\[Rule]", 
       RowBox[{"10", "mm"}]}], ",", "filamentparams"}], "}"}]}]}], "//.", 
  "filamentparams"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForce", "@", 
   RowBox[{"Flatten", "@", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Displacement", "\[Rule]", 
       RowBox[{"10", "mm"}]}], ",", 
      RowBox[{"CoilThickness", "\[Rule]", 
       RowBox[{"0.5", "mm"}]}], ",", "filamentparams"}], "}"}]}]}], "//.", 
  "filamentparams"}]}], "Input",
 CellChangeTimes->{{3.525565997227015*^9, 3.525566042779338*^9}, {
  3.527287675088017*^9, 3.527287692616922*^9}, {3.527287765851552*^9, 
  3.527287781058097*^9}, {3.5272878161049547`*^9, 3.527287816585288*^9}}],

Cell[BoxData[
 RowBox[{"-", "0.25096957575971945`"}]], "Output",
 CellChangeTimes->{
  3.523852085855826*^9, 3.523855586664403*^9, {3.5254845847641697`*^9, 
   3.525484610764722*^9}, {3.525566009123337*^9, 3.52556604392514*^9}, {
   3.526266065299343*^9, 3.5262660885617533`*^9}, 3.527287702305901*^9, {
   3.527287750478094*^9, 3.527287758463147*^9}, 3.527287970203721*^9, 
   3.527288725469844*^9, 3.52728880484662*^9, {3.529272845923213*^9, 
   3.529272858877344*^9}, 3.52927343832587*^9}],

Cell[BoxData[
 RowBox[{"-", "0.26012834261341344`"}]], "Output",
 CellChangeTimes->{
  3.523852085855826*^9, 3.523855586664403*^9, {3.5254845847641697`*^9, 
   3.525484610764722*^9}, {3.525566009123337*^9, 3.52556604392514*^9}, {
   3.526266065299343*^9, 3.5262660885617533`*^9}, 3.527287702305901*^9, {
   3.527287750478094*^9, 3.527287758463147*^9}, 3.527287970203721*^9, 
   3.527288725469844*^9, 3.52728880484662*^9, {3.529272845923213*^9, 
   3.529272858877344*^9}, 3.529273438329406*^9}],

Cell[BoxData[
 RowBox[{"-", "0.25266565004805713`"}]], "Output",
 CellChangeTimes->{
  3.523852085855826*^9, 3.523855586664403*^9, {3.5254845847641697`*^9, 
   3.525484610764722*^9}, {3.525566009123337*^9, 3.52556604392514*^9}, {
   3.526266065299343*^9, 3.5262660885617533`*^9}, 3.527287702305901*^9, {
   3.527287750478094*^9, 3.527287758463147*^9}, 3.527287970203721*^9, 
   3.527288725469844*^9, 3.52728880484662*^9, {3.529272845923213*^9, 
   3.529272858877344*^9}, 3.529273438368909*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"xmax", ":=", 
  RowBox[{"40", "mm"}]}], "\[IndentingNewLine]", 
 RowBox[{"dx", ":=", 
  RowBox[{"1", "mm"}]}], "\[IndentingNewLine]", 
 RowBox[{"Dx", ":=", 
  RowBox[{"2", "dx"}]}], "\[IndentingNewLine]", 
 RowBox[{"thinthin1", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1000", "x"}], ",", 
      RowBox[{"MagnetCoilForce", "@", 
       RowBox[{"Flatten", "@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
          RowBox[{"Method", "\[Rule]", "\"\<Filament\>\""}], ",", 
          "filamentparams"}], "}"}]}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "dx", ",", "xmax", ",", "Dx"}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"thinthin2", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1000", "x"}], ",", 
      RowBox[{"MagnetCoilForce", "@", 
       RowBox[{"Flatten", "@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Displacement", "\[Rule]", "x"}], ",", "filamentparams"}], 
         "}"}]}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "xmax", ",", "dx"}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"thinthin3", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1000", "x"}], ",", 
       RowBox[{
        RowBox[{"MagnetCoilForce", "@", 
         RowBox[{"Flatten", "@", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
            RowBox[{"CoilThickness", "\[Rule]", 
             RowBox[{"0.5", "mm"}]}], ",", "filamentparams"}], "}"}]}]}], "//.",
         "thickfilamentparams"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"2", "dx"}], ",", "xmax", ",", "Dx"}], "}"}]}], "]"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"thinthin4", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1000", "x"}], ",", 
        RowBox[{
         RowBox[{"MagnetCoilForce", "@", 
          RowBox[{"Flatten", "@", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
             RowBox[{"CoilThickness", "\[Rule]", 
              RowBox[{"1", "mm"}]}], ",", "filamentparams"}], "}"}]}]}], "//.",
          "thickfilamentparams"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{"3", "dx"}], ",", "xmax", ",", "Dx"}], "}"}]}], "]"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBmIQ7VJR8e+u4ytH3z02zPeAtNDZST4g+pJTcgqI3ic/
KRdEG7xvzgfRv315S0D0pAkbwTSb/KF6EH1YJ60VRKsfkJwEouNrIsC03Kaw
AyD62+8XYLrx7KrV2s6vHJfN/7wGTGtu2Qmid/Is3AWiOU75iugAaS3dTDEQ
vVak7TSI/pc2E0zLhu9rCHZ55ch96QCYLpnmdL0aSC9ddO4xiN7faxteA6R3
rDeNANFX0tTngGi/rOqFIJqv/94aEJ2gZLkeRHfYugk3AOkNghMkQfTuIyLm
INrvnCSYLjDvrzvi/srxWJJ6A4gGAHhOluY=
  "]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.023068831597685857`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3.`", ",", 
     RowBox[{"-", "0.0745111637912423`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5.`", ",", 
     RowBox[{"-", "0.14479593883018502`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7.`", ",", 
     RowBox[{"-", "0.21198684609879848`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"9.000000000000002`", ",", 
     RowBox[{"-", "0.2445016977642073`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"11.`", ",", 
     RowBox[{"-", "0.25167657987695236`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13.000000000000002`", ",", 
     RowBox[{"-", "0.23501242654879778`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"15.`", ",", 
     RowBox[{"-", "0.187110913775953`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"17.`", ",", 
     RowBox[{"-", "0.12396124455266305`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"19.000000000000004`", ",", 
     RowBox[{"-", "0.0850189579660251`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"21.`", ",", 
     RowBox[{"-", "0.06061429469937136`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"23.`", ",", 
     RowBox[{"-", "0.04434906204591905`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"25.`", ",", 
     RowBox[{"-", "0.033105024613838754`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"27.000000000000004`", ",", 
     RowBox[{"-", "0.025133819103899575`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"29.`", ",", 
     RowBox[{"-", "0.019370337094972923`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"31.`", ",", 
     RowBox[{"-", "0.015132764255200373`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"33.`", ",", 
     RowBox[{"-", "0.011970294321763131`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"35.`", ",", 
     RowBox[{"-", "0.009577800613599176`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"37.00000000000001`", ",", 
     RowBox[{"-", "0.007744880614262151`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"39.`", ",", 
     RowBox[{"-", "0.0063241245961718085`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.524195115227697*^9, 3.5241951980599213`*^9, 3.524195242938348*^9, 
   3.524195296024626*^9, {3.5241953346562634`*^9, 3.524195341385236*^9}, 
   3.5241953770463657`*^9, 3.524195484624962*^9, 3.524195855662176*^9, 
   3.524877683923644*^9, 3.5248777927990913`*^9, 3.524878553058675*^9, 
   3.524879980819289*^9, 3.525484690892411*^9, 3.525565078287965*^9, 
   3.5255652016344843`*^9, 3.5255662482425137`*^9, 3.525567179757433*^9, {
   3.525567743934214*^9, 3.5255677667100477`*^9}, 3.5255678743030043`*^9, 
   3.525574868649303*^9, 3.5255751060041027`*^9, 3.527287988060841*^9, 
   3.527288851360241*^9, 3.529272866977179*^9, 3.52927344939554*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.024708048113987184`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2.`", ",", 
     RowBox[{"-", "0.050810762697027854`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3.`", ",", 
     RowBox[{"-", "0.07997224793012328`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4.`", ",", 
     RowBox[{"-", "0.11438690596715125`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5.`", ",", 
     RowBox[{"-", "0.155293687216296`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6.`", ",", 
     RowBox[{"-", "0.19435740040789348`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7.`", ",", 
     RowBox[{"-", "0.2231617471011868`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8.`", ",", 
     RowBox[{"-", "0.24269372846992054`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"9.000000000000002`", ",", 
     RowBox[{"-", "0.25469669381771703`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10.`", ",", 
     RowBox[{"-", "0.26012834261341344`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"11.`", ",", 
     RowBox[{"-", "0.259318545147836`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"12.`", ",", 
     RowBox[{"-", "0.2520496143430523`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13.000000000000002`", ",", 
     RowBox[{"-", "0.2374809480298787`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"14.`", ",", 
     RowBox[{"-", "0.21399652362000668`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"15.`", ",", 
     RowBox[{"-", "0.18075308583649174`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"16.`", ",", 
     RowBox[{"-", "0.14633480359679857`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"17.`", ",", 
     RowBox[{"-", "0.11927904983879536`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"18.000000000000004`", ",", 
     RowBox[{"-", "0.09859761056448395`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"19.`", ",", 
     RowBox[{"-", "0.08241642321723593`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"20.`", ",", 
     RowBox[{"-", "0.06949426875683246`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"21.`", ",", 
     RowBox[{"-", "0.05901669537312712`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"22.`", ",", 
     RowBox[{"-", "0.05042319525065704`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"23.`", ",", 
     RowBox[{"-", "0.04331113408608786`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"24.`", ",", 
     RowBox[{"-", "0.0373815338722854`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"25.`", ",", 
     RowBox[{"-", "0.032406744211792196`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"26.000000000000004`", ",", 
     RowBox[{"-", "0.028210081500075302`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"27.`", ",", 
     RowBox[{"-", "0.024652378726016753`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"28.`", ",", 
     RowBox[{"-", "0.021622736746469227`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"29.`", ",", 
     RowBox[{"-", "0.019031950338024937`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"30.`", ",", 
     RowBox[{"-", "0.01680770719700471`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"31.`", ",", 
     RowBox[{"-", "0.01489100384009773`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"32.`", ",", 
     RowBox[{"-", "0.01323342199473042`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"33.`", ",", 
     RowBox[{"-", "0.011795028916539484`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"34.`", ",", 
     RowBox[{"-", "0.010542739663946622`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"35.`", ",", 
     RowBox[{"-", "0.009449027367909965`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"36.00000000000001`", ",", 
     RowBox[{"-", "0.0084908994023985`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"37.`", ",", 
     RowBox[{"-", "0.007649079119957365`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"38.`", ",", 
     RowBox[{"-", "0.006907348054714695`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"39.`", ",", 
     RowBox[{"-", "0.006252014406790371`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"40.`", ",", 
     RowBox[{"-", "0.005671481588718266`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.524195115227697*^9, 3.5241951980599213`*^9, 3.524195242938348*^9, 
   3.524195296024626*^9, {3.5241953346562634`*^9, 3.524195341385236*^9}, 
   3.5241953770463657`*^9, 3.524195484624962*^9, 3.524195855662176*^9, 
   3.524877683923644*^9, 3.5248777927990913`*^9, 3.524878553058675*^9, 
   3.524879980819289*^9, 3.525484690892411*^9, 3.525565078287965*^9, 
   3.5255652016344843`*^9, 3.5255662482425137`*^9, 3.525567179757433*^9, {
   3.525567743934214*^9, 3.5255677667100477`*^9}, 3.5255678743030043`*^9, 
   3.525574868649303*^9, 3.5255751060041027`*^9, 3.527287988060841*^9, 
   3.527288851360241*^9, 3.529272866977179*^9, 3.5292734494567957`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2.`", ",", 
     RowBox[{"-", "0.05108261411092234`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4.`", ",", 
     RowBox[{"-", "0.11357174345145385`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6.`", ",", 
     RowBox[{"-", "0.18813018176190233`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"8.`", ",", 
     RowBox[{"-", "0.23522243159501244`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10.`", ",", 
     RowBox[{"-", "0.25266565004805713`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"12.`", ",", 
     RowBox[{"-", "0.24485810681089873`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"14.`", ",", 
     RowBox[{"-", "0.20835321797528564`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"16.`", ",", 
     RowBox[{"-", "0.1463953910728147`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"18.000000000000004`", ",", 
     RowBox[{"-", "0.1000726418380844`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"20.000000000000004`", ",", 
     RowBox[{"-", "0.07098086861859389`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"22.`", ",", 
     RowBox[{"-", "0.05171832764728035`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"24.`", ",", 
     RowBox[{"-", "0.03846596178819943`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"26.000000000000004`", ",", 
     RowBox[{"-", "0.029105230243036816`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"28.000000000000004`", ",", 
     RowBox[{"-", "0.022358173633331927`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"30.`", ",", 
     RowBox[{"-", "0.017411847208762304`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"32.`", ",", 
     RowBox[{"-", "0.013730881311368577`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"34.`", ",", 
     RowBox[{"-", "0.010953921812542433`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"36.00000000000001`", ",", 
     RowBox[{"-", "0.008832353814007918`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"38.00000000000001`", ",", 
     RowBox[{"-", "0.007192353784233012`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"40.`", ",", 
     RowBox[{"-", "0.0059106428259433404`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.524195115227697*^9, 3.5241951980599213`*^9, 3.524195242938348*^9, 
   3.524195296024626*^9, {3.5241953346562634`*^9, 3.524195341385236*^9}, 
   3.5241953770463657`*^9, 3.524195484624962*^9, 3.524195855662176*^9, 
   3.524877683923644*^9, 3.5248777927990913`*^9, 3.524878553058675*^9, 
   3.524879980819289*^9, 3.525484690892411*^9, 3.525565078287965*^9, 
   3.5255652016344843`*^9, 3.5255662482425137`*^9, 3.525567179757433*^9, {
   3.525567743934214*^9, 3.5255677667100477`*^9}, 3.5255678743030043`*^9, 
   3.525574868649303*^9, 3.5255751060041027`*^9, 3.527287988060841*^9, 
   3.527288851360241*^9, 3.529272866977179*^9, 3.529273449658196*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"LinSpace", "[", 
   RowBox[{"x_", ",", "y_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Range", "[", 
   RowBox[{"x", ",", "y", ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"y", "-", "x"}], ")"}], "/", "n"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"lx", ":=", 
  RowBox[{"21", "+", 
   RowBox[{"{", 
    RowBox[{"0", ",", "2", ",", "4"}], "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ly", "=", 
   RowBox[{"LinSpace", "[", 
    RowBox[{
     RowBox[{"-", "0.15"}], ",", 
     RowBox[{"-", "0.25"}], ",", "4"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"styles", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Darker", "[", "Green", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"Dashed", ",", " ", "Red"}], "}"}], ",", "Blue", ",", 
     "Orange"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"joined", "=", 
   RowBox[{"{", 
    RowBox[{"False", ",", "True", ",", "False"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"markers", "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Automatic", ",", "6"}], "}"}], ",", "None", ",", 
      RowBox[{"{", 
       RowBox[{"Automatic", ",", "6"}], "}"}]}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"compareThinPlot", "=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ListPlot", "[", 
       RowBox[{
        RowBox[{"{", "thinthin1", "}"}], ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{
         "markers", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{
         "styles", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"Joined", "\[Rule]", 
         RowBox[{
         "joined", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}],
        "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"ListPlot", "[", 
       RowBox[{
        RowBox[{"{", "thinthin2", "}"}], ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{
         "markers", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{
         "styles", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"Joined", "\[Rule]", 
         RowBox[{
         "joined", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}]}],
        "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"ListPlot", "[", 
       RowBox[{
        RowBox[{"{", "thinthin3", "}"}], ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{
         "markers", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{
         "styles", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"Joined", "\[Rule]", 
         RowBox[{
         "joined", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}]}],
        "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"ListPlot", "[", 
         RowBox[{
          RowBox[{"{", "thinthin4", "}"}], ",", 
          RowBox[{"PlotMarkers", "\[Rule]", 
           RowBox[{
           "markers", "\[LeftDoubleBracket]", "4", 
            "\[RightDoubleBracket]"}]}], ",", 
          RowBox[{"PlotStyle", "\[Rule]", 
           RowBox[{
           "styles", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}]}],
           ",", 
          RowBox[{"Joined", "\[Rule]", 
           RowBox[{
           "joined", "\[LeftDoubleBracket]", "4", 
            "\[RightDoubleBracket]"}]}]}], "]"}], ","}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "}"}], "}"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{
         "markers", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{
         "styles", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"Joined", "\[Rule]", 
         RowBox[{
         "joined", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}],
        "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], "}"}], "}"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{
         "markers", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{
         "styles", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"Joined", "\[Rule]", 
         RowBox[{
         "joined", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}]}],
        "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}], "}"}], "}"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{
         "markers", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{
         "styles", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
        ",", 
        RowBox[{"Joined", "\[Rule]", 
         RowBox[{
         "joined", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}]}],
        "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"lx", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"ly", "[", 
                RowBox[{"[", "4", "]"}], "]"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"lx", "[", 
                RowBox[{"[", "2", "]"}], "]"}], ",", 
               RowBox[{"ly", "[", 
                RowBox[{"[", "4", "]"}], "]"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"lx", "[", 
                RowBox[{"[", "3", "]"}], "]"}], ",", 
               RowBox[{"ly", "[", 
                RowBox[{"[", "4", "]"}], "]"}]}], "}"}]}], "}"}], "}"}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"PlotMarkers", "\[Rule]", 
           RowBox[{
           "markers", "\[LeftDoubleBracket]", "4", 
            "\[RightDoubleBracket]"}]}], ",", 
          RowBox[{"PlotStyle", "\[Rule]", 
           RowBox[{
           "styles", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}]}],
           ",", 
          RowBox[{"Joined", "\[Rule]", 
           RowBox[{
           "joined", "\[LeftDoubleBracket]", "4", 
            "\[RightDoubleBracket]"}]}]}], "]"}], ","}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\< Filament\>\"", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"lx", "[", 
            RowBox[{"[", "3", "]"}], "]"}], ",", 
           RowBox[{"ly", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\< Shell\>\"", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"lx", "[", 
            RowBox[{"[", "3", "]"}], "]"}], ",", 
           RowBox[{"ly", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\< 0.5 mm thick\>\"", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"lx", "[", 
            RowBox[{"[", "3", "]"}], "]"}], ",", 
           RowBox[{"ly", "[", 
            RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}]}], 
     "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{"\"\< 1 mm thick\>\"", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"lx", "[", 
            RowBox[{"[", "3", "]"}], "]"}], ",", 
           RowBox[{"ly", "[", 
            RowBox[{"[", "4", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}], "*)"}], 
     "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ImageSize", "\[Rule]", 
     RowBox[{"8", " ", 
      RowBox[{"72", "/", "2.54"}]}]}], ",", 
    RowBox[{"(*", " ", 
     RowBox[{"6", "cm"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{"Frame", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<Displacement, mm\>\"", ",", "\"\<Force, N\>\""}], "}"}]}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.525567381233439*^9, 3.525567572510372*^9}, {
   3.525567616364772*^9, 3.525567616788227*^9}, {3.525567679475011*^9, 
   3.525567685921509*^9}, {3.525567789280285*^9, 3.525567833096798*^9}, {
   3.525568172580879*^9, 3.525568189411601*^9}, {3.525574762672323*^9, 
   3.5255747640934563`*^9}, {3.52557493935835*^9, 3.525574984754119*^9}, {
   3.5255750618393784`*^9, 3.5255750679116583`*^9}, {3.5255757107836237`*^9, 
   3.525575713202506*^9}, 3.527287958362409*^9, {3.527288822732325*^9, 
   3.5272888305482388`*^9}, {3.5272888640952253`*^9, 3.527288867334216*^9}, {
   3.529631503443142*^9, 3.5296315377442083`*^9}, {3.529631581179988*^9, 
   3.529631655191921*^9}, {3.529631721783494*^9, 3.529631743088028*^9}, {
   3.5327667757271357`*^9, 3.5327667763245153`*^9}}],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGDQAGIQDQEf7DcUCYtGz5++H8LncDg7ffVjTeHNUL6Iw2fn
93FrOg5B+TIOMTOm9SYpnd7PCOYrOfDG3H56xfs8VF7NYV3ae9EK6QtQeS2H
cwri8o9FzkHl9Ry+W26Sdvh8HMo3dDhiEzz5ze79UPXGDhrcjDlnD2+Fyps6
nDxzzuoax3oo39yB8anBtNdblkH5lg6viiV9VD4vgOq3dng634dn3Z6ZUHlb
B9uS8MCLVydD+fYOvx9Nt3n7tw/Cb3BwOJsvuvVhcweU7+ggkmi5uG1uM8S8
BicHhieG/RI766Hyzg6fXvO7Bj+v3D8afpSFHwCxbcsq
    "], {
     {RGBColor[0, 
       NCache[
        Rational[2, 3], 0.6666666666666666], 0], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 21], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 22], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 23], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 24], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 25], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 26], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 27], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 28], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 29], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 30], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 31], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 32], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 33], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 34], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 35], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 36], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 37], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 38], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 39], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 40]}, {}}], {{}, {}, 
    {RGBColor[1, 0, 0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJxTTMoPSmViYGDQBGIQjR18sLd7mcwl6DtzP1TAQVtS2eMe8yoon8PBqa9o
LX/lFihfwOGpscWVUo+9UL6IQ4bz+7srHxyG8iUcdqfacW95cALKl3F4534o
a8KEM1C+gsOUupjeaQLn9zOC+UoO+7WnxXz2uQCVV3FgSxaK/LgUxldzEGVY
dX7NDBhfw0Fu5ZItUxQvQPVrOfjzz55yLPUcVF7HIfa18WeblNNQvp7DnXMa
J14pHYfyDRyW1d/2ldx9CMo3dJC6op4k1LkPap6RQ/GUaxkbbXdC5Y0dePnj
lewkt0L5Jg41bydGJ5zaCOWbOqwJFd+Yb74OyjdzkMqXXyt4cSWUb+4Qs/h5
wwOdZVC+hYNzzEulNcqLoXxLh1Mlck/nTl8Atd/KYW7s9Vapx3Og8tYOkfHT
FCvtYPFj4+AZssXHVWUalG/rsHO5/ESZ2slQvp3DlsK3SlxmE6F8e4ekvt+b
der7oHwHB8sTxVflpboh/AYHh53Nc3//Ve+Ayjs6iPzvmJA3vRUq7+hQ0Wb4
hymqGeo+JweFrT8t1yY3QuWdHPL2L+JaElIP1e/skCP8Rf6hVw1U3tlBbMGU
sG2zK6HyLg5mZhL7f1mV7wcAVt298g==
      "]]}}, 
   GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGDQAGIQDQUO0/wYpBrUV+2HcAUcJq26v5pTeC+UL+FQkcke
uUzkBJSv4MBgXXf0iPQ5KF/F4eWtj91rtC9A+RoOrc9K1jSFn4fydRwe6s+p
CFx9Cso3cPi8vqJHdO+h/YxgvpHDApvXwTHzdkL5Jg43T0THntXeBFVv5rBg
wSbpy1Uw91k4XMs5Xn9+42KoeisHNxfVD4fOzoXybRwW7jZbxP90GlS9nUN1
vI9n5pWJUL6DA8um7I3v5XqgfEeHy7JT65ny26D6nRwsBC3LNok1QfnODpNq
eo/sKq2FqndxaBPTkTpkWrF/NPwoCz8A16bAAg==
    "], {
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 21], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 22], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 23], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 24], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 25], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 26], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 27], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 28], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 29], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 30], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 31], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 32], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 33], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 34], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 35], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 36], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 37], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 38], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 39], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 40]}, {}}], 
   GraphicsComplexBox[{{21., -0.15}, {23., -0.15}, {25., -0.15}, {
    21., -0.15}, {23., -0.15}, {25., -0.15}}, {
     {RGBColor[0, 
       NCache[
        Rational[2, 3], 0.6666666666666666], 0], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 4], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 5], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 6]}, {}}], {{}, {}, 
    {RGBColor[1, 0, 0], Dashing[{Small, Small}], 
     LineBox[{{21., -0.175}, {23., -0.175}, {25., -0.175}}]}}, 
   GraphicsComplexBox[{{21., -0.2}, {23., -0.2}, {25., -0.2}, {21., -0.2}, {
    23., -0.2}, {25., -0.2}}, {
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 4], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 5], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 6]}, {}}], 
   InsetBox["\<\" Filament\"\>", {25, -0.15}, {-1, 0}], 
   InsetBox["\<\" Integral\"\>", {25, -0.175}, {-1, 0}], 
   InsetBox["\<\" 0.5 mm thick\"\>", {25, -0.2}, {-1, 0}]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  Frame->True,
  FrameLabel->{
    FormBox["\"Displacement, mm\"", TraditionalForm], 
    FormBox["\"Force, N\"", TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->All,
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.525567358642232*^9, 3.525567399992653*^9, {3.525567436978498*^9, 
   3.525567519680789*^9}, 3.525567573390931*^9, 3.525567618562364*^9, 
   3.5255676875851517`*^9, {3.525567769772876*^9, 3.525567833798024*^9}, 
   3.525568144113998*^9, {3.525568179581223*^9, 3.525568190035246*^9}, 
   3.525574871756144*^9, {3.525574940488765*^9, 3.525574985172071*^9}, 
   3.525575068946329*^9, 3.525575108526195*^9, 3.5255757169203253`*^9, 
   3.527287991715209*^9, {3.527288834841929*^9, 3.5272888683469973`*^9}, 
   3.529273453387203*^9, {3.529631484997254*^9, 3.529631539267852*^9}, {
   3.529631608517016*^9, 3.529631655624748*^9}, 3.529631694454578*^9, {
   3.529631738607073*^9, 3.529631743788554*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"PSfragExport", "[", 
   RowBox[{"\"\<fig/thin-compare\>\"", ",", "compareThinPlot"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.525567623584237*^9, 3.525567626988958*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Thick coil comparison", "Section",
 CellChangeTimes->{{3.529631462762744*^9, 3.52963146453685*^9}}],

Cell[BoxData[
 RowBox[{"thickfilamentparams", ":=", 
  RowBox[{"{", "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Displacement", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"Eccentricity", "\[Rule]", 
     RowBox[{"0", "mm"}]}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{"MagnetRadius", "\[Rule]", 
     RowBox[{"9", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"MagnetLength", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"Magnetisation", "\[Rule]", "1"}], ",", "\[IndentingNewLine]", 
    "\[IndentingNewLine]", 
    RowBox[{"CoilRadius", "\[Rule]", 
     RowBox[{"10", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilLength", "\[Rule]", 
     RowBox[{"20", "mm"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilThickness", "\[Rule]", 
     RowBox[{"5", "mm"}]}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{"CoilTurns", "\[Rule]", "100"}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilTurnsR", "\[Rule]", "5"}], ",", "\[IndentingNewLine]", 
    RowBox[{"CoilTurnsZ", "\[Rule]", "20"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Current", "\[Rule]", "1"}]}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", "}"}]}]], "Input",
 CellChangeTimes->{{3.5255655574528017`*^9, 3.5255655731587543`*^9}, {
  3.526269603414728*^9, 3.526269608893355*^9}, {3.526270548461328*^9, 
  3.5262705487411203`*^9}, {3.527287062863903*^9, 3.5272870729977837`*^9}, {
  3.5280806748430967`*^9, 3.5280807166124363`*^9}, {3.528080896225556*^9, 
  3.528080896830866*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{"thickfilamentparams", "\[IndentingNewLine]", 
 RowBox[{"xmax", ":=", 
  RowBox[{"40", "mm"}]}], "\[IndentingNewLine]", 
 RowBox[{"dx", ":=", 
  RowBox[{"1.5", "mm"}]}], "\[IndentingNewLine]", 
 RowBox[{"Dx", ":=", 
  RowBox[{"2", "dx"}]}], "\[IndentingNewLine]", 
 RowBox[{"thinthick1", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1000", "x"}], ",", 
      RowBox[{"MagnetCoilForce", "[", 
       RowBox[{"Flatten", "@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Method", "\[Rule]", "\"\<Filament\>\""}], ",", 
          RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
          RowBox[{"IntegrationPrecision", "\[Rule]", "3"}], ",", 
          "thickfilamentparams"}], "}"}]}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"0", "dx"}], ",", "xmax", ",", "dx"}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"thinthick2", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1000", "x"}], ",", 
      RowBox[{"MagnetCoilForce", "[", 
       RowBox[{"Flatten", "@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Method", "\[Rule]", "\"\<Shell\>\""}], ",", 
          RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
          RowBox[{"IntegrationPrecision", "\[Rule]", "3"}], ",", 
          "thickfilamentparams"}], "}"}]}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"1", "dx"}], ",", "xmax", ",", "Dx"}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"thinthick3", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1000", "x"}], ",", 
      RowBox[{
       RowBox[{"MagnetCoilForce", "[", 
        RowBox[{"Flatten", "@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
           RowBox[{"Method", "\[Rule]", "Automatic"}], ",", 
           RowBox[{"IntegrationPrecision", "\[Rule]", "3"}], ",", 
           "thickfilamentparams"}], "}"}]}], "]"}], "//.", 
       "thickfilamentparams"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"2", "dx"}], ",", "xmax", ",", "Dx"}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"thinthick4", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1000", "x"}], ",", 
      RowBox[{
       RowBox[{"MagnetCoilForce", "[", 
        RowBox[{"Flatten", "@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
           RowBox[{"Method", "\[Rule]", "\"\<Babic\>\""}], ",", 
           RowBox[{"IntegrationPrecision", "\[Rule]", "3"}], ",", 
           "thickfilamentparams"}], "}"}]}], "]"}], "//.", 
       "thickfilamentparams"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "xmax", ",", "dx"}], "}"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.524194291764681*^9, 3.524194329897986*^9}, {
  3.524194916587533*^9, 3.524195107101907*^9}, {3.524195180566375*^9, 
  3.5241951961229477`*^9}, {3.524195232415769*^9, 3.5241952375488977`*^9}, {
  3.524195326066287*^9, 3.524195371192964*^9}, {3.524195472804706*^9, 
  3.524195474765182*^9}, {3.524195842709243*^9, 3.524195847280757*^9}, {
  3.524877661337586*^9, 3.5248776716132383`*^9}, {3.524877773630084*^9, 
  3.524877781032803*^9}, {3.524878498430912*^9, 3.524878515286763*^9}, {
  3.524879964190024*^9, 3.5248799647938223`*^9}, {3.525483525948134*^9, 
  3.52548352602564*^9}, {3.526266280018209*^9, 3.5262662852124043`*^9}, {
  3.526271326926477*^9, 3.526271334003601*^9}, {3.527288486718754*^9, 
  3.527288498453455*^9}, {3.5292735730403976`*^9, 3.529273573319426*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Displacement", "\[Rule]", "0.01`"}], ",", 
   RowBox[{"Eccentricity", "\[Rule]", "0.`"}], ",", 
   RowBox[{"MagnetRadius", "\[Rule]", "0.009000000000000001`"}], ",", 
   RowBox[{"MagnetLength", "\[Rule]", "0.01`"}], ",", 
   RowBox[{"Magnetisation", "\[Rule]", "1"}], ",", 
   RowBox[{"CoilRadius", "\[Rule]", "0.01`"}], ",", 
   RowBox[{"CoilLength", "\[Rule]", "0.02`"}], ",", 
   RowBox[{"CoilThickness", "\[Rule]", "0.005`"}], ",", 
   RowBox[{"CoilTurns", "\[Rule]", "100"}], ",", 
   RowBox[{"CoilTurnsR", "\[Rule]", "5"}], ",", 
   RowBox[{"CoilTurnsZ", "\[Rule]", "20"}], ",", 
   RowBox[{"Current", "\[Rule]", "1"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.524195115227697*^9, 3.5241951980599213`*^9, 3.524195242938348*^9, 
   3.524195296024626*^9, {3.5241953346562634`*^9, 3.524195341385236*^9}, 
   3.5241953770463657`*^9, 3.524195484624962*^9, 3.524195855662176*^9, 
   3.524877683923644*^9, 3.5248777927990913`*^9, 3.524878553058675*^9, 
   3.524879980819289*^9, 3.525484690892411*^9, 3.525565078287965*^9, 
   3.5255652016344843`*^9, 3.526271382995364*^9, 3.527288935752054*^9, 
   3.528080645705431*^9, {3.5280806801472597`*^9, 3.5280807180832787`*^9}, {
   3.528080912176641*^9, 3.52808091746607*^9}, 3.5280810282489243`*^9, {
   3.529273573960421*^9, 3.529273579309136*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.5`", ",", 
     RowBox[{"-", "0.07618611779924976`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3.`", ",", 
     RowBox[{"-", "0.15606944782380167`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4.500000000000001`", ",", 
     RowBox[{"-", "0.24311575621537604`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6.`", ",", 
     RowBox[{"-", "0.33479964265314505`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7.5`", ",", 
     RowBox[{"-", "0.4088187587755334`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"9.000000000000002`", ",", 
     RowBox[{"-", "0.45629811385663877`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10.5`", ",", 
     RowBox[{"-", "0.4781135996204566`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"12.`", ",", 
     RowBox[{"-", "0.4750768287174702`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13.5`", ",", 
     RowBox[{"-", "0.4470581865780007`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"15.`", ",", 
     RowBox[{"-", "0.3941227299602045`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"16.5`", ",", 
     RowBox[{"-", "0.3249800719235231`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"18.000000000000004`", ",", 
     RowBox[{"-", "0.26357978610659527`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"19.5`", ",", 
     RowBox[{"-", "0.21389650550651304`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"21.`", ",", 
     RowBox[{"-", "0.17423142134776023`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"22.5`", ",", 
     RowBox[{"-", "0.14263181940134534`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"24.`", ",", 
     RowBox[{"-", "0.1174074988500238`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"25.500000000000004`", ",", 
     RowBox[{"-", "0.09719506745142528`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"27.`", ",", 
     RowBox[{"-", "0.08092266656874883`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"28.5`", ",", 
     RowBox[{"-", "0.06775544009537472`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"30.`", ",", 
     RowBox[{"-", "0.05704457746783944`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"31.5`", ",", 
     RowBox[{"-", "0.04828531696307778`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"33.`", ",", 
     RowBox[{"-", "0.04108390701246187`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"34.5`", ",", 
     RowBox[{"-", "0.03513210241149789`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"36.00000000000001`", ",", 
     RowBox[{"-", "0.030187623140788884`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"37.5`", ",", 
     RowBox[{"-", "0.026059214785827553`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"39.`", ",", 
     RowBox[{"-", "0.022595222806447074`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.524195115227697*^9, 3.5241951980599213`*^9, 3.524195242938348*^9, 
   3.524195296024626*^9, {3.5241953346562634`*^9, 3.524195341385236*^9}, 
   3.5241953770463657`*^9, 3.524195484624962*^9, 3.524195855662176*^9, 
   3.524877683923644*^9, 3.5248777927990913`*^9, 3.524878553058675*^9, 
   3.524879980819289*^9, 3.525484690892411*^9, 3.525565078287965*^9, 
   3.5255652016344843`*^9, 3.526271382995364*^9, 3.527288935752054*^9, 
   3.528080645705431*^9, {3.5280806801472597`*^9, 3.5280807180832787`*^9}, {
   3.528080912176641*^9, 3.52808091746607*^9}, 3.5280810282489243`*^9, 
   3.529273573960421*^9, 3.529273620085088*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.5`", ",", 
     RowBox[{"-", "0.08658538430803767`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4.500000000000001`", ",", 
     RowBox[{"-", "0.2750400589130219`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7.5`", ",", 
     RowBox[{"-", "0.44195745583944285`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10.5`", ",", 
     RowBox[{"-", "0.4997359601434368`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13.5`", ",", 
     RowBox[{"-", "0.44946078978065307`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"16.5`", ",", 
     RowBox[{"-", "0.31444881585453266`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"19.500000000000004`", ",", 
     RowBox[{"-", "0.20690171792107784`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"22.500000000000004`", ",", 
     RowBox[{"-", "0.13830457713054153`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"25.500000000000004`", ",", 
     RowBox[{"-", "0.09451237812371703`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"28.5`", ",", 
     RowBox[{"-", "0.06606649509369567`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"31.5`", ",", 
     RowBox[{"-", "0.0472018637514838`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"34.5`", ",", 
     RowBox[{"-", "0.03442328494828292`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"37.50000000000001`", ",", 
     RowBox[{"-", "0.025586314645558574`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.524195115227697*^9, 3.5241951980599213`*^9, 3.524195242938348*^9, 
   3.524195296024626*^9, {3.5241953346562634`*^9, 3.524195341385236*^9}, 
   3.5241953770463657`*^9, 3.524195484624962*^9, 3.524195855662176*^9, 
   3.524877683923644*^9, 3.5248777927990913`*^9, 3.524878553058675*^9, 
   3.524879980819289*^9, 3.525484690892411*^9, 3.525565078287965*^9, 
   3.5255652016344843`*^9, 3.526271382995364*^9, 3.527288935752054*^9, 
   3.528080645705431*^9, {3.5280806801472597`*^9, 3.5280807180832787`*^9}, {
   3.528080912176641*^9, 3.52808091746607*^9}, 3.5280810282489243`*^9, 
   3.529273573960421*^9, 3.52927362013107*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3.`", ",", 
     RowBox[{"-", "0.17885387695833901`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6.`", ",", 
     RowBox[{"-", "0.367523491028598`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"9.000000000000002`", ",", 
     RowBox[{"-", "0.4806513250600631`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"12.`", ",", 
     RowBox[{"-", "0.480758877363449`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"15.`", ",", 
     RowBox[{"-", "0.38491360322427187`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"18.`", ",", 
     RowBox[{"-", "0.2572698220778697`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"21.`", ",", 
     RowBox[{"-", "0.1702459441066184`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"24.`", ",", 
     RowBox[{"-", "0.11483223643345958`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"27.`", ",", 
     RowBox[{"-", "0.07923152007148841`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"30.`", ",", 
     RowBox[{"-", "0.05591547340369309`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"33.`", ",", 
     RowBox[{"-", "0.04031595223214119`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"36.00000000000001`", ",", 
     RowBox[{"-", "0.029655082316968333`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"39.00000000000001`", ",", 
     RowBox[{"-", "0.022218715330895672`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.524195115227697*^9, 3.5241951980599213`*^9, 3.524195242938348*^9, 
   3.524195296024626*^9, {3.5241953346562634`*^9, 3.524195341385236*^9}, 
   3.5241953770463657`*^9, 3.524195484624962*^9, 3.524195855662176*^9, 
   3.524877683923644*^9, 3.5248777927990913`*^9, 3.524878553058675*^9, 
   3.524879980819289*^9, 3.525484690892411*^9, 3.525565078287965*^9, 
   3.5255652016344843`*^9, 3.526271382995364*^9, 3.527288935752054*^9, 
   3.528080645705431*^9, {3.5280806801472597`*^9, 3.5280807180832787`*^9}, {
   3.528080912176641*^9, 3.52808091746607*^9}, 3.5280810282489243`*^9, 
   3.529273573960421*^9, 3.529273620313897*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.5`", ",", 
     RowBox[{"-", "0.0875786888764272`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3.`", ",", 
     RowBox[{"-", "0.17884492641324018`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4.500000000000001`", ",", 
     RowBox[{"-", "0.2751894679935808`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6.`", ",", 
     RowBox[{"-", "0.3675083896236442`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7.5`", ",", 
     RowBox[{"-", "0.43784353157490635`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"9.000000000000002`", ",", 
     RowBox[{"-", "0.4805844440871045`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10.5`", ",", 
     RowBox[{"-", "0.49586543994368043`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"12.`", ",", 
     RowBox[{"-", "0.48419441661172424`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13.5`", ",", 
     RowBox[{"-", "0.44602613328186974`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"15.`", ",", 
     RowBox[{"-", "0.3848775270259281`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"16.5`", ",", 
     RowBox[{"-", "0.3167589294295129`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"18.000000000000004`", ",", 
     RowBox[{"-", "0.2573111586151906`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"19.5`", ",", 
     RowBox[{"-", "0.20893839413769316`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"21.`", ",", 
     RowBox[{"-", "0.17026035245651092`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"22.5`", ",", 
     RowBox[{"-", "0.1394378662173643`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"24.`", ",", 
     RowBox[{"-", "0.11483142570471476`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"25.500000000000004`", ",", 
     RowBox[{"-", "0.09511098020527216`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"27.`", ",", 
     RowBox[{"-", "0.07923052222543195`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"28.5`", ",", 
     RowBox[{"-", "0.06637591710195183`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"30.`", ",", 
     RowBox[{"-", "0.05591493632613287`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"31.5`", ",", 
     RowBox[{"-", "0.04735597936241158`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"33.`", ",", 
     RowBox[{"-", "0.04031568055664397`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"34.5`", ",", 
     RowBox[{"-", "0.03449395941221041`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"36.00000000000001`", ",", 
     RowBox[{"-", "0.02965494358752876`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"37.5`", ",", 
     RowBox[{"-", "0.02561240664623627`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"39.`", ",", 
     RowBox[{"-", "0.02221864327336728`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.524195115227697*^9, 3.5241951980599213`*^9, 3.524195242938348*^9, 
   3.524195296024626*^9, {3.5241953346562634`*^9, 3.524195341385236*^9}, 
   3.5241953770463657`*^9, 3.524195484624962*^9, 3.524195855662176*^9, 
   3.524877683923644*^9, 3.5248777927990913`*^9, 3.524878553058675*^9, 
   3.524879980819289*^9, 3.525484690892411*^9, 3.525565078287965*^9, 
   3.5255652016344843`*^9, 3.526271382995364*^9, 3.527288935752054*^9, 
   3.528080645705431*^9, {3.5280806801472597`*^9, 3.5280807180832787`*^9}, {
   3.528080912176641*^9, 3.52808091746607*^9}, 3.5280810282489243`*^9, 
   3.529273573960421*^9, 3.529273621313253*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"lx", ":=", 
  RowBox[{"{", 
   RowBox[{"26", ",", "28", ",", "30"}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ly", "=", 
   RowBox[{"Table", "[", 
    RowBox[{"i", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"-", "0.25"}], ",", 
       RowBox[{"-", "0.4"}], ",", 
       RowBox[{"-", "0.05"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"comparePlot", "=", 
  RowBox[{"Show", "@", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"{", "thinthick4", "}"}], ",", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Darker", "[", "Green", "]"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"ImageSize", "\[Rule]", 
        RowBox[{"8", " ", 
         RowBox[{"72", "/", "2.54"}]}]}], ",", 
       RowBox[{"(*", " ", 
        RowBox[{"6", "cm"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"Frame", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<Displacement, mm\>\"", ",", "\"\<Force, N\>\""}], 
         "}"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"thinthick2", ",", "thinthick3"}], "}"}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Blue", ",", "Orange"}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"{", "thinthick1", "}"}], ",", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Dashed", ",", "Red"}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "4", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "4", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "4", "]"}], "]"}]}], "}"}]}], "}"}]}], 
        "\[IndentingNewLine]", "}"}], ",", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Dashed", ",", "Red"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Dashing", "[", "1", "]"}], ",", 
            RowBox[{"Darker", "[", "Green", "]"}]}], "}"}]}], "}"}]}]}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"lx", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"ly", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}], "}"}]}], 
        "\[IndentingNewLine]", "}"}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Blue", ",", "Orange"}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\< Filament\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"lx", "[", 
           RowBox[{"[", "3", "]"}], "]"}], ",", 
          RowBox[{"ly", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\< Shell\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"lx", "[", 
           RowBox[{"[", "3", "]"}], "]"}], ",", 
          RowBox[{"ly", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\< Integral\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"lx", "[", 
           RowBox[{"[", "3", "]"}], "]"}], ",", 
          RowBox[{"ly", "[", 
           RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{"\"\< Babic\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"lx", "[", 
           RowBox[{"[", "3", "]"}], "]"}], ",", 
          RowBox[{"ly", "[", 
           RowBox[{"[", "4", "]"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}]}]}], 
    "}"}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.524195056117908*^9, 3.524195056622604*^9}, {
   3.5241951340926332`*^9, 3.524195163538514*^9}, 3.524195203903841*^9, {
   3.524195246539789*^9, 3.5241952723027687`*^9}, {3.524195522939631*^9, 
   3.524195701014695*^9}, {3.5241957324524307`*^9, 3.524195877125457*^9}, {
   3.524195957683879*^9, 3.524195961656314*^9}, {3.524196685995064*^9, 
   3.524196718947332*^9}, {3.524197643522938*^9, 3.524197725458014*^9}, {
   3.524197784295594*^9, 3.524197805653969*^9}, {3.524197840263143*^9, 
   3.524197845539424*^9}, {3.524198003915935*^9, 3.524198010707597*^9}, {
   3.5241982148568153`*^9, 3.5241982248933773`*^9}, {3.5248738277761106`*^9, 
   3.524873855071908*^9}, {3.52487762600912*^9, 3.524877631982111*^9}, {
   3.5248784447826777`*^9, 3.5248784707026854`*^9}, {3.524878506211789*^9, 
   3.5248785063747387`*^9}, {3.524879539604516*^9, 3.5248795404895773`*^9}, {
   3.524879582811501*^9, 3.524879853444289*^9}, {3.5248798933432493`*^9, 
   3.524879917207478*^9}, {3.524880419518641*^9, 3.524880421499208*^9}, 
   3.525575730139488*^9, {3.527288906828319*^9, 3.5272889110350237`*^9}}],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 
     NCache[
      Rational[2, 3], 0.6666666666666666], 0], LineBox[CompressedData["
1:eJxTTMoPSmViYGCQBmIQjR38sFdi6p3Sl71tP4TP4fCIh+F38pNj+xnBfCGH
fdL+nlvmXITKSzjUSM954dh6HcqXc+h0t3iwgPUOVL2SQ3DTFMenB+9B5VUd
Vk79mOq05z6Ur+GwwVhgCtdfmLy2Q/+NZ4Ybu+9A+XoO9asfRV5deAPCbzBw
kMtYbnfc9QrUfCOHNVUnLU6WX4DKGzu0zbvpUbfvFFS/qYP9tLNR4qePQuXN
HMSV5s6UvHkQKm/hwJz3ZeH01L0Q8xosHZbeqyg1jNwBlbd2mLTg7NRijy1Q
/TYOwU8MJZn/bYDK2zkoPDj49uGCNVB5ewezJOe/XPYroHwHB/Ny5sOOS5ZA
+A6ODlUt2uaGKxdC3e/k8Fy2bvLd2HkQ+QNODpdKVjcrW82C6nd22P704Yvq
A9P2AwBQnIij
     "]]}, GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQDQE/7KVUXm0s1dq2nxHMF3IQPtI+z3HWxf0QeTmH
vwWH37B73oHyVR3OPIjpXPP7PpSv7TDD18bg+2GofIOBwx8Z8bdv5a9AzGsw
duBt/5V+sPoUlG/moH7tysNvmw9C+ZYO309KT/1muAOq38ZhxZuTt3e/3ADl
2zu8eWG367rWCgjfwdEh54jqsbfzF0L0H3ByKH/R1ORjPAvqHg6HRkc1gfVP
jkH5Eg7pG+fZN7Zeh/pPyeFsXNWxv4fuQeU1HGoyHz08cATG13NY3p+xKmfR
DSjfyGH/PJYb0uUXoHxTB/Zy6dp5p45C+RYOU2SYbJam7oXyrR2Ef7QEt3hs
gfLtHBa/Nv/8ecEaqH8cHHYab3jgtWQJ1D1ODv+W3Hj0LHYeNDycHSycLu1r
ODBt/2j8DO74AQDw2SK/
    "], {
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 27], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 28], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 29], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 30], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 31], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 32], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 33], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 34], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 35], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 36], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 37], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 38], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 39]}, 
     {RGBColor[1, 0.5, 0], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 40], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 41], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 42], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 43], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 44], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 45], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 46], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 47], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 48], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 49], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 50], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 51], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 52]}, {}}], 
   {RGBColor[1, 0, 0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJxTTMoPSmViYGCQBmIQjR38sJdPjv7yrmHzfgifw+FQjnq26K/D+xnBfCEH
ez3u41ly56HyEg61Er+qonOvQvlyDktW7lYX07oFVa/kkJXBz/jX+C5UXtXh
t2LnicyZ96B8DYf+4tlLVqTD+NoO2/du+DBzzh0oX89Br+VTl5/lTQi/wcDB
7qaBeeWZK1DzjRzSdvkeqL13ASpv7GDd8eH914TTUP2mDsuf+UuY+x6Dyps5
iAXczD7keAgqb+FwadPd71l8+yDmNVg6TArgXHLs4Q6ovLXDoV9ZfJHbt0D1
2zjc/Bi6PDtiI1TezmHxK7fUTyZrofL2Dm8ay5Nu7VgB5Ts4vAlj1frMsRTC
d3B0aAg8vffqn4VQ9zs5FN3rkGh8OQ8if8DJQec1n6/L+llQ/c4OEs3/dxsr
T98PAJX2ikI=
     "]]}, {
    {RGBColor[1, 0, 0], Dashing[{Small, Small}], 
     LineBox[{{26., -0.25}, {28., -0.25}, {30., -0.25}}]}, 
    {RGBColor[0, 
      NCache[
       Rational[2, 3], 0.6666666666666666], 0], Dashing[1], 
     LineBox[{{26., -0.4}, {28., -0.4}, {30., -0.4}}]}}, 
   GraphicsComplexBox[{{26., -0.3}, {28., -0.3}, {30., -0.3}, {26., -0.35}, {
    28., -0.35}, {30., -0.35}, {26., -0.3}, {28., -0.3}, {30., -0.3}, {
    26., -0.35}, {28., -0.35}, {30., -0.35}}, {
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 7], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 8], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 9]}, 
     {RGBColor[1, 0.5, 0], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 10], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 11], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 12]}, {}}], 
   InsetBox["\<\" Filament\"\>", {30, -0.25}, {-1, 0}], 
   InsetBox["\<\" Shell\"\>", {30, -0.3}, {-1, 0}], 
   InsetBox["\<\" Integral\"\>", {30, -0.35}, {-1, 0}], 
   InsetBox["\<\" Babic\"\>", {30, -0.4}, {-1, 0}]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{
    FormBox["\"Displacement, mm\"", TraditionalForm], 
    FormBox["\"Force, N\"", TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->Automatic,
  PlotRangeClipping->True]], "Output",
 CellChangeTimes->{
  3.524195204872847*^9, {3.52419525797318*^9, 3.524195272722447*^9}, 
   3.524195305576558*^9, 3.524195344219942*^9, 3.524195380627103*^9, 
   3.524195487622579*^9, {3.524195576183766*^9, 3.524195615919348*^9}, {
   3.524195666181089*^9, 3.524195689573907*^9}, {3.5241957512956963`*^9, 
   3.5241958184052963`*^9}, {3.524195859745822*^9, 3.524195877848345*^9}, 
   3.524195963485445*^9, {3.524196697173341*^9, 3.5241967199797688`*^9}, {
   3.524197693442253*^9, 3.524197726216732*^9}, {3.524197796060946*^9, 
   3.524197806645759*^9}, {3.524197841172781*^9, 3.524197846107337*^9}, 
   3.5241980117421713`*^9, {3.52419821999529*^9, 3.524198225844144*^9}, 
   3.5248776338093967`*^9, 3.52487771839433*^9, 3.5248784194883633`*^9, {
   3.524878463113698*^9, 3.524878471621685*^9}, 3.524878560599001*^9, 
   3.5248795419080276`*^9, {3.524879663385305*^9, 3.524879748824171*^9}, {
   3.524879804869821*^9, 3.524879829611492*^9}, {3.524879895804399*^9, 
   3.5248799178740177`*^9}, 3.524880422225493*^9, 3.525484698192417*^9, 
   3.525565082489925*^9, 3.5255652058968887`*^9, 3.525575732610827*^9, 
   3.527288942598406*^9, 3.528080650235495*^9, 3.528081031551509*^9, 
   3.529273689078971*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"PSfragExport", "[", 
   RowBox[{"\"\<fig/thickthin-compare\>\"", ",", "comparePlot"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.524195966227098*^9, 3.524195988121998*^9}, {
   3.524196048036046*^9, 3.524196049752448*^9}, {3.5241962302634897`*^9, 
   3.524196231013565*^9}, {3.5241966327435417`*^9, 3.524196643830638*^9}, 
   3.5241980193467484`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Electromagnet optimisation", "Section",
 CellChangeTimes->{{3.529719432948333*^9, 3.529719433850789*^9}, {
  3.53276682270732*^9, 3.53276682561195*^9}}],

Cell[CellGroupData[{

Cell["Outer coil configuration", "Subsection",
 CellChangeTimes->{{3.5297194386800337`*^9, 3.529719441955373*^9}}],

Cell["\<\
These are the only fixed variables:\
\>", "Text",
 CellChangeTimes->{{3.5317045917463617`*^9, 3.53170459777395*^9}}],

Cell[BoxData[{
 RowBox[{"GapRadius", ":=", 
  RowBox[{"0.5", "mm"}]}], "\n", 
 RowBox[{"WireResistivity", " ", ":=", " ", 
  RowBox[{"1.7", "*", 
   RowBox[{"10", "^", 
    RowBox[{"-", "8", " ", 
     RowBox[{"(*", " ", "copper", " ", "*)"}]}]}]}]}]}], "Input",
 CellChangeTimes->{3.5317045889295273`*^9}],

Cell[BoxData[{
 RowBox[{"MagVolume", ":=", 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"20", "mm"}], ")"}], "3"]}], "\[IndentingNewLine]", 
 RowBox[{"CoilResistance", ":=", 
  RowBox[{"4", "ohms"}]}], "\[IndentingNewLine]", 
 RowBox[{"WireDiam", ":=", 
  RowBox[{"1", "mm"}]}]}], "Input",
 CellChangeTimes->{{3.529719907133342*^9, 3.529719916113104*^9}, {
   3.529720365184144*^9, 3.5297203670724583`*^9}, {3.529720418714933*^9, 
   3.529720418850844*^9}, {3.529720554743701*^9, 3.529720557855049*^9}, {
   3.529720717404449*^9, 3.529720863893064*^9}, {3.529720935764885*^9, 
   3.5297209627630177`*^9}, {3.529721050200921*^9, 3.5297210724420757`*^9}, {
   3.529721380839862*^9, 3.529721397567935*^9}, {3.5308679776713543`*^9, 
   3.530867984465651*^9}, {3.5310228354742813`*^9, 3.53102283589914*^9}, 
   3.5310229035060368`*^9, {3.531704581852942*^9, 3.531704586468792*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"WireLength", "=", 
  RowBox[{"Nz", " ", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"2", "\[Pi]", 
      RowBox[{"(", 
       RowBox[{"rc", "+", 
        RowBox[{"dw", 
         RowBox[{"(", 
          RowBox[{"n", "+", 
           RowBox[{"1", "/", "2"}]}], ")"}]}]}], ")"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "0", ",", 
       RowBox[{"Nr", "-", "1"}]}], "}"}]}], 
    "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"lc", ">", "0"}], "}"}], ",", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"lw", "==", "WireLength"}], "//.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Nz", "\[Rule]", 
          RowBox[{"lc", "/", "dw"}]}], ",", 
         RowBox[{"Nr", "\[Rule]", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Rc", "-", "rc"}], ")"}], "/", "dw"}]}]}], "}"}]}], ",", 
      "Rc"}], "]"}], "//", "FullSimplify"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.531021389101947*^9, 3.531021416533976*^9}, {
  3.531021648707464*^9, 3.531021663110693*^9}, {3.531021707053648*^9, 
  3.5310217130027733`*^9}, {3.5310217686093063`*^9, 3.531021788409176*^9}, {
  3.531021913708954*^9, 3.531022083344963*^9}}],

Cell[BoxData[
 RowBox[{"Nr", " ", "Nz", " ", "\[Pi]", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"dw", " ", "Nr"}], "+", 
    RowBox[{"2", " ", "rc"}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.531021417268556*^9, {3.531021655994117*^9, 3.531021663800349*^9}, 
   3.531021713552977*^9, 3.5310217893315077`*^9, 3.5310219882742643`*^9, {
   3.531022033412694*^9, 3.5310220839720297`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Rc", "\[Rule]", 
     RowBox[{"-", 
      SqrtBox[
       RowBox[{
        FractionBox[
         RowBox[{
          SuperscriptBox["dw", "2"], " ", "lw"}], 
         RowBox[{"lc", " ", "\[Pi]"}]], "+", 
        SuperscriptBox["rc", "2"]}]]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Rc", "\[Rule]", 
     SqrtBox[
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["dw", "2"], " ", "lw"}], 
        RowBox[{"lc", " ", "\[Pi]"}]], "+", 
       SuperscriptBox["rc", "2"]}]]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.531021417268556*^9, {3.531021655994117*^9, 3.531021663800349*^9}, 
   3.531021713552977*^9, 3.5310217893315077`*^9, 3.5310219882742643`*^9, {
   3.531022033412694*^9, 3.531022084018194*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"magparam", "[", 
   RowBox[{"\[Alpha]_", ",", "\[Beta]_", ",", "d_", ",", "R_", ",", "V_"}], 
   "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "cr", ",", "cR", ",", "mr", ",", "wa", ",", "wl", ",", "cl", ",", "ct"}],
      "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mr", "=", 
      SuperscriptBox[
       RowBox[{"(", 
        FractionBox["V", 
         RowBox[{"\[Pi]", " ", "\[Alpha]"}]], ")"}], 
       RowBox[{"1", "/", "3"}]]}], ";", " ", 
     RowBox[{"(*", " ", 
      RowBox[{"magnet", " ", "radius"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"cr", "=", 
      RowBox[{"mr", "+", "GapRadius"}]}], ";", " ", 
     RowBox[{"(*", " ", 
      RowBox[{"coil", " ", "radius"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"wa", " ", "=", " ", 
      RowBox[{"\[Pi]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "/", "2"}], ")"}], "^", "2"}]}]}], ";", " ", 
     RowBox[{"(*", " ", 
      RowBox[{"wire", " ", "area"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"wl", "=", 
      RowBox[{"R", " ", 
       RowBox[{"wa", " ", "/", " ", "WireResistivity"}]}]}], ";", " ", 
     RowBox[{"(*", " ", 
      RowBox[{"wire", " ", "length"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"cl", "=", " ", 
      RowBox[{"cr", " ", "\[Beta]"}]}], ";", 
     RowBox[{"(*", " ", 
      RowBox[{"coil", " ", "length"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"cR", "=", 
      SqrtBox[
       RowBox[{
        FractionBox[
         RowBox[{
          SuperscriptBox["d", "2"], " ", "wl"}], 
         RowBox[{"cl", " ", "\[Pi]"}]], "+", 
        SuperscriptBox["cr", "2"]}]]}], ";", 
     RowBox[{"(*", " ", 
      RowBox[{"coil", " ", "outer", " ", "radius"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"ct", "=", 
      RowBox[{"cR", "-", "cr"}]}], ";", " ", 
     RowBox[{"(*", " ", 
      RowBox[{"coil", " ", "thickness"}], " ", "*)"}], "\[IndentingNewLine]", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Magnetisation", "\[Rule]", "1"}], ",", 
       RowBox[{"Current", "\[Rule]", 
        RowBox[{"-", "1"}]}], ",", 
       RowBox[{"MagnetLength", "\[Rule]", 
        RowBox[{"mr", " ", "\[Alpha]"}]}], ",", 
       RowBox[{"MagnetRadius", "\[Rule]", "mr"}], ",", 
       RowBox[{"CoilRadius", "\[Rule]", "cr"}], ",", "\[IndentingNewLine]", 
       RowBox[{"CoilLength", "\[Rule]", "cl"}], ",", 
       RowBox[{"CoilThickness", "\[Rule]", "ct"}], ",", 
       RowBox[{"CoilTurnsR", "\[Rule]", 
        RowBox[{"Round", "[", 
         RowBox[{"ct", "/", "d"}], "]"}]}], ",", 
       RowBox[{"CoilTurnsZ", "\[Rule]", 
        RowBox[{"Round", "[", 
         RowBox[{"cl", "/", "d"}], "]"}]}]}], "}"}]}]}], " ", 
   "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"magparam", "[", 
   RowBox[{"\[Alpha]_", ",", "\[Beta]_", ",", "d_"}], "]"}], ":=", 
  RowBox[{"magparam", "[", 
   RowBox[{
   "\[Alpha]", ",", "\[Beta]", ",", "d", ",", "CoilResistance", ",", 
    "MagVolume"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"magparam", "[", 
   RowBox[{"\[Alpha]_", ",", "\[Beta]_"}], "]"}], ":=", 
  RowBox[{"magparam", "[", 
   RowBox[{
   "\[Alpha]", ",", "\[Beta]", ",", "WireDiam", ",", "CoilResistance", ",", 
    "MagVolume"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"magparam", "[", 
  RowBox[{"4.75", ",", "2.6"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"magparam", "[", 
  RowBox[{"4.75", ",", "2.6", ",", 
   RowBox[{"0.5", "mm"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"magparam", "[", 
  RowBox[{"4.75", ",", "2.6", ",", 
   RowBox[{"0.5", "mm"}], ",", 
   RowBox[{"8", "ohms"}], ",", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"20", "mm"}], ")"}], "3"]}], "]"}]}], "Input",
 CellChangeTimes->{{3.5308677728924637`*^9, 3.530867778270687*^9}, {
  3.530867878274317*^9, 3.5308678787034082`*^9}, {3.530913040352851*^9, 
  3.530913052640807*^9}, {3.530996850455081*^9, 3.530996860621992*^9}, {
  3.531000012734997*^9, 3.53100013236687*^9}, {3.531022322618884*^9, 
  3.531022324998724*^9}, {3.531022386372211*^9, 3.531022404300662*^9}, {
  3.5310224420375757`*^9, 3.531022486195818*^9}, {3.531022785438821*^9, 
  3.531022785952286*^9}, {3.531022984793416*^9, 3.53102301092432*^9}, {
  3.531525545627799*^9, 3.531525581245061*^9}, {3.531525618266749*^9, 
  3.531525626423832*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Magnetisation", "\[Rule]", "1"}], ",", 
   RowBox[{"Current", "\[Rule]", 
    RowBox[{"-", "1"}]}], ",", 
   RowBox[{"MagnetLength", "\[Rule]", 
    RowBox[{"1.9293563503789133`", " ", 
     SuperscriptBox["MagVolume", 
      RowBox[{"1", "/", "3"}]]}]}], ",", 
   RowBox[{"MagnetRadius", "\[Rule]", 
    RowBox[{"0.4061802842902975`", " ", 
     SuperscriptBox["MagVolume", 
      RowBox[{"1", "/", "3"}]]}]}], ",", 
   RowBox[{"CoilRadius", "\[Rule]", 
    RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
     RowBox[{"0.4061802842902975`", " ", 
      SuperscriptBox["MagVolume", 
       RowBox[{"1", "/", "3"}]]}]}]}], ",", 
   RowBox[{"CoilLength", "\[Rule]", 
    RowBox[{"2.6`", " ", 
     RowBox[{"(", 
      RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.4061802842902975`", " ", 
        SuperscriptBox["MagVolume", 
         RowBox[{"1", "/", "3"}]]}]}], ")"}]}]}], ",", 
   RowBox[{"CoilThickness", "\[Rule]", 
    RowBox[{
     RowBox[{"-", "0.0005`"}], "-", 
     RowBox[{"0.4061802842902975`", " ", 
      SuperscriptBox["MagVolume", 
       RowBox[{"1", "/", "3"}]]}], "+", 
     SqrtBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
          RowBox[{"0.4061802842902975`", " ", 
           SuperscriptBox["MagVolume", 
            RowBox[{"1", "/", "3"}]]}]}], ")"}], "2"], "+", 
       FractionBox[
        RowBox[{"5.656108597285068`*^6", " ", "CoilResistance", " ", 
         SuperscriptBox["WireDiam", "4"]}], 
        RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.4061802842902975`", " ", 
          SuperscriptBox["MagVolume", 
           RowBox[{"1", "/", "3"}]]}]}]]}]]}]}], ",", 
   RowBox[{"CoilTurnsR", "\[Rule]", 
    RowBox[{"Round", "[", 
     FractionBox[
      RowBox[{
       RowBox[{"-", "0.0005`"}], "-", 
       RowBox[{"0.4061802842902975`", " ", 
        SuperscriptBox["MagVolume", 
         RowBox[{"1", "/", "3"}]]}], "+", 
       SqrtBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
            RowBox[{"0.4061802842902975`", " ", 
             SuperscriptBox["MagVolume", 
              RowBox[{"1", "/", "3"}]]}]}], ")"}], "2"], "+", 
         FractionBox[
          RowBox[{"5.656108597285068`*^6", " ", "CoilResistance", " ", 
           SuperscriptBox["WireDiam", "4"]}], 
          RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
           RowBox[{"0.4061802842902975`", " ", 
            SuperscriptBox["MagVolume", 
             RowBox[{"1", "/", "3"}]]}]}]]}]]}], "WireDiam"], "]"}]}], ",", 
   RowBox[{"CoilTurnsZ", "\[Rule]", 
    RowBox[{"Round", "[", 
     FractionBox[
      RowBox[{"2.6`", " ", 
       RowBox[{"(", 
        RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.4061802842902975`", " ", 
          SuperscriptBox["MagVolume", 
           RowBox[{"1", "/", "3"}]]}]}], ")"}]}], "WireDiam"], "]"}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.5308677434820004`*^9, 3.530867778789256*^9, 3.530867879310422*^9, {
   3.5309130410555077`*^9, 3.530913053319745*^9}, {3.531000124735517*^9, 
   3.5310001330438633`*^9}, 3.531022349951119*^9, 3.5310224943767357`*^9, 
   3.531022789295512*^9, {3.5310229980580683`*^9, 3.531023011176262*^9}, {
   3.531525553080307*^9, 3.53152558275309*^9}, 3.531525627196446*^9, 
   3.531704623537663*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Magnetisation", "\[Rule]", "1"}], ",", 
   RowBox[{"Current", "\[Rule]", 
    RowBox[{"-", "1"}]}], ",", 
   RowBox[{"MagnetLength", "\[Rule]", 
    RowBox[{"1.9293563503789133`", " ", 
     SuperscriptBox["MagVolume", 
      RowBox[{"1", "/", "3"}]]}]}], ",", 
   RowBox[{"MagnetRadius", "\[Rule]", 
    RowBox[{"0.4061802842902975`", " ", 
     SuperscriptBox["MagVolume", 
      RowBox[{"1", "/", "3"}]]}]}], ",", 
   RowBox[{"CoilRadius", "\[Rule]", 
    RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
     RowBox[{"0.4061802842902975`", " ", 
      SuperscriptBox["MagVolume", 
       RowBox[{"1", "/", "3"}]]}]}]}], ",", 
   RowBox[{"CoilLength", "\[Rule]", 
    RowBox[{"2.6`", " ", 
     RowBox[{"(", 
      RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.4061802842902975`", " ", 
        SuperscriptBox["MagVolume", 
         RowBox[{"1", "/", "3"}]]}]}], ")"}]}]}], ",", 
   RowBox[{"CoilThickness", "\[Rule]", 
    RowBox[{
     RowBox[{"-", "0.0005`"}], "+", 
     SqrtBox[
      RowBox[{
       FractionBox[
        RowBox[{"3.5350678733031663`*^-7", " ", "CoilResistance"}], 
        RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.4061802842902975`", " ", 
          SuperscriptBox["MagVolume", 
           RowBox[{"1", "/", "3"}]]}]}]], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
          RowBox[{"0.4061802842902975`", " ", 
           SuperscriptBox["MagVolume", 
            RowBox[{"1", "/", "3"}]]}]}], ")"}], "2"]}]], "-", 
     RowBox[{"0.4061802842902975`", " ", 
      SuperscriptBox["MagVolume", 
       RowBox[{"1", "/", "3"}]]}]}]}], ",", 
   RowBox[{"CoilTurnsR", "\[Rule]", 
    RowBox[{"Round", "[", 
     RowBox[{"2000.`", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "0.0005`"}], "+", 
        SqrtBox[
         RowBox[{
          FractionBox[
           RowBox[{"3.5350678733031663`*^-7", " ", "CoilResistance"}], 
           RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
            RowBox[{"0.4061802842902975`", " ", 
             SuperscriptBox["MagVolume", 
              RowBox[{"1", "/", "3"}]]}]}]], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
             RowBox[{"0.4061802842902975`", " ", 
              SuperscriptBox["MagVolume", 
               RowBox[{"1", "/", "3"}]]}]}], ")"}], "2"]}]], "-", 
        RowBox[{"0.4061802842902975`", " ", 
         SuperscriptBox["MagVolume", 
          RowBox[{"1", "/", "3"}]]}]}], ")"}]}], "]"}]}], ",", 
   RowBox[{"CoilTurnsZ", "\[Rule]", 
    RowBox[{"Round", "[", 
     RowBox[{"5200.`", " ", 
      RowBox[{"(", 
       RowBox[{"0.0005`", "\[VeryThinSpace]", "+", 
        RowBox[{"0.4061802842902975`", " ", 
         SuperscriptBox["MagVolume", 
          RowBox[{"1", "/", "3"}]]}]}], ")"}]}], "]"}]}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5308677434820004`*^9, 3.530867778789256*^9, 3.530867879310422*^9, {
   3.5309130410555077`*^9, 3.530913053319745*^9}, {3.531000124735517*^9, 
   3.5310001330438633`*^9}, 3.531022349951119*^9, 3.5310224943767357`*^9, 
   3.531022789295512*^9, {3.5310229980580683`*^9, 3.531023011176262*^9}, {
   3.531525553080307*^9, 3.53152558275309*^9}, 3.531525627196446*^9, 
   3.531704623550768*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Magnetisation", "\[Rule]", "1"}], ",", 
   RowBox[{"Current", "\[Rule]", 
    RowBox[{"-", "1"}]}], ",", 
   RowBox[{"MagnetLength", "\[Rule]", "0.03858712700757828`"}], ",", 
   RowBox[{"MagnetRadius", "\[Rule]", "0.008123605685805953`"}], ",", 
   RowBox[{"CoilRadius", "\[Rule]", "0.008623605685805953`"}], ",", 
   RowBox[{"CoilLength", "\[Rule]", "0.022421374783095478`"}], ",", 
   RowBox[{"CoilThickness", "\[Rule]", "0.01143405977379686`"}], ",", 
   RowBox[{"CoilTurnsR", "\[Rule]", "23"}], ",", 
   RowBox[{"CoilTurnsZ", "\[Rule]", "45"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.5308677434820004`*^9, 3.530867778789256*^9, 3.530867879310422*^9, {
   3.5309130410555077`*^9, 3.530913053319745*^9}, {3.531000124735517*^9, 
   3.5310001330438633`*^9}, 3.531022349951119*^9, 3.5310224943767357`*^9, 
   3.531022789295512*^9, {3.5310229980580683`*^9, 3.531023011176262*^9}, {
   3.531525553080307*^9, 3.53152558275309*^9}, 3.531525627196446*^9, 
   3.531704623556404*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{"dx", ":=", 
  RowBox[{"1", "mm"}]}], "\[IndentingNewLine]", 
 RowBox[{"xmax", ":=", 
  RowBox[{"40", "mm"}]}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Various functions we need", "Subsection",
 CellChangeTimes->{{3.531013625134231*^9, 3.5310136293468313`*^9}, {
  3.53276686395765*^9, 3.5327668681871433`*^9}}],

Cell["\<\
This code evolved somewhat organically so there\[CloseCurlyQuote]s a bit of \
overlap and experimentation in here.
Sorry if it\[CloseCurlyQuote]s all a bit of a mess.\
\>", "Text",
 CellChangeTimes->{{3.532766869556202*^9, 3.532766892593606*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"MagnetCoilForceFn", "[", 
   RowBox[{
    RowBox[{"x_", "?", "NumberQ"}], ",", 
    RowBox[{"a_", "?", "NumberQ"}], ",", 
    RowBox[{"b_", "?", "NumberQ"}], ",", 
    RowBox[{"d_", "?", "NumberQ"}], ",", 
    RowBox[{"R_", "?", "NumberQ"}], ",", 
    RowBox[{"V_", "?", "NumberQ"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"s", ",", "p"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"s", "=", 
      RowBox[{"magparam", "[", 
       RowBox[{"a", ",", "b", ",", "d", ",", "R", ",", "V"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"p", "=", 
      RowBox[{"Flatten", "@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
         RowBox[{"Method", "\[Rule]", "\"\<Shell\>\""}], ",", "s"}], 
        "}"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"MagnetCoilForce", "[", "p", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForceFn", "[", 
   RowBox[{
    RowBox[{"x_", "?", "NumberQ"}], ",", 
    RowBox[{"a_", "?", "NumberQ"}], ",", 
    RowBox[{"b_", "?", "NumberQ"}], ",", 
    RowBox[{"d_", "?", "NumberQ"}]}], "]"}], ":=", 
  RowBox[{"MagnetCoilForceFn", "[", 
   RowBox[{
   "x", ",", "a", ",", "b", ",", "d", ",", "CoilResistance", ",", 
    "MagVolume"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForceFn", "[", 
   RowBox[{
    RowBox[{"x_", "?", "NumberQ"}], ",", 
    RowBox[{"a_", "?", "NumberQ"}], ",", 
    RowBox[{"b_", "?", "NumberQ"}]}], "]"}], ":=", 
  RowBox[{"MagnetCoilForceFn", "[", 
   RowBox[{
   "x", ",", "a", ",", "b", ",", "WireDiam", ",", "CoilResistance", ",", 
    "MagVolume"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.530395141070418*^9, 3.5303951667145147`*^9}, {
   3.530395223892181*^9, 3.530395277472005*^9}, {3.5303956110384893`*^9, 
   3.530395679115591*^9}, {3.530395821257185*^9, 3.530395821598509*^9}, {
   3.530405483662218*^9, 3.530405483905305*^9}, {3.53040757124535*^9, 
   3.530407572912415*^9}, {3.530407914679078*^9, 3.530407964033022*^9}, {
   3.530408059447954*^9, 3.530408073417036*^9}, {3.530408128983395*^9, 
   3.530408129549343*^9}, {3.530408301082837*^9, 3.53040830273778*^9}, {
   3.530408456146613*^9, 3.530408502392315*^9}, 3.5304153977843447`*^9, 
   3.530415863889744*^9, {3.530486168739999*^9, 3.5304861691351223`*^9}, {
   3.53048634271419*^9, 3.530486357394219*^9}, {3.530486392617415*^9, 
   3.530486406675949*^9}, {3.531000303686986*^9, 3.5310003687755527`*^9}, {
   3.531000416026626*^9, 3.5310004169841013`*^9}, {3.5310005756089973`*^9, 
   3.5310005756806087`*^9}, {3.531023767284873*^9, 3.531023774232245*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"DisplMax", "[", 
    RowBox[{"\[Alpha]_", ",", "\[Beta]_", ",", "d_", ",", "R_", ",", "V_"}], 
    "]"}], ":=", 
   RowBox[{
    RowBox[{"5", 
     RowBox[{"Max", "[", 
      RowBox[{"{", 
       RowBox[{"MagnetLength", ",", "CoilLength"}], "}"}], "]"}]}], "//.", 
    RowBox[{"magparam", "[", 
     RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "d", ",", "R", ",", "V"}], 
     "]"}]}]}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForceMax", "[", 
   RowBox[{
    RowBox[{"a_", "?", "NumberQ"}], ",", 
    RowBox[{"b_", "?", "NumberQ"}], ",", 
    RowBox[{"d_", "?", "NumberQ"}], ",", 
    RowBox[{"R_", "?", "NumberQ"}], ",", 
    RowBox[{"V_", "?", "NumberQ"}]}], "]"}], ":=", 
  RowBox[{"Max", "@", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"MagnetCoilForceFn", "[", 
      RowBox[{"x", ",", "a", ",", "b", ",", "d", ",", "R", ",", "V"}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"2", "mm"}], ",", 
       RowBox[{"DisplMax", "[", 
        RowBox[{"a", ",", "b", ",", "d", ",", "R", ",", "V"}], "]"}], ",", 
       RowBox[{
        RowBox[{"DisplMax", "[", 
         RowBox[{"a", ",", "b", ",", "d", ",", "R", ",", "V"}], "]"}], "/", 
        "100"}]}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForceMax", "[", 
   RowBox[{
    RowBox[{"a_", "?", "NumberQ"}], ",", 
    RowBox[{"b_", "?", "NumberQ"}], ",", 
    RowBox[{"d_", "?", "NumberQ"}]}], "]"}], ":=", 
  RowBox[{"MagnetCoilForceMax", "[", 
   RowBox[{"a", ",", "b", ",", "d", ",", "CoilResistance", ",", "MagVolume"}],
    "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MagnetCoilForceMax", "[", 
    RowBox[{
     RowBox[{"a_", "?", "NumberQ"}], ",", 
     RowBox[{"b_", "?", "NumberQ"}]}], "]"}], ":=", 
   RowBox[{"MagnetCoilForceMax", "[", 
    RowBox[{
    "a", ",", "b", ",", "WireDiam", ",", "CoilResistance", ",", "MagVolume"}],
     "]"}]}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForceMaxAlt", "[", 
   RowBox[{
    RowBox[{"a_", "?", "NumberQ"}], ",", 
    RowBox[{"b_", "?", "NumberQ"}], ",", 
    RowBox[{"d_", "?", "NumberQ"}], ",", 
    RowBox[{"R_", "?", "NumberQ"}], ",", 
    RowBox[{"V_", "?", "NumberQ"}]}], "]"}], ":=", 
  RowBox[{"FindMaxValue", "[", 
   RowBox[{
    RowBox[{"MagnetCoilForceFn", "[", 
     RowBox[{"x", ",", "a", ",", "b", ",", "d", ",", "R", ",", "V"}], "]"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{
       RowBox[{"DisplMax", "[", 
        RowBox[{"a", ",", "b", ",", "d", ",", "R", ",", "V"}], "]"}], "/", 
       "2"}], ",", 
      RowBox[{"1", "mm"}], ",", 
      RowBox[{"DisplMax", "[", 
       RowBox[{"a", ",", "b", ",", "d", ",", "R", ",", "V"}], "]"}]}], "}"}], 
    ",", 
    RowBox[{"AccuracyGoal", "\[Rule]", "2"}], ",", 
    RowBox[{"PrecisionGoal", "\[Rule]", "2"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForceMaxAlt", "[", 
   RowBox[{
    RowBox[{"a_", "?", "NumberQ"}], ",", 
    RowBox[{"b_", "?", "NumberQ"}], ",", 
    RowBox[{"d_", "?", "NumberQ"}]}], "]"}], ":=", 
  RowBox[{"MagnetCoilForceMaxAlt", "[", 
   RowBox[{"a", ",", "b", ",", "d", ",", "CoilResistance", ",", "MagVolume"}],
    "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MagnetCoilForceMaxAlt", "[", 
    RowBox[{
     RowBox[{"a_", "?", "NumberQ"}], ",", 
     RowBox[{"b_", "?", "NumberQ"}]}], "]"}], ":=", 
   RowBox[{"MagnetCoilForceMaxAlt", "[", 
    RowBox[{
    "a", ",", "b", ",", "WireDiam", ",", "CoilResistance", ",", "MagVolume"}],
     "]"}]}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MagnetCoilForceMaxScaled", "[", 
   RowBox[{
    RowBox[{"a_", "?", "NumberQ"}], ",", 
    RowBox[{"b_", "?", "NumberQ"}], ",", 
    RowBox[{"d_", "?", "NumberQ"}], ",", 
    RowBox[{"R_", "?", "NumberQ"}], ",", 
    RowBox[{"V_", "?", "NumberQ"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"CurrentFromDiam", "[", "d", "]"}], "\[Times]", 
   RowBox[{"MagnetCoilForceMaxAlt", "[", 
    RowBox[{"a", ",", "b", ",", "d", ",", "R", ",", "V"}], 
    "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.530866939929921*^9, 3.530867111085322*^9}, {
   3.5308671614199743`*^9, 3.5308672135618677`*^9}, {3.5308673014293213`*^9, 
   3.530867326611278*^9}, {3.530869167062359*^9, 3.530869177153694*^9}, {
   3.530869209767724*^9, 3.5308692299241543`*^9}, {3.530869270454965*^9, 
   3.530869302143712*^9}, {3.530869371732486*^9, 3.5308694295577917`*^9}, {
   3.530869787468231*^9, 3.530869821996155*^9}, {3.530870055440539*^9, 
   3.5308700585788403`*^9}, {3.5308701023523693`*^9, 3.530870102435564*^9}, {
   3.530870508243445*^9, 3.5308705100043077`*^9}, {3.530870545541462*^9, 
   3.5308705740676203`*^9}, {3.5310004341032257`*^9, 3.531000516936474*^9}, {
   3.531000641105632*^9, 3.531000684879459*^9}, {3.531000728233267*^9, 
   3.531000797285927*^9}, {3.531001672492416*^9, 3.531001674444566*^9}, {
   3.5310146729576693`*^9, 3.531014676540773*^9}, 3.531015041689137*^9, {
   3.5310237810852118`*^9, 3.531023796274798*^9}, {3.53102387386658*^9, 
   3.5310238740994797`*^9}, {3.53137183626397*^9, 3.5313718749233217`*^9}, {
   3.5317047315996323`*^9, 3.531704784349339*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Min/max coil ratios", "Subsection",
 CellChangeTimes->{{3.532766906707871*^9, 3.532766917011195*^9}}],

Cell["\<\
I found early on that the optimisations were hampered by pathological cases \
of very large or small coil ratios; we want to ensure it doesn\
\[CloseCurlyQuote]t exceed the natural limits imposed by having to physically \
wind a wire.\
\>", "Text",
 CellChangeTimes->{{3.5327669210604887`*^9, 3.532766967725329*^9}}],

Cell["\<\
Maximum ratio: (only one radial turn; coil thickness = wire diameter)\
\>", "Text",
 CellChangeTimes->{{3.531003011940989*^9, 3.5310030139057817`*^9}, {
  3.531525340259554*^9, 3.531525360123562*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SqrtBox[
    RowBox[{
     FractionBox[
      RowBox[{
       SuperscriptBox["dw", "2"], " ", "lw"}], 
      RowBox[{"lc", " ", "\[Pi]"}]], "+", 
     SuperscriptBox["rc", "2"]}]], "-", "rc"}], "\[Equal]", 
  "dw"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "//.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"lw", "->", 
      RowBox[{"R", " ", 
       RowBox[{"aw", " ", "/", " ", "\[Rho]"}]}]}], ",", 
     RowBox[{"aw", " ", "->", " ", 
      RowBox[{"\[Pi]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"dw", "/", "2"}], ")"}], "^", "2"}]}]}], ",", 
     RowBox[{"rm", "->", 
      SuperscriptBox[
       RowBox[{"(", 
        FractionBox["V", 
         RowBox[{"\[Pi]", " ", "\[Alpha]"}]], ")"}], 
       RowBox[{"1", "/", "3"}]]}], ",", 
     RowBox[{"rc", "->", 
      RowBox[{"rm", "+", "rg"}]}], ",", 
     RowBox[{"lc", "->", 
      RowBox[{"rc", " ", "\[Beta]"}]}]}], "}"}]}], "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{"Solve", "[", 
  RowBox[{"%", ",", "\[Beta]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.531002129320777*^9, 3.531002136516582*^9}, {
   3.531002219728058*^9, 3.531002315704068*^9}, {3.531002349992877*^9, 
   3.5310024166500673`*^9}, 3.531002503340366*^9, {3.5314575293339367`*^9, 
   3.53145753336451*^9}, 3.531457564408263*^9, {3.5314576012568808`*^9, 
   3.531457634789913*^9}, {3.5314576675211487`*^9, 3.531457689405526*^9}, {
   3.531457724410865*^9, 3.531457738921939*^9}, {3.531481666705831*^9, 
   3.531481667357782*^9}, {3.531525408972889*^9, 3.531525513499733*^9}, {
   3.531525650128832*^9, 3.5315258309153023`*^9}, {3.53152610235983*^9, 
   3.5315261204565477`*^9}, {3.531526240195448*^9, 3.5315262403596888`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "rc"}], "+", 
   SqrtBox[
    RowBox[{
     FractionBox[
      RowBox[{
       SuperscriptBox["dw", "2"], " ", "lw"}], 
      RowBox[{"lc", " ", "\[Pi]"}]], "+", 
     SuperscriptBox["rc", "2"]}]]}], "\[Equal]", "dw"}]], "Output",
 CellChangeTimes->{{3.531002392241859*^9, 3.531002418374352*^9}, 
   3.5310025052582417`*^9, 3.5314815464286833`*^9, 3.5314817043607807`*^9, {
   3.531525442052788*^9, 3.5315254511708393`*^9}, {3.5315254874927473`*^9, 
   3.531525516264883*^9}, {3.53152565547746*^9, 3.5315257163301897`*^9}, {
   3.5315257844914207`*^9, 3.531525831795061*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"rg", "+", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          FractionBox["V", "\[Alpha]"], ")"}], 
         RowBox[{"1", "/", "3"}]], 
        SuperscriptBox["\[Pi]", 
         RowBox[{"1", "/", "3"}]]]}], ")"}], "2"], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["dw", "4"], " ", 
      SuperscriptBox["\[Pi]", 
       RowBox[{"1", "/", "3"}]], " ", "R"}], 
     RowBox[{
      RowBox[{"4", " ", 
       SuperscriptBox["\[Pi]", 
        RowBox[{"1", "/", "3"}]], " ", "rg", " ", "\[Beta]", " ", "\[Rho]"}], 
      "+", 
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         FractionBox["V", "\[Alpha]"], ")"}], 
        RowBox[{"1", "/", "3"}]], " ", "\[Beta]", " ", "\[Rho]"}]}]]}]], 
  "\[Equal]", 
  RowBox[{"dw", "+", "rg", "+", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["V", "\[Alpha]"], ")"}], 
     RowBox[{"1", "/", "3"}]], 
    SuperscriptBox["\[Pi]", 
     RowBox[{"1", "/", "3"}]]]}]}]], "Output",
 CellChangeTimes->{{3.531002392241859*^9, 3.531002418374352*^9}, 
   3.5310025052582417`*^9, 3.5314815464286833`*^9, 3.5314817043607807`*^9, {
   3.531525442052788*^9, 3.5315254511708393`*^9}, {3.5315254874927473`*^9, 
   3.531525516264883*^9}, {3.53152565547746*^9, 3.5315257163301897`*^9}, {
   3.5315257844914207`*^9, 3.531525812515388*^9}, 3.5315258505804453`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"\[Beta]", "\[Rule]", 
  FractionBox[
   RowBox[{
    SuperscriptBox["dw", "3"], " ", 
    SuperscriptBox["\[Pi]", 
     RowBox[{"2", "/", "3"}]], " ", "R"}], 
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Pi]", 
        RowBox[{"1", "/", "3"}]], " ", "rg"}], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        FractionBox["V", "\[Alpha]"], ")"}], 
       RowBox[{"1", "/", "3"}]]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Pi]", 
        RowBox[{"1", "/", "3"}]], " ", 
       RowBox[{"(", 
        RowBox[{"dw", "+", 
         RowBox[{"2", " ", "rg"}]}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         FractionBox["V", "\[Alpha]"], ")"}], 
        RowBox[{"1", "/", "3"}]]}]}], ")"}], " ", "\[Rho]"}]]}]], "Input",
 CellChangeTimes->{{3.531526149051284*^9, 3.531526152069353*^9}, {
   3.531526198888969*^9, 3.531526265202918*^9}, 3.531526397459875*^9}],

Cell["\<\
Minimum ratio (only one axial turn; coil length = wire diameter):\
\>", "Text",
 CellChangeTimes->{{3.531003503374901*^9, 3.53100350701278*^9}, {
  3.5315253205015373`*^9, 3.531525378763671*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          FractionBox["V", 
           RowBox[{"\[Pi]", " ", "\[Alpha]"}]], ")"}], 
         RowBox[{"1", "/", "3"}]], "+", "rg"}], ")"}], " ", "\[Beta]"}], 
     "\[Equal]", "d"}], ",", "\[Beta]"}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.53100355671668*^9, 3.531003586851054*^9}, {
  3.531481592300336*^9, 3.531481592384404*^9}, {3.531481631166162*^9, 
  3.53148163840938*^9}, {3.531481737121668*^9, 3.531481767388123*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"\[Beta]", "\[Rule]", 
    FractionBox[
     RowBox[{"d", " ", 
      SuperscriptBox["\[Pi]", 
       RowBox[{"1", "/", "3"}]]}], 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Pi]", 
        RowBox[{"1", "/", "3"}]], " ", "rg"}], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        FractionBox["V", "\[Alpha]"], ")"}], 
       RowBox[{"1", "/", "3"}]]}]]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{
  3.531003588060356*^9, 3.531481593961506*^9, 3.531481639882821*^9, {
   3.531481748406596*^9, 3.5314817680688467`*^9}}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"coilRatioMax", "[", 
   RowBox[{"\[Alpha]_", ",", "d_", ",", "R_", ",", "V_"}], "]"}], ":=", 
  FractionBox[
   RowBox[{
    SuperscriptBox["d", "3"], " ", 
    SuperscriptBox["\[Pi]", 
     RowBox[{"2", "/", "3"}]], " ", "R"}], 
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Pi]", 
        RowBox[{"1", "/", "3"}]], " ", "GapRadius"}], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        FractionBox["V", "\[Alpha]"], ")"}], 
       RowBox[{"1", "/", "3"}]]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Pi]", 
        RowBox[{"1", "/", "3"}]], " ", 
       RowBox[{"(", 
        RowBox[{"d", "+", 
         RowBox[{"2", " ", "GapRadius"}]}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         FractionBox["V", "\[Alpha]"], ")"}], 
        RowBox[{"1", "/", "3"}]]}]}], ")"}], " ", 
    "WireResistivity"}]]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"coilRatioMin", "[", 
   RowBox[{"\[Alpha]_", ",", "d_", ",", "V_"}], "]"}], ":=", 
  FractionBox["d", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["V", 
       RowBox[{"\[Pi]", " ", "\[Alpha]"}]], ")"}], 
     RowBox[{"1", "/", "3"}]], "+", "GapRadius"}]]}]}], "Input",
 CellChangeTimes->{{3.531002450298849*^9, 3.531002534053406*^9}, {
  3.531003611316386*^9, 3.531003634993681*^9}, {3.5310036792848797`*^9, 
  3.531003732607746*^9}, {3.5310137358431673`*^9, 3.5310137413084917`*^9}, {
  3.5315263196376534`*^9, 3.531526370907646*^9}, {3.531526427142396*^9, 
  3.531526441173978*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"coilRatioMax", "[", 
  RowBox[{"0.1", ",", 
   RowBox[{"0.2", "mm"}], ",", "2", ",", "0.00022"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"coilRatioMin", "[", 
  RowBox[{"0.1", ",", 
   RowBox[{"0.2", "mm"}], ",", "0.00022"}], "]"}]}], "Input",
 CellChangeTimes->{{3.5310025643232317`*^9, 3.531002573436692*^9}, {
  3.531003709901993*^9, 3.531003737105974*^9}}],

Cell[BoxData["0.014735661227836564`"], "Output",
 CellChangeTimes->{
  3.531002574543169*^9, {3.531003715372233*^9, 3.531003744193101*^9}, {
   3.5315263640222073`*^9, 3.5315263741102247`*^9}, {3.531526432820065*^9, 
   3.531526443116887*^9}}],

Cell[BoxData["0.0022395836922033036`"], "Output",
 CellChangeTimes->{
  3.531002574543169*^9, {3.531003715372233*^9, 3.531003744193101*^9}, {
   3.5315263640222073`*^9, 3.5315263741102247`*^9}, {3.531526432820065*^9, 
   3.5315264431221*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Force versus displacement while varying coil and magnet ratios\
\>", "Subsection",
 CellChangeTimes->{{3.5310215436922903`*^9, 3.531021545290001*^9}, {
  3.532766978741037*^9, 3.532766993461816*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"betarange", "=", 
   RowBox[{"{", 
    RowBox[{"0.2", ",", "0.5", ",", "1", ",", "2", ",", "4", ",", "8"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outercoil1", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Alpha]", "=", "2"}], ",", 
       RowBox[{"MagVolume", "=", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"20", "mm"}], ")"}], "3"]}], ",", 
       RowBox[{"CoilResistance", "=", 
        RowBox[{"4", "ohms"}]}], ",", 
       RowBox[{"WireDiam", "=", 
        RowBox[{"1", "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"1000", "x"}], ",", 
           RowBox[{"MagnetCoilForce", "@", 
            RowBox[{"Flatten", "@", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"Method", "\[Rule]", "\"\<Shell\>\""}], ",", 
               RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
               RowBox[{"IntegrationPrecision", "\[Rule]", "3"}], ",", 
               RowBox[{"magparam", "[", 
                RowBox[{
                "\[Alpha]", ",", "\[Beta]", ",", "WireDiam", ",", 
                 "CoilResistance", ",", "MagVolume"}], "]"}]}], "}"}]}]}]}], 
          "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", 
           RowBox[{"2", "xmax"}], ",", "dx"}], "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Beta]", ",", "betarange"}], "}"}]}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.529721523734128*^9, 3.529721603326482*^9}, {
   3.529721643727309*^9, 3.529721649274551*^9}, {3.5297217343906116`*^9, 
   3.529721773833557*^9}, {3.5297218284058857`*^9, 3.529721909151206*^9}, {
   3.5297219418944263`*^9, 3.529721941997654*^9}, {3.529721972464243*^9, 
   3.529721982011956*^9}, {3.529722262331712*^9, 3.5297222626401243`*^9}, {
   3.529722348460517*^9, 3.529722348663426*^9}, {3.529808970150538*^9, 
   3.5298090014820223`*^9}, {3.530348588994258*^9, 3.530348590079336*^9}, {
   3.530502857444809*^9, 3.5305028576412277`*^9}, {3.530502911233264*^9, 
   3.5305029120968237`*^9}, {3.5310226696364307`*^9, 
   3.5310227532382317`*^9}, {3.531022809541533*^9, 3.531022811044488*^9}, {
   3.531022859818091*^9, 3.531022874232539*^9}, {3.531023049976616*^9, 
   3.53102305024972*^9}, {3.531023091449778*^9, 3.5310230987124*^9}, {
   3.531023139443898*^9, 3.531023145274685*^9}, 3.531792178872346*^9, {
   3.531792218581098*^9, 3.531792331019415*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"annot", "[", "s_", "]"}], ":=", 
  RowBox[{"PSfrag", "[", 
   RowBox[{"s", ",", 
    RowBox[{"TeXCommand", "\[Rule]", 
     RowBox[{
     "\"\<\\\\fboxsep=1pt\\\\footnotesize\\\\colorbox{white}{\>\"", "<>", "s",
       "<>", "\"\<}\>\""}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"coilratioPlot", "=", 
  RowBox[{"Show", "[", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"Reverse", "@", "outercoil1"}], ",", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"Frame", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Displacement, mm\>\"", ",", "\"\<Force per amp, N/A\>\""}], 
         "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"ImageSize", "\[Rule]", 
        RowBox[{"8", " ", 
         RowBox[{"72", "/", "2.54"}]}]}]}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{
        RowBox[{"PSfrag", "[", 
         RowBox[{"\"\<\[Beta] = 000\>\"", ",", 
          RowBox[{"TeXCommand", "\[Rule]", 
           RowBox[{
           "\"\<\\\\fboxsep=1pt\\\\footnotesize\\\\colorbox{white}{$\\\\beta=\
\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{
             "betarange", "\[LeftDoubleBracket]", "1", 
              "\[RightDoubleBracket]"}], "]"}], "<>", "\"\<$}\>\""}]}]}], 
         "]"}], ",", 
        RowBox[{"outercoil1", "\[LeftDoubleBracket]", 
         RowBox[{"1", ",", "30"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{
        RowBox[{"annot", "@", 
         RowBox[{"ToString", "[", 
          RowBox[{
          "betarange", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
           "]"}]}], ",", 
        RowBox[{"outercoil1", "\[LeftDoubleBracket]", 
         RowBox[{"2", ",", "32"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{
        RowBox[{"annot", "@", 
         RowBox[{"ToString", "[", 
          RowBox[{
          "betarange", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
           "]"}]}], ",", 
        RowBox[{"outercoil1", "\[LeftDoubleBracket]", 
         RowBox[{"3", ",", "32"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{
        RowBox[{"annot", "@", 
         RowBox[{"ToString", "[", 
          RowBox[{
          "betarange", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}],
           "]"}]}], ",", 
        RowBox[{"outercoil1", "\[LeftDoubleBracket]", 
         RowBox[{"4", ",", "25"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{
        RowBox[{"annot", "@", 
         RowBox[{"ToString", "[", 
          RowBox[{
          "betarange", "\[LeftDoubleBracket]", "5", "\[RightDoubleBracket]"}],
           "]"}]}], ",", 
        RowBox[{"outercoil1", "\[LeftDoubleBracket]", 
         RowBox[{"5", ",", "33"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Graphics", "@", 
      RowBox[{"Text", "[", 
       RowBox[{
        RowBox[{"annot", "@", 
         RowBox[{"ToString", "[", 
          RowBox[{
          "betarange", "\[LeftDoubleBracket]", "6", "\[RightDoubleBracket]"}],
           "]"}]}], ",", 
        RowBox[{"outercoil1", "\[LeftDoubleBracket]", 
         RowBox[{"6", ",", "55"}], "\[RightDoubleBracket]"}]}], "]"}]}]}], 
    "\[IndentingNewLine]", "}"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.529809085073724*^9, 3.529809212081563*^9}, {
   3.529809255445449*^9, 3.5298092976867247`*^9}, {3.5298093335881233`*^9, 
   3.529809346304268*^9}, {3.529809416071454*^9, 3.529809417053248*^9}, {
   3.529809661076013*^9, 3.5298096658455877`*^9}, 3.52980969975294*^9, {
   3.529809791157524*^9, 3.529809800358873*^9}, {3.52980991469665*^9, 
   3.529809948326025*^9}, {3.529810029296039*^9, 3.529810029766173*^9}, {
   3.5298100617913427`*^9, 3.529810084342622*^9}, {3.529810123248456*^9, 
   3.529810201183525*^9}, {3.529810252130142*^9, 3.529810266704001*^9}, {
   3.530502959684078*^9, 3.5305030419506693`*^9}, {3.5305030958413477`*^9, 
   3.530503138972239*^9}, {3.530503184865293*^9, 3.530503196521578*^9}, {
   3.5305032532682962`*^9, 3.530503254524678*^9}, {3.530503293108841*^9, 
   3.530503299056409*^9}, {3.5310229404815474`*^9, 3.531022941573008*^9}, {
   3.5317923468569613`*^9, 3.5317923495527973`*^9}}],

Cell[BoxData[
 GraphicsBox[{{
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxtk3lME2kYh6u1lHpE0KLEGFBRLC4SVlSwCP0Bpdyll7oim6qgaaKGSJCy
8WDA2wR0QwKYyooHqaBogg0uEZRKrK4KS1gNhza1XiCiAlpZQMSj7/znJJPJ
M09+8877fe83f3OGastEDoej/HZ/f/786pccyszKUheUSugFdPzSYOtfF4hd
ETq9MM+3oIbYDUbP+tZ9L/8mFmLPhfX3r226SewJD+26qhpeE/FcFPnPuxqZ
bSGeh+fRHpb7/HuSCT94AXzWW5u7bjaTXwiFOX7XL22txL6Q9h423rG2EYtQ
EaAdX+jyiPJLsN287cZreTt5f0w1vbvOr+8kDsAl7p/i28onxIGY2GAWW4Q2
4l+xyr3KewrPTt9bBt+16SEnDc/IByFV+mhcoHxBvByDZfsTykSviFdAnxcj
f7yoh3glHOMtO08m9hIHY63fsuov5X3EIai83Cg4vPg98SqkdEX/nz/YT/XF
2LU0V6vOGCQfCkOqZp9j6kfi1ShpOletanEQh+GKYCQwp3aIOBxphd6lx+8O
E0tw8IV0rI3/mRiYM6oP2a3/4mQG6A7inlAVcOD0EZBpRh+4jExwMhMBR36O
UZTDhfP/IlGWEtA3pXMS+UjYhAc3tLq7UD4KXcd4aww+fPJRkCu9Tl+c7kpe
CtVQbFJQOzEjxaeef0vidgvIR6Pr2hZv+2diJhoHIgcCzKmTycuwekeUe56B
mJGhkDs0cKCO9TFYFPHb8Ggt62MQnnU221bE+li4KQy8D8msj0V22PWa271s
/TjERT0INqUJnP0ycZCVXjrUV+dK/cdjrm54PLOP7S8e4jMGh98I238CRPnP
jj6x88gnoFK9Mld+ltaLk4jzDe2e/4i55BNhDbYZ62/RenOScKp4ICgwjUP1
k9AxbB+bFUb7xZHj8qkl/5lejdL+ybEzv0Hr2cjudzJmP7RummaheWCSEX5M
YEp3sPOigM8Mk9ekBJonKMAU2e70N9G8MQrclYd2r/l9wMmNCpQPut07Iuyn
vBK5W93qPFreUl4J0VO9XljxhvJKdM/cqLtV+do5z41KNPjbPvh1dNN8qxBo
MVcb/en8QIXy0xvHXp6j88V840qvlA7xc6qvwlZNrab2nZ3qq5GbqSv5VP+U
8mqk7y0YKpLReWbUaNYVlyyVWCmvRu+sqj96tI8pr0FGqGJDYkWn5CubQ3Gh

      "]]}, 
    {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJxtlHlQE2cYh0Pw4CyIVhGxHIUggigRCIohv3AIgUQ2QUSRyAgTrWAF0Qpj
aLPV1qgMjgidpniOeI4cDo4iKsiAUxwR4sUxWJ0I2NJSUVCoYos98u5/3Zmd
nWef/X3vt/u933qkZ6s0fB6Pp/zn/Pf6/8criYtjf/FnstsSuoHrzKaClNXd
xFYYXbzPZXhlL7EjSqVN+11KB4hnQPdNi7fz8BCxM6xe61PP1I0Qu6Ijx9fV
ctoYsTtK7NY1iHa9k1j8x55g9+uF9Y5/kfeC4qZ+8GUaD2YWwD3lhDpJxiee
B03d3VndEZNgzs9Hb+mzJQXeU8j7Y077ar/fuqcSB2DTUO3cFrU18SKEOR2t
Fl+2IQ6EviglbIvJlsYTYmJb2WyNyY78Yti/7Zw5/Yo9cRCyjYK42PSPiIPR
+0t5+cXnHIegWau8ch8OxCL0ND3Osd7OcSjWCSx/KNJxvAS5IblvsjIcqP5S
3Pgq2LXQjfNhKHPrs664xI2/DPrvknZtduNYDGO9+FhcOje/cFxsK+45r+Xm
L8HZ7QKmJ9OWGOi96qH1XEjvzwI2jV2lHS1W5KXwbDlVOBBO34+Volz0fbu+
aTLNLwL5DbXLjqonkY/A09QqhdzBkvKRWDX3Q+aBTgvykZDuYHzlNdx6RiH8
iNe1togP5vVmo5BrHBfqorn1jwY/0G4iL/M9+WhsPFbLL6l+R345LrVmBd10
ekt+OWS6+s/5h7j+isGj9fF+Wr9R8jFoFhZtXf/kNflYWGiSZkw9Q/3JxmKk
+Y+9XbuHycvQOSFOT/7ilbk/WRluWSS4xowMUb/G4Wqx8z3TnBeUj0NG+by1
0zSDlI+HQZbbbGj9lXw8ZqeNytMU3H6R49vzXT7bBn4mL8e5DHw65fBz8gqM
V2o3Z2X0U30FqrV9aR+jj/wK5AxmZ5cspP3IrsDXwkKPg4uekU9AgNePvkap
iXwCRKF7QpNvPSXPoEonyPf3eGJmMGgUld32MTym5xkE8Awz8316zNzIwL2i
qGFLG/c/UMImjx0r2NtFeSUqWc2GwOROyitxPO/Q78GhHeb5NypxUvOwUDH/
EX0/FX4arGBV/g8pr0LNmmRtkPgB5VXYeNr+z7G196m+CrNYtbFaf4/qJ2LB
zi8ZW52R8omoGT95QT25nfKJSO3XeZtO3KV8IsINdd0hTCvlV2K6b6buE6c7
kr8Bl59dEw==
      "]]}, 
    {Hue[0.1421359549995791, 0.6, 0.6], LineBox[CompressedData["
1:eJxtkXtMU2cYhwvIxZZlgBgYY2xiUSFMNjbBSaG/Si8iAm0p4gIKdjIMDCHD
BJwtbbIQtjlnmEOdMgVcmBGrGKK2xEtJ5hAmSAfUVbdRbqNuYGqlyGXILn3P
f2tycvKc5zzn/b6vq5Sl8gJ3Fosl++f69/7/Pzu/5rrQuDLtVz49wGmj/ovC
vEliH5ypzqlb0kwT+2Gy44ZqTDxPHIhH/WplQ80ScTDa+kwBt8Pd4eJQ+A+/
cvKpYBnxa3j3QGGFLdwLbv9xOIJXf9Xh3+lNnovbBWV7DTHLiddALTvU5raL
TbwOPUEdm0blHOqjMD3lYEW/6Es+GhVhBq3mKMProYoQf51tZfgN+OXlvO81
w/CbmFw7EWMY8KXvxWJ/xI1elYrxb6EnTBsvnOIQvw39829UjjUMb8BLmnRb
USSzvjiozWXzsU4f4nh4mD+vDPuW2d9GJOYuDGjyvYjfwXcX/Nz1r3vS/E0Y
qm60KjnMeSWgiFsfbZ9lzpOH1sjQDXqnG3EiTBuPS7zdGE7C0P7iCW4Ii5gP
811v78va5/T/AO7bO6Ms43+5WAv07jij3K1cIC/Avpf9rzmn58gL8NFJYZu0
fpbvWt9mGDw1qbzsZ+Q340Stnfcld4b6ZMxlX3qB5+Ekn4yZrOKuWMdT8kKc
yF88UmV3kBciofagZ9ziE/Ii3D9feEkdRKwVoVRVIalJspMXo7lRFBg++pi8
GFlsI7s7b4q8BOz3UgZbn/xJXoI9+Phu3fE/yG9BbrttPlX6iPwWRFUuTOhD
bORTEJyUUd7v/N21X20Kkq/obzVZx2n/W1FrqVqf+GCM+q0osUQWdQ6PUp8K
S9ZObfnMCPlUdFf5sbVBxKxtaCop3B0vHCa/DSWhxWOPD1rJp0Fiaq/xVQ3R
/DQkqgfbe1b8Rj4dEp2HwOvmL9SnYwmi+ZYDD8lnYDZg8dr3ogfkM3Dqan1S
wSoLeSlKm45lt3J+djGksOdUOwY97tP7UvD2fqIIWG52sVEKxQc2hS5kkHoZ
Rq5zm+fiB6iX4YfPTkUp8vuplyHiMEex+uhPrvUbZdDtSzhsvmei85Nj4eqr
uvGVJurl0IlHPuzb00e9HMkN5YG8yns0X45nLf4xhrhemp+Jsy2fHhMs66E+
E6r2nbNs64/UZ+JC3cMj27u6qc8E95yovOxWF/UK5LZFT17suMP/G8CGbV4=

      "]]}, 
    {Hue[0.37820393249936934`, 0.6, 0.6], LineBox[CompressedData["
1:eJxtlHtMU2cYh1tiIwylyqqiYoTiEAsoziooQn8t9wItbREvMUQbdSoIooKE
iBYVFOXilCxK6B/I1GbZRoYj0XgJCrWiUFFAQYQWVO4CiliheNv69j9P8uXk
OU9+532/9zs5rspk+XYbBoMh+7b+u3//GhFsbmGbC0IeCegB9D3lv7lIDMS2
mJP+uZCd3UM8A+wvvDetScPEHMSFnWcpKkaJnaDRlef9nmUidkaNbpFpcfME
sQs0mheFhsufBMz/mYvekwUpf5i+kF+EgHtzM4WvGLCwOzRn76z3zGcSe8Cc
uHW5+QMTljwPs5TOmXZcG/JeELac/fOig5WXYsY1Ht/xljXvg9mdIv0Dbysv
xxme0mFdIoPe9zPa/boX1kut/azA3UN2VxtbPhHzMSQrM3AOTBKvRIT2sCTa
1Uy8Cjk+L/NNhnFiX2RoVa47yz8S++Gi8cShjgLrfFZDuXewvzDzA81jDWrj
XBr+zhgj7w9udkl2cvZ74rVoTOGW2Kut8w5Ann+qubrqHXEgKrWlK7qG3xIL
sMBtfPX1JVYGLnmU3Li0Z8TCKmCHY2m4c5r1PIVgFTcYa7hD5IVID/2l0tc4
SP2JUL3Dvzv9rwHyIjAniyXq3H7KByFws/hk7f4+8kG40+XR75PQSz4YY55Z
uZxk+p5UwUgfbBPdPdxNPgSl/+bUn7vwmnwINNL7bW9uvyIfiiR5U7hg6CX5
ULg964tlLCZmhGGa26/D/+zsIh8G3/pidntFJ/lwnAqNcnJkEavC0Z9vE9kb
byQfAZbflNyJIwbLflURKPIM+qFP2kH7F0N3eoI15NVOeTGOez88tX32C8pH
QlRxYmalfRv5SIjjUpvE056Tj0K7/gpfPaeVfBTs9XwvnncL+WhsTODUpUU9
o/rR0O1X1zikPiUvwfTOMtOxy82Ul6BjdN2PScYm8lLsem+3cjeXWCXFtmbh
uXmJjeRj0NPkv9T91hMLIwbj1Qdz/GYRq2LgGnhlA/PAYwtXxUCUkB/Pb22g
vAxX3eM5ucHEkGFgNHlKagL9P1QyVNvednTYqLf0XyVD5b66XVtk9TQ/OcYG
zfPGFHWUl2Pvgkjjpi0PKS/HkrlTH/2U9oDqy3HGKZ49XFRL9RUQ56Wcj715
n/IKbAsoHPk4oKO8AkXzRW+PuhFXKVAeO1KgVt6jfCyWXdCWmTRawVcTknjV

      "]]}, 
    {Hue[0.6142719099991583, 0.6, 0.6], LineBox[CompressedData["
1:eJxt03lMk3ccx/EHWpdhrJGkJspwDJSpTLCuFOTQfiiFQivYp7VJGRihCDom
ioR5jz2aOcFuEyV4TeOBLJ5x1AvUOQ61cghSLmVFXAEdYJFjEOYR3eTb/9bk
SfN63vk+v+f0NKzTpDgzDMP+u/33//+/Aam5LFunk5ZLaQc+81BlGuUN5A8h
PJT21jT8iDwFz4tMuwb0f5KF4BcbDEXl3eRpyLhaEHJS0kt2h2DkhH/LlX7y
J/iH3+326uig1Om9vXCmNjVl7c/D1GfherzuRrPLCPlTTFRd8CnpdngO6sWv
5qvnj9K8DyzFwoOGZ44+D21uxoIud4f9oHgacMRV/DdZBHbv1NFAjWO9BdDb
f5i6c+sQHe9zhGVG/LLg4iB1MebeGHroYx8g+8N60gUiicMS5JdW+9YIXpAD
0KSUB96stJMDwZakhkUZn5MXwiotz9iW3EcOQtrp5bkpql5aPxi3F0Y1CGU9
1EMwoeRyXGHkX+RQOPfv9vDXPyMvgu/aKu3I10/Ji2EV5H7geczxPKTIWalO
fGPpIgOr72Tstk4hc4C5aeIKd30n9TCIewOGvU/bqIfhXIOXSzpjo/OToWxu
cta6RHr+nAwrD49tvGx+QvPh0P22fmathMyF40DXQMrRZR3U5difXtNzLfox
dTnqot3SJcp26hEQyY2ZGTor9Qjcqv4qsT7tD+qR6Nj48U+VuW3UI+E0TZZ8
r9jxfioQyuPnGGwPqSsw6XW7t8dHZCYKpsG2wPz4VupRgP3Lni8KW6hHY7nq
d+fpQ83j18tF43xCRdIGBZlRol2cl+1Z2ETzSvS2nnh5n09mVCi1XAryX9NI
XYXms4d5ro8s1JfAGBrq560ic0vASx+LlVQ4vrcYJO1oyXRZ3EDrx2DLJqt1
za0H1GMxedSp2jeczMVidtzmrOCD9dSXYtX2pG0zTHXUl4JnF+37tfE+dTWU
WTzBqde144Yae4ddV52fR+bU6Cz4dnR7cs24y9SY8Z393JPj1TTPIljEtuzq
qqJ5FpsU+/qC/Mgci4S6nbqb39wbP/8yFjl7bLJWi5nunwZ9Hs1bdb5mmtdA
X2QL4v94l+Y1SOUeOF8avEPra9A5EvO9OJ7MaMHaZmXHVd2meS0ava7mvQwh
c1ok3i2Q9JsqaV6L4x3CIwI/MrMMm8fy8sUXKqTvAOmte9c=
      "]]}, 
    {Hue[0.8503398874989481, 0.6, 0.6], LineBox[CompressedData["
1:eJxt1H1ME2ccB/ATEgYKCqh1Fod0TnSTiqIrVCz9ghZqAenLJIiSRhRihoxo
EDOxSXkREEKMRiokQ1yWWRtNDIWiCMyCFhG1WJwg2gkTl2axI75gpUzr3Pjd
f7vkcvnc97733PPc5XjZBcocL4ZhFB/3f4//v70QFy7Njtq5Ry+mE5iq+mFb
cd5lsi80qhPN+Q3d5EA4dFUFBypvkheAZ6+x95XeJX+KvUhIdffayEswVfc0
LOPBr+QwrG/nzbY4h8Wz/vPnMDW7GhX+jyn/AsV8c7O33xNyOFo/eV317swo
eSU6a3OtWamj1P8KTVrXYdM89voInBSWPw4ZtJNX48Oq6v6is+z914B/o/Tb
2COPyGvx2YTE8T5nhO4XBTe/b8SW9ZDydeAH57wq3j1MXo9izbjueNEQ+WuY
fIrul+gekAXwrizzaeth5xuNpJKKUxXT98kxsB7Mn5SJWAuRGSkY31E9SONv
wNX0yxLO7+z6xWLIzV34G1hvhMdsOc013COL8IZziRO/mHUcmjyGHB1vgCzG
H0zY80OhVjIQvtipFq2k96UF+Kf99wfE3aE8HhmRIctD1bcpj8eiZ/XzWqv7
6fkSYBUoD/LMtyhPwEXTwjyLp4/6m7AgWvJmdiJZuwnBAWMRe+vY72UzjiVy
8rOcvZRvhlWVUVMoIzMSnLMFhuCShXIJDNParkwumUmEvW9Mk1tzg/JEdJyq
FPp7kZkkSDUt8//WXKc8CbF+2ZMOTw/lUhQ01wXVlpO1UhTWqksNc9l8C3o6
3I+Yxu6Z+Wq3wKxLsmVGkhkZgm4F/1lqMVNfBsnc3SukajKTjHRPe7176hrl
yXAZIsKdQjKTgsE1HX8tKvmF8hQMHHdNr7V2UZ6KqM53cybDumj8VPBGFNFL
vu+kfCteQ9qwb6iD+lsRsEvqeyGGzKQhVK4cONt0lfI0JGdv/I47h8zIYRfw
1I7D7TOGHJa3x8ZGJq7Q9XK0Gx3Xe3PJZjlsMb44+pT9Hyhw4jz35sQuMhTY
IPsw+uRZG/UViDU2eq3La5t5fvNHN7wo6p800fopoRf9LKgoMVFfiV7vl/ek
QWStEts59XLXT600vhJGxfCyQiGZUWFcz/Ex2Fqor8JOvej5kX1krQrvhWXO
h35kswrcL38U6A1G6n+DrGsp7sFko/gfeap52A==
      "]]}}, InsetBox[
    FormBox["\<\"\[Beta] = 000\"\>",
     TraditionalForm], {29., 0.2515413972399608}, {0, 0}, Automatic, {1, 0}], 
   InsetBox[
    FormBox["\<\"0.5\"\>",
     TraditionalForm], {31., 0.6507555848248936}, {0, 0}, Automatic, {1, 0}], 
   InsetBox[
    FormBox["\<\"1\"\>",
     TraditionalForm], {31., 1.072686411340505}, {0, 0}, Automatic, {1, 0}], 
   InsetBox[
    FormBox["\<\"2\"\>",
     TraditionalForm], {24., 2.7971188201150015`}, {0, 0}, Automatic, {1, 0}],
    InsetBox[
    FormBox["\<\"4\"\>",
     TraditionalForm], {32., 3.273014986006374}, {0, 0}, Automatic, {1, 0}], 
   InsetBox[
    FormBox["\<\"8\"\>",
     TraditionalForm], {54., 2.7352839252710233`}, {0, 0}, Automatic, {1, 0}]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{
    FormBox["\"Displacement, mm\"", TraditionalForm], 
    FormBox["\"Force per amp, N/A\"", TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->Automatic,
  PlotRangeClipping->True]], "Output",
 CellChangeTimes->{
  3.529721851520268*^9, 3.529721912611534*^9, 3.52972194778815*^9, 
   3.529721993482246*^9, 3.529722274240143*^9, 3.529722362524856*^9, 
   3.529808978562749*^9, 3.5298090412060328`*^9, {3.5298091554955463`*^9, 
   3.5298092125570803`*^9}, 3.529809272318616*^9, {3.529809340188225*^9, 
   3.529809347017859*^9}, 3.5298094181118183`*^9, 3.529809669920579*^9, 
   3.5298097005761757`*^9, {3.5298097965115128`*^9, 3.529809800912767*^9}, {
   3.5298099311720123`*^9, 3.5298099489526033`*^9}, 3.529810031393676*^9, {
   3.529810073699915*^9, 3.529810086243137*^9}, 3.529810157682193*^9, 
   3.529810205774354*^9, {3.5298102547662287`*^9, 3.529810267150775*^9}, 
   3.5305029030419064`*^9, {3.53050295222544*^9, 3.530503042484676*^9}, {
   3.530503124957962*^9, 3.530503139793277*^9}, 3.530503199955455*^9, {
   3.53050325529379*^9, 3.5305032663289537`*^9}, 3.530503299836842*^9, 
   3.531022827252331*^9, 3.531022880566578*^9, {3.531022916276321*^9, 
   3.5310229421277657`*^9}, 3.5310230382751207`*^9, 3.53102311162036*^9, 
   3.531023154764955*^9, 3.531792184298356*^9, 3.531792287619998*^9, {
   3.531792340553447*^9, 3.531792353009461*^9}, 3.5317980582497053`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"GuessTag", ",", 
    RowBox[{"MaximumTagLength", "\[Rule]", "1"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"PSfragExport", "[", 
  RowBox[{"\"\<fig/magcoil-coilratio\>\"", ",", "coilratioPlot"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.529809303599979*^9, 3.529809311312736*^9}, {
  3.529810259426181*^9, 3.529810264002078*^9}, {3.531023657793407*^9, 
  3.531023665675201*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"fig/magcoil-coilratio\"\>", 
   ",", "\<\"fig/magcoil-coilratio-psfrag.eps\"\>", 
   ",", "\<\"fig/magcoil-coilratio-psfrag.tex\"\>"}], "}"}]], "Output",
 CellChangeTimes->{
  3.5298094487765903`*^9, 3.529809680307023*^9, 3.529809809686823*^9, {
   3.529809940583035*^9, 3.5298099577125597`*^9}, 3.529810043829143*^9, {
   3.529810085559052*^9, 3.5298100965717173`*^9}, 3.5298101670982924`*^9, 
   3.5298102145277023`*^9, 3.529810276160879*^9, 3.5298103280076237`*^9, 
   3.5305030555090523`*^9, 3.5305032141326714`*^9, 3.530503276955986*^9, 
   3.5305033141429996`*^9, 3.531022963415571*^9, 3.531023661445265*^9, 
   3.531798076624819*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"alpharange", "=", 
   RowBox[{"{", 
    RowBox[{"0.2", ",", "0.5", ",", "1", ",", "2", ",", "4"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outercoil2", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[Beta]", "=", "1"}], ",", 
       RowBox[{"MagVolume", "=", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"20", "mm"}], ")"}], "3"]}], ",", 
       RowBox[{"CoilResistance", "=", 
        RowBox[{"4", "ohms"}]}], ",", 
       RowBox[{"WireDiam", "=", 
        RowBox[{"1", "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"1000", "x"}], ",", 
           RowBox[{"MagnetCoilForce", "@", 
            RowBox[{"Flatten", "@", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"Method", "\[Rule]", "\"\<Shell\>\""}], ",", 
               RowBox[{"Displacement", "\[Rule]", "x"}], ",", 
               RowBox[{"IntegrationPrecision", "\[Rule]", "3"}], ",", 
               RowBox[{"magparam", "[", 
                RowBox[{
                "\[Alpha]", ",", "\[Beta]", ",", "WireDiam", ",", 
                 "CoilResistance", ",", "MagVolume"}], "]"}]}], "}"}]}]}]}], 
          "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", "0", ",", "xmax", ",", "dx"}], "}"}]}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"\[Alpha]", ",", "alpharange"}], "}"}]}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.529722031311343*^9, 3.529722042035968*^9}, {
   3.5297227104640636`*^9, 3.529722734686067*^9}, {3.529722766517045*^9, 
   3.52972276669412*^9}, {3.52972280577337*^9, 3.529722808778788*^9}, {
   3.5297229163361464`*^9, 3.52972292196251*^9}, {3.529723002424171*^9, 
   3.529723025212903*^9}, {3.529810720567577*^9, 3.5298107313268747`*^9}, {
   3.531023044635893*^9, 3.531023074297576*^9}, 3.531023174163457*^9, {
   3.531023223268777*^9, 3.5310232233314342`*^9}, {3.531792360783758*^9, 
   3.531792377520928*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"annot", "[", "s_", "]"}], ":=", 
  RowBox[{"PSfrag", "[", 
   RowBox[{"s", ",", 
    RowBox[{"TeXCommand", "\[Rule]", 
     RowBox[{
     "\"\<\\\\fboxsep=1pt\\\\footnotesize\\\\colorbox{white}{\>\"", "<>", "s",
       "<>", "\"\<}\>\""}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"magratioPlot", "=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ListPlot", "[", 
       RowBox[{"outercoil2", ",", 
        RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{
         RowBox[{"annot", "@", 
          RowBox[{"ToString", "[", 
           RowBox[{
           "alpharange", "\[LeftDoubleBracket]", "1", 
            "\[RightDoubleBracket]"}], "]"}]}], ",", 
         RowBox[{"outercoil2", "\[LeftDoubleBracket]", 
          RowBox[{"1", ",", "11"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",",
       "\[IndentingNewLine]", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{
         RowBox[{"annot", "@", 
          RowBox[{"ToString", "[", 
           RowBox[{
           "alpharange", "\[LeftDoubleBracket]", "2", 
            "\[RightDoubleBracket]"}], "]"}]}], ",", 
         RowBox[{"outercoil2", "\[LeftDoubleBracket]", 
          RowBox[{"2", ",", "12"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",",
       "\[IndentingNewLine]", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{
         RowBox[{"annot", "@", 
          RowBox[{"ToString", "[", 
           RowBox[{
           "alpharange", "\[LeftDoubleBracket]", "3", 
            "\[RightDoubleBracket]"}], "]"}]}], ",", 
         RowBox[{"outercoil2", "\[LeftDoubleBracket]", 
          RowBox[{"3", ",", "11"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",",
       "\[IndentingNewLine]", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{
         RowBox[{"annot", "@", 
          RowBox[{"ToString", "[", 
           RowBox[{
           "alpharange", "\[LeftDoubleBracket]", "4", 
            "\[RightDoubleBracket]"}], "]"}]}], ",", 
         RowBox[{"outercoil2", "\[LeftDoubleBracket]", 
          RowBox[{"4", ",", "14"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",",
       "\[IndentingNewLine]", 
      RowBox[{"Graphics", "@", 
       RowBox[{"Text", "[", 
        RowBox[{
         RowBox[{"annot", "[", 
          RowBox[{"\"\<$\\\\alpha=\>\"", "<>", 
           RowBox[{"ToString", "[", 
            RowBox[{
            "alpharange", "\[LeftDoubleBracket]", "5", 
             "\[RightDoubleBracket]"}], "]"}], "<>", "\"\<$\>\""}], "]"}], 
         ",", 
         RowBox[{"outercoil2", "\[LeftDoubleBracket]", 
          RowBox[{"5", ",", "10"}], "\[RightDoubleBracket]"}]}], "]"}]}]}], 
     "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Frame", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"None", ",", "\"\<Force per amp, N/A\>\""}], "}"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"ImageSize", "\[Rule]", 
     RowBox[{"8", " ", 
      RowBox[{"72", "/", "2.54"}]}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Automatic", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "3.1"}], "}"}]}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.529723029431357*^9, 3.5297232148053207`*^9}, {
  3.5298089256569147`*^9, 3.52980893452147*^9}, {3.5298107647607603`*^9, 
  3.529810868303986*^9}, {3.529810956835265*^9, 3.529810979344338*^9}, {
  3.529811097798086*^9, 3.529811236769058*^9}, {3.5298112931748953`*^9, 
  3.529811294484275*^9}, {3.52981149541881*^9, 3.529811542698914*^9}, {
  3.531023186959499*^9, 3.531023349037293*^9}, {3.531792387883234*^9, 
  3.531792390407345*^9}, {3.531798275440837*^9, 3.53179828007865*^9}}],

Cell[BoxData[
 GraphicsBox[{{
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGDQBGIQjR18sJ+4U1m23uycPVTAYVKumF205T0on8PhsOoz
h651z6B8AYf8nlOPqlzeQfkiDrcq4uaeef8JypdwuHryiF3CgW9QvozDFSt/
Wespv6B8BQfvNEPGzOx/9oxgvpJDxqY/heeUGB0g8ioO74TF+GT+M0H5ag7W
DFn8X1JZoHwNh71yWwWemrM6QPRrOexqKJ9wp4QVKq/j8PmeGO89WRhfz6Fc
au6LlQkw/QYOj29FPReZwwzlGzqIfO1PVb/NBDXPyIHb561EhRzMfmOHj52d
TZVpMPeZOCw+9yilYTMDlG/qYHHRIvAkK4xv5rAqK43h965/UP+aO3C27hXK
Lf0L5Vs4HHQXqmWy/gPlWzpM+LH36n7u39DwsHK4vnmjy4znP6Hy1g5/g0rr
Cs79gPJtHPbyWDgrHvgO5ds6TD9rWlu5Bxbedg7vPRsFHQ5/hfLtHXL3Pykw
vfIFyndwYLv7+5PI+88QfoODw5RKZ+ZpwlA+g6PD7SBztVxHaHw2ODr8UCy8
7VPxEeo+JwfL7lcJz7Z/gMo7Ofi/vLz1yv/3UP3ODqe+Rx0XWgBNHw3ODkZd
4twv/d9C5V0cPgm+d97N98YeAKs0sT8=
      "]]}, 
    {Hue[0.9060679774997897, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGDQBGIQjR18sG9pn/hU9sB1e6iAg9eZsPc7DjyH8jkcHqb6
LVe8+BHKF3BwyQ1Kf3/wO5Qv4rDV52Bg/aS/UL6Ew9tpudZFIYwOEL6MQ8jB
gHu7c5ihfAWH7dmcU28JsjowgvlKDo8XFL376swGlVdxcLAwNE5lZofy1Rwi
tJi7VwXA+BoO7Odb7lXqsEP1aznYNc1iPTYDpl/HYY4Ix1zHg6xQvp7D2Sau
0y0vWKB8A4efnF49E0VgfEOHCRvEZoi7MUPNM3IQvbZaq7OWCSpv7CBU3SNp
sBvmHxOHdt4EKcv/DFC+qcOxK5Zquz1hfDOH3Cw+w0rT/9DwMHc4uVL/XO4T
WPhYOGxV4rxntOAPlG/pkPVkmf30jN/2EPutHI6fjUmNtf0Flbd2SGRxEreT
+wnl2zjc0SzY9IrrB5Rv66BSvpyRnxkWH3YOceZSR4LYvkH59g7zum58jBL5
CuU7ONw/tvbkKZ0vEH6Dg8OpxpUbXfw/Q+UdHTzV13zMqv4ElXd0cFn8wvfB
ho9Q9zk5fBaxafN4/wEq7+SwrT5zn4cZlM/g7OA5x2EpP/d7qLyzww3tWLPY
nW+h8i4OLnZLnS+UvbEHAKR9pok=
      "]]}, 
    {Hue[0.1421359549995791, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGDQBGIQjR18sFdXZ+iTsrxrDxVw4Hkd+2vzu9dQPodDGoP4
n9tvP0P5Ag4MR7zDvhn+hPJFHCYb+11MPfIXypdwqHaXixO/zOAA4cs4zH5b
udEqhwnKV3CYs+F+jvE6ZgdGMF/JIejvgohpM1mg8ioOf8U3Lf2ixwrlqzks
qSkJuZcH42s4hKbcvP0ykhWqX8vBWtRd7ugHmH4dh7TLlj5m5jC+noPVpOOv
r8UyQ/kGDhoMlkeku2DuMXTQrFXW3LiXEWqekYNewbTVz3/A3G/s8EMjLv+e
NYxv4rBit+1ddd7/UP+aOqj8Fp70YCPM/2YO7quXWMxI/QPlmzuseOw98ZX6
byjfwuGd7PUDKT9g4WfpMP9dSFLXtR/2EPutHPKuyQbdPPgdKm/tYBG9t/PM
zm9Qvo3DhOuNHvf2foXybR32bVPc3Hn2C5Rv57DLbSqnzwtYfNk7eH6Y2LWU
F8Z3cDiSt1hMyvYThN/g4JDM+HyBYdlHqLyjg+jcwoqKHR+g8o4Or9sUiuuZ
P0Dd5+QgsGsK446176DyTg62U3dGBKS8hep3dpjeukSOVesNVN7ZIaiHrUzm
3yuovIuD9g5z06f3X9oDAKeHr9o=
      "]]}, 
    {Hue[0.37820393249936934`, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGDQBGIQjR18sI+5wf+7z/W8PVTA4dzz9dMU/O5D+RwO4hX/
+vlbn0P5Ag78/7Xe3sx7D+WLOIS5z2AN3vQZypdwWHF8fc+Sxu9QvozDkeMq
39Wv/oLyFRxWrLjTf3/ZX3tGMF/J4UVHX+Gq7/+h8ioOtsckax2fMDhA+GoO
KyYdDNfuZYTyNRx+5yQa/v7G6ADRr+UgmiRTy6nEBJXXcXC8MWnNIj4YX89B
YIeWidBemH4DB7GHTudO6cL4hg4TtJL4QnMYoOYZOdy1eCZ/1h/mHmOHQzWc
my/f+Avlmzi8C1x8X6TkD5Rv6uB5tM7PV/E3lG/m0GbwuPf7/Z9QvrlD1dEG
xYz1P6B8C4dFD9pr7vXBwsfSIangzav+2m/Q8LByOBmmcGFd1VeovLWDUuuc
1vzWL1C+jcPlQqU53HNh4W3r0GNd+vvwgU9Qvp3D1qMLjR+9/wjl2zvIKv+0
3KkJ4zs4LNWYs3tp7gcIv8HBIU1ooYdMGSw+HR1YZ114cETpHVTe0aHCLX2r
+YM3UPc5ORxOs35WsfY1VN7JgfHPLL+5na+g+p0d7GK8Ok4Wv4TKOzscfKTx
yiD7BVTexeGrdmOnSP5zewBmiLnh
      "]]}, 
    {Hue[0.6142719099991583, 0.6, 0.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGDQBGIQjR18sN+54eaEqwt22kMFHMJ3+/L+W3MSyudwuHjl
y8/vAZehfAGHdY+lPh2/exPKF3GQuv00b7LjAyhfwoEhKyN93eLHUL6MQ4mX
mkCcxHMoX8FB3X9C4slFr+wZwXwlhzmlidfY3N5B5VUctJMfxjQyfYTy1RzW
KBu3anz8BOVrOPRMvTznwfsvUP1aDgEWZq5CX75B5XUcdmns+2G05QeUr+cw
g9XqcoHILyjfwOHbVzfrzSK/oXxDh7k3Cyz8tv6GmmfkcP3ygvPt32Dyxg4d
EcYrO27A+CYOgdnVy5KiYXxTB+dtH1LN6mHmmzn0d8fd2e39E8o3d3iWFtvw
nQ3mHguHr7qr/D2vwNxr6TDvdeTj75u/Qu23ctj8ybXMa+kXqLy1w5a+l+eD
ln+G8m0cFsmXfuHeCQsPW4foWrvw7Juw8LJziGOu/prFDuPbO5ybssmD3+kD
lO/g0KP1yOmqwHsIv8HBwUB66WrbzW+h8o4OxwQTN+7OfAOVd3RYEjzL/LLR
a6j7nByS2ir/q/G+gso7Obwp3v2v7PsLqH5nh7cpk3PmfYDGd4OzQ9NVrfUT
vz+Dyrs4SEgrucVwP7MHAJRtwW8=
      "]]}}, InsetBox[
    FormBox["\<\"0.2\"\>",
     TraditionalForm], {10., 2.374565229462397}, {0, 0}, Automatic, {1, 0}], 
   InsetBox[
    FormBox["\<\"0.5\"\>",
     TraditionalForm], {11., 2.9143877849904065`}, {0, 0}, Automatic, {1, 0}],
    InsetBox[
    FormBox["\<\"1\"\>",
     TraditionalForm], {10., 2.6479275650954137`}, {0, 0}, Automatic, {1, 0}],
    InsetBox[
    FormBox["\<\"2\"\>",
     TraditionalForm], {13.000000000000002`, 2.266619660786328}, {0, 0}, 
    Automatic, {1, 0}], InsetBox[
    FormBox["\<\"$\\\\alpha=4$\"\>",
     TraditionalForm], {9.000000000000002, 0.9460481822131217}, {0, 0}, 
    Automatic, {1, 0}]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{None, 
    FormBox["\"Force per amp, N/A\"", TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->{Automatic, {0, 3.1}},
  PlotRangeClipping->True]], "Output",
 CellChangeTimes->{
  3.529722049823474*^9, 3.52972274005791*^9, 3.52972277163168*^9, 
   3.529722813239011*^9, 3.529722929333362*^9, 3.529723013096768*^9, {
   3.529723048678212*^9, 3.529723134002199*^9}, {3.529723166558072*^9, 
   3.529723215639434*^9}, 3.529808942805984*^9, {3.529810815019244*^9, 
   3.529810868792377*^9}, {3.529810961514556*^9, 3.529810979898343*^9}, {
   3.529811237511187*^9, 3.529811241017638*^9}, {3.529811279107985*^9, 
   3.529811307686949*^9}, {3.529811399125222*^9, 3.529811402602186*^9}, 
   3.529811447388714*^9, {3.529811514202334*^9, 3.529811543190441*^9}, 
   3.5310231904946327`*^9, {3.5310232277824383`*^9, 3.531023349770215*^9}, {
   3.5317923751447067`*^9, 3.531792390917531*^9}, 3.5317982808130836`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"GuessTag", ",", 
    RowBox[{"MaximumTagLength", "\[Rule]", "1"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"PSfragExport", "[", 
  RowBox[{"\"\<fig/magcoil-magratio\>\"", ",", "magratioPlot"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.529810790343165*^9, 3.529810794575123*^9}, {
   3.5310234797440367`*^9, 3.531023483686715*^9}, {3.531023567944633*^9, 
   3.531023634405796*^9}, 3.531023668411827*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"fig/magcoil-magratio\"\>", 
   ",", "\<\"fig/magcoil-magratio-psfrag.eps\"\>", 
   ",", "\<\"fig/magcoil-magratio-psfrag.tex\"\>"}], "}"}]], "Output",
 CellChangeTimes->{
  3.529810890139901*^9, 3.5298110041357117`*^9, {3.529811321555726*^9, 
   3.529811368064636*^9}, 3.529811409318844*^9, 3.529811457714217*^9, 
   3.529811556056909*^9, 3.5310232676678457`*^9, 3.5310233616505613`*^9, 
   3.53102349717562*^9, {3.531023578713791*^9, 3.531023634735045*^9}, 
   3.5317980849526367`*^9, 3.5317982907610188`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Wire diameter vs peak force", "Subsection",
 CellChangeTimes->{{3.5304829355000467`*^9, 3.5304829421499662`*^9}}],

Cell["\<\
From http://www.powerstream.com/Wire_Size.htm\
\>", "Text",
 CellChangeTimes->{{3.530484889671361*^9, 3.530484890519912*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"wireDiam", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "11.684`", ",", "10.40384`", ",", "9.26592`", ",", "8.25246`", ",", 
      "7.34822`", ",", "6.54304`", ",", "5.82676`", ",", "5.18922`", ",", 
      "4.62026`", ",", "4.1148`", ",", "3.66522`", ",", "3.2639`", ",", 
      "2.90576`", ",", "2.58826`", ",", "2.30378`", ",", "2.05232`", ",", 
      "1.8288`", ",", "1.62814`", ",", "1.45034`", ",", "1.29032`", ",", 
      "1.15062`", ",", "1.02362`", ",", "0.91186`", ",", "0.8128`", ",", 
      "0.7239`", ",", "0.64516`", ",", "0.57404`", ",", "0.51054`", ",", 
      "0.45466`", ",", "0.40386`", ",", "0.36068`", ",", "0.32004`", ",", 
      "0.28702`", ",", "0.254`", ",", "0.22606`", ",", "0.2032`", ",", "0.2`",
       ",", "0.18`", ",", "0.16002`", ",", "0.14224`", ",", "0.14`", ",", 
      "0.127`", ",", "0.125`", ",", "0.1143`", ",", "0.112`", ",", "0.1016`", 
      ",", "0.1`", ",", "0.0889`", ",", "0.07874`"}], "}"}], "mm"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.5304848974638166`*^9, 3.53048491355019*^9}, {
  3.530485700975645*^9, 3.530485701562028*^9}, {3.530485770356678*^9, 
  3.530485770711952*^9}, {3.530485963493528*^9, 3.530485963637562*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"wireCurr", "=", 
   RowBox[{"{", 
    RowBox[{
    "380", ",", "328", ",", "283", ",", "245", ",", "211", ",", "181", ",", 
     "158", ",", "135", ",", "118", ",", "101", ",", "89", ",", "73", ",", 
     "64", ",", "55", ",", "47", ",", "41", ",", "35", ",", "32", ",", "28", 
     ",", "22", ",", "19", ",", "16", ",", "14", ",", "11", ",", "9", ",", 
     "7", ",", "4.7`", ",", "3.5`", ",", "2.7`", ",", "2.2`", ",", "1.7`", 
     ",", "1.4`", ",", "1.2`", ",", "0.86`", ",", "0.7`", ",", "0.53`", ",", 
     "0.51`", ",", "0.43`", ",", "0.33`", ",", "0.27`", ",", "0.26`", ",", 
     "0.21`", ",", "0.2`", ",", "0.17`", ",", "0.163`", ",", "0.13`", ",", 
     "0.126`", ",", "0.11`", ",", "0.09`"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.530484902055738*^9, 3.530484914722308*^9}, {
  3.5304856805884666`*^9, 3.530485706856761*^9}, {3.5304857737911663`*^9, 
  3.530485774102854*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"maxcurrPlot", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{
    RowBox[{"Transpose", "@", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"wireDiam", "/", "mm"}], ",", "wireCurr"}], "}"}]}], ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"PlotMarkers", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
    RowBox[{"Frame", "\[Rule]", "True"}], ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<Wire diameter, mm\>\"", ",", "\"\<Maximum current, A\>\""}], 
      "}"}]}], ",", 
    RowBox[{"ImageSize", "\[Rule]", 
     RowBox[{"8", " ", 
      RowBox[{"72", "/", "2.54"}]}]}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "2.5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "55"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.530483069811493*^9, 3.5304831194379053`*^9}, {
  3.530483197046229*^9, 3.53048324448711*^9}, {3.530484928577766*^9, 
  3.5304849323854113`*^9}, {3.530485094642906*^9, 3.530485119466517*^9}, {
  3.530485217708946*^9, 3.5304852184100847`*^9}, {3.53048529469398*^9, 
  3.530485299860901*^9}, {3.530485971325615*^9, 3.530485971911425*^9}, {
  3.530486022008966*^9, 3.530486078922285*^9}, {3.5317990479346447`*^9, 
  3.531799080380858*^9}}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGBQBWIQPfFtjb1pnLoDAwg0eDuIxKxZcXALC5wfmZdpqpLH
BOW7O7zJbN+vls0A5bs4/D797uRhp7/2EL6jQ/nnkPY7PL8gfAYHB18vx+3T
Tb5D+TYOt/5Nnaq29AuUb+bw4Wdiy4eMT1C+sYN0Y0Xs/oQPUL6BQ+SvC6xf
dd9C+ToOy7ncVEuZXkH5ag41iUahBlrPoXwlqPlPoHwZBwXBYKXOuEf2Z8+A
gJCD14zM3+FhD6DyPA7bNTY7qknetZ81EwRYHZLTueZev3ETymd0uNUvMjFR
9Lq9MRj8tp8WHcXbWX3FPg0MvtlDzL8Elf9sv+KYt3mn4wX71teBO+RaX9sX
t5bv7Hh/Bqr+mT3E/afsv2nE9B/6+gBqz0n7IJDqwAf2XNcXF9hyHYfqvw21
74i9PFjgqv2dXPFZr00O2Xs8rBJZ537R/g3YmoP2S4C6ri++ALX/gD1Y+uEp
qD8PwO2BxNde+8NfQQ44al/5Us2QY80e+/J986X07x6Bum8X1DyYvp32IY+X
zj6icMA+yHNug9qhbfaaIOdr7LHnE338x1djC9Td2+0BRK3rEQ==
   "], {{{}, {}, 
     {Hue[0.67, 0.6, 0.6], 
      LineBox[{2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
        20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
       37}]}}, {
     {Hue[0.67, 0.6, 0.6], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 3], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 4], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 5], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 6], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 7], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 8], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 9], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 10], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 11], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 12], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 13], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 14], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 15], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 16], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 17], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 18], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 19], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 20], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 21], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 22], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 23], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 24], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 25], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 26], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 27], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 28], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 29], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 30], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 31], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 32], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 33], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 34], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 35], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 36], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 37]}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  Frame->True,
  FrameLabel->{
    FormBox["\"Wire diameter, mm\"", TraditionalForm], 
    FormBox["\"Maximum current, A\"", TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->{{0, 2.5}, {0, 55}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.530483083342112*^9, 3.530483121007759*^9}, {
   3.530483225034247*^9, 3.530483245186542*^9}, 3.530484938231185*^9, {
   3.530485095654759*^9, 3.5304851316949577`*^9}, 3.530485219285267*^9, 
   3.530485300566902*^9, 3.530485711011551*^9, 3.530485779706295*^9, 
   3.53048597312766*^9, {3.530486037631773*^9, 3.530486079511153*^9}, 
   3.5317045429718657`*^9, {3.531799048632831*^9, 3.5317990805901833`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"GuessTag", ",", 
    RowBox[{"MaximumTagLength", "\[Rule]", "1"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"PSfragExport", "[", 
  RowBox[{"\"\<fig/magcoil-maxcurr\>\"", ",", "maxcurrPlot"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.530485124825267*^9, 3.5304851292010937`*^9}, {
  3.5310239587815437`*^9, 3.531023959229908*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"fig/magcoil-maxcurr\"\>", 
   ",", "\<\"fig/magcoil-maxcurr-psfrag.eps\"\>", 
   ",", "\<\"fig/magcoil-maxcurr-psfrag.tex\"\>"}], "}"}]], "Output",
 CellChangeTimes->{3.5304851378830833`*^9, 3.530485233829032*^9, 
  3.530485313811183*^9, 3.530485721193996*^9, 3.530485790736479*^9, 
  3.530486096377694*^9, 3.531023968801198*^9, 3.531704548404608*^9, 
  3.531799089605689*^9}]
}, Open  ]],

Cell["\<\
Verify that the interpolation function is working:\
\>", "Text",
 CellChangeTimes->{{3.5317991229185677`*^9, 3.531799132932968*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CurrentFromDiam", "=", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"wireCurr", ",", "wireDiam"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"CurrentFromDiam", "[", 
    RowBox[{"x", " ", "mm"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0.1", ",", "2"}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.530485534567195*^9, 3.530485537997287*^9}, {
   3.530485617872755*^9, 3.530485644160096*^9}, {3.5304858051178093`*^9, 
   3.530485825358951*^9}, {3.530485986371888*^9, 3.530485986699666*^9}, 
   3.530486094523608*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVV3c4Fd4bNytFtlAiK3tf2e+5ViEilFC0rIbKDNlbIokvJaNEMlMyI3uE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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0.5, 0},
  PlotRange->{{0.1, 2}, {0., 39.51360515337517}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.530485646362669*^9, {3.530485721453743*^9, 3.530485727314487*^9}, {
   3.5304857926778927`*^9, 3.530485826781165*^9}, {3.530485977689883*^9, 
   3.530485987261629*^9}, 3.53048609655575*^9, 3.531704548662509*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Trying to improve the efficiency\
\>", "Subsection",
 CellChangeTimes->{{3.530690464569697*^9, 3.5306904683728867`*^9}, {
  3.531024377303055*^9, 3.531024381990385*^9}}],

Cell["\<\
The examples below may no longer be relevant but I was attempting to find the \
fast way to calculate the maximum/peak values of the force functions.\
\>", "Text",
 CellChangeTimes->{{3.5327670625200987`*^9, 3.5327670881272383`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FindMaximum", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceFn", "[", 
       RowBox[{"x", ",", "a", ",", "b", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "mm"}], "<", "x"}], ",", 
      RowBox[{
       RowBox[{"0.01", "<", "b"}], "&&", 
       RowBox[{"0.01", "<", "a"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "b", ",", "a"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530862488230544*^9, 3.530862506766666*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"13.773925000001327`", ",", 
   RowBox[{"{", 
    RowBox[{"55.45361597895198`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"x", "\[Rule]", "0.020227187279649617`"}], ",", 
       RowBox[{"b", "\[Rule]", "1.719697096488665`"}], ",", 
       RowBox[{"a", "\[Rule]", "0.22671061442689913`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.530862521531398*^9, 3.530862548341291*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"NMaximize", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceFn", "[", 
       RowBox[{"x", ",", "a", ",", "b", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "mm"}], "<", "x"}], ",", 
      RowBox[{
       RowBox[{"0.01", "<", "b"}], "&&", 
       RowBox[{"0.01", "<", "a"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "b", ",", "a"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530864886206271*^9, 3.530864894797144*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"150.78161900000123`", ",", 
   RowBox[{"{", 
    RowBox[{"58.785386186212`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"x", "\[Rule]", "0.020219738157784366`"}], ",", 
       RowBox[{"b", "\[Rule]", "1.792121788647017`"}], ",", 
       RowBox[{"a", "\[Rule]", "0.26038495659192173`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5308650509919786`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FindMaximum", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceFn", "[", 
       RowBox[{"x", ",", "a", ",", "b", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "mm"}], "<", "x", "<", 
       RowBox[{"100", "mm"}]}], ",", 
      RowBox[{
       RowBox[{"0.01", "<", "b"}], "&&", 
       RowBox[{"0.01", "<", "a"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "b", ",", "a"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5308651916019173`*^9, 3.530865192753655*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaximum", "::", "eit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The algorithm does not converge to the tolerance of \
\[NoBreak]\\!\\(4.806217383937354`*^-6\\)\[NoBreak] in \[NoBreak]\\!\\(500\\)\
\[NoBreak] iterations. The best estimated solution, with feasibility \
residual, KKT residual, or complementary residual of \
\[NoBreak]\\!\\({2.544085795486628`*^-6, 6.242805966048464`, \
8.26830419358586`*^-7}\\)\[NoBreak], is returned. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaximum\\\", ButtonNote -> \
\\\"FindMaximum::eit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.530865408598225*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"178.83637100000124`", ",", 
   RowBox[{"{", 
    RowBox[{"74.66969108536381`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"x", "\[Rule]", "0.022912617292714498`"}], ",", 
       RowBox[{"b", "\[Rule]", "2.81314652515862`"}], ",", 
       RowBox[{"a", "\[Rule]", "0.7360576789692791`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5308654086299887`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FindMaximum", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceFn", "[", 
       RowBox[{"x", ",", "a", ",", "b", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "mm"}], "<", "x", "<", 
       RowBox[{"100", "mm"}]}], ",", 
      RowBox[{
       RowBox[{"0.01", "<", "b", "<", "10"}], "&&", 
       RowBox[{"0.01", "<", "a", "<", "10"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "b", ",", "a"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5308663767230263`*^9, 3.530866379695109*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaximum", "::", "eit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The algorithm does not converge to the tolerance of \
\[NoBreak]\\!\\(4.806217383937354`*^-6\\)\[NoBreak] in \[NoBreak]\\!\\(500\\)\
\[NoBreak] iterations. The best estimated solution, with feasibility \
residual, KKT residual, or complementary residual of \
\[NoBreak]\\!\\({2.541827786970408`*^-8, 6.7917707678645725`, \
4.659017177215318`*^-9}\\)\[NoBreak], is returned. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaximum\\\", ButtonNote -> \
\\\"FindMaximum::eit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.530866577076173*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"190.51233200000206`", ",", 
   RowBox[{"{", 
    RowBox[{"77.58267675675975`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"x", "\[Rule]", "0.022808141250405735`"}], ",", 
       RowBox[{"b", "\[Rule]", "2.8556566493882576`"}], ",", 
       RowBox[{"a", "\[Rule]", "0.7885860894874697`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.530866577308971*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"MagnetCoilForceMax", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"1", "mm"}]}], "]"}], "//.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"b", "\[Rule]", "4.75"}], ",", 
     RowBox[{"a", "\[Rule]", "2.6"}]}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"MagnetCoilForceMaxAlt", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"1", "mm"}]}], "]"}], "//.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"b", "\[Rule]", "4.75"}], ",", 
     RowBox[{"a", "\[Rule]", "2.6"}]}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.5308691134693527`*^9, 3.53086913354839*^9}, {
  3.5308692353841763`*^9, 3.530869238103901*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.5977159999893047`", ",", "85.04274351659757`"}], "}"}]], "Output",\

 CellChangeTimes->{{3.5308691151336613`*^9, 3.5308691346835213`*^9}, 
   3.530869239399302*^9, {3.530869282421072*^9, 3.530869306880842*^9}, {
   3.53086941134513*^9, 3.530869432326076*^9}, {3.530869791542008*^9, 
   3.5308698316040783`*^9}, 3.5308705165700817`*^9, 3.5308705800716457`*^9, 
   3.531000585707981*^9, 3.5310006452110043`*^9, 3.5310006892950907`*^9, 
   3.531000737996599*^9, {3.531000779813098*^9, 3.53100080048206*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.18691299999773037`", ",", "85.0557091643692`"}], "}"}]], "Output",\

 CellChangeTimes->{{3.5308691151336613`*^9, 3.5308691346835213`*^9}, 
   3.530869239399302*^9, {3.530869282421072*^9, 3.530869306880842*^9}, {
   3.53086941134513*^9, 3.530869432326076*^9}, {3.530869791542008*^9, 
   3.5308698316040783`*^9}, 3.5308705165700817`*^9, 3.5308705800716457`*^9, 
   3.531000585707981*^9, 3.5310006452110043`*^9, 3.5310006892950907`*^9, 
   3.531000737996599*^9, {3.531000779813098*^9, 3.531000800672209*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"NMaximize", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceMax", "[", 
       RowBox[{"aa", ",", "bb", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "<", "bb", "<", "10"}], "&&", 
       RowBox[{"0.1", "<", "aa", "<", "10"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"aa", ",", "bb"}], "}"}], ",", 
    RowBox[{"AccuracyGoal", "\[Rule]", "3"}], ",", 
    RowBox[{"PrecisionGoal", "\[Rule]", "2"}], ",", 
    RowBox[{"MaxIterations", "\[Rule]", "10"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530868055961205*^9, 3.530868070672183*^9}, {
  3.530869444286397*^9, 3.530869444698388*^9}, {3.530870157240322*^9, 
  3.5308702327656393`*^9}, {3.530870276880289*^9, 3.530870278559849*^9}, {
  3.5308703557579947`*^9, 3.530870379700316*^9}, {3.5308704814534197`*^9, 
  3.530870483699584*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(10\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.530870420082218*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"38.55670899999677`", ",", 
   RowBox[{"{", 
    RowBox[{"85.28241039956202`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"aa", "\[Rule]", "1.9678401839243036`"}], ",", 
       RowBox[{"bb", "\[Rule]", "4.203765689369992`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.5308691071267242`*^9, 3.530869773878765*^9, {3.530870194767611*^9, 
   3.530870237722581*^9}, 3.5308702869858227`*^9, 3.530870380580577*^9, 
   3.5308704200854673`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"FindMaximum", "[", 
   RowBox[{
    RowBox[{"MagnetCoilForceMaxAlt", "[", 
     RowBox[{"aa", ",", "bb", ",", 
      RowBox[{"1", "mm"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"aa", ",", "3", ",", "0.1", ",", "10"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"bb", ",", "3", ",", "0.1", ",", "10"}], "}"}], ",", 
    RowBox[{"AccuracyGoal", "\[Rule]", "3"}], ",", 
    RowBox[{"PrecisionGoal", "\[Rule]", "2"}], ",", 
    RowBox[{"MaxIterations", "\[Rule]", "10"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530870489243659*^9, 3.530870489579605*^9}, {
  3.530870826254547*^9, 3.530870882056559*^9}, {3.530870954030046*^9, 
  3.530870955808296*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "lstol"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The line search decreased the step size to within the \
tolerance specified by AccuracyGoal and PrecisionGoal but was unable to find \
a sufficient increase in the function. You may need more than \
\[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digits of working precision to \
meet these tolerances. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaxValue\\\", ButtonNote -> \
\\\"FindMaxValue::lstol\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.5308709621445837`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "lstol"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The line search decreased the step size to within the \
tolerance specified by AccuracyGoal and PrecisionGoal but was unable to find \
a sufficient increase in the function. You may need more than \
\[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digits of working precision to \
meet these tolerances. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaxValue\\\", ButtonNote -> \
\\\"FindMaxValue::lstol\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.530870962734087*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "lstol"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The line search decreased the step size to within the \
tolerance specified by AccuracyGoal and PrecisionGoal but was unable to find \
a sufficient increase in the function. You may need more than \
\[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digits of working precision to \
meet these tolerances. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaxValue\\\", ButtonNote -> \
\\\"FindMaxValue::lstol\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.530870963382037*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMaxValue :: lstol\\), \\\"MessageName\\\"]\
\\)\[NoBreak] will be suppressed during this calculation. \\!\\(\\*ButtonBox[\
\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.5308709633858833`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "sdprec"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Line search unable to find a sufficient increase in the \
function value with \[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digit \
precision. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\
\\\", ButtonFrame->None, ButtonData:>\\\"paclet:ref/FindMaxValue\\\", \
ButtonNote -> \\\"FindMaxValue::sdprec\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.530870986589137*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "sdprec"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Line search unable to find a sufficient increase in the \
function value with \[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digit \
precision. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\
\\\", ButtonFrame->None, ButtonData:>\\\"paclet:ref/FindMaxValue\\\", \
ButtonNote -> \\\"FindMaxValue::sdprec\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.5308709868957872`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "sdprec"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Line search unable to find a sufficient increase in the \
function value with \[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digit \
precision. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\
\\\", ButtonFrame->None, ButtonData:>\\\"paclet:ref/FindMaxValue\\\", \
ButtonNote -> \\\"FindMaxValue::sdprec\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.530870995545967*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMaxValue :: sdprec\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.530870995549213*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaximum", "::", "lstol"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The line search decreased the step size to within the \
tolerance specified by AccuracyGoal and PrecisionGoal but was unable to find \
a sufficient increase in the function. You may need more than \
\[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digits of working precision to \
meet these tolerances. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaximum\\\", ButtonNote -> \
\\\"FindMaximum::lstol\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.530870490630981*^9, 3.530870585946617*^9, {3.5308708565866537`*^9, 
   3.5308708841703863`*^9}, 3.53087101988735*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"58.30300100000022`", ",", 
   RowBox[{"{", 
    RowBox[{"82.18143467476561`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"aa", "\[Rule]", "1.372009767688457`"}], ",", 
       RowBox[{"bb", "\[Rule]", "4.381009149151438`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.530870497207679*^9, 3.530870616105389*^9, 
  3.53087085659086*^9, 3.530870941272724*^9, 3.5308710198907337`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"NMaximize", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceMax", "[", 
       RowBox[{"aa", ",", "bb", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "<", "bb", "<", "10"}], "&&", 
       RowBox[{"0.1", "<", "aa", "<", "10"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"aa", ",", "bb"}], "}"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530871242828513*^9, 3.53087124300246*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.530871509441078*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"251.57535799999823`", ",", 
   RowBox[{"{", 
    RowBox[{"86.26973513530518`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"aa", "\[Rule]", "2.035409600899757`"}], ",", 
       RowBox[{"bb", "\[Rule]", "4.301437772864553`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.53087150944368*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"NMaximize", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceMax", "[", 
       RowBox[{"aa", ",", "bb", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "<", "aa", "<", "10"}], "&&", 
       RowBox[{"0.1", "<", "bb", "<", "10"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"aa", ",", "bb"}], "}"}], ",", 
    RowBox[{"AccuracyGoal", "\[Rule]", "4"}], ",", 
    RowBox[{"PrecisionGoal", "\[Rule]", "3"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530871761131874*^9, 3.5308718088402367`*^9}, {
  3.530872640218534*^9, 3.5308726510176067`*^9}, {3.530872686633894*^9, 
  3.5308726943572483`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"68.11798699999781`", ",", 
   RowBox[{"{", 
    RowBox[{"85.91937439317188`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"aa", "\[Rule]", "2.0662395758134764`"}], ",", 
       RowBox[{"bb", "\[Rule]", "4.322114989428625`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.530871791848919*^9, 3.530872635826132*^9, 
  3.5308727672767878`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"NMaximize", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceMax", "[", 
       RowBox[{"aa", ",", "bb", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "<", "aa", "<", "10"}], "&&", 
       RowBox[{"0.1", "<", "bb", "<", "10"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"aa", ",", "bb"}], "}"}], ",", 
    RowBox[{"AccuracyGoal", "\[Rule]", "3"}], ",", 
    RowBox[{"PrecisionGoal", "\[Rule]", "2"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530871761131874*^9, 3.5308718088402367`*^9}, {
  3.530872640218534*^9, 3.5308726510176067`*^9}, {3.530872686633894*^9, 
  3.5308726943572483`*^9}, {3.530911993778913*^9, 3.530911995443512*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"68.60433300000295`", ",", 
   RowBox[{"{", 
    RowBox[{"85.91937439317188`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"aa", "\[Rule]", "2.0662395758134764`"}], ",", 
       RowBox[{"bb", "\[Rule]", "4.322114989428625`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.530871791848919*^9, 3.530872635826132*^9, 
  3.5308727672767878`*^9, 3.5309120649231997`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"NMaximize", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceMaxAlt", "[", 
       RowBox[{"aa", ",", "bb", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "<", "aa", "<", "10"}], "&&", 
       RowBox[{"0.1", "<", "bb", "<", "10"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"aa", ",", "bb"}], "}"}], ",", 
    RowBox[{"AccuracyGoal", "\[Rule]", "3"}], ",", 
    RowBox[{"PrecisionGoal", "\[Rule]", "2"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530871761131874*^9, 3.5308718088402367`*^9}, {
  3.530872640218534*^9, 3.5308726510176067`*^9}, {3.530872686633894*^9, 
  3.5308726943572483`*^9}, {3.530911993778913*^9, 3.530911995443512*^9}, {
  3.530912748707921*^9, 3.5309127490506268`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "lstol"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The line search decreased the step size to within the \
tolerance specified by AccuracyGoal and PrecisionGoal but was unable to find \
a sufficient increase in the function. You may need more than \
\[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digits of working precision to \
meet these tolerances. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaxValue\\\", ButtonNote -> \
\\\"FindMaxValue::lstol\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5309127503250723`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "lstol"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The line search decreased the step size to within the \
tolerance specified by AccuracyGoal and PrecisionGoal but was unable to find \
a sufficient increase in the function. You may need more than \
\[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digits of working precision to \
meet these tolerances. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaxValue\\\", ButtonNote -> \
\\\"FindMaxValue::lstol\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.530912751648212*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "lstol"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The line search decreased the step size to within the \
tolerance specified by AccuracyGoal and PrecisionGoal but was unable to find \
a sufficient increase in the function. You may need more than \
\[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digits of working precision to \
meet these tolerances. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaxValue\\\", ButtonNote -> \
\\\"FindMaxValue::lstol\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.53091275194324*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMaxValue :: lstol\\), \\\"MessageName\\\"]\
\\)\[NoBreak] will be suppressed during this calculation. \\!\\(\\*ButtonBox[\
\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5309127519477453`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "sdprec"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Line search unable to find a sufficient increase in the \
function value with \[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digit \
precision. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\
\\\", ButtonFrame->None, ButtonData:>\\\"paclet:ref/FindMaxValue\\\", \
ButtonNote -> \\\"FindMaxValue::sdprec\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.530912757600439*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"45.83918199999971`", ",", 
   RowBox[{"{", 
    RowBox[{"86.00401369794892`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"aa", "\[Rule]", "2.0662395758134764`"}], ",", 
       RowBox[{"bb", "\[Rule]", "4.322114989428625`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.530871791848919*^9, 3.530872635826132*^9, 
  3.5308727672767878`*^9, 3.5309120649231997`*^9, 3.530912795687297*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"NMaximize", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"MagnetCoilForceMax", "[", 
       RowBox[{"aa", ",", "bb", ",", 
        RowBox[{"1", "mm"}]}], "]"}], ",", 
      RowBox[{
       RowBox[{"0.1", "<", "aa", "<", "10"}], "&&", 
       RowBox[{"0.1", "<", "bb", "<", "10"}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"aa", ",", "bb"}], "}"}], ",", 
    RowBox[{"AccuracyGoal", "\[Rule]", "2"}], ",", 
    RowBox[{"PrecisionGoal", "\[Rule]", "2"}], ",", 
    RowBox[{"MaxIterations", "\[Rule]", "10"}]}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.530871761131874*^9, 3.5308718088402367`*^9}, {
   3.530872640218534*^9, 3.5308726510176067`*^9}, {3.530872686633894*^9, 
   3.5308726943572483`*^9}, {3.530911993778913*^9, 3.530911995443512*^9}, 
   3.530912375851425*^9, {3.530912459848711*^9, 3.530912464455961*^9}, 
   3.530912508348041*^9, {3.5309125450568237`*^9, 3.530912545161757*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(10\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5309125009706717`*^9, 3.53091253480902*^9, 
  3.5309125842700777`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"38.70689100000163`", ",", 
   RowBox[{"{", 
    RowBox[{"85.28241039956202`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"aa", "\[Rule]", "1.9678401839243036`"}], ",", 
       RowBox[{"bb", "\[Rule]", "4.203765689369992`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.530871791848919*^9, 3.530872635826132*^9, 
  3.5308727672767878`*^9, 3.5309120649231997`*^9, 3.5309124453087053`*^9, 
  3.530912500974553*^9, 3.530912534813087*^9, 3.5309125842735167`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Calculating tables of peak/maximum forces\
\>", "Subsection",
 CellChangeTimes->{{3.531013707496604*^9, 3.531013712282455*^9}, {
  3.532767199035295*^9, 3.532767210095948*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Drange", "=", 
   RowBox[{"Range", "[", 
    RowBox[{
     RowBox[{"0.2", "mm"}], ",", 
     RowBox[{"2.5", "mm"}], ",", 
     RowBox[{"0.1", "mm"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rrange", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"2", "ohms"}], ",", 
     RowBox[{"4", "ohms"}], ",", 
     RowBox[{"8", "ohms"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Vrange", "=", 
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"15", "mm"}], ")"}], "3"], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"20", "mm"}], ")"}], "3"], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"25", "mm"}], ")"}], "3"]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ND", "=", 
   RowBox[{"Length", "@", "Drange"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NR", "=", 
   RowBox[{"Length", "@", "Rrange"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NV", "=", 
   RowBox[{"Length", "@", "Vrange"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"amin", "=", "0.1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"amax", "=", "10.1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Nratios", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"magRatios", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"amin", ",", "amax", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"amax", "-", "amin"}], ")"}], "/", "Nratios"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.531000890307617*^9, 3.531001183701652*^9}, {
   3.531001517637619*^9, 3.53100152641973*^9}, 3.531001732344817*^9, {
   3.5310138028736343`*^9, 3.531013971615902*^9}, {3.531014041003152*^9, 
   3.531014142629984*^9}, {3.531014741780023*^9, 3.531014778819468*^9}, 
   3.531015017469843*^9, {3.531025691384624*^9, 3.5310257030506067`*^9}, {
   3.531375236315611*^9, 3.531375249072425*^9}, {3.531375492351269*^9, 
   3.531375538174203*^9}, {3.531377880078582*^9, 3.531377884444112*^9}, {
   3.531449942376875*^9, 3.531449948243217*^9}, {3.531456141774374*^9, 
   3.531456143390842*^9}, 3.531456521574184*^9, {3.531531764087036*^9, 
   3.5315317652255573`*^9}, {3.5315322497056427`*^9, 3.531532252679487*^9}, {
   3.531791003757653*^9, 3.531791003798374*^9}}],

Cell["\<\
An example of how long these things take to calculate:\
\>", "Text",
 CellChangeTimes->{{3.53276717459369*^9, 3.532767183478939*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Timing", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"ratiodata", "[", 
       RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}], "=", 
      RowBox[{"Block", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"bmin", ",", "bmax", ",", "coilRatios"}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Table", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"bmin", "=", 
            RowBox[{"Max", "[", 
             RowBox[{
              RowBox[{"coilRatioMin", "[", 
               RowBox[{"a", ",", "dd", ",", "vv"}], "]"}], ",", "0.1"}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"bmax", "=", 
            RowBox[{"Min", "[", 
             RowBox[{
              RowBox[{"coilRatioMax", "[", 
               RowBox[{"a", ",", "dd", ",", "rr", ",", "vv"}], "]"}], ",", 
              "10"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"coilRatios", "=", 
            RowBox[{"Range", "[", 
             RowBox[{"bmin", ",", "bmax", ",", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"bmax", "-", "bmin"}], ")"}], "/", "Nratios"}]}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"a", ",", "b", ",", 
               RowBox[{"MagnetCoilForceMaxScaled", "[", 
                RowBox[{"a", ",", "b", ",", "dd", ",", "rr", ",", "vv"}], 
                "]"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"b", ",", "coilRatios"}], "}"}]}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"a", ",", "magRatios"}], "}"}]}], "]"}]}], 
       "\[IndentingNewLine]", "]"}]}], ";"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"dd", ",", 
     RowBox[{"{", 
      RowBox[{"1", "mm"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"rr", ",", 
     RowBox[{"{", 
      RowBox[{"4", "ohms"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"vv", ",", 
     RowBox[{"{", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"20", "mm"}], ")"}], "3"], "}"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.531531792306746*^9, 3.531531835268137*^9}, {
  3.5317909184152117`*^9, 3.531790923627264*^9}, {3.5317909756415777`*^9, 
  3.531790991206016*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"105.00176499999725`", ",", "Null"}], "}"}], "}"}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.53153181534988*^9, 3.53153204420755*^9, 
  3.5317909934383097`*^9, 3.5317910268527403`*^9, 3.5317911654558992`*^9}]
}, Open  ]],

Cell["\<\
This did not end up being used; it takes a long long time to calculate:\
\>", "Text",
 CellChangeTimes->{{3.5327672726178923`*^9, 3.532767293031299*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"datafile", "=", "\"\<data/Thick-Coil-Magnet-Data.mx\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"FindFile", "[", "datafile", "]"}], "===", "$Failed"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"regenerate", " ", "data"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Table", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Timing", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"ratiodata", "[", 
          RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}], "=", 
         RowBox[{"Block", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"bmin", ",", "bmax", ",", "coilRatios"}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Table", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"bmin", "=", 
               RowBox[{"Max", "[", 
                RowBox[{
                 RowBox[{"coilRatioMin", "[", 
                  RowBox[{"a", ",", "dd", ",", "vv"}], "]"}], ",", "0.1"}], 
                "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"bmax", "=", 
               RowBox[{"Min", "[", 
                RowBox[{
                 RowBox[{"coilRatioMax", "[", 
                  RowBox[{"a", ",", "dd", ",", "rr", ",", "vv"}], "]"}], ",", 
                 "10"}], "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"coilRatios", "=", 
               RowBox[{"Range", "[", 
                RowBox[{"bmin", ",", "bmax", ",", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"bmax", "-", "bmin"}], ")"}], "/", "Nratios"}]}], 
                "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"Table", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"a", ",", "b", ",", 
                  RowBox[{"MagnetCoilForceMaxScaled", "[", 
                   RowBox[{"a", ",", "b", ",", "dd", ",", "rr", ",", "vv"}], 
                   "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"b", ",", "coilRatios"}], "}"}]}], "]"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"a", ",", "magRatios"}], "}"}]}], "]"}]}], 
          "\[IndentingNewLine]", "]"}]}], ";"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dd", ",", "Drange"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"rr", ",", "Rrange"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"vv", ",", "Vrange"}], "}"}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"Save", "[", 
     RowBox[{"datafile", ",", 
      RowBox[{"{", "ratiodata", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
   ",", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"load", " ", "data"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"DumpGet", "[", "datafile", "]"}], ";"}]}], "\[IndentingNewLine]",
   "]"}]}], "Input",
 CellChangeTimes->{
  3.531014781701407*^9, {3.531014829072331*^9, 3.53101489073297*^9}, {
   3.531014999570187*^9, 3.531015000044561*^9}, {3.531017710921466*^9, 
   3.5310177124727716`*^9}, {3.531025206721374*^9, 3.5310253401355963`*^9}, {
   3.5310253844432793`*^9, 3.531025388788477*^9}, {3.531025510584298*^9, 
   3.531025510709333*^9}, {3.5310259423390102`*^9, 3.531025949075136*^9}, {
   3.5310259835148973`*^9, 3.531025983802842*^9}, {3.531026080554593*^9, 
   3.531026099084464*^9}, {3.531026285236916*^9, 3.531026345609716*^9}, {
   3.5313712550479317`*^9, 3.5313712551502323`*^9}, {3.531371886020668*^9, 
   3.531371894609222*^9}, {3.5313719405631*^9, 3.531371940620125*^9}, {
   3.531374942881824*^9, 3.5313749474504004`*^9}, {3.531375033699106*^9, 
   3.531375104203504*^9}, {3.531375300699626*^9, 3.531375319807626*^9}, {
   3.531375389817379*^9, 3.531375442747094*^9}, {3.5313755892825737`*^9, 
   3.53137559797615*^9}, {3.5313776835169764`*^9, 3.531377684761633*^9}, {
   3.531377893832996*^9, 3.531377893902883*^9}, {3.531449890624711*^9, 
   3.531449899792684*^9}, {3.531703602417076*^9, 3.5317036028280697`*^9}, {
   3.531703730248193*^9, 3.5317038094115953`*^9}, 3.531790497756925*^9, {
   3.531790830420824*^9, 3.531790831509234*^9}, {3.531791017241932*^9, 
   3.531791018360804*^9}, {3.53179143120988*^9, 3.5317914314042177`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Block", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"dd", "=", 
      RowBox[{"1", "mm"}]}], ",", 
     RowBox[{"rr", "=", 
      RowBox[{"4", "ohms"}]}], ",", 
     RowBox[{"vv", "=", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"20", "mm"}], ")"}], "3"]}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"size", "=", 
     RowBox[{"6", " ", 
      RowBox[{"72", "/", "2.54"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"data", "=", 
     RowBox[{"Partition", "[", 
      RowBox[{
       RowBox[{"Flatten", "@", 
        RowBox[{"ratiodata", "[", 
         RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}], ",", "3"}], "]"}]}], 
    ";", "\[IndentingNewLine]", 
    RowBox[{"max", "=", 
     RowBox[{"Max", "[", 
      RowBox[{"data", "\[LeftDoubleBracket]", 
       RowBox[{"All", ",", "3"}], "\[RightDoubleBracket]"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"maxpos", "=", 
     RowBox[{"Position", "[", 
      RowBox[{"data", ",", "max"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"g1", "=", 
     RowBox[{"Show", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ListDensityPlot", "[", 
          RowBox[{"data", ",", 
           RowBox[{"InterpolationOrder", "\[Rule]", "0"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"ListPlot", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"data", "\[LeftDoubleBracket]", 
             RowBox[{
              RowBox[{"maxpos", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], 
            "}"}], ",", 
           RowBox[{"PlotMarkers", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"\[FivePointedStar]", ",", "Medium"}], "}"}]}], ",", 
           RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
        "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<Magnet ratio\>\"", ",", 
            RowBox[{
            "TeXCommand", "->", "\"\<Magnet ratio, $\\\\ratioMag$\>\""}]}], 
           "]"}], ",", 
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<Coil ratio\>\"", ",", 
            RowBox[{
            "TeXCommand", "->", "\"\<Coil ratio, $\\\\ratioCoil$\>\""}]}], 
           "]"}], ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
       RowBox[{"ImageSize", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "size"}], "}"}]}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"g2", "=", 
     RowBox[{"DensityPlot", "[", 
      RowBox[{"y", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "0", ",", "max"}], "}"}], ",", 
       RowBox[{"AspectRatio", "\[Rule]", "10"}], ",", 
       RowBox[{"ImageSize", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "size"}], "}"}]}], ",", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{"0.5", ",", 
             RowBox[{"PSfrag", "[", 
              RowBox[{"\"\<~\>\"", ",", 
               RowBox[{"TeXCommand", "\[Rule]", "\"\<\>\""}]}], "]"}]}], 
            "}"}], "}"}], ",", "Automatic", ",", "None", ",", "None"}], 
         "}"}]}], ",", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<~\>\"", ",", 
            RowBox[{"TeXCommand", "\[Rule]", "\"\<\>\""}]}], "]"}], ",", 
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<~\>\"", ",", 
            RowBox[{"TeXCommand", "\[Rule]", "\"\<\>\""}]}], "]"}], ",", 
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<fmax\>\"", ",", 
            RowBox[{
            "TeXCommand", "\[Rule]", 
             "\"\<$\\\\fpeak/\\\\currentCoil$\>\""}]}], "]"}], ",", 
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<~\>\"", ",", 
            RowBox[{"TeXCommand", "\[Rule]", "\"\<\>\""}]}], "]"}]}], "}"}]}],
        ",", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "max"}], "}"}]}], "}"}]}]}], "]"}]}], ";"}]}], 
  "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Row", "[", 
   RowBox[{"{", 
    RowBox[{"g1", ",", "g2"}], "}"}], "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"Block", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"dd", "=", 
      RowBox[{"1", "mm"}]}], ",", 
     RowBox[{"rr", "=", 
      RowBox[{"4", "ohms"}]}], ",", 
     RowBox[{"vv", "=", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"20", "mm"}], ")"}], "3"]}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"size", "=", 
     RowBox[{"6", " ", 
      RowBox[{"72", "/", "2.54"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"data", "=", 
     RowBox[{
      RowBox[{
       RowBox[{"#", "*", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1", ",", 
          RowBox[{"1", "/", 
           RowBox[{"CurrentFromDiam", "[", "dd", "]"}]}]}], "}"}]}], "&"}], "/@", 
      RowBox[{"Partition", "[", 
       RowBox[{
        RowBox[{"Flatten", "@", 
         RowBox[{"ratiodata", "[", 
          RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}], ",", "3"}], 
       "]"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"max", "=", 
     RowBox[{"Max", "[", 
      RowBox[{"data", "\[LeftDoubleBracket]", 
       RowBox[{"All", ",", "3"}], "\[RightDoubleBracket]"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"maxpos", "=", 
     RowBox[{"Position", "[", 
      RowBox[{"data", ",", "max"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"g1", "=", 
     RowBox[{"Show", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"ListDensityPlot", "[", 
          RowBox[{"data", ",", 
           RowBox[{"InterpolationOrder", "\[Rule]", "0"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"ListPlot", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"data", "\[LeftDoubleBracket]", 
             RowBox[{
              RowBox[{"maxpos", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], 
            "}"}], ",", 
           RowBox[{"PlotMarkers", "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"\[FivePointedStar]", ",", "Medium"}], "}"}]}], ",", 
           RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Graphics", "@", 
          RowBox[{"Text", "[", 
           RowBox[{
            RowBox[{"PSfrag", "[", 
             RowBox[{"\"\<fmax\>\"", ",", 
              RowBox[{"TeXCommand", "\[Rule]", "\"\<$\\\\fnmax$\>\""}]}], 
             "]"}], ",", 
            RowBox[{"data", "\[LeftDoubleBracket]", 
             RowBox[{
              RowBox[{"maxpos", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], 
            ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "2"}], ",", "0"}], "}"}]}], "]"}]}]}], 
        "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<Magnet ratio\>\"", ",", 
            RowBox[{
            "TeXCommand", "->", "\"\<Magnet ratio, $\\\\ratioMag$\>\""}]}], 
           "]"}], ",", 
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<Coil ratio\>\"", ",", 
            RowBox[{
            "TeXCommand", "->", "\"\<Coil ratio, $\\\\ratioCoil$\>\""}]}], 
           "]"}], ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
       RowBox[{"ImageSize", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "size"}], "}"}]}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"g2", "=", 
     RowBox[{"DensityPlot", "[", 
      RowBox[{"y", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "0", ",", "max"}], "}"}], ",", 
       RowBox[{"AspectRatio", "\[Rule]", "10"}], ",", 
       RowBox[{"ImageSize", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "size"}], "}"}]}], ",", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{"0.5", ",", 
             RowBox[{"PSfrag", "[", 
              RowBox[{"\"\<~\>\"", ",", 
               RowBox[{"TeXCommand", "\[Rule]", "\"\<\>\""}]}], "]"}]}], 
            "}"}], "}"}], ",", "Automatic", ",", "None", ",", "None"}], 
         "}"}]}], ",", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<~\>\"", ",", 
            RowBox[{"TeXCommand", "\[Rule]", "\"\<\>\""}]}], "]"}], ",", 
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<~\>\"", ",", 
            RowBox[{"TeXCommand", "\[Rule]", "\"\<\>\""}]}], "]"}], ",", 
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<norm fmax\>\"", ",", 
            RowBox[{"TeXCommand", "\[Rule]", "\"\<$\\\\fnpeak$\>\""}]}], 
           "]"}], ",", 
          RowBox[{"PSfrag", "[", 
           RowBox[{"\"\<~\>\"", ",", 
            RowBox[{"TeXCommand", "\[Rule]", "\"\<\>\""}]}], "]"}]}], "}"}]}],
        ",", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "max"}], "}"}]}], "}"}]}]}], "]"}]}], ";"}]}], 
  "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Row", "[", 
  RowBox[{"{", 
   RowBox[{"g1", ",", "g2"}], "}"}], "]"}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJwdzk0og3EAx/GH0Czs4mBJGpKUraa00Gy1iyGMJi+ztUy2FMawqU0tIe+3
ZRITKeXAylrCwUs7eCtvwxxWptiiLAtT/r/n8O1z/XLUXTJNLEVR6SQYHPKW
bFW/iu966stgtjBNAuUbPimM+c5ohIt/7WpYGV3ogOfahE5omHWYIXfuYBS6
sk4noVt7tASfXsbWYPzP/gW0HfC98N0leICmtiT2NnHPl8mB3bprLnRaU3iQ
8lMCWtZmMfx4TC924qdQQbvu66yDqQXTE7C/aFm+Qww82xtgNKpUQZtJpIZC
xWwfzEtmGaDn5dfCq3kV+6uZw5DBP2Y3E6scEVqdxD4AyxvctIOPvMQWYkij
Z0K9MfzWSsyR6YPw5nOlVkmUnvTK4aHKo4dnoV0jnGevXrqJ5saRK3gbya/w
Epssk7RxpblWyJgJjENR+H4KOhhftP991d3N
  "]],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{
     GraphicsComplexBox[CompressedData["
1:eJyVnXlczekfxZOEi0uSNEkLkmyl7HQfsmZrsWQnKfvSgqxFspZdyZYka5YS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      "], {{
        GrayLevel[0.8], 
        EdgeForm[None], 
        GraphicsGroupBox[{
          
          PolygonBox[{{2417, 24, 25, 2416}, {2481, 1, 2401, 2561}, {449, 450, 
           451, 452}, {2413, 18, 19, 2412}, {485, 486, 487, 488}, {323, 324, 
           325, 326}, {509, 510, 511, 512}, {2433, 48, 49, 2432}, {2488, 181, 
           182, 2487}, {437, 438, 439, 440}, {2549, 2400, 2480, 2564}}], 
          PolygonBox[CompressedData["
1:eJwt1FdwVkUYBuAVVFoo11SpIQFB0gBB1BBAsZD8CQklwEwIid6IhpCo6A2g
WChKFxSkCAoIM4jOSFdpQkiodrA3QLCNDQSfdbh4cr53c/7dPbt7TvuSiYkH
6oUQCrmOM41C2NYghO1s5Uf5Gw5dH0INB/la/oiz14ZwjjN8KO+hjc7a0o73
5C/J95sCEnwh326cbAaSwyB+1r6+YQgbWMcFeRV3cTf3sJJXuJdh5LKa3+hI
JzrwK7vjvdeEsIa17JK/Z6BnyiGb7+SfSDdeBmmckx+hszl1pBMPy58y3gKV
UsIn8nIGMZghLGMFd3AnQ3mZeXEMMshkLpfZYQ472cVZ+d/Yn3owQ/hBXkAW
venDfBbRl5vpx0Ke5yZ6kcZzXInjeaYsenNenkUq3ejOTOZwIz3oyWwucdh+
1VLHt/LpuA6e/bP4/JySLzLc/wsp4it5Cl2tWxeSeVR+Ij6DOpMspstLuZXb
4plgCS+RHfeJHF7kr7hPzth5LvCx/DcTzKGM8jgn+R9OqU/zeSS/QH9uYQCL
mRb3whzSSGeqPCOup7oPfXlSfjyukzqFVB6T15JHIp5r1rCJMYxlHBt5jYK4
NhTyKpOcw0omU0U1w8gljwT5lBprAmWUcx9HOcZxTnCSEYxkFKMpjle/L2YM
YxnHHu172cd+DpBPAcOvvvtFvGmOm13fYAtvyW/H86KPbnRnq3yCCvswiUpO
ynVxj+tbN2ZwJJ4fpqqnMZ1a+WjcJ/VFLnFMfp/G6iYkcVA+RFN1M5pTIycZ
vynNaE4L/tQ+2RyqqOYD+Y/4LvnNZa5wXN4ez7j7e9CTbXKub1QeCfIpIH4M
J/IgD1HBePfqOsTvpO5CSdxLWqtb0oqKOA/aqdvQlkq5mg7qG2hPlbyeIkYw
knXxLNPg6jfYaxRK5dcZxWiK2UBorN27vJFN/BLfdxNLJ4NMsihv9P+twS0h
/qzMn/oannbtTBd+51lrMZNZzGYO+7Uv0MdCFnFA3stc9Tzmsy/24ZpMV1JI
5R3tLV1b0Zp35fvjXppDE5LieZZrjHOYWuo4wg7tK11XsZqdcX4kx+8JKTxD
Pc/xlGs/ffVns/o/l6TZww==
           "]], 
          PolygonBox[CompressedData["
1:eJwV2XOgVmccAOCbbdtaWlxYtldbXnYtrbbstWzbtlat5Vq2bdtatef88dz7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           "]], 
          PolygonBox[CompressedData["
1:eJwV0Lk33GEUBuDfSFBHBrVaEJUlCGr7sccyM6okzARZ6GP9Y2zJOcRSWpJU
iZ3QC4kOEYVniqf47nvee8/5cmKJpnhKEARh0nnh0UU3PfQSIZqcyXvoJUKU
GH00h4KghVbaaKeDTop0iymhlGeUUc6V/Job/nHLf+7oZ4A4Cd4wyBBn/Oac
C/7wl0uG7X3LO97zgRFGk3fl5VTwnEqqqCYkT+EBD0kljXRyeUIe+RTwlEIe
6WbwmDCZZJFNjb+ppY56GmikiZe84jX9DBAnwQ/dn2yzwy577PORMcaZYJIp
pjngkCOO+cUJpyzau8QXlllhlTVm5LPMMc8Cn/jMOhtsssVXvvGde9irRrE=

           "]], 
          
          PolygonBox[{{441, 442, 443, 444, 445, 446, 447, 448}, {201, 202, 
           203, 204, 205, 206, 207, 208}, {327, 328, 329, 330, 331, 332, 333, 
           334}}]}]}, {}, {}, {}, {}}, VertexColors -> CompressedData["
1:eJzV3fk3Vf37P3BJoYgKcYcoSSGzMl4KpZJCCCEiU1HJkDmZydBEmkRlyFFo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       "]], 
     GraphicsComplexBox[{{1.6, 3.5650000000000004`}, {1.6, 
      3.5650000000000004`}, {1.6, 3.5650000000000004`}, {1.6, 
      3.5650000000000004`}}, {{
        RGBColor[1, 0, 0], 
        InsetBox[
         StyleBox[
         "\[FivePointedStar]", StripOnInput -> False, FontSize -> Medium], 3], 
        InsetBox[
         StyleBox[
         "\[FivePointedStar]", StripOnInput -> False, FontSize -> Medium], 
         4]}, {}}]}, AspectRatio -> 1, Frame -> True, FrameLabel -> {
      FormBox[
       FormBox["\"Magnet ratio\"", TraditionalForm], TraditionalForm], 
      FormBox[
       FormBox["\"Coil ratio\"", TraditionalForm], TraditionalForm], 
      FormBox["\"\"", TraditionalForm], 
      FormBox["\"\"", TraditionalForm]}, 
    ImageSize -> {Automatic, 170.0787401574803}, 
    PlotRange -> {{0.1, 10.1}, {0.1, 10.}}, PlotRangeClipping -> True, 
    PlotRangePadding -> {
      Scaled[0.02], 
      Scaled[0.02]}],GraphicsBox[
    GraphicsComplexBox[CompressedData["
1:eJxl179qFFEYhvExhRcg1oJHu3SnsDBiwkEnhNEEhDSWgliskspesLPR7BWo
rdhI9AIGIkKqDEQSUwgigogm2An+28zO88r3ukTkl8PszMPhfGFO31i7dnOm
qqofk39H//tnfHZp8vNiPrqV91/dmfxs2Xon98v7b+V301/Iv/8cfd7L6/31
H+QzRxx/lF/2l3+SF/sbfpb3+u//Io/6B/wqTz+H5sl12xur64snFuiNbmV6
43on04vpxfRiejG9mF5ML6YX04vpjaY3WW+y3mS9yXqT9SbrTdabrDdZb7Le
ZL3JepP1JutN6n19/mF1fTSr3uhWpjeudzK9mF5ML6YX04vpxfRiejG9mN5o
9jfb/mbb32z7m21/s+1vtv3Ntr/Z9jfb/mbb32z7m21/s+1vtv3N6l05np7d
3Tun3uhWpjeudzK9mF5ML6YX04vpxfRiejG9mN7oqtrs93tOvdGtTG9c72R6
Mb2YXkwvphfTi+nF9GJ6Mb3RVTW39nP14q959Ua3Mr1xvZPpxfRiejG9mF5M
L6YX04vpxfRGc36Lnd9i57fY+S12foud32Lnt9j5LXZ+i53fYue32Pktdn6L
nd9i57eo9+Tt0b37jy6pN7qV6Y3rnUwvphfTi+nF9GJ6Mb2YXkwvpjeaeVXb
vKptXtU2r2qbV7XNq9rmVW3zqrZ5Vdu8qm1e1TavaptXtc2r2uZVrd4Hj3d3
bqUl9Ua3Mr1xvZPpxfRiejG9mF5ML6YX04vpxfRGM58bm8+NzefG5nNj87mx
+dzYfG5sPjc2nxubz43N58bmc2PzubH53Nh8btRb7VyevbJxVb3Rraz3hbDe
yXpfGKz3hcF6Xxis94XBel8YrPeFwXpfGKz3hcF6XxhMb3RVXejn9Yp6o1uZ
3rjeyfRiejG9mF5ML6YX04vpxfRieuPzPh3m9Yz9fZoZnv+brR/KXH8qPVnY
3F3W9Zjr3X5/rmMds+7Pz/dN18fD83yfn+7Hgcz68zfLW8dG7NeBHHv/XR/v
P/7v/lzPehU+h7o/v/H7831/AVn7tKM=
     "], {{
       GrayLevel[0.8], 
       EdgeForm[None], 
       GraphicsGroupBox[
        PolygonBox[CompressedData["
1:eJwBCQX2+iFib1JiAgAAAKgBAAADAAAA4gIRFQUGEgIDEwMEFgYHFwcIGgoL
HAwNGAgJGQkKFAQFKhobGwsMKRkaIxMUIBARIRESJBQVJRUWKBgZ7+XkJhYX
JxcYHQ0OKxscLBwdPy8wRzc4MSEiLR0eMiIjNCQlNycoMyMkNSUmNiYnLx8g
MCAhOCgpOSkqQDAxPCwtPi4vQTEyIhITQjIzRDQ1OiorOyssQzM0RTU2gHBx
RjY3f29wTj4/Sjo7Szs8Tz9AUEBBU0NEVUVGUUFCUkJDTT0+Y1NUVERFYlJT
XExNWEhJWUlKXU1OXk5PYVFSV0dIX09QYFBRVkZHZFRVZVVWd2doSTk6aVla
ZlZXa1tcbV1ecGBhbFxdbl5fb19gZ1dYaFhZcWFicmJjeGhpdWVmdmZnempr
SDg5e2tsfW1uc2NkdGRlfGxtfm5vWkpLgXFyzb2+hnZ3g3N0hHR1nIyNh3d4
inp7i3t8jHx9jX1+hXV2iXl6jn5/kICBlYWGkoKDk4OEloaHmIiJm4uMkYGC
mYmKmoqLj3+A8PjnnY2On4+Qo5OUoJCRoZGSno6PpJSVp5eYqJiZqZmaqpqb
opKTq5ucrJydrZ2er5+gsKChsaGisqKjlISFs6OkuKiprp6ftqant6eotKSl
gnJzuamqu6uswLCxva2uvq6vwbGywrKzxra3yLi5w7O0xbW2v6+w1sbHx7e4
1cXGzr6/y7u8zLy9z7/A0MDB5sXUyrq70cHC0sLDybm618fI2MjJCBgXvKyt
3MzN2cnK3c3O38/Q4+IQ3s7P4NDR7ejS2srL28vMAhIRAxMSCRkYBhYVBxcW
ChoZuqqrCxsaDR0cBBQTBRUUDBwb5B0OECAfESEgEiIhFCQjFSUkFiYlFycm
GCgnGSkoGiopGysqHCwrHS0sHy8uIDAvITEwIjIxIzMyJDQzJTU0JjY1Jzc2
KDg3KTk4Kjo5Kzs6LDw7Lj49Lz8+M0NCMEA/MUFASVlYNERDNkZFN0dGOEhH
OUlIMkJBOkpJO0tKPU1MP09OQFBPQVFQQlJRSFhXQ1NSR1dWPk5NRVVURlZV
RFRTNUVESlpZbn59TV1cTl5dT19eUGBfUWFgUmJhU2NiVGRjVWVkVmZlV2dm
WGhnWWloW2tqXGxrXW1sXm5tX29uYHBvYXFwYnJxY3NyZHRzZXV0ZnZ1Z3d2
bHx7aHh3anp5TFxbbX18b39+cIB/cYGAcoKBa3t6c4OCdISDdYWEd4eGeYmI
eoqJe4uKgZGQfIyLgJCPdoaFfo6Nf4+OfY2MEyMigpKRhJSTiZmYhZWUhpaV
ipqZi5uajp6dkKCfjJybjZ2ciJiXnq6tj5+ena2sl6emk6OilKSjmKinmamo
nKyrkqKhmqqpm6uqkaGgn6+uoLCvssLBu8vKpLSzobGwpra1qLi3q7u6p7e2
qbm4qrq5orKxo7OyrLy7rb28s8PCsMC/scHAtcXElaWktsbFuMjHrr69r7++
t8fGucnIusrJvMzLg5OSwdHQvs7Nv8/OwtLR8Ofmx9fWydnYxdXUxtbVwNDP
yNjXytrZ6+QOzNzbzd3czt7dz9/eHeQe0ODf8u7jy9va7uriEOIR6ODRpZWW
5OUe+fHhvc3M8eno7ObU6Ong8+/r9u4B4uoC9PDs9fHt5ufE0ujR4+7i9+8P
7vIB7PDm7fHo7/MPxebE8PTT7vbq6+/k+PDT8fXh7/fl8fnpl2heKQ==
         
         "]]]}, {}, {}, {}, {}}, VertexColors -> CompressedData["
1:eJxTTMoPSmViYGD4CcTMQFz2kP158PFL9rFM3pwyKWvtd7U453/88MB+VJy6
4jFRmlbraq7ZF9Wel4u1O2J/U2VTo0bP41FxKos7Xfbq5TC6Za8mu2zuzYkX
7H8cUZgSr/5sVJzK4ho+OQ+Snt+1n1L9ZHULy3V7idT+ZdMOvRgVp7J4+6Nj
M+POPbBnvKW0S7/8rr05678dZ2Nfj4pTWZw/e7mcev0je7W9xSd3zXloLykR
zOoi+3ZUnMri+7rVo9TuPLbX8I9pqDj72L5JoPSoheqoOLXF89Ysn6pq+dRe
+6GLudm/p/avOKa36uqOilNbXO6s+kWVac/s9Yp1333We2EfzLjTVcl0VJza
4ufeLudR+fzc3oBVbOnG+Ff2e37eYhW3HRWntnjKD74/2RYv7VOu+QV7Jry2
/3ssUlTp0ZtRcSqLvyvPdL1R+Mre8Lj+37MnXtuv+dfmsyN0VJza4hXfD/e5
rH5t/2e7wPIgwzf2UWZbmv1Ovh4Vp7I4U7ncjQ1P3tifXPEx4PrMN/bseQ93
PbEZFae2eM+3CkVZuXf2U2de+hXN9NZ+61K+T1UbXo2KU1n85dwZAoyLrthz
2SkppkzfbX/qStWPHrtHVBM/tu30n7TGt/Z6io+WSp57Y3+Ps+La+m+vSBbH
5X5SxydxmYPL3vX2F6pXbblsP+f+Y6GdKlvtD+ncN94//SFO8ZVhSvyHzr+1
T4yNumD14o39qXDnlKmXX+EUx2UOqe7HZT4uc0h1Py57AcxjtVU=
      "]], AspectRatio -> 
    10, Frame -> True, FrameLabel -> {
      FormBox[
       FormBox["\"~\"", TraditionalForm], TraditionalForm], 
      FormBox[
       FormBox["\"~\"", TraditionalForm], TraditionalForm], 
      FormBox[
       FormBox["\"fmax\"", TraditionalForm], TraditionalForm], 
      FormBox[
       FormBox["\"~\"", TraditionalForm], TraditionalForm]}, 
    FrameTicks -> {{{0.5, 
        FormBox[
         FormBox["\"~\"", TraditionalForm], TraditionalForm]}}, Automatic, 
      None, None}, ImageSize -> {Automatic, 170.0787401574803}, 
    PlotRange -> {{0., 1.}, {0, 63.97846484018234}}, PlotRangeClipping -> 
    True, PlotRangePadding -> {
      Scaled[0.02], Automatic}]},
  "Row",
  DisplayFunction->(RowBox[{#, "\[InvisibleSpace]", #2}]& ),
  InterpretationFunction->(RowBox[{"Row", "[", 
     RowBox[{"{", 
       RowBox[{#, ",", #2}], "}"}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.531445791803351*^9, 3.531445895796611*^9}, {
   3.531445966334722*^9, 3.531445979443829*^9}, 3.531446539760621*^9, 
   3.531446650787305*^9, 3.531446748333639*^9, 3.531448789017064*^9, {
   3.531449084931744*^9, 3.5314491509204206`*^9}, 3.53144919245894*^9, {
   3.531449272941903*^9, 3.531449418833249*^9}, 3.531530658494975*^9, 
   3.5317012541830263`*^9, 3.531790519010234*^9, 3.5317911940587063`*^9, 
   3.531791297049189*^9, 3.531792479114287*^9, 3.531798372984351*^9, {
   3.53179919658955*^9, 3.531799220487289*^9}, {3.531799451390073*^9, 
   3.531799502654282*^9}, 3.531986599835832*^9, 3.532048989342044*^9, {
   3.5320494758473253`*^9, 3.532049488014544*^9}, 3.532049520301751*^9}],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{
     GraphicsComplexBox[CompressedData["
1:eJyVnXlczekfxZOEi0uSNEkLkmyl7HQfsmZrsWQnKfvSgqxFspZdyZYka5YS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      "], {{
        GrayLevel[0.8], 
        EdgeForm[None], 
        GraphicsGroupBox[{
          
          PolygonBox[{{2417, 24, 25, 2416}, {2481, 1, 2401, 2561}, {449, 450, 
           451, 452}, {2413, 18, 19, 2412}, {485, 486, 487, 488}, {323, 324, 
           325, 326}, {509, 510, 511, 512}, {2433, 48, 49, 2432}, {2488, 181, 
           182, 2487}, {437, 438, 439, 440}, {2549, 2400, 2480, 2564}}], 
          PolygonBox[CompressedData["
1:eJwt1FdwVkUYBuAVVFoo11SpIQFB0gBB1BBAsZD8CQklwEwIid6IhpCo6A2g
WChKFxSkCAoIM4jOSFdpQkiodrA3QLCNDQSfdbh4cr53c/7dPbt7TvuSiYkH
6oUQCrmOM41C2NYghO1s5Uf5Gw5dH0INB/la/oiz14ZwjjN8KO+hjc7a0o73
5C/J95sCEnwh326cbAaSwyB+1r6+YQgbWMcFeRV3cTf3sJJXuJdh5LKa3+hI
JzrwK7vjvdeEsIa17JK/Z6BnyiGb7+SfSDdeBmmckx+hszl1pBMPy58y3gKV
UsIn8nIGMZghLGMFd3AnQ3mZeXEMMshkLpfZYQ472cVZ+d/Yn3owQ/hBXkAW
venDfBbRl5vpx0Ke5yZ6kcZzXInjeaYsenNenkUq3ejOTOZwIz3oyWwucdh+
1VLHt/LpuA6e/bP4/JySLzLc/wsp4it5Cl2tWxeSeVR+Ij6DOpMspstLuZXb
4plgCS+RHfeJHF7kr7hPzth5LvCx/DcTzKGM8jgn+R9OqU/zeSS/QH9uYQCL
mRb3whzSSGeqPCOup7oPfXlSfjyukzqFVB6T15JHIp5r1rCJMYxlHBt5jYK4
NhTyKpOcw0omU0U1w8gljwT5lBprAmWUcx9HOcZxTnCSEYxkFKMpjle/L2YM
YxnHHu172cd+DpBPAcOvvvtFvGmOm13fYAtvyW/H86KPbnRnq3yCCvswiUpO
ynVxj+tbN2ZwJJ4fpqqnMZ1a+WjcJ/VFLnFMfp/G6iYkcVA+RFN1M5pTIycZ
vynNaE4L/tQ+2RyqqOYD+Y/4LvnNZa5wXN4ez7j7e9CTbXKub1QeCfIpIH4M
J/IgD1HBePfqOsTvpO5CSdxLWqtb0oqKOA/aqdvQlkq5mg7qG2hPlbyeIkYw
knXxLNPg6jfYaxRK5dcZxWiK2UBorN27vJFN/BLfdxNLJ4NMsihv9P+twS0h
/qzMn/oannbtTBd+51lrMZNZzGYO+7Uv0MdCFnFA3stc9Tzmsy/24ZpMV1JI
5R3tLV1b0Zp35fvjXppDE5LieZZrjHOYWuo4wg7tK11XsZqdcX4kx+8JKTxD
Pc/xlGs/ffVns/o/l6TZww==
           "]], 
          PolygonBox[CompressedData["
1:eJwV2XOgVmccAOCbbdtaWlxYtldbXnYtrbbstWzbtlat5Vq2bdtatef88dz7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           "]], 
          PolygonBox[CompressedData["
1:eJwV0Lk33GEUBuDfSFBHBrVaEJUlCGr7sccyM6okzARZ6GP9Y2zJOcRSWpJU
iZ3QC4kOEYVniqf47nvee8/5cmKJpnhKEARh0nnh0UU3PfQSIZqcyXvoJUKU
GH00h4KghVbaaKeDTop0iymhlGeUUc6V/Job/nHLf+7oZ4A4Cd4wyBBn/Oac
C/7wl0uG7X3LO97zgRFGk3fl5VTwnEqqqCYkT+EBD0kljXRyeUIe+RTwlEIe
6WbwmDCZZJFNjb+ppY56GmikiZe84jX9DBAnwQ/dn2yzwy577PORMcaZYJIp
pjngkCOO+cUJpyzau8QXlllhlTVm5LPMMc8Cn/jMOhtsssVXvvGde9irRrE=

           "]], 
          
          PolygonBox[{{441, 442, 443, 444, 445, 446, 447, 448}, {201, 202, 
           203, 204, 205, 206, 207, 208}, {327, 328, 329, 330, 331, 332, 333, 
           334}}]}]}, {}, {}, {}, {}}, VertexColors -> CompressedData["
1:eJzV3fk31e/3P36hEKJCKkRJCpmFsCVKJTKHIiJDAyWSOZnJ1EBpEJUhJBok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       "]], 
     GraphicsComplexBox[{{1.6, 3.5650000000000004`}, {1.6, 
      3.5650000000000004`}, {1.6, 3.5650000000000004`}, {1.6, 
      3.5650000000000004`}}, {{
        RGBColor[1, 0, 0], 
        InsetBox[
         StyleBox[
         "\[FivePointedStar]", StripOnInput -> False, FontSize -> Medium], 3], 
        InsetBox[
         StyleBox[
         "\[FivePointedStar]", StripOnInput -> False, FontSize -> Medium], 
         4]}, {}}], 
     InsetBox[
      FormBox["\"fmax\"", TraditionalForm], {1.6, 3.5650000000000004`}, {-2, 
      0}, Automatic, {1, 0}]}, AspectRatio -> 1, Frame -> True, FrameLabel -> {
      FormBox[
       FormBox["\"Magnet ratio\"", TraditionalForm], TraditionalForm], 
      FormBox[
       FormBox["\"Coil ratio\"", TraditionalForm], TraditionalForm], 
      FormBox["\"\"", TraditionalForm], 
      FormBox["\"\"", TraditionalForm]}, 
    ImageSize -> {Automatic, 170.0787401574803}, 
    PlotRange -> {{0.1, 10.1}, {0.1, 10.}}, PlotRangeClipping -> True, 
    PlotRangePadding -> {
      Scaled[0.02], 
      Scaled[0.02]}],GraphicsBox[
    GraphicsComplexBox[CompressedData["
1:eJxl1r9rFEEYh/FxQZbgRqcINsEiaGFnEzshUwiSxsI/QdCAEqzs7CwCUQSv
FQStrQJqJzhgJSSwRUBTCIMExB8J0wRF1MvePW/4vh4R+WTY23sc951buH7n
2o0mhPBz/Ofwb/8anVse/2wsqbN559Xq+Oe9W+/Nw/LOtvnj5BfmP38PX5/M
j4fri/nsIUefzS+Hy3fNV4YbfjF/GN7/q/n28AG/myevfecQ2pnFzctv+iV6
1dlMr673ZnoxvZheTC+mF9OL6cX0YnoxvWp6i+strre43uJ6i+strre43uJ6
i+strre43uJ6i+strvfo/9H67NWLK2e+Wa86m+nV9d5ML6YX04vpxfRiejG9
mF5ML6ZXzf5Wt7/V7W91+1vd/la3v9Xtb3X7W93+Vre/1e1vdftb3f5Wt7/V
7W+13uUTb49trx5Yrzqb6dX13kwvphfTi+nF9GJ6Mb2YXkwvplcdwoNhv39b
rzqb6dX13kwvphfTi+nF9GJ6Mb2YXkwvplcdwu7a6uK9myHRq85menW9N9OL
6cX0YnoxvZheTC+mF9OL6VXz/DbWq85mfX4b68X6/DZJn98m6fPbJH1+m6TP
b5P0+W2SPr9N0ue3Sfr8NtarDuH8+q+V2dPHrVedzfTqem+mF9OL6cX0Ynox
vZheTC+mF9OrZl611qvOZp1XrfVinVdt0nnVJp1XbdJ51SadV23SedUmnVdt
0nnVJp1XrfWqQ7j1cO3J03cz1qvOZnp1vTfTi+nF9GJ6Mb2YXkwvphfTi+lV
M58761Vns87nznqxzucu6Xzuks7nLul87pLO5y7pfO6Szucu6Xzuks7nznrV
Ibx4NLd14e5J61VnM7263pvpxfRiejG9mF5ML6YX04vpxfSqOY+iO4+iO4+i
O4+iO4+iO4+iO4+iO4+iO4+iO4+iO4+iO4+iO4+iO4+iO4/4vM+n8zq775d5
+v3hh1vfN3P9pa2D+blnp+zfD3O9t78/17GOWfefn/ebrI+mn2dj+n1oz8x6
fr24cH+e/doza+/R9Xr/0X/353rWg7z27f78xt+f9/sHph5Mog==
     "], {{
       GrayLevel[0.8], 
       EdgeForm[None], 
       GraphicsGroupBox[
        PolygonBox[CompressedData["
1:eJwBCQX2+iFib1JiAgAAAKgBAAADAAAA4gIRFQUGEgIDEwMEFgYHFwcIGgoL
HAwNGAgJGQkKFAQFKhobGwsMKRkaIxMUIBARIRESJBQVJRUWKBgZ7+XkJhYX
JxcYHQ0OKxscLBwdPy8wRzc4MSEiLR0eMiIjNCQlNycoMyMkNSUmNiYnLx8g
MCAhOCgpOSkqQDAxPCwtPi4vQTEyIhITQjIzRDQ1OiorOyssQzM0RTU2gHBx
RjY3f29wTj4/Sjo7Szs8Tz9AUEBBU0NEVUVGUUFCUkJDTT0+Y1NUVERFYlJT
XExNWEhJWUlKXU1OXk5PYVFSV0dIX09QYFBRVkZHZFRVZVVWd2doSTk6aVla
ZlZXa1tcbV1ecGBhbFxdbl5fb19gZ1dYaFhZcWFicmJjeGhpdWVmdmZnempr
SDg5e2tsfW1uc2NkdGRlfGxtfm5vWkpLgXFyzb2+hnZ3g3N0hHR1nIyNh3d4
inp7i3t8jHx9jX1+hXV2iXl6jn5/kICBlYWGkoKDk4OEloaHmIiJm4uMkYGC
mYmKmoqLj3+A8PjnnY2On4+Qo5OUoJCRoZGSno6PpJSVp5eYqJiZqZmaqpqb
opKTq5ucrJydrZ2er5+gsKChsaGisqKjlISFs6OkuKiprp6ftqant6eotKSl
gnJzuamqu6uswLCxva2uvq6vwbGywrKzxra3yLi5w7O0xbW2v6+w1sbHx7e4
1cXGzr6/y7u8zLy9z7/A0MDB5sXUyrq70cHC0sLDybm618fI2MjJCBgXvKyt
3MzN2cnK3c3O38/Q4+IQ3s7P4NDR7ejS2srL28vMAhIRAxMSCRkYBhYVBxcW
ChoZuqqrCxsaDR0cBBQTBRUUDBwb5B0OECAfESEgEiIhFCQjFSUkFiYlFycm
GCgnGSkoGiopGysqHCwrHS0sHy8uIDAvITEwIjIxIzMyJDQzJTU0JjY1Jzc2
KDg3KTk4Kjo5Kzs6LDw7Lj49Lz8+M0NCMEA/MUFASVlYNERDNkZFN0dGOEhH
OUlIMkJBOkpJO0tKPU1MP09OQFBPQVFQQlJRSFhXQ1NSR1dWPk5NRVVURlZV
RFRTNUVESlpZbn59TV1cTl5dT19eUGBfUWFgUmJhU2NiVGRjVWVkVmZlV2dm
WGhnWWloW2tqXGxrXW1sXm5tX29uYHBvYXFwYnJxY3NyZHRzZXV0ZnZ1Z3d2
bHx7aHh3anp5TFxbbX18b39+cIB/cYGAcoKBa3t6c4OCdISDdYWEd4eGeYmI
eoqJe4uKgZGQfIyLgJCPdoaFfo6Nf4+OfY2MEyMigpKRhJSTiZmYhZWUhpaV
ipqZi5uajp6dkKCfjJybjZ2ciJiXnq6tj5+ena2sl6emk6OilKSjmKinmamo
nKyrkqKhmqqpm6uqkaGgn6+uoLCvssLBu8vKpLSzobGwpra1qLi3q7u6p7e2
qbm4qrq5orKxo7OyrLy7rb28s8PCsMC/scHAtcXElaWktsbFuMjHrr69r7++
t8fGucnIusrJvMzLg5OSwdHQvs7Nv8/OwtLR8Ofmx9fWydnYxdXUxtbVwNDP
yNjXytrZ6+QOzNzbzd3czt7dz9/eHeQe0ODf8u7jy9va7uriEOIR6ODRpZWW
5OUe+fHhvc3M8eno7ObU6Ong8+/r9u4B4uoC9PDs9fHt5ufE0ujR4+7i9+8P
7vIB7PDm7fHo7/MPxebE8PTT7vbq6+/k+PDT8fXh7/fl8fnpl2heKQ==
         
         "]]]}, {}, {}, {}, {}}, VertexColors -> CompressedData["
1:eJxTTMoPSmViYGD4CcTMQFz2kP158PFL9rFM3pwyKWvtd7U453/88MB+VJy6
4jFRmlbraq7ZF9Wel4u1O2J/U2VTo0bP41FxKos7Xfbq5TC6Za8mu2zuzYkX
7H8cUZgSr/5sVJzK4ho+OQ+Snt+1n1L9ZHULy3V7idT+ZdMOvRgVp7J4+6Nj
M+POPbBnvKW0S7/8rr05678dZ2Nfj4pTWZw/e7mcev0je7W9xSd3zXloLykR
zOoi+3ZUnMri+7vVo9TuPLbX9I9pqDj72L5JoPSoheqoOLXF89Ysn6pq+dRe
+6GLudm/p/avOKa36uqOilNbXO6s+kWVac/s9Yp1333We2EfzLjTVcl0VJza
4uffLudR+fzc3ohVbOnG+Ff2e37eYhW3HRWntnjKD74/2RYv7VOu+QV7Jry2
/3ssUlTp0ZtRcSqLvyvPdL1R+Mre8Lj+37MnXtuv+dfmsyN0VJza4pXfD/e5
rH5t/2e7wPIgwzf2kWZbmv1Ovh4Vp7I4U7ncjQ1P3tifXPEx4PrMN/bseQ93
PbEZFae2ePe3CkVZuXf2U2Ze+hXN9NZ+21K+T1UbXo2KU1n85dwZAoyLrthz
2SkppkzfbX/qStWPHrtHVBM/tu30n7TGt/Z6io+WSp57Y3+Ps+La+m+vSBbH
5X5SxydxmYPL3vX2F6pXbblsP+f+Y6GdKlvtD+ncN94//SFO8ZVhSvyHzr+1
T4yNumD14o39qXDnlKmXX+EUx2UOqe7HZT4uc0h1Py57AQnntY0=
      "]], AspectRatio -> 
    10, Frame -> True, FrameLabel -> {
      FormBox[
       FormBox["\"~\"", TraditionalForm], TraditionalForm], 
      FormBox[
       FormBox["\"~\"", TraditionalForm], TraditionalForm], 
      FormBox[
       FormBox["\"norm fmax\"", TraditionalForm], TraditionalForm], 
      FormBox[
       FormBox["\"~\"", TraditionalForm], TraditionalForm]}, 
    FrameTicks -> {{{0.5, 
        FormBox[
         FormBox["\"~\"", TraditionalForm], TraditionalForm]}}, Automatic, 
      None, None}, ImageSize -> {Automatic, 170.0787401574803}, 
    PlotRange -> {{0., 1.}, {0, 4.100083178696854}}, PlotRangeClipping -> 
    True, PlotRangePadding -> {
      Scaled[0.02], Automatic}]},
  "Row",
  DisplayFunction->(RowBox[{#, "\[InvisibleSpace]", #2}]& ),
  InterpretationFunction->(RowBox[{"Row", "[", 
     RowBox[{"{", 
       RowBox[{#, ",", #2}], "}"}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.531445791803351*^9, 3.531445895796611*^9}, {
   3.531445966334722*^9, 3.531445979443829*^9}, 3.531446539760621*^9, 
   3.531446650787305*^9, 3.531446748333639*^9, 3.531448789017064*^9, {
   3.531449084931744*^9, 3.5314491509204206`*^9}, 3.53144919245894*^9, {
   3.531449272941903*^9, 3.531449418833249*^9}, 3.531530658494975*^9, 
   3.5317012541830263`*^9, 3.531790519010234*^9, 3.5317911940587063`*^9, 
   3.531791297049189*^9, 3.531792479114287*^9, 3.531798372984351*^9, {
   3.53179919658955*^9, 3.531799220487289*^9}, {3.531799451390073*^9, 
   3.531799502654282*^9}, 3.531986599835832*^9, 3.532048989342044*^9, {
   3.5320494758473253`*^9, 3.532049488014544*^9}, 3.532049520572236*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"PSfragExport", "[", 
   RowBox[{"\"\<fig/magratio-maxforce\>\"", ",", "g1"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PSfragExport", "[", 
   RowBox[{"\"\<fig/magratio-maxforce-cb\>\"", ",", "g2"}], "]"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.531446003102043*^9, 3.531446010716894*^9}, {
  3.531446206175181*^9, 3.5314462424150753`*^9}, {3.531447418971744*^9, 
  3.531447419350978*^9}, {3.5314476671821127`*^9, 3.5314476675174427`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Maximum peak force versus wire diameter\
\>", "Subsection",
 CellChangeTimes->{{3.531013707496604*^9, 3.531013712282455*^9}, {
  3.5317007786874313`*^9, 3.531700789535866*^9}, {3.532767313821266*^9, 
  3.532767320367345*^9}}],

Cell["\<\
As we discussed above, need to ensure the coil ratio doesn\[CloseCurlyQuote]t \
exceed certain bounds:\
\>", "Text",
 CellChangeTimes->{{3.532767330143702*^9, 3.532767353328958*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MagnetCoilForceMaxScaledBound", "[", 
   RowBox[{"a_", ",", "b_", ",", "dd_", ",", "rr_", ",", "vv_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"coilRatioMin", "[", 
        RowBox[{"a", ",", "dd", ",", "vv"}], "]"}], "<", "b", "<", 
       RowBox[{"coilRatioMax", "[", 
        RowBox[{"a", ",", "dd", ",", "rr", ",", "vv"}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"MagnetCoilForceMaxScaled", "[", 
       RowBox[{"a", ",", "b", ",", "dd", ",", "rr", ",", "vv"}], "]"}], ",", 
      "\[IndentingNewLine]", "0"}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.531702469733382*^9, 3.5317025725029716`*^9}, {
  3.5317026280987186`*^9, 3.531702641392272*^9}}],

Cell["\<\
This is how long it takes to calculate a single data point:\
\>", "Text",
 CellChangeTimes->{{3.532767358953096*^9, 3.5327673667929792`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"dd", "=", 
       RowBox[{"1", "mm"}]}], ",", 
      RowBox[{"rr", "=", 
       RowBox[{"4", "ohms"}]}], ",", 
      RowBox[{"vv", "=", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"20", "mm"}], ")"}], "3"]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"NMaximize", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"MagnetCoilForceMaxScaledBound", "[", 
         RowBox[{"a", ",", "b", ",", "dd", ",", "rr", ",", "vv"}], "]"}], ",", 
        RowBox[{"a", ">", "0.1"}], ",", 
        RowBox[{"b", ">", "0.1"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"a", ",", "0.1", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"b", ",", "0.1", ",", "10"}], "}"}]}], "}"}]}], "]"}]}], 
   "\[IndentingNewLine]", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.5317026029761457`*^9, 3.531702671521059*^9}, {
  3.531702744157432*^9, 3.5317028409758673`*^9}, {3.531702878517693*^9, 
  3.5317028951197968`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"86.32609699999999`", ",", 
   RowBox[{"{", 
    RowBox[{"65.46775346330693`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2067930264114124`"}], ",", 
       RowBox[{"b", "\[Rule]", "2.9640566817688576`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.53170266208169*^9, {3.531702741622149*^9, 3.531702774149312*^9}, {
   3.5317028161943417`*^9, 3.5317028428531227`*^9}, 3.5317028966883173`*^9, 
   3.531702982909288*^9, 3.5317045581240187`*^9, 3.531704607269784*^9, 
   3.531704646023295*^9, 3.531704890999691*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"CalcRatioMax", "[", 
   RowBox[{"dd_", ",", "rr_", ",", "vv_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"ValueQ", "@", 
       RowBox[{"ratiomax", "[", 
        RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", "true", " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Already calculated for d=\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"dd", "/", "mm"}], "]"}], "<>", "\"\< mm, r=\>\"", "<>", 
        RowBox[{"ToString", "[", "rr", " ", "]"}], "<>", "\"\< ohm, v=(\>\"", 
        "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", "vv", ")"}], 
           RowBox[{"1", "/", "3"}]], "/", "mm"}], "]"}], "<>", 
        "\"\<mm\!\(\*SuperscriptBox[\()\), \(3\)]\)\>\""}], "]"}], ",", " ", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", "false", " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Calculating for d=\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"dd", "/", "mm"}], "]"}], "<>", "\"\< mm, r=\>\"", "<>", 
         RowBox[{"ToString", "[", "rr", " ", "]"}], "<>", "\"\< ohm, v=(\>\"",
          "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", "vv", ")"}], 
            RowBox[{"1", "/", "3"}]], "/", "mm"}], "]"}], "<>", 
         "\"\<mm\!\(\*SuperscriptBox[\()\), \(3\)]\)\>\""}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"ratiomax", "[", 
         RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}], "=", 
        RowBox[{"NMaximize", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"MagnetCoilForceMaxScaledBound", "[", 
             RowBox[{"a", ",", "b", ",", "dd", ",", "rr", ",", "vv"}], "]"}], 
            ",", 
            RowBox[{"a", ">", "0.1"}], ",", 
            RowBox[{"b", ">", "0.1"}]}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"a", ",", "0.1", ",", "10"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"b", ",", "0.1", ",", "10"}], "}"}]}], "}"}]}], 
         "\[IndentingNewLine]", "]"}]}]}]}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.531706608705106*^9, 3.531706686368783*^9}, {
  3.5317067439000883`*^9, 3.531706802637002*^9}, {3.531706855118711*^9, 
  3.5317068581098843`*^9}}],

Cell["\<\
To save sanity, this data was not calculated in one continuous run; rather, \
data sets were calculated as necessary to produce the desired plots for \
publication. Still, more data was generated than presented in the publication.\
\>", "Text",
 CellChangeTimes->{{3.53276738107316*^9, 3.5327674504890137`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Get", "[", 
   RowBox[{"\"\<data/magratio-max.mx\>\"", ",", 
    RowBox[{"{", "ratiomax", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"rr", "=", 
      RowBox[{"32", "ohms"}]}], ",", 
     RowBox[{"vv", "=", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"15", "mm"}], ")"}], "3"]}], ",", 
     RowBox[{"Drange", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
       "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"CalcRatioMax", "[", 
      RowBox[{"#", ",", "rr", ",", "vv"}], "]"}], "&"}], "/@", "Drange"}]}], 
  "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Save", "[", 
  RowBox[{"\"\<data/magratio-max.mx\>\"", ",", 
   RowBox[{"{", "ratiomax", "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.531706493526801*^9, 3.5317065745602827`*^9}, {
   3.531706696685379*^9, 3.5317067301249027`*^9}, {3.531707175898985*^9, 
   3.531707198874715*^9}, {3.531708002067355*^9, 3.531708012827004*^9}, {
   3.531708793901289*^9, 3.5317087974122963`*^9}, {3.5317094578444233`*^9, 
   3.531709463943623*^9}, {3.531709984495932*^9, 3.531709985439969*^9}, {
   3.531712199107966*^9, 3.5317122841051607`*^9}, {3.531721238809808*^9, 
   3.531721279747238*^9}, {3.531730308325708*^9, 3.5317303110500927`*^9}, {
   3.531949788040578*^9, 3.531949795541567*^9}, {3.531949846806779*^9, 
   3.531950002737095*^9}, 3.531950165982705*^9, {3.531987724542244*^9, 
   3.531987733894985*^9}, {3.531991555053258*^9, 3.531991570914847*^9}, 
   3.532767376805167*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Get", "::", "notencode"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Warning: The file \[NoBreak]\\!\\(\\\"data/magratio-max.mx\
\\\"\\)\[NoBreak] is not encoded. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/Get/notencode\\\", \
ButtonNote -> \\\"Get::notencode\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.5319915775453253`*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Already calculated for d=0.5 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577599113*^9}],

Cell[BoxData["\<\"Already calculated for d=0.6 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577602642*^9}],

Cell[BoxData["\<\"Already calculated for d=0.7 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577606847*^9}],

Cell[BoxData["\<\"Already calculated for d=0.8 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577611514*^9}],

Cell[BoxData["\<\"Already calculated for d=0.9 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577616239*^9}],

Cell[BoxData["\<\"Already calculated for d=1. mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577621031*^9}],

Cell[BoxData["\<\"Already calculated for d=1.1 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577625819*^9}],

Cell[BoxData["\<\"Already calculated for d=1.2 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577629664*^9}],

Cell[BoxData["\<\"Already calculated for d=1.3 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577634036*^9}],

Cell[BoxData["\<\"Already calculated for d=1.4 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577638546*^9}],

Cell[BoxData["\<\"Already calculated for d=1.5 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577643093*^9}],

Cell[BoxData["\<\"Already calculated for d=1.6 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577647562*^9}],

Cell[BoxData["\<\"Already calculated for d=1.7 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319915776520023`*^9}],

Cell[BoxData["\<\"Already calculated for d=1.8 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577656406*^9}],

Cell[BoxData["\<\"Already calculated for d=1.9 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577660796*^9}],

Cell[BoxData["\<\"Already calculated for d=2. mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531991577665182*^9}],

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319915776690598`*^9}],

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531992125378439*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMaxValue", "::", "lstol"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The line search decreased the step size to within the \
tolerance specified by AccuracyGoal and PrecisionGoal but was unable to find \
a sufficient increase in the function. You may need more than \
\[NoBreak]\\!\\(MachinePrecision\\)\[NoBreak] digits of working precision to \
meet these tolerances. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/FindMaxValue\\\", ButtonNote -> \
\\\"FindMaxValue::lstol\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.531992143856752*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531992540947063*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531992979183621*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=32 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53199327603115*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", 
   ",", "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", ",", 
   "Null", ",", "Null", ",", "Null", ",", "Null", ",", "Null", ",", 
   RowBox[{"{", 
    RowBox[{"48.89795161214758`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3069715947114884`"}], ",", 
       RowBox[{"b", "\[Rule]", "21.814248302234734`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"43.27343238844906`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "0.24770003536689778`"}], ",", 
       RowBox[{"b", "\[Rule]", "15.067989491977464`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"47.25046237098698`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2282103747408595`"}], ",", 
       RowBox[{"b", "\[Rule]", "26.39736641337699`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"46.746294062793524`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2325902900804167`"}], ",", 
       RowBox[{"b", "\[Rule]", "24.472345120787445`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"46.43990691665021`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2358613093587358`"}], ",", 
       RowBox[{"b", "\[Rule]", "25.26452313745402`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.531993633224141*^9}],

Cell[BoxData["\<\"Calculating for d=0.5 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531993633227933*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.531993709709584*^9}],

Cell[BoxData["\<\"Calculating for d=0.6 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53199370971338*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.531993808747538*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.7 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531993808751308*^9}],

Cell[BoxData["\<\"Calculating for d=0.8 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319939950211906`*^9}],

Cell[BoxData["\<\"Calculating for d=0.9 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531994201466024*^9}],

Cell[BoxData["\<\"Calculating for d=1. mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319943246581097`*^9}],

Cell[BoxData["\<\"Calculating for d=1.1 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531994519174089*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.5319947054285727`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(NMaximize \
:: cvmit\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.5319947054322977`*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=1.2 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531994705435848*^9}],

Cell[BoxData["\<\"Calculating for d=1.3 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319948598015823`*^9}],

Cell[BoxData["\<\"Calculating for d=1.4 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531995117969452*^9}],

Cell[BoxData["\<\"Calculating for d=1.5 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319954877545424`*^9}],

Cell[BoxData["\<\"Calculating for d=1.6 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319958495181227`*^9}],

Cell[BoxData["\<\"Calculating for d=1.7 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531996105033597*^9}],

Cell[BoxData["\<\"Calculating for d=1.8 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531996410736898*^9}],

Cell[BoxData["\<\"Calculating for d=1.9 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531996643186183*^9}],

Cell[BoxData["\<\"Calculating for d=2. mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531996903842979*^9}],

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531997403245201*^9}],

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531997601095068*^9}],

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531997915761237*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531998222422311*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=32 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531998564742156*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"43.310868369938184`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.8272337680432895`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.320512843315262`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"63.86959183168526`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3114419681157927`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.028746685616759`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"83.88628608778716`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4540651106034925`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.1504773393804517`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"94.3922267786212`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4650407412966038`"}], ",", 
       RowBox[{"b", "\[Rule]", "5.142341215585039`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"105.47201930156912`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.8689050103620022`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.486141912719765`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"106.67477721662804`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4795657800466506`"}], ",", 
       RowBox[{"b", "\[Rule]", "5.887534232447642`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"107.83220920418927`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.843748341617452`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.176905114735723`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"108.72492235374828`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4646860915404538`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.19197738838413`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"109.30686153423623`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4643903514378551`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.874068554762285`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"115.55142180012199`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.591323612948191`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.81419111589205`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"118.05980806876379`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4943674446436899`"}], ",", 
       RowBox[{"b", "\[Rule]", "11.39075268429021`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"115.27247314303754`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6721819292065536`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.72888904542729`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"111.651369131019`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4958697177025067`"}], ",", 
       RowBox[{"b", "\[Rule]", "13.050322503801278`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"105.3599415020394`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5705037015570054`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.654732237080331`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"105.26676931161639`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2332554535292202`"}], ",", 
       RowBox[{"b", "\[Rule]", "15.433816434100969`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"104.8513702730339`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6011981497346643`"}], ",", 
       RowBox[{"b", "\[Rule]", "15.526763731373169`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"86.27211379320546`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "0.1325664663303344`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.351125760097179`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"102.2935424584767`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2158643578711696`"}], ",", 
       RowBox[{"b", "\[Rule]", "14.810827109312772`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"101.46219166337015`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3199294925415925`"}], ",", 
       RowBox[{"b", "\[Rule]", "19.627201270274128`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"100.5358351913744`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2107929023314858`"}], ",", 
       RowBox[{"b", "\[Rule]", "18.479301853154855`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"99.96352778272205`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.235607151010407`"}], ",", 
       RowBox[{"b", "\[Rule]", "19.18592839161499`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.531998971384445*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.5 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319989713893757`*^9}],

Cell[BoxData["\<\"Calculating for d=0.6 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531999044320182*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.531999143832941*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.7 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531999143836625*^9}],

Cell[BoxData["\<\"Calculating for d=0.8 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531999263324287*^9}],

Cell[BoxData["\<\"Calculating for d=0.9 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5319994000160017`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.53199955236412*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=1. mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531999552367737*^9}],

Cell[BoxData["\<\"Calculating for d=1.1 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.531999719238871*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.5319998635292273`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(NMaximize \
:: cvmit\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.531999863533003*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=1.2 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53199986353662*^9}],

Cell[BoxData["\<\"Calculating for d=1.3 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320000725157022`*^9}],

Cell[BoxData["\<\"Calculating for d=1.4 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320002349249763`*^9}],

Cell[BoxData["\<\"Calculating for d=1.5 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532000437769693*^9}],

Cell[BoxData["\<\"Calculating for d=1.6 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320007390874367`*^9}],

Cell[BoxData["\<\"Calculating for d=1.7 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532000981063602*^9}],

Cell[BoxData["\<\"Calculating for d=1.8 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320012118222923`*^9}],

Cell[BoxData["\<\"Calculating for d=1.9 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532001423470208*^9}],

Cell[BoxData["\<\"Calculating for d=2. mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532002017685134*^9}],

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532002463558519*^9}],

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532002758483433*^9}],

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532003280860672*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532003697479885*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=32 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320039937505913`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"62.02895774018678`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "2.1498704406124345`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.4424426449636214`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"99.55550834716381`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.302627525994199`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.068877952501246`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"138.62722031469292`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7994710310289919`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.3116974164732493`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"154.76174798926527`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3690919250795062`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.04928999317246`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"178.53862404416446`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3838231185129621`"}], ",", 
       RowBox[{"b", "\[Rule]", "4.523383753221855`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"181.9733696969624`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.484469177751889`"}], ",", 
       RowBox[{"b", "\[Rule]", "4.817798997858463`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"185.84478958953122`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7377732040027798`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.249170202087723`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"188.3228620807413`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4946462468970598`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.805166668625594`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"190.39663983179105`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4261801747576777`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.427559705264453`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"202.1622207315714`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6494681519415897`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.85131925897037`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"207.65592593786874`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6014926356350676`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.493187390985454`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"203.8738294699548`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1967981274824158`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.120839143662035`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"197.4949039080424`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.304475440253809`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.859027669660735`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"189.59509487907405`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2729284457351784`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.35982965596395`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"184.84125141638916`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1189183465387729`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.027807246256451`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"187.79299035704008`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5273402088681862`"}], ",", 
       RowBox[{"b", "\[Rule]", "12.074016874301273`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"182.16176721027819`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3601634032677492`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.703822639246287`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"184.17570785597383`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3794322567305008`"}], ",", 
       RowBox[{"b", "\[Rule]", "14.797234424532585`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"182.12210053497327`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3092557631486943`"}], ",", 
       RowBox[{"b", "\[Rule]", "14.497069139686513`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"181.00558654931518`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3308067171128817`"}], ",", 
       RowBox[{"b", "\[Rule]", "15.656828914824882`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"177.5084675347381`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "2.223962270223882`"}], ",", 
       RowBox[{"b", "\[Rule]", "21.0831638067435`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.532004700891738*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.5 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320047008967743`*^9}],

Cell[BoxData["\<\"Calculating for d=0.6 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532004802013072*^9}],

Cell[BoxData["\<\"Calculating for d=0.7 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532004911676436*^9}],

Cell[BoxData["\<\"Calculating for d=0.8 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53200504472445*^9}],

Cell[BoxData["\<\"Calculating for d=0.9 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532005235413773*^9}],

Cell[BoxData["\<\"Calculating for d=1. mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53200531105091*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532005446997616*^9}],

Cell[BoxData["\<\"Calculating for d=1.1 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532005447001367*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532005605981391*^9}],

Cell[BoxData["\<\"Calculating for d=1.2 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532005605985053*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.5320058481298447`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(NMaximize \
:: cvmit\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532005848133724*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=1.3 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53200584813734*^9}],

Cell[BoxData["\<\"Calculating for d=1.4 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532006083468684*^9}],

Cell[BoxData["\<\"Calculating for d=1.5 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532006350847011*^9}],

Cell[BoxData["\<\"Calculating for d=1.6 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532006701468978*^9}],

Cell[BoxData["\<\"Calculating for d=1.7 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532006893654891*^9}],

Cell[BoxData["\<\"Calculating for d=1.8 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532007432862103*^9}],

Cell[BoxData["\<\"Calculating for d=1.9 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53200771164707*^9}],

Cell[BoxData["\<\"Calculating for d=2. mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532008147051745*^9}],

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532008620641653*^9}],

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320089532871027`*^9}],

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320092411441317`*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532009770805017*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=28 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532010082905592*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"23.278825818316662`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4182985567704696`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.093929667311938`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"31.51016925611852`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "0.9688121743477796`"}], ",", 
       RowBox[{"b", "\[Rule]", "2.847794352661223`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"42.63248067392832`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7529967061186622`"}], ",", 
       RowBox[{"b", "\[Rule]", "5.642626332008687`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"46.44967296644522`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6060694796180546`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.532881166889529`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"51.44710790601541`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6353796073640605`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.99991009273322`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"51.3622533664883`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.44819735420979`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.217800541753022`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"51.482010038046425`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7587689746399153`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.345322817074797`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"51.482182770042`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2773705690346402`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.719082217042462`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"51.57018194944457`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.429751850693358`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.911157562923087`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"54.23766077222172`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4720162029440274`"}], ",", 
       RowBox[{"b", "\[Rule]", "12.006302475751175`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"55.18442482459456`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4009868893405903`"}], ",", 
       RowBox[{"b", "\[Rule]", "12.198285481502785`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"53.820908009435215`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2110284837335155`"}], ",", 
       RowBox[{"b", "\[Rule]", "13.533137879873383`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"51.44675678032288`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "2.192163308121855`"}], ",", 
       RowBox[{"b", "\[Rule]", "17.336454603241393`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"49.60307258589568`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.295238013382727`"}], ",", 
       RowBox[{"b", "\[Rule]", "19.190340407015608`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"48.881584033234965`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2783629569763513`"}], ",", 
       RowBox[{"b", "\[Rule]", "18.6428842991873`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"48.26052780665408`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.171209175138355`"}], ",", 
       RowBox[{"b", "\[Rule]", "21.436541246509417`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"48.16373793351286`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.216689956571614`"}], ",", 
       RowBox[{"b", "\[Rule]", "20.70124925492241`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"47.21385573817422`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.221461509034376`"}], ",", 
       RowBox[{"b", "\[Rule]", "20.84101012424944`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"46.41623525666591`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7819231605702965`"}], ",", 
       RowBox[{"b", "\[Rule]", "27.11832950866964`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"46.23330061094612`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6197707137480974`"}], ",", 
       RowBox[{"b", "\[Rule]", "27.719909932018616`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"45.79011152003182`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2236372554754813`"}], ",", 
       RowBox[{"b", "\[Rule]", "25.681360494163613`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.532010442589737*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.5 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532010442594728*^9}],

Cell[BoxData["\<\"Calculating for d=0.6 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532010521729109*^9}],

Cell[BoxData["\<\"Calculating for d=0.7 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320106536665363`*^9}],

Cell[BoxData["\<\"Calculating for d=0.8 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320108065349903`*^9}],

Cell[BoxData["\<\"Calculating for d=0.9 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532010994896847*^9}],

Cell[BoxData["\<\"Calculating for d=1. mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532011156436036*^9}],

Cell[BoxData["\<\"Calculating for d=1.1 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532011313207892*^9}],

Cell[BoxData["\<\"Calculating for d=1.2 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532011491912942*^9}],

Cell[BoxData["\<\"Calculating for d=1.3 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320117532870407`*^9}],

Cell[BoxData["\<\"Calculating for d=1.4 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320119672856894`*^9}],

Cell[BoxData["\<\"Calculating for d=1.5 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320122965057917`*^9}],

Cell[BoxData["\<\"Calculating for d=1.6 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532012552291356*^9}],

Cell[BoxData["\<\"Calculating for d=1.7 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532012827640955*^9}],

Cell[BoxData["\<\"Calculating for d=1.8 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532013042264771*^9}],

Cell[BoxData["\<\"Calculating for d=1.9 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532013273487084*^9}],

Cell[BoxData["\<\"Calculating for d=2. mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320135443471317`*^9}],

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532013955676537*^9}],

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532014355526075*^9}],

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532014556965282*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532014771909786*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=28 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320150101870747`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"41.10294112018688`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.8081120427664112`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.2666839288970344`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"61.424306184775666`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.315698029283178`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.031893451201166`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"81.43501317206228`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.512455804142183`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.1339498272914654`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"90.66988024964517`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5499239283225235`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.8699669936697303`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"102.46214830899801`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2976306158369681`"}], ",", 
       RowBox[{"b", "\[Rule]", "5.633740311970956`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"103.96779596583802`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3143877134611932`"}], ",", 
       RowBox[{"b", "\[Rule]", "5.745647742407978`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"105.44151322395435`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.782720103597441`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.247862087436897`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"106.05457812581291`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.383083690372546`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.0082327093345045`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"106.88281414476096`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.401858948823743`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.853310580230536`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"113.19245589915698`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3961496757811014`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.191644217884683`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"115.6406939673392`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.732418294717475`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.299028656598482`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"113.60973340894178`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5705382855605634`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.907905224066514`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"109.29488209980508`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.631152689900107`"}], ",", 
       RowBox[{"b", "\[Rule]", "13.135672253686213`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"102.33013445994872`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.490195193606935`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.321226557102117`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"99.27977639069549`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6092992232447694`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.458640258450957`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"103.31673001456873`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.296582212026209`"}], ",", 
       RowBox[{"b", "\[Rule]", "16.04835367909504`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"101.48894979768461`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "2.336629478877739`"}], ",", 
       RowBox[{"b", "\[Rule]", "19.558288112641577`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"83.95958127115566`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "0.1425016857621236`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.894589616259005`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"99.33725996485178`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1218749754140687`"}], ",", 
       RowBox[{"b", "\[Rule]", "18.29262001569522`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"98.81579706794045`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2038618234723613`"}], ",", 
       RowBox[{"b", "\[Rule]", "18.085389571393634`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"98.47420316777101`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1947669958517662`"}], ",", 
       RowBox[{"b", "\[Rule]", "18.98016226639438`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.5320154704321136`*^9}],

Cell[BoxData["\<\"Calculating for d=0.5 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320154704372883`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.53201560775884*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.6 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532015607762603*^9}],

Cell[BoxData["\<\"Calculating for d=0.7 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320156847207203`*^9}],

Cell[BoxData["\<\"Calculating for d=0.8 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53201577874144*^9}],

Cell[BoxData["\<\"Calculating for d=0.9 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320159253813753`*^9}],

Cell[BoxData["\<\"Calculating for d=1. mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53201606367857*^9}],

Cell[BoxData["\<\"Calculating for d=1.1 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53201616320446*^9}],

Cell[BoxData["\<\"Calculating for d=1.2 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320163379963303`*^9}],

Cell[BoxData["\<\"Calculating for d=1.3 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320164930624027`*^9}],

Cell[BoxData["\<\"Calculating for d=1.4 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532016707173315*^9}],

Cell[BoxData["\<\"Calculating for d=1.5 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532016884879445*^9}],

Cell[BoxData["\<\"Calculating for d=1.6 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532017166327558*^9}],

Cell[BoxData["\<\"Calculating for d=1.7 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320174172272387`*^9}],

Cell[BoxData["\<\"Calculating for d=1.8 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532017663911007*^9}],

Cell[BoxData["\<\"Calculating for d=1.9 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532017933759501*^9}],

Cell[BoxData["\<\"Calculating for d=2. mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532018137294545*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532018456028822*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532018456032578*^9}],

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532018697736229*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532019048156538*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(NMaximize \
:: cvmit\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.5320190481605186`*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320190481642237`*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532019427986805*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=28 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532019887419772*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"60.51650577602135`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6317876994097675`"}], ",", 
       RowBox[{"b", "\[Rule]", "1.4298257994939914`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"95.03301623650303`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.778151476980296`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.2694391100717404`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"132.7373841487066`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.429602440202627`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.140070696278109`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"150.3296918555634`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.349281730110075`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.03497450122962`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"167.64475253915168`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.0876219680933505`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.0266136573376254`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"176.81635499085064`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5068946539144548`"}], ",", 
       RowBox[{"b", "\[Rule]", "4.711189972359046`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"179.57520624071464`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.0960354087540367`"}], ",", 
       RowBox[{"b", "\[Rule]", "4.998318121951609`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"183.42942341723383`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.720377786855667`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.306947144729977`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"185.8096612301564`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6195753367383634`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.565546257963609`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"197.84407400046192`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5988412412805861`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.650106578993804`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"203.34931060381817`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.570165299829238`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.642738043830272`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"200.11176078592433`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6804339859803492`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.099698839249449`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"193.1653183472552`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2735574661585618`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.73661994875744`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"186.80311357710488`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3189133121313827`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.252053840086058`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"184.4443486836763`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3614199449706035`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.694047503913438`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"184.07686762649442`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2148500964368212`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.971432862040036`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"182.830779169582`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1523264235943196`"}], ",", 
       RowBox[{"b", "\[Rule]", "11.45001168103501`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"179.64214042442205`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.64845452395368`"}], ",", 
       RowBox[{"b", "\[Rule]", "11.846244908652114`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"179.0550507373129`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4595122088176193`"}], ",", 
       RowBox[{"b", "\[Rule]", "14.285361722441714`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"177.9581444045229`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5524589419407848`"}], ",", 
       RowBox[{"b", "\[Rule]", "16.2973068803`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"175.78690113292197`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "2.0612276182878104`"}], ",", 
       RowBox[{"b", "\[Rule]", "18.05909891146763`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.5320204562273827`*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.5 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320204562325163`*^9}],

Cell[BoxData["\<\"Calculating for d=0.6 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532020564765705*^9}],

Cell[BoxData["\<\"Calculating for d=0.7 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532020758188048*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.5320208697550583`*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.8 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320208697588797`*^9}],

Cell[BoxData["\<\"Calculating for d=0.9 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532021001897306*^9}],

Cell[BoxData["\<\"Calculating for d=1. mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320211705329323`*^9}],

Cell[BoxData["\<\"Calculating for d=1.1 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532021266627191*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532021437251903*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=1.2 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532021437255672*^9}],

Cell[BoxData["\<\"Calculating for d=1.3 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532021565716002*^9}],

Cell[BoxData["\<\"Calculating for d=1.4 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532021840478777*^9}],

Cell[BoxData["\<\"Calculating for d=1.5 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532022126125308*^9}],

Cell[BoxData["\<\"Calculating for d=1.6 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532022427154558*^9}],

Cell[BoxData["\<\"Calculating for d=1.7 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53202278875193*^9}],

Cell[BoxData["\<\"Calculating for d=1.8 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532023221032374*^9}],

Cell[BoxData["\<\"Calculating for d=1.9 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53202364461195*^9}],

Cell[BoxData["\<\"Calculating for d=2. mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532024036477412*^9}],

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532024447333952*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532024863513852*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(NMaximize \
:: cvmit\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.53202486351856*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532024863522602*^9}],

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532025232763476*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532025474952149*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=24 ohm, \
v=(15.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320258085727587`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"22.334895038433853`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6217055341996518`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.173366849255286`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"30.990688461211384`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3527338386892054`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.104330786989653`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"41.205425703057095`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.8521690727465725`"}], ",", 
       RowBox[{"b", "\[Rule]", "4.949291691071632`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"44.94041420956999`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.788079828137517`"}], ",", 
       RowBox[{"b", "\[Rule]", "5.504513632526887`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"49.818926447756226`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3531829457364226`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.054640165914582`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"49.75703293861551`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4294065504856754`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.141420513738984`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"50.336672645583285`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.419360426869123`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.639108212173804`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"50.27969204055656`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5597155062468375`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.194829606437569`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"50.11545972817536`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "2.0053341516564447`"}], ",", 
       RowBox[{"b", "\[Rule]", "13.042706138892646`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"52.60347058546626`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2139193938581336`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.77141987708361`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"54.189992184966314`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4641723448874606`"}], ",", 
       RowBox[{"b", "\[Rule]", "12.84220863925094`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"52.76132528573778`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.175596047837514`"}], ",", 
       RowBox[{"b", "\[Rule]", "12.622377731610761`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"48.66913303917458`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4871145154608094`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.780667492367114`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"48.82687416742102`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7799501613363988`"}], ",", 
       RowBox[{"b", "\[Rule]", "18.40028918644508`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"48.016379644061224`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.292302203646213`"}], ",", 
       RowBox[{"b", "\[Rule]", "16.40459387860976`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"47.542469877010106`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1781666583997241`"}], ",", 
       RowBox[{"b", "\[Rule]", "17.946407886963826`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"47.00853708285216`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.9304653909320084`"}], ",", 
       RowBox[{"b", "\[Rule]", "21.06146027462216`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"46.65780398100393`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3941022435069084`"}], ",", 
       RowBox[{"b", "\[Rule]", "22.414076732060522`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"45.714309382388414`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2269758515040818`"}], ",", 
       RowBox[{"b", "\[Rule]", "19.30620756958232`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"44.416936258811475`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "0.7147361012406594`"}], ",", 
       RowBox[{"b", "\[Rule]", "21.58104110694744`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"44.828336478000125`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1294182294436321`"}], ",", 
       RowBox[{"b", "\[Rule]", "25.762989635483336`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.532026050135754*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.5 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532026050139852*^9}],

Cell[BoxData["\<\"Calculating for d=0.6 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532026146165669*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.5320262610468693`*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.7 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532026261050725*^9}],

Cell[BoxData["\<\"Calculating for d=0.8 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320263793541727`*^9}],

Cell[BoxData["\<\"Calculating for d=0.9 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532026517103489*^9}],

Cell[BoxData["\<\"Calculating for d=1. mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532026699425543*^9}],

Cell[BoxData["\<\"Calculating for d=1.1 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532026843400703*^9}],

Cell[BoxData["\<\"Calculating for d=1.2 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532026996139041*^9}],

Cell[BoxData["\<\"Calculating for d=1.3 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320271079436703`*^9}],

Cell[BoxData["\<\"Calculating for d=1.4 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532027280297367*^9}],

Cell[BoxData["\<\"Calculating for d=1.5 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532027470893635*^9}],

Cell[BoxData["\<\"Calculating for d=1.6 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532027679817789*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532027964968004*^9}],

Cell[BoxData["\<\"Calculating for d=1.7 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532027964971881*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532028200536894*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(NMaximize \
:: cvmit\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.53202820054093*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=1.8 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532028200544786*^9}],

Cell[BoxData["\<\"Calculating for d=1.9 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532028562043787*^9}],

Cell[BoxData["\<\"Calculating for d=2. mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532028940356864*^9}],

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320292327229433`*^9}],

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532029628980268*^9}],

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532032299240315*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532032535117022*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=24 ohm, \
v=(20.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320329092400846`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"39.10192513409084`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2528268086514611`"}], ",", 
       RowBox[{"b", "\[Rule]", "1.8796609779461944`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"58.595268393856074`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4017735386310266`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.094071226165609`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"78.69501491037637`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3076761888381676`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.106045442862934`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"85.78409255681002`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3797682384193217`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.164598064723122`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"99.3702811292119`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2413109145152392`"}], ",", 
       RowBox[{"b", "\[Rule]", "4.736368294299169`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"100.90040514204979`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7685669101180284`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.233012303132978`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"102.61839225083185`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6282223716291722`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.677628482023726`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"103.2301899897883`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5677436059935006`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.980432750709603`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"103.90139061569023`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5252377154851295`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.207139990407748`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"110.33160517069511`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7620551516514036`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.020375296332727`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"113.11701233973355`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3620525847667058`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.537089982517774`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"110.77392434678202`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2385469022969926`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.38273230830528`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"107.1511871611429`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.296303262731545`"}], ",", 
       RowBox[{"b", "\[Rule]", "11.421189255246881`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"103.07045234747827`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2900537194791462`"}], ",", 
       RowBox[{"b", "\[Rule]", "12.902253906765422`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"101.7063098938339`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6566645206623416`"}], ",", 
       RowBox[{"b", "\[Rule]", "14.438433137919809`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"101.44651395821693`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.2568158279122794`"}], ",", 
       RowBox[{"b", "\[Rule]", "13.9122748033345`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"100.0333511519397`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.8025565371189436`"}], ",", 
       RowBox[{"b", "\[Rule]", "19.26091769733936`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"98.60900820835676`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7378604836280933`"}], ",", 
       RowBox[{"b", "\[Rule]", "18.459847474414186`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"81.22100316859192`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "0.14015440817247038`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.997216220401174`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"97.3765418616744`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3192557751467577`"}], ",", 
       RowBox[{"b", "\[Rule]", "16.770929651337998`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"96.60066498611562`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3039448592852267`"}], ",", 
       RowBox[{"b", "\[Rule]", "17.4044774738487`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.532033139419767*^9}],

Cell[BoxData["\<\"Calculating for d=0.5 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53203313942558*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532033234537299*^9}],

Cell[BoxData["\<\"Calculating for d=0.6 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320332345528593`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532033328799901*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.7 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320333288037767`*^9}],

Cell[BoxData["\<\"Calculating for d=0.8 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532033428217968*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"NMaximize", "::", "cvmit"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Failed to converge to the requested accuracy or precision \
within \[NoBreak]\\!\\(100\\)\[NoBreak] iterations. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/NMaximize\\\", ButtonNote -> \
\\\"NMaximize::cvmit\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532033577609527*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(NMaximize \
:: cvmit\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.531707250737571*^9, 3.5317081816550493`*^9, 
  3.531709196543458*^9, 3.531709691520874*^9, 3.5317100988962107`*^9, 
  3.53171232314461*^9, 3.531721468267434*^9, 3.531730557263322*^9, 
  3.531950145864275*^9, 3.5319877402606697`*^9, 3.532033577614173*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Calculating for d=0.9 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532033577618153*^9}],

Cell[BoxData["\<\"Calculating for d=1. mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532033745531289*^9}],

Cell[BoxData["\<\"Calculating for d=1.1 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532033881320897*^9}],

Cell[BoxData["\<\"Calculating for d=1.2 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532033992945854*^9}],

Cell[BoxData["\<\"Calculating for d=1.3 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532034158472028*^9}],

Cell[BoxData["\<\"Calculating for d=1.4 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532034355870973*^9}],

Cell[BoxData["\<\"Calculating for d=1.5 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53203450381504*^9}],

Cell[BoxData["\<\"Calculating for d=1.6 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532034678537599*^9}],

Cell[BoxData["\<\"Calculating for d=1.7 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320349844564037`*^9}],

Cell[BoxData["\<\"Calculating for d=1.8 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53203520739257*^9}],

Cell[BoxData["\<\"Calculating for d=1.9 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532035481613048*^9}],

Cell[BoxData["\<\"Calculating for d=2. mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.53203571849682*^9}],

Cell[BoxData["\<\"Calculating for d=2.1 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.5320360109556217`*^9}],

Cell[BoxData["\<\"Calculating for d=2.2 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532036393984969*^9}],

Cell[BoxData["\<\"Calculating for d=2.3 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532036694346826*^9}],

Cell[BoxData["\<\"Calculating for d=2.4 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532036905051322*^9}],

Cell[BoxData["\<\"Calculating for d=2.5 mm, r=24 ohm, \
v=(25.mm\\!\\(\\*SuperscriptBox[\\()\\), \\(3\\)]\\)\"\>"], "Print",
 CellChangeTimes->{3.5317068920155888`*^9, 3.5317072035881853`*^9, 
  3.5317080219967527`*^9, 3.531708836825247*^9, 3.5317094688051023`*^9, 
  3.531709986167589*^9, 3.5317122886977*^9, 3.531721316369472*^9, 
  3.5317303120326777`*^9, 3.5319501458718853`*^9, 3.531987740266015*^9, 
  3.532037191514529*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"60.116256309199386`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.541296078101849`"}], ",", 
       RowBox[{"b", "\[Rule]", "1.7509653996312906`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"89.15482209630773`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.6873244237695597`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.214705077857743`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"127.24679147791534`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.3076179025884405`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.0633590456818376`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"143.7717152709358`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.9860358700929361`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.3807226009185114`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"162.6349093582016`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1230954621769498`"}], ",", 
       RowBox[{"b", "\[Rule]", "3.005019179933404`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"170.6921623604361`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7051855040888402`"}], ",", 
       RowBox[{"b", "\[Rule]", "4.632188563176273`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"174.5554966413758`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4327457497419336`"}], ",", 
       RowBox[{"b", "\[Rule]", "4.676681183072121`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"178.17069511525253`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.43994919310017`"}], ",", 
       RowBox[{"b", "\[Rule]", "5.724845127470161`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"180.7599452048034`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5057895042566387`"}], ",", 
       RowBox[{"b", "\[Rule]", "6.629838270779613`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"191.273316718601`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1391025435617212`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.189331192890132`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"198.07463321563208`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5060916719647461`"}], ",", 
       RowBox[{"b", "\[Rule]", "7.650307131627816`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"195.30479383072603`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.376634987450997`"}], ",", 
       RowBox[{"b", "\[Rule]", "8.734936928070763`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"188.89151222521488`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.7197628592378291`"}], ",", 
       RowBox[{"b", "\[Rule]", "9.866393080345109`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"182.43129233729198`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.5321524015957073`"}], ",", 
       RowBox[{"b", "\[Rule]", "11.23043194577148`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"179.92638214814156`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.8963393031345614`"}], ",", 
       RowBox[{"b", "\[Rule]", "12.431206148152802`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"180.2134871806533`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.8291910082842828`"}], ",", 
       RowBox[{"b", "\[Rule]", "13.072914713490817`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"179.11209948831697`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.1785929685456076`"}], ",", 
       RowBox[{"b", "\[Rule]", "10.777453463888731`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"177.27135458063714`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.763608667841079`"}], ",", 
       RowBox[{"b", "\[Rule]", "15.114558590635259`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"175.33743733596134`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4460587707464576`"}], ",", 
       RowBox[{"b", "\[Rule]", "13.494861668026758`"}]}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"174.66434794727058`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "1.4735171224368913`"}], ",", 
       RowBox[{"b", "\[Rule]", "15.25114869296161`"}]}], "}"}]}], "}"}], ",", 
   
   RowBox[{"{", 
    RowBox[{"167.4888704354576`", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"a", "\[Rule]", "0.7895954654945072`"}], ",", 
       RowBox[{"b", "\[Rule]", "17.331745471795262`"}]}], "}"}]}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5317070984757137`*^9, 3.531707757130727*^9, 
  3.5317083837785177`*^9, 3.5317091965536137`*^9, 3.531709691537621*^9, 
  3.531711667597616*^9, 3.53171680889186*^9, 3.531723950042823*^9, 
  3.531733285245955*^9, 3.531950145868682*^9, 3.531991548544888*^9, 
  3.532037491663423*^9}]
}, Closed]],

Cell["\<\
This is a general overview of the data calculated:\
\>", "Text",
 CellChangeTimes->{{3.532767462652763*^9, 3.5327674733465233`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"g1", "=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Vrange", "=", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], "3"]}],
       ",", 
      RowBox[{"Drange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
        "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"dd", "/", "mm"}], ",", 
            RowBox[{"First", "@", 
             RowBox[{"ratiomax", "[", 
              RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"rr", ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
              "2", ",", "4", ",", "8", ",", "12", ",", "16", ",", "20"}], 
              "}"}], "ohms"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Joined", "\[Rule]", "True"}], ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
        RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
        RowBox[{"Axes", "\[Rule]", "False"}], ",", 
        RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "]"}], 
      "\[IndentingNewLine]", ",", 
      RowBox[{"{", 
       RowBox[{"vv", ",", "Vrange"}], "}"}]}], "]"}]}], "\[IndentingNewLine]",
    "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"g2", "=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Rrange", "=", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"2", ",", "4", ",", "8", ",", "12", ",", "16", ",", "20"}], 
         "}"}], "ohms"}]}], ",", 
      RowBox[{"Drange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
        "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"dd", "/", "mm"}], ",", 
            RowBox[{"First", "@", 
             RowBox[{"ratiomax", "[", 
              RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"vv", ",", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], 
             "3"]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Joined", "\[Rule]", "True"}], ",", 
        RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
        RowBox[{"Axes", "\[Rule]", "False"}], ",", 
        RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "]"}], 
      "\[IndentingNewLine]", ",", 
      RowBox[{"{", 
       RowBox[{"rr", ",", "Rrange"}], "}"}]}], "]"}]}], "\[IndentingNewLine]",
    "]"}]}]}], "Input",
 CellChangeTimes->{{3.531788440002974*^9, 3.5317885494151297`*^9}, {
  3.5317885995951138`*^9, 3.53178860042345*^9}, {3.531789044777679*^9, 
  3.531789270066319*^9}, {3.531985922827238*^9, 3.531985953988789*^9}, {
  3.531986036924321*^9, 3.531986062880549*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxN0wlMU3ccwPEClUFWHeHSAS4K4baO+wi077V97/WVYSgoImDAgFQOhSkg
MI1yyVSmwkDRMRvuCRsV5L4EBsyD4gEFBSap49qEITBADgcTLL/ul7y8vOT/
//6Tz8t/d0CER5AihUI59+FZf38cKXI4K/5Mzv5dqPXGDCP+mvEq+LIZKtiY
MeRUsEFjYJIV+sOt9fkLmX0sFjgv26FPxOsziQyPBu9brHdCP/amkQtXphPf
LDjL1s8gSQsN9dkGTFn/H+S7rbrSL7WZsv1zyL5nifNdfyCy8xaQnK+dc00P
sWS9RaTUmDZ2aIUl6y0h2Ntpt8gZlqy3gmzTELbyU1iy3nvEpV3cWd+Lynqr
iOS6zs+M96isR0EXf1N80k1joV0b6yloiMdJiRlts6+AjpOv7q31b+5XRDt8
kr0WqlDZeUrow1vntb/p3OxR0cxrZawD7ZvfUuSBVdqF22ID8FTTs83hv7ME
T2YHvY69yxE8Wyu7CWcdBngGvbKw21mNgmf8URd1myUUPMuqUmIvT7HAE/VL
Fa66ssFzqPSOnWEhGzyZESMDS1IOePJduZfyzmHgKepqU55DMfC8T/3Fpq6V
A578sZVTnroc8Hx6/KVCxzgbPAd/7DO5M88GT76akZewiQ2exYXVd1uEbPCc
HrXR0zrDBk+jh5q8Yww2eM4KU12izdjgaREelflV1F7wNG4I6K24bweemO0W
Qb4jAp4mWfRVhisLPNPMbVW+H+SAp0ecjeclJgaei8tFmF4NBp7F9GIt89cY
eI6a1NPK6Th4egYrJotiCPDcFrqa2aXDBc+WVP7pYhEBnpJKURL/EQ6eqlKn
5vIdOHhqOv891T2MgeevruePnSzBwNM+1DeElo6BJ4s+n6/44f9temYWSkcq
TDHw1Mca1Xj9HPA0icibyV/mgGfGZOJZdQdL8EQlzb65I47guXV7p4hhzAJP
FjULGShgg+eyOMwtpQIHT5uqucaoQRw8J6ZODNIYBHiSNWS6/QUCPAN3aNvF
FBPg6e9uWOekRIKnV5yDrySdBM/xZ+bk1TUueKq7Ka/EMbngmV4bKh3yJOT3
vZbRoqxEgKdo92zWjTQcPJVN3D87LcDBM/r4J6rN4Th4GhrV/Z5gi4OnyK8n
T4uCg2d/R3yI/SgGnkNhw9eqBdbg2WYWWRap7wye7frBf148wAZPJ9fX8ZLb
GHjGvtQgTb/ggmdGYYJ5uCEXPAX7dYXxUVzwXFmsNfgpmwueRzTdLUYauOBZ
130D2anAA8/e58lr17/lgedAVNuI6jwJnnbZnUudpiR4dtFo1n0MLnha9hBI
0ZDcc6LE/ubBHAI8kwbct6Q1yu+7TzlPxcOVAE+TyoRoP0LuaRtJrRlqkXsy
n24fZcbh4NlklB1U+sgGPE9Q3jgGCBngmf851V+jngOeNzm95ckxOHhOHlWq
ohiR4BnqXeJNpZPgmX+2IMNCQILnZYMrA8ZhJHgOOATd3RNIgqf+pxRGhETu
WR2iRk/xcwHPmMgXkv4HPPBsiu3V+XdI7vn2ca2mz3O5p3Tvux5xEhc8Ixav
Wh1054JnAYt3TzuBAM/cKbpp34v/eZZ6ZxaFEOB5eEI0Q6gT4Ll2RLdmTxGG
/gfxJU7j
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], 
        LineBox[{64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 
         79, 80, 81, 82, 83, 84}]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], 
        LineBox[{85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 
         100, 101, 102, 103, 104, 105}]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], 
        LineBox[{106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 
         118, 119, 120, 121, 122, 123, 124, 125, 126}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 64], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 65], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 66], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 67], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 68], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 69], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 70], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 71], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 72], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 73], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 74], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 75], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 76], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 77], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 78], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 79], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 80], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 81], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 82], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 83], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 84]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 85], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 86], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 87], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 88], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 89], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 90], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 91], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 92], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 93], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 94], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 95], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 96], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 97], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 98], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 99], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 100], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 101], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 102], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 103], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 104], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 105]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 106], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 107], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 108], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 109], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 110], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 111], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 112], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 113], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 114], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 115], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 116], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 117], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 118], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 119], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 120], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 121], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 122], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 123], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 124], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 125], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 126]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 10.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {8.163333758780645, 52.73846972752965}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF1HkwnGccwHGy2CTiiKQik1LboHUkqTOVyL6v+1h0j/fdd49sIiruFB0G
NUhpJTZxDKXbGFcJ66gMRYkKcoqg2pUo4kxJ0ggbV0pFWrJ++5t55/3neb7P
O59556H4hjHPblNQUEj4/9l4v5txhJzv/NfJWkPUcnOeIKrWCfhxijXqvznT
CPZnjspc6TH0yg8b8xwhzbubIhJHtLd7Y2aQ7F9oa69rXdF3PSnS3rRL3SvM
Q7b+FVIl5pcY0Txl/QUk9eGc06zQS7Z/ERkLOGqXWe8tO28Z2U1OFGYI6bLe
P8i5aH9SOJch660g1+2m4k/+wZD1/kVeBGhfno9gyHpryNCka5bbfoast47E
5ATlD2gyZD0F1JHoD2I5MNCezfUKKKPVt9uEttVXROO8cv2jDbb2b0MHI+0j
3VW2ziOht5IxiXTvVk8JTY9w+blveet7x5EyD7pz/LA5eGqV9AVO9JwAT5ub
Ie21xk7gWZE9nS8RuoPnBzeVX1bMe4Nnwdin1XMCOng2I8LdJmQGeJ5fGX/z
60EGeE4+7gw0+IQJniFnM8TMqyzwXOfprWlcwMCzpU6/qjYZA8+7nbePFGhi
4HlH5bf4RoIFnpGvkrQH+SzwVNTIyo6NY4FnWdgxPdtvWeBpVmBaXOrLAk8r
qrt3ihULPD1sq5+bHWGBZ+V9seR3WxZ4hiY3XR5ZsALP+Vt1l0R+DuBZaSmN
mhyggWcOmvfZTj86eGLf0+gpY0zwvCRc7RqlsMBzSDu87XEEJv8/Rc0d301i
4JlXF8+P8sTB8yCXa2H0kA2exeNDe4IjCPD0JNew7zIJ8IwoahSI0tngqROT
YXF1EQfPsqN8u/JaHDy7E89FP23FwdPNdlXSko2D50vKqK5zKg6eAwLP1SI3
HDzXyPuqL5rj4Kl4zTqLZICDZxX/m6/7z1DBs6mqMtsn0AU8u0yX94aa08ET
uX3l/EAYEzwHMePClh8x8OxtsjyVE46DZ9lC0IdR9mzwLPpiqn+1mQ2ehka6
oqntBHialWDToQkc8HR78GX5kisXPE/4Vi09lXLAMzj9Yt5hAw54XlNrmejk
EeA5Uq8rjllig6dbXf5ikBYBnu8fUkX3rLDB07jn3oQgiQ2e8T1NB8pt2eCZ
+cyzsf8wGzzJIcFUuicbPCPTeg9o6tiDp+FM7gTm4w6e/ubMUXEzAzwdRqcX
/JpZ4PnIsrXLpwsHT6tuhslpCwI8VQWP3n7eRICn3t/2PL+3BHh6pdeYFttw
wBN1JFHqS7ngOdkyyVlO5IGna+sDjK/PA8/ZXOX1RBcueAbWZ5af4nLA8/WK
1CZlhQDP1pobDEKdA55oYX+euTIBnuzeamrME7mn5kjf6UwlAjzLydT5CRMC
PL9adzo0okKAp+GMxs4d+XJPrWzRDTyVBp4z+m67cruZ4Dn87J5PQCEGniUh
Nl5ODQR4JgnT93e4cMCTOkqSdNzngOcOtTMftZG54BkgXl268zEXPM1DEYFS
Ow88S814DTWxfPA8Th8PF6nzwZNE2T7mqsOT359pSpSKfVzwTLO37VRr4IDn
T7kWWhnX5Z7kuOHYhCIOeIqzKtJNZuWetDb/rGap/P6sfHGB+0Yi93xP27mN
Wkug/wF6Z1cZ
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], 
        LineBox[{64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 
         79, 80, 81, 82, 83, 84}]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], 
        LineBox[{85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 
         100, 101, 102, 103, 104, 105}]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], 
        LineBox[{106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 
         118, 119, 120, 121, 122, 123, 124, 125, 126}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 64], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 65], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 66], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 67], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 68], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 69], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 70], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 71], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 72], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 73], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 74], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 75], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 76], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 77], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 78], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 79], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 80], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 81], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 82], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 83], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 84]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 85], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 86], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 87], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 88], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 89], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 90], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 91], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 92], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 93], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 94], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 95], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 96], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 97], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 98], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 99], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 100], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 101], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 102], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 103], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 104], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 105]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 106], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 107], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 108], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 109], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 110], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 111], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 112], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 113], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 114], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 115], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 116], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 117], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 118], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 119], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 120], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 121], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 122], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 123], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 124], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 125], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 126]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 20.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {10.846411832330444`, 109.90128835493175`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF0AlMk2ccx/GOcgYLTBDnmMYqbHgwRDkiUN63OJXJKaVytMWX94CJYehU
VKZgnATnZIgM5dCIlLDNAxw6I5dEhoGRAt7jyigiHTBCLCDMysqm1l//yZMn
T/I83yf5COnUSM6Ex+Nl/L9e729HTagff6XqznUj172ZQYKeP3XuO4Efmfhm
NIRFjoK+fn8TWVz0ekaIZtt2qY1/KNmhej1jxCHuZqVwfST5tvecCK4sr1dd
jTTc1xJzTnE3BkKlhv4kce32XcsgSbTh/RSRJc31CquNMfw3TcTrRKta4mWG
3j+Exr8y4o61wtB7SVyS6baZHVYYeq8Iu+Bo9USa3NCbJe4taSqOSJcbenqi
Y2ZmKGtObujxSGetaolSqiDb39znkWvHOvLOlr3rv0euDB2oD29UGN6bkJuf
ZDdF/PLuPz7ZVX5MqelWGHqmZDPPV5g5+u6sJjKd3LKJIW94RrfxJ+qqP4On
6e4duVfMI+D5UdEQ7TwogeeIOONko3k0PDfus589nR4LzxX69g/db8vgWfFw
5BnrqoCnkw0ltlwQD0+fvZn+jicpeMp8Hg9Z/k7Bs7tnA3ltWwI8A7rplDV6
Cp7Sw74HSs5Q8CxrLoi1K6Xg+aOr9+L93RQ8K3LKPTdOUPBcarJmMGOYgmcd
53k55xEFz0cJXbkBtRQ8nfzKHuwcpOA5uurOC65HDM/lWdZx+VuC4JnfX/Iz
MSmBZ9axYeWANBaeag9lRuPxeHiqbkVp1zlS8BwPcbYPbqbguS5vl9LtIQXP
yYmL+u6+BHj2yP9oGQtj4Ok5Ws8u+JiF5xL70btWH7Dw3L6nqeH97xl4iryJ
zER7Bp78hohk3RQNzynH3VqhKQNPv859oj8XMfA8YjsrEqloeJZZZEs+vUfD
U7BQJtPX0/Dsas1+1XaHhqfo/IikUEvCs2shoTarioCnLOPEVs/cGHieae6t
ttqlgOdBpkFV/XkCPL88kCcufJQAT842PCvSjYbn3hS7RrsABp4188xn464z
8DxYrbIROXPwnJAFFj54wsEzZ9NqE5dTHDz5WhMHr0UcPKO0eusVaSw8BYKl
FtOBLDzlDnl9qRIWnjvCW3fmill4OmnsNF/MZ+EpZm+JhU8ZeFockQWZPWPg
GeXX761sZeBZd6mkmL0cBE/3J7Z8XYgEnt6i+orxdBk8iw9VpSZ5UvDcblVQ
caGEhueel6tWL39Kw1PgWzm014yFp0t63+2Ugyw855b+Oi7rZOEZO+gb2EEk
wnNZX8x4qU0SPI/IPXcVViXCszaofP88s0SjZ2NyfwvDwfOcalOphx8HT31e
VY7Ai4PnqWMdW54v5+A5mS8YFrYZPWtbxobP32Thmen1SdTRGyw8c+T+Hhol
C0+q4EWk+4VgeNbMNGz0OBoFz1Ona9b4/KuAp4THXxbokABP8T5/4cKjDDyF
L/6y3bqHheffUpd2Pz4HT/XA+s3OHAfP+pthguNnOHj+1tx7nA5JgudU3emZ
Np3Rs7fh/uNvv0mCZ+vgiZgkndFTc7/vbJhbIjzbHKOLrkwbPV2Tgy1W6oye
J05WNAWPGD0LU74O77zIwTPtp6RUHzkHz7WLr07/sIWDZ0Dj84BMe478D3PR
Wf0=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], 
        LineBox[{64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 
         79, 80, 81, 82, 83, 84}]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], 
        LineBox[{85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 
         100, 101, 102, 103, 104, 105}]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], 
        LineBox[{106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 
         118, 119, 120, 121, 122, 123, 124, 125, 126}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 64], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 65], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 66], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 67], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 68], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 69], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 70], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 71], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 72], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 73], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 74], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 75], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 76], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 77], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 78], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 79], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 80], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 81], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 82], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 83], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 84]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 85], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 86], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 87], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 88], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 89], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 90], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 91], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 92], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 93], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 94], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 95], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 96], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 97], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 98], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 99], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 100], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 101], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 102], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 103], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 104], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 105]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 106], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 107], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 108], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 109], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 110], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 111], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 112], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 113], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 114], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 115], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 116], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 117], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 118], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 119], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 120], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 121], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 122], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 123], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 124], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 125], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 126]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 50.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {14.780958546655654`, 191.8372762552168}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5317885412988443`*^9, 3.5317885501076183`*^9}, 
   3.531788601308609*^9, {3.531789048214293*^9, 3.531789132388371*^9}, {
   3.531789172136693*^9, 3.531789271147477*^9}, {3.531985935878749*^9, 
   3.53198595534121*^9}, {3.531986042047785*^9, 3.531986064040893*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJw90HtIU1EcwPGlpYLTrEQoQRpa0SQCwyK9es7Nt+vq7nTN3K7Bwql/hM4k
QzPFR5Fp62WiZVj6RySKS3uw7KHNirwO81EuiKbmIpOcy6ZNyVLvfj84HA6c
8z3wESizJelOPB4P/V8r+9qYkKK2pLAxaTvetzoT6Jh3iVuUXYhVq2NGuZn+
XcfLgnB93cp8R9Z3rIqw78cGdmWm0cRkJrWgC8VrPQuqqLaUTtkI7v4sKrM9
1d30D+f6v1CVh69pr084934OUQOlv/vHEfefDTXmEHd2p5BcbwG17uKbUxZJ
rvcHRc5YEk/OklxvEXluud0tPkdyvSUUr2f7dCOY6/1FwzXbWsKWMNfj4YXX
ToZBPon7V+/zcJZEPSzkO/rr8LfYzw+WjY73Trg3tVxme4i5/5zx27pin4I+
R289vq5pJ5P1jrMJuTZEfVVod4Cne/BZaaggGDyTR2tcZppDwNPZGheIhiLA
89pj0dK8NgY8Xz7he1LZ8eDZck/etFN0GDwvjMxE/qykwPNLxgHicmcCeG5y
La7UVIrB80S+yjnnKA2eOmKySDFIg+ePDJ8qq5oGz0/jMVdjt9Lgebomq+Gj
Fw2eEbLhrKRDNHjSz5SsUESDZyF1Q5UfQIOnMY/Mi3OhwfNVWfKQxZsGz0vq
6I4Bmxg8TSO5rFGzBzyVm+duXfQIBU/XakbZ8T4aPPUb+6WeBAWeZ9IftQkO
SsBT1NbcxbZKwHPZN7VzjJKCZ/vzXrfYJBl4Vkg1wQm6FPBMs4cFvkmTg6eZ
aBN3uzPgeV9uP7KhiAFPL5HMZD2lAM8Bv556cYECPA3z85MVywrwDJhl/Zqk
DHgGTRuu1N5lwFNIjXUlvmDAM+bD+R6xlgHP0ebyJrORAU89L0RQPMXgf+Yy
sv0=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 20.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {8.163333758780645, 119.66400938667636`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7I8bTWide0bZwRgMHtsLyJguCPhu6JAGBs/s
7Y7q7nRSsHSYNRMEXtof3HLJzUbK1uHcGRB4Y59618BMdpuDA8S8D/YNKV5C
Jj8doOo/2m/Y2lbR9c4Rav5ne4e47nn/fJyg+r/Y31u7wkx1qRPUvm/2dvlP
bv184Aw174d9gI9756I6F6h5P+3XnT3M9sXBBWreb/t9LGtMdh50hpr3xz7g
2e+iUGlnqHn/7M/n3GA8+twJah6Dw+051zRWfHVyOAtWz+AQIKAWPm+vE9R8
RoeVS7etPzAP5h4mhw9PTWREq52g9jE7qJ0Q8Uy3hZnH4vBpXrdXqRaM/8B+
mVeAa+1tQ3h4Ci2+kPHwrC08PM0OZR/YqOkCD8+Vk5/NvdzlCQ9P+UOsb1d+
8oOH57z7FmvexwbAw3OnfZegFnsgPDwbfj74u0c5EB6ej+6cyFAxCIKHZ3Zq
/4qgpcHw8PwXJfeHvz0EHp67Nyms3tgcAg/PYyeO6M8TCIGH51G287XbwoPh
4VnysUnsZnQwPDwZ+SdNrqwOhofnsnwrOcvWYHh46szTXrgkKRgeniZ2nn5t
JsHw8PSyXPNSRz8YHp6rTq64fNEyGB6e9dK67fZPzeDhGX6K+dPuTS7w8GQp
zOxfwxYAD0+ZmU+TVB4Hw8PzpWNdz362cHh4upYK/5lUFQkPT81/Z6X090XD
w3PZ5ZdPUjRi4eEpzZfgyCEaBw9P85J6G7GeBHh4RptffcpxMgEenjdvOTts
CEuEh6fdzaRcg38J8PAMrbWqmD0tAR6ei45MjRRYkAAPz+UaZrLlNxMQ4dm7
xMT1UwI8PBWYDB7XvUiAh+fuVJPVvVcS4OF5JfFGv92uBHh4SlsvupT9OMEB
AHW5qrU=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 20.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {9.899639295437181, 138.70901501896768`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]},
    ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7A3ySqZ4l+g5GIPBY3v13UlXN+8zc0gDg2f2
LqasaYst7R1mzQSBl/Ya03X/2fo4Opw7AwJv7Cdom3JMuu3sADHvg31QpUlo
p50LVP1H+x+/lrnIbHeBmv/ZfqXuSlHthy5Q/V/sn2rs4tmo6wq175t9aAZT
y7pyN6h5P+z5sv5NOSvlDjXvp/2B7oCylevcoOb9tr+yZV1zwElXqHl/7Dkf
WO/fKAEz75+9iM3bd5ceu0DNY3A45FOfXrjKxeEsWD2Dg3lWdCbPRJh7GR0c
db8uZqpzgepncpiy9MGTzZow9zM7KLnsEfC8CfMvi4NG/qKPi3/B+A/sc5p3
9Nz9bAIPz0+HN3XPSHGCh+cq4w+lj657w8NzqsNsf66UAHh4hkz3Dmi7HwQP
z+6uX6fuKQbDw/OWWMH+O4Uh8PDsnLHz4JRHIfDwnL2pNrrUJxQensqRkUZq
V8Pg4bnwwS3hrMJweHj6sK8LOxYUDg/PwgXbYmf0hcHDU6Ki32jpl1B4eC4z
j7ZZvjEUHp5n6nPLn+8NhYenh+Wvy7snh8LD863iPVnXzlB4eF6P9fm1wCMU
Hp5/2MXXdBiGwsOTcb3pJGaVUHh4vtI++DX1liM8PJVbuaMme3nAw3Py/dkr
7T8Hw8OzteXF4oehkfDwfGC4uG5/Rxw8PM/sCPloLJYAD893PirC3kcS4OFp
PLFgse7lBHh4fv608N/NO4nw8LwVc/34G79keHiavNqTIqqWAg9POeFXRzkl
UuDhGV98aK9gXzI8PG3N7OvThJPh4cm8NyDr15ckeHh+ESv8qMiSDA9P6/Ol
tvckk+Hh2cD/x9b2TBI8PBextwfrXUiChyeveHT0vz1J8PC8caL996mDSQ4A
itauaQ==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 20.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {15.227139128887842`, 161.19011151215966`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJw90HtIU1EcB/C16ZboUpZlYUKGs8QHiG9kO+dKmzcQNnXeLZeW3k2cb0LN
QYlKRgSpQ9JlSKWRlaAwe2xY2B9GklOiLpVppaUzScL5Ih9YqXe/LxzOP7/z
Ped8AnNL0/RcDoeD/q/tfTeTqHm+7qIoPhJH7eQHwsyA9u50As7biQMJ/YZ7
JMcJ3HZzO3OIcGtFn+8l4VH7dubRur1QcaVPhnf7FlD0k+Xn5eMydt6Jfv0u
HveSyNn+JUQ+I01x9XL2/DKiDx2MvfBQzt63is6mim2JPJLt+4PUxngtYyLZ
vjU0+zaUbPibzPZtIJGCv2GUJrN9m8hkLZj8muHq20KMVfKSz5OzfRzcE7jY
2tIkwyM78xzMP5HqXZnneu8eXFEk8BgokbHnuVgcbJuojZGx9/FwT/b7jgMc
13/d8NirGkPczEns8uzWXq5lcqTgae1+1HwuXw6eb0JXfYsileCJBttqPpam
geeYKuR2f4cKPEetUdk3yjLA8/6S4VgFQYHnnZIZZt1Ggac4OMA8s1cNnmGd
KkdRtQY8yeHzXSvJp8FTktu9MrugAc+Chqu3IoI04Nkr7J8aylSD55fHAQ+q
VijwJC3tywaRGjyPhHvi/WsUeIaMvJ7KqqPA89KI1b8rgQLPpp8pT5kICjwF
hQVSZQoFnpL2uXSzE4PnJz806d6rBE9t9bXU6EYNeLYMjls8yrLA00i/sFtO
5YBnSZWJMDM54Kn3VtSnheeCZ3mxz4CPlAZPmxd/M7OPBk+jxb5PEqQHz0Vt
kvndBz14XpeHccVNevDkObm+MYf14KlybnmGVOrAUyg8KlhN0oHnGV/TRGm6
DjwNiqHCRkIHnv4OH0e+SAeehM5KBH6nwVNQoyXdp2nwVCV+i+0covE/032i
Mw==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 20.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {17.219032079662224`, 174.71320736847662`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJw90G1IU1EYB/CpWyWpCQUFkjhFSlfUmi7wZedoereFfZhb3m2ajnl3laIM
tkglmJZUX6yFKOYyjUzDQguW+V5gIPgye7ERoss5lVUyVpa5jK3U6/OHw+HA
Of+H8+NqS7J1gSwWC/1f6/tmZpH9nPN2Fy3Ago040VC8/pk+OgXTG1lEb6KL
XTcV6bjh7nq+oOQsR8VkYwa2jq1nCZV+2i2JixTjzT4PqnlUybsQK2buf0e0
POJ+hUHM9C+jtdXumDazmHn/E2n2yI7O94mZeSuo530d2h8gZfpW0cd3Vf7a
G1Kmz4umDEPzwb8kTN8aEppHvaNxEqbvLxoPCRHYUrf6fIj/gUCtdoLpY+Fv
7cfrc5oJPL5xn4WvTck4pv6t/wVg9XPpjuwsgnkfiA9aKi/lE5nMvCCcqGe/
tL/OZPrYWDSxd0FUtnWeRYZqa0T4vjTwjF2qcyg0UvCk+dn2xz0y8Ey3Ly5T
PXLwtAkGRjQjp8EzYUwWX3CMBM+dZ2z+wm4SPCO/pqkpPwmep2518B4IleCJ
TwRxLS0q8Jzrm1OuGNXgKR4YVeRGqcHTXcfxGQkVeBZbTG35KiV4/vZ6hNe9
JHgOdAzKyDAleOKmSTOfQ4JnjvWpqNSZA57hM28LTGwSPNu2i3444knwLPdl
HJ7ZRoJnX7u5gXoiAc8jtl1Bf7Lk4ClM7W91l+eCZ8OVzpKiBA14FgTXtjaZ
teCp9/IOxcxpwTM0qWPBwKHAM7Z8evB8GQWe/qgX7twJCjxVzqR0K6LBM3pa
6W4OKwLPiryEi/WdNHj2Slouh3Bo8FS8Ovt5uFAHnvfGiGZ+sg48fXc6q0MT
deBpqrKe9MTowHO5JtTFHaHAs3d4ydXYRYGnMfGA4qqFAs/qvBT+4kMK/wMx
Y65E
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 50.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {19.40122317602424, 184.4571449027581}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJw90HtIk1EcxvE3NzArM9Q0i6BZXhJXapGJuXOWuk1XltftnVrezoaaLJAy
MYo0SkdazEuzKBNFVJphMEtrLSvxghaiJrmwhWCZw5S8tbKL+vp74OX955zv
gQ8nRRlFrCiKQv+/5f/qTEjvfpdouw/iAysbQ1nUt4CU+0FYvrJxVOPCPu3Q
FozvVC5vAmmCh5qv5oTit73LMyNzGktHuYvwam8aZdCNNJsrYs7PoJqLtaU+
chHT/4FUu4tHPDJFzP1ZNHKYPPJOFTHvzSPXjVSQcjCM6S2ilvQt3Gunwpne
T5STPTz4oTOM6f1C+gtD25dG13q/0feep46yfiHT+4NM+xYGeguETI/CysUS
v7hIIe5bOU/hWn7YY6crAqa/DldPcfe+HxYw962wp5Yuq0sXMO+xcMJk04zA
XsD02Phv0o4n3nUheM3TzWy3weYeHzztSzUvYovE4GneJdpU0RsFnsavnUmK
qhjwrMk8dDxEJwHPfFWJS7tACp68UdZAe7cUPG1skz0M1jR4Kuotcx2eNHj6
nkGJ7Jcy8Kz1lumacuPBM/Ck6axmczx4sjjrPwm3ycCzo5jNaXCmwbOYH9Bl
q5OCp7bCz/5mmxQ8rfOMuZceSMGzXt1Q4jUVB55ig1zdOh0Nno2T1+mlAQl4
bnUKNfCaJeCZVD4Xtb9KDJ6tC/pQ3/wY8LylbvXxX0oEz2iK5XrUMRk8+eeO
cJzzU8GTM/fFLjI7DTwnY936AlkEPE2fA4R7CAHP5y0RtoUVBDxfvzEWphxT
gOfsM/VCj0UBnkZ9/1BRgQI8u8ZUUoVFDp7j/R9vR3Dl4NnjJKl8OE/A0zND
bO1lIeCpulH3SjxBwFOTlXfiXTUBz/P1CqV/AgFPv53a+bJwAp48wzTvsgPB
/wBrqahE
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 50.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {20.787802076700917`, 191.8372762552168}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5317885412988443`*^9, 3.5317885501076183`*^9}, 
   3.531788601308609*^9, {3.531789048214293*^9, 3.531789132388371*^9}, {
   3.531789172136693*^9, 3.531789271147477*^9}, {3.531985935878749*^9, 
   3.53198595534121*^9}, {3.531986042047785*^9, 3.531986064708846*^9}}]
}, Open  ]],

Cell["\<\
And here it\[CloseCurlyQuote]s done for normalised force instead:\
\>", "Text",
 CellChangeTimes->{{3.532767479964542*^9, 3.532767489598234*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"g1", "=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Vrange", "=", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], "3"]}],
       ",", 
      RowBox[{"Drange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
        "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"dd", "/", "mm"}], ",", 
            RowBox[{
             RowBox[{"First", "@", 
              RowBox[{"ratiomax", "[", 
               RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}], "/", 
             RowBox[{"CurrentFromDiam", "[", "dd", "]"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"rr", ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
              "2", ",", "4", ",", "8", ",", "12", ",", "16", ",", "20"}], 
              "}"}], "ohms"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Joined", "\[Rule]", "True"}], ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
        RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
        RowBox[{"Axes", "\[Rule]", "False"}], ",", 
        RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "]"}], 
      "\[IndentingNewLine]", ",", 
      RowBox[{"{", 
       RowBox[{"vv", ",", "Vrange"}], "}"}]}], "]"}]}], "\[IndentingNewLine]",
    "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"g2", "=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Rrange", "=", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"2", ",", "4", ",", "8", ",", "12", ",", "16", ",", "20"}], 
         "}"}], "ohms"}]}], ",", 
      RowBox[{"Drange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
        "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"dd", "/", "mm"}], ",", 
            RowBox[{
             RowBox[{"First", "@", 
              RowBox[{"ratiomax", "[", 
               RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}], "/", 
             RowBox[{"CurrentFromDiam", "[", "dd", "]"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"vv", ",", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], 
             "3"]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Joined", "\[Rule]", "True"}], ",", 
        RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
        RowBox[{"PlotMarkers", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
        RowBox[{"Axes", "\[Rule]", "False"}], ",", 
        RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "]"}], 
      "\[IndentingNewLine]", ",", 
      RowBox[{"{", 
       RowBox[{"rr", ",", "Rrange"}], "}"}]}], "]"}]}], "\[IndentingNewLine]",
    "]"}]}]}], "Input",
 CellChangeTimes->{{3.531788440002974*^9, 3.5317885494151297`*^9}, {
  3.5317885995951138`*^9, 3.53178860042345*^9}, {3.531789044777679*^9, 
  3.531789270066319*^9}, {3.531792649101386*^9, 3.531792659909355*^9}, {
  3.531987107137422*^9, 3.531987115792983*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF03kwnGccwPF3l7XuY7ssUyRaEkWrSsfd501UNKrITiZ1TMbNdEjEOaJC
ohm1qWIiTREVx9QRVTNF3FpH6gjKIBiVda21LBZrLeto2fXb38wzzz/P831m
PvO+BoGR9BAihmHJ/6+TXTyzSKXJuCm9QAa3PJ0FdC8rJHq9VAYPPZ0l9JD6
XnpzFgHPzzsZDuKlvmmqNyfgQwMnw0Wkuetlbx0xXNzjoWTSrB9We4jE5zfR
CLE0e9ZAhMT9bcSefseRSBMi8X0+yu0JfLg2KUDi9wSoVv21g8hpB4l7QjS6
89v3EwXbkt4eujQWu3PBfkvSEyGlVvOZdD5P0jtACYNuONNyQ9I7QheVrA1J
OWuSHoZPur2vXPPLKho8PY/h9JA7c3nzHEmfgE/9PRrJsFuW3CfiEyZO7UMD
S5L3ZPAU62B6cyVL0pPFq87del4Ut4jOPFlpivYBvWTwdGOqOI4aK4Lnq0wb
VvY4GTwzzbwPH3aTwPN8eOh2YI4MeCoKLNwiws78N1EnZdwjxxDDzzyr+g4q
gtwPwXNhFDP9rHEfPLsVlAmJfkLw7BY1diT4CMAzKI19xegJX+rp1HXkp7cN
ntWJC/7cuk3wbE5x8bkVxwPP9NREdlT9Onh+mbdr0BnLBc/a67b6ya9WwNPi
Ju1zRWsOeNZGBdTPpbHBU2ua+EORyRJ4Jmm5h0W4UsCzwuHDJI0sdfBM0ql/
WlWoAp7t+Xp+XrMK4FmcZKi5dJkMnsyiLfZRhyx42tzoe6paRgRP4cHsQG8D
hp95vi28HVDufQSeVBOSAyFDBJ7yjtmqjpZ74Bmf6Rk0prQLng6FPWRN6g54
Tt632dzJ2AbPg6Fn1i3mW+BZbNowF9PKA8+/2mr8U+02wHNYt1U2M34NPMNy
l7iPU1fBk+lhz7+QzwHPIP0me4WAZfAMvvbzOO93KnhONLaWDE5RwNOu5+PC
RGd18LTVHH49vaAEngKyp9qf4QrgWSeX8iLoGznwPFfhG6xNkgXPhq7jT3Xl
ieDp0GxVs+WA4Wee2nWRV0R3DsFTp7BD0WVgHzwf5dRgBoFC8Cwv0fbQvycA
z9sbc4GsP/jgmTH5Lqvrg23wvPqye3Q8fhM8GaqON10QDzxDvnOLqx5eB8+P
TBvrBRtc8DS2eMYN1VoFz/1/ro04p3DAc95Lrm/vKxp4Jh38a5b1LRU8owuK
Ar4I0gDPvt5DM+cJVfAsbHOXe0RVAs9cI+Z4SwsZPMmp4fN39UjgaUQXMZTZ
Us9LOqU3fiIQwDOetvH15Yoj8DSh2mbSJ0Xg+WCdysvw3QPPZMWI7n7/XfAU
tHWFzzB2wLOSFm2+uyz1FLVXu3PoW+C5YnW++mKM9H9PGwyVLyLzwLNHfs5O
KLcGnjr7iwwNNS54rld2EtmWK+DpPWyr4yvUBk9XNWa5cbYmeDJyPWlRLyng
iUR8nzhVdfD0epMYk39VGTyPZ8rLCf3y4GlMxOgqu1JP3faGAIsfZcBzepyT
EX1X6pn3POZXn6pj8HwQk0vt6DwAz4EZhf52i33wbCkZK9ZWF4JnmUuK6WOK
ADxj2+7j7Ql86ffpqjWIz0s9tXx8WVOam+D5gksbWQ2XejZS3ayCn6yD56LM
IoOSIPVkLWQueX3CQf8B2gV5WA==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], 
        LineBox[{64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 
         79, 80, 81, 82, 83, 84}]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], 
        LineBox[{85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 
         100, 101, 102, 103, 104, 105}]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], 
        LineBox[{106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 
         118, 119, 120, 121, 122, 123, 124, 125, 126}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 64], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 65], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 66], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 67], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 68], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 69], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 70], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 71], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 72], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 73], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 74], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 75], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 76], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 77], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 78], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 79], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 80], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 81], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 82], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 83], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 84]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 85], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 86], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 87], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 88], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 89], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 90], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 91], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 92], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 93], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 94], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 95], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 96], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 97], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 98], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 99], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 100], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 101], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 102], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 103], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 104], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 105]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 106], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 107], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 108], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 109], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 110], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 111], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 112], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 113], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 114], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 115], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 116], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 117], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 118], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 119], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 120], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 121], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 122], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 123], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 124], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 125], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 126]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 1.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.6393588174623707, 6.242535013345411}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF03lM02cYB/Afbe1BoZZWKrQCBTymEuZRBKfy/iQBzBgI3aQIqFAQLBEk
Ax33xDGtgzFOkUNd5kCGm8jhioMoKGwgiKgQVpRjVrASplQrh21xA9qHb/Lm
/ed5v2/eT/Laio8JDxMwDEv7f83vixlBRdP7z11hGONbF6JAR9XFMdxWBh6x
kDGUfIdft9uDhpcUz+climnGN7tUmuLdXfOZQL/alvhea6Hji32TyKEzWvqO
S9fPq9CqzR513yfQ9P1vUUKb07ZTVyj682p04S6xNBgj6++bQnZbPe6NepL0
fTOotKU9FU0Q9H2zqECaEJvDI+j7NMicXsbpHcH0fVpE2DsoWVHzAS32zSG5
a+GM4ncd0j8YD39rESpSa9D9hXkMH8z4os3N7T1a7DfCv+pKrCgpn9GfJ+DC
qCP9EZ9Mo8X7iHiWOiEihTql7yPh1kOXJTvq1MjgGdKYFbszzhw8G2/I+5pq
zcFTZVZ5RMhngyc77VOHh1w2eP4g8Gvy4bPAs8Fqtty5nQmeHU/aBTmVDPA8
4886e+hLOngSCX+05DKNwVP0xFnkz6GCZ5apcPiXMDJ4/uZ+k/GunwSezN38
6noJETx3ntU9GJkwwg2eXdsP1CcPYLjBs3dAlrQp+wN4ht87bdF/WQeeIarR
vfbDGvDcnuQjYAjeg+dzhZ+i8dYMePq/frCq+fg0eM5gV9ZRN1qC502uvVT1
Fxc88+VtztESLnhqTqpl2gwL8DwRnaeq4HHAk3mf8jfvIhs8TcuHLsgiWeBJ
GsoSjzcwwfNkSFXVrW8Z4DkZZhNVMkcHz++OBRyvz6WBZ5onKTi/lgKe1UZr
298+WgaeuUEvUzoDSOC5wc3XZ+IZATxHClCrtNQIN3juGGsNrQjFcINn98GV
1J6pOfB8vYuVbWqsA08xO1b4sb8GPB0dCo+Odc6C51qPLNE+ZAeeV6OxGhdf
Pni6VAXWnG+wBk+7R1aj2QU88LwXFly+xcECPC8/KPLu1piDp+fTSR6TtgI8
74qbXYO2sMBTNEBVfJ7OBM/p25I9ZCoDPGu9Tacm8ujg+ZnbN5RAEQ08TZKX
84KDKODpYFPHqfl5GXjWejWl9WmJ4KnjRhZ4VRPAM2fwKftNrBF4WsSERSU5
YrjBM6Vjg88L4Rx4/km9ZvOiTAueo8oTvHgnDXiulx3oSZWuBs91/+TlpCTa
gWdP178NB3v44Hk79E1nZroVeD72ue6XcdoSPC86Ww1hHSvBM2icGVcUaA6e
7jVbM5Xb2OCZu88688x+M/Ac85ex+2KXg+fIlumS7S9MwPMxa/zx1Uxj8CSF
C+Jc46ngWZja63kphwyeJ5Vy8Y5hEnjesUwJ8UsmgmfFk5HINUlLnrTCINme
+CXP1kOvFM2RH8Dz2eTADeIzHXi+ejcm5Vhql/672Dvd+aM14ElvT7ThmKwG
T58Qk8T1fbbgWVyyWX5Jbg2eu7xUAZltPPAM35gvPfzcAjy7PcyuV/tywFM2
RDqfZ7wCPG0w/o9UCgs83aUKe8sIJnieq50cLJo2Bc+Yg5L0xlQ6eGYYdbg6
IRp4ko3HXWZwCniS19eR0k4tA8+4+MGmDjIJPJX7lAFHagng+VOZGM0mL3lq
y0983SfQgGf/JlapfdMceD5UO+p0Si36D5u3TWk=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], 
        LineBox[{64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 
         79, 80, 81, 82, 83, 84}]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], 
        LineBox[{85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 
         100, 101, 102, 103, 104, 105}]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], 
        LineBox[{106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 
         118, 119, 120, 121, 122, 123, 124, 125, 126}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 64], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 65], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 66], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 67], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 68], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 69], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 70], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 71], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 72], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 73], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 74], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 75], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 76], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 77], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 78], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 79], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 80], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 81], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 82], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 83], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 84]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 85], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 86], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 87], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 88], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 89], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 90], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 91], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 92], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 93], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 94], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 95], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 96], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 97], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 98], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 99], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 100], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 101], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 102], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 103], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 104], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 105]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 106], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 107], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 108], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 109], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 110], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 111], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 112], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 113], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 114], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 115], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 116], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 117], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 118], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 119], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 120], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 121], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 122], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 123], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 124], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 125], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 126]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {1.2937578321997374`, 10.578548470207792`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF03tQlFUcxvEFlxXcXfbGCiyw7OIioKwyIrTC4PuCioAJZkFUSEBclFA0
FEkcoDBiwAuhMl4puZgSIpoNJrGIq4DcZiFrQpIQWOVm3BdQNwp8e/jNnDn/
nPM9M5+ZI42I3x6lT6PRUv5bc/ub6SLWSd9NVtTwSOf56SEcfO+OaG8IyOj5
eUYk0xXy0GIhee7s3PQT2gb/QfUTIdnSNDdDhHpDoWtVsJB80xshmqqX8Mzs
BNT5USLfOCj+oVxA9ceJBLMta0XX+NT9CeL2FaHiaQWPek9LqPJKlXF1XKo3
TYxVjFySlnGo3gyRtm3cwFbLpnqviMlSMt8hi0n1XhNBE8VKceQSqvcP4Zzh
GNa83ojq0cjMo25xV2sWk83z52lkpar/2/pUBtXXI9Pr/RSZEQbUfX3y7kcc
g1OedOq9RWSb+5EHKeQiqkcnKyM+0J0g9cn/PS8mFmuS0yzgyX/M+c51mRie
MuW25ykxYniqNhkkzlZbwfP8X3payxkRPOMNu/0YEhE8c3V3Qi7PmMGz0Ue/
/EKwKTwrehmh7ONCeK5LYqnv+5rAMzWrs7zanw9Pt1Nc176LPHgyig8PPNrL
hWe1vK+yS2kMTx/nTveMMRY8o9qLGlP9mPAcSNoesbnSCJ6t3KxN4QGG8Pyx
8MTP2VoGPH/dPMVNO2kAz9/jHay/OU+HZx/JHh7aZwvPnOxauvd1KTz3XHB+
XjpkA0+1zcSGt8Zt4KnRrWve1C2F5/KDiRYlKRJ4egr6NA5OYniqzpTohY6K
4MkP9vZxOWsOz6HgaPHOUlN4cnYHlPTMCOFZFDumzY01gaf158WK6EE+PONi
g9fnXuPB83Wkh92LNC48nbqHZb1NxvDsy/Ctf0Rjw5OZpfLvZTDhab/Gq8H2
ByN4yo56Zu5faQjPT1z4/n49DHhKotZW+Xwog+fXNVEv1H728Kwa6S+c2mIH
z3u7IkJv0pfD805zR+BGUgbPsomX0nAfG3huLO+YMt0hgaf81YWsPWoxPDu9
vZ8luVvCU3Ok27Nt0ByeGptIRlerKTy3pudzKvqE8NTvP3ji5i4TeHYG2YuC
tHx4eli/un+9kAfPgJW1t3QHuPC0k0d7qGuM4alNaDnd3sGC50iy10+DEiY8
Dy27+2D2eyN46thOMQYyQ3hqct5RzV6Sw9Np11P+VpEjPKcuKxtJ3gp4Hi+8
ybjjYQ/PvNUXuyybbOE5HROw417IMni+P3UjVMKwgefTwEuNtyMl8Cxs3c1K
arOCZ0sZ3yp1swU8q79qUE7amsMzf23zmjqWKTxVe2kmXVuF8Gx9LDuZ3SaA
Z+a9B/6NyXx4ng0rK7jlyIPnHyFFG2tZ3IX/7rJCQetlwzOOrRz5zYsFz/Yq
et7grSXwTF2lXJq72gieXy7+om0ozwme5od/eZj5pxyeTE7BZPpnjvC8Upyg
OXPUAZ5e7w0PmDxeDs+KZ4KPraZl8AzrSdOdHljwPOW3grFfKoXnpy6BaZph
MTx7j10915hjCU/vzuHRLDsRPN8OI+2sexc8OTdejHq6L4XnvgM1ev7HTOB5
pq5kRt9aAM9DL0P+Dmxa8HTVlU+fL1jw3KBYFW4m4sAz0zTfrU7GhmeOR4Jv
SiITnk/idmqUFUbkvydmRRQ=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], 
        LineBox[{64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 
         79, 80, 81, 82, 83, 84}]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], 
        LineBox[{85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 
         100, 101, 102, 103, 104, 105}]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], 
        LineBox[{106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 
         118, 119, 120, 121, 122, 123, 124, 125, 126}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, 
       {Hue[0.37820393249936934`, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 64], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 65], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 66], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 67], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 68], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 69], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 70], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 71], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 72], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 73], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 74], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 75], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 76], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 77], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 78], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 79], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 80], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 81], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 82], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 83], InsetBox[
         StyleBox["\<\"\[FilledUpTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 84]}, 
       {Hue[0.6142719099991583, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 85], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 86], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 87], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 88], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 89], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 90], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 91], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 92], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 93], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 94], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 95], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 96], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 97], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 98], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 99], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 100], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 101], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 102], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 103], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 104], InsetBox[
         StyleBox["\<\"\[FilledDownTriangle]\"\>",
          StripOnInput->False,
          FontSize->6], 105]}, 
       {Hue[0.8503398874989481, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 106], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 107], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 108], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 109], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 110], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 111], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 112], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 113], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 114], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 115], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 116], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 117], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 118], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 119], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 120], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 121], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 122], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 123], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 124], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 125], InsetBox[
         StyleBox["\<\"\[EmptyCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 126]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {2.2815246384528485`, 16.589551121236134`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5317885412988443`*^9, 3.5317885501076183`*^9}, 
   3.531788601308609*^9, {3.531789048214293*^9, 3.531789132388371*^9}, {
   3.531789172136693*^9, 3.531789271147477*^9}, 3.531792661822454*^9, 
   3.531987117016985*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF0HtIk1EcxvHXd1umy1tzJRbW7GKGIrLSpen5qWBUUrLEkqLLUMnwRhda
mBo2aoWhkFnei+GNliGjTC1zmWk6xVJSidnMtESt5S5ayy6695weeHn/Oed7
4COQpIkTaIqi0L9v8W+dDjk0bmmUl7JAuLQxlJmXcPqrggWJS5tAMldPeVOe
DRQXLW4S6XPeNT7ys4FezeKmEWc0pmokhAJrT4+yOLpjlGoBWc9/R29oRb5O
YEHWvgF9fs8LoVfPI+t9I7rTIZHNDJmR9T0zUjl377REmJC1N4/6Tcqrg6UG
pvcDhQ2cNW0OnmV6FsR96qeVG/VM7xeS9kTBB+E3pvcbeXEDN3JuzjA9Coai
Nqx4WDaFepbOUyBOSB8t+jjJ9G1g+FV/2rWgL8x9Gga3RrT0aiaY91iQHRgv
bqodZ3psuL8upeLuuU8Ie96eiyusdrQnnsnGolT3l47EM+PFelVYpB3xTG0F
f1GNA/FUCoqj69Rc4unTnSI3uXMBe671j1TdkNoB9pS2bw/IqbYF7FnWxio5
Qi0D7OkpjOwa38UG7Fmi7sxE0zRgzwK5ND1/DQ3Yk88tXTWgowB70vu1Sa71
f4jncOit+bHHC8Qz3uB24qDRQjy1spj28PCfxPO85kJVceU88RSfOjmYGDRH
PHON0sSLy83E02NEkRSsMhLPHYIDGSK1C/H03t2qN9fz/nuyRb5HK/nE09y1
b6pPyyeefRGKgGeH+MRT89zexc2LRzzLHWPTXvvyiOcZt73b3B+sJJ5Pavii
0QYX4tlWqGxJ7nAmnrMN+nuCOifieSnawNlkdiCeJiWUe1/nEs9YY2WLR7w9
YE/hFZ/jPaF2gD3luUHJtWpbwJ7NbZMVndnLAHte7twjkks4gD1bDztxCsLY
gD3fBsvas4AF2LNZEreQBzT8BXhgsBY=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 1.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.6393588174623707, 5.466617275890476}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJw90X8s1GEcwPHv3XE4XOfkx4TOaOuHtsyNLup5WolkJ7dGWfIr1KqhYnYa
1tqVHzvEP5LNJk1rloX5cXbC2YiSIcPm1DmH3cpFx+Eovt/v57M9e/55nvez
vR6vpHRJCpMgCPR/7e3kzCGtjBOUOGCF/fdHgyLU9mfHjnJw6v4soH75aW3Z
hBV+VbU3S0jue8P8TGWJvw7vjR4J7qWuJlWwMNlbQRyjX8T9NAZ13oB6+ROR
FT4E1V9F7we3G5LFZkTeX0OaMeLEufZNRL5nRCobO4Y0fgORvQ2k2mrvyYk1
IrJnQsky3aUjlWuI7G0h2wt9O/Eeq1RvGzVKNQn6FgPV20Gd+aGxD7JWqB6B
XzyV6jJbf6Ev++cJfKVq3av3sZ7qM3DzNZFnXv8ydZ+J/eJcLnICl6j3WLg5
M7H1h0xH9Syw8wyzuPb4AqI9ExQlGcGPnMBT0To10fXRCTwNDg13JAJH8HTM
C/cddXMEz1JhVJdYwAfPdg9TfeAADzwHZwaEZQ1c8HwezS+Mf2iLaU8Ws7On
nEf/nxHFzATGRDtbY9qzxF6ifpfMxrRnY0gH9++kBaY9eecFH1rusjDtGVxo
HpnTMzDtOSyKa8mdJjDtOT7dJj0l3wXP259lrpN1ZvBMMGgjvdVb4CmSioVc
4SZ4zmuiNArlBnhG/x5x/5S1Dp412fXa3IJD4MmfPlAb4O0Jnj7Kq7q8NE/w
7AuxzN7t9gDPajXD6G5yA89065/hbIEbeL40d958a3IFz6EwZtPr6y7g2TbP
vmUvdwJPUY7dN9Xlg+CZXzTb1C3mg+eZSl7AYo0DeLLrnyyPZ/DAs/vkomJO
yQXPMP/ZINkfO/BMmXozlB9ui2nP5RxJUqjCBtOeo7yikMRIa0x7NteVdhQb
2Zj2HAtd5xVUWGLa83v6scPl1Rb4H53Tru0=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 1.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.692703505998767, 7.60107003622355}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJw90ltI01EcwPH/1l+3NbbmvOUsTfNSWimJaOk6RyMrH6b4IIvCnJbW8FJU
i2DIkAqxYTpLdNpISI1pCFEsyaTQ0vKChA/2osvLnDmZc9Mcm6N0///vB4fz
cs73wIcTUVSZd51JEAT6v3Z27xiRMkhSWpYtxEm7M49epx9X+j0V4JLdMSFl
yPumbh0Pa1t2ZhkNaA9elRo5eGJsZyyoXRkVaMpkYW9vDc2+XF/yfCGp8zaU
mv+9id/JpPp2tOU2jo0YCOq+A83oKmRdlzzI+94mCojzSWeoXcjb20JscT1f
nORE3p4TKepyi6e4f5G350LpumFWYMAG8vbcaFqVattQ26meB7knWlM+JqxT
PQK3xxt+3+lfQ+O75wn8+VNvYfVpK9Vn4MkD/WSdYpW6z8SlzSaLpnqFem8P
ns1Jc8Rol6keiYvD+tI4MjOiPbeIrlh2fAh49okO19iGReDZ+OtrSvlNEXi6
VA6D++F+8FSUa2ydoUHgKRhnTYfq/MGT1zHzwlAqBE9yRl3054MAPFWFev3A
Iz6mPdeKw+VaDxfTnrWV0nvvGjiY9qw6T15pfMvCtGcvI2bE/tMH054Nl5eV
o1IS055xmbkSyxwT057GZ2ioppWBac8005CsU0Zg2nOiIJg9uekBT6tYWMfb
uw2eRf638hLyXeB54tjzMtOoEzzNmGe13I4Gz/on38is3gjwrGhLWuqxRILn
ZKTjbIo9EjwXt0+Nn5uLAM+Y+4pQfdUh8MzwNy8eTQwDz8FmPaPAJgJPoTTr
QnJLCHhapCVhN3qCwXNfeY5+3hkInq/k65saeQB4hj/oSC1ZEYJnmVx6RvPG
Dzzd18SxqyoBeCbOWaMWxvjgaX58cWSK4IEnt3ZQsuDLxbTnkZOZP6K7OZj2
jFJn1NyNZ2P4n8lCSfa8L/4H5E+qqg==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.7618452230899034, 10.72251078640734}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7FMCp1/9sE7EwRgMHttf37Fn0dmbQg5pYPDM
3uq4wbwqVwGHWTNB4KW9peiF07cfczucOwMCb+y/sQfw78/mdICY98F+C1v9
quRMNqj6j/byK6JTJFhZoOZ/tt9++L+pDAcTVP8Xe5tdJus/2TBA7ftmL7El
3+13wV97iHk/7CXnHeRyP/PLHmLeT/uuyesZFJN+2EPM+22/fJGEv1ztN3uI
eX/s894/THq66Ys9xLx/9j03pJ8e1vwMNY/BwXPbkctXyz7anwWrZ3Do5LON
dbf/ADWf0SG12ad07YV3UP1MDnraO7Z+e/8Gah+zg4bh7DdpYq+h5rE4/Dof
eNG1/qU9LDzV3HrCQ+2V4OG5Opdho0WAAjw8LVZFbZyxQw4enkqXZJ/2TZGG
h+ep5JilRjoS8PBcfH6677nfovDwdL/zQVqAUwQenoeTDthFGwnBwzP8Fsfj
4EYBeHh+35/pwcbB5wALz02+vN/eTOJ2gIWnj1Mze1Q4pwMsPHmq+aVjotkd
YOGpI79ZbOMSVgdYeG7y3lN39Q+zAyw8/0qlT/Fez+QAC88Jd+8IfypgdICF
p0ReclaVHix+mRxqTmr5PQ/6Bw/PYxzr5J/P+QMPz6cvyqRLTH/Dw1Mh1WSv
R5QKPDzbD6a+veClAQ/PvR9eLv7urQ4Pz0OZSXGbWNTg4bnr7O1QFwcVeHiu
+/JLMdFDCR6eLhtufxePVYCHp+7vOV15F+Tg4XnPze1ZhbUMPDyftjxyvPRa
Eh6eT5VS2B5cFIeHp2/zPP7tL0Th4cn0srx/U6YIPDzvhWlIhX0Tgoenrfzv
I+sXC8LD01/72Ja/pQLw8FTXTbO9cJAPHp7fis9NvXmbBx6eH6qdtr5W4HaA
hWeV8oGj/5fD8iOLw19eg3RWFQ4HAF+hq2k=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.7967862213395158, 12.14612523905886}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7B9FsJ386SvuYAwGj+1r/tzR6a8WcUgDg2f2
RXMWJHokCzrMmgkCL+1Pnvir43qdz+HcGRB4Yz9vrx9blwi3A8S8D/YzVO9f
3b2bHar+oz17U/ajSllWqPmf7VWDfnfyPGeC6v9i7yi5OGwqIyPUvm/2ZeLv
w51W/LOHmPfDXkvEsi/oxm97iHk/7RvfiXzoif5pDzHvt30dV86RUwnf7SHm
/bH/tvdw9t3Or/YQ8/7ZrxQv0v/+4jPUPAaH3/vW+r0M+mR/FqyeweGVicJa
9eI3UPMZHdrOpnEsYP8A1c/kcJzjodUPtrdQ+5gdJH896RTkfwM1j8Xh3cpD
TM+NX9nDwlNze+yF2g4VeHiqP5w0oaZSCR6eF8683RF3QQEenvsTP53ubpSF
h+dlvw2BLW2S8PCcZy57j+GkODw8o18JFE+PEoWHp+tG4+4XZsLw8JwYKtfd
HikID89nYduFrxbwO8DC84HR91mWz3kcYOF5WejV5dXdXA6w8GRJMSm2K+Fw
gIXn1Nor7vMnsDnAwrPhxc0k6/ssDrDwPCRZkxBYzewAC89ltx+kq1YxOsDC
k3Nq9HaPEgYHWHgeiX/3+ED6f3h4Pvpwayvzo7+I8Pz6rENM8g88PJ9OCDz8
f6EuPDwNMh8K+UrpwMPz+7J9px0EteDh2bd4E9suWw14eE7Tn/tA5owqPDx/
pPvHHopRhodn+PeNcQpsSvDwfBi68PSOFAV4eC6+mMtTcUkWHp7n1gnJ1rtL
w8Nzf+upfV9VJeHhOc/krNFxHnF4eB4uYBB54CsKD8+Lt1Qmd18Shodnx6Gj
fqerheDhOTNh3aItOoLw8LwRs8TlGI8APDwvmmpZMDzhhYdnDu++D1edeBxg
4XlzL8u011u4HGDhWa+3T2ySPqcDAFKmscY=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.8188357404627007, 14.816404441553866`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJw90H8s1HEcx/HvnXN+HN055+5IXReFlFGUavp8qqWRzCzbRSsM07Qah6HC
qT9O0UzSOS11m5T+SNq6NrpNWxNhaampTq27M3Th6uKLO4r7ft+v7bvvP5/P
87M9pJkXkrOZBEGg/9/q37Hv6OS7vb5ppBjvWpsBxXO/tQXX+eCctY2jalWS
KP85H6ubVjeJ0JI1tWgdDw8NrM6MZB/L5Oo4D+zozaIVfVsbo9+VOm9BwUwi
2XPemer/Qf46bUZErRN134q+jEzWFJQyqPfmUFOLvDX18Qpy9EikkKsEPa9s
yNFbQAN6t35dxCJy9JZQl+bDfTGPRI6eDT04WhFaz59Djt4yKnxZiXUlVqpH
4Lh44SD+8RsNrp0nsDA1zTTqY6H6DNxuFg3/zJul7jPxC0FCZFbDNPWeEzY6
Gav5JWaqx8Imw41x2c5JRHuSmccVe4K3gCfnTalE6BEInonpHqUhI1LwbFJH
jLaMbgTPmGMW2fXX68EzK/SmMtsoBs+hWK+OJ0lC8NSOsVT17gLwlBCb7rm6
8MHziNIQ4JvDw7RnY+es/va8J6Y9z58+q+i6zMG051VG34Eo5IZpT7b7VDSJ
XTDtyQ55xiqvcsa0p7xQ393HZmHacyJlQpbbycS0p+ZOJlq4SGDa09ZaXDES
uQSen8L5zQHdy+A5bA2z2yds4FnlonhvbgwHT99L3X3KrzvAk8PV/L1SsB08
H7bKTaqaEPA8dGJmSvB5K3hqx73PbCADwTPdUGm/NbUZPBvit7ELpVLwzItK
qTTNbARPY+0j9ds6f/CMHZuxXAvyA8+EdBwkMYrAk/v0l+XgfiF45hf1MBJr
BeCp6m1fYEq8wbNs8dR0yoAXeO62d5DNGh54Ho4OyxD7ccFTKbq7rzfQE9Oe
dTHyuPJiDqY99edyTTqtG/4HBtSp1g==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.7873959067429693, 16.589551121236134`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.5317885412988443`*^9, 3.5317885501076183`*^9}, 
   3.531788601308609*^9, {3.531789048214293*^9, 3.531789132388371*^9}, {
   3.531789172136693*^9, 3.531789271147477*^9}, 3.531792661822454*^9, 
   3.5319871178603067`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Graphics for publication", "Subsection",
 CellChangeTimes->{{3.532767568446211*^9, 3.532767571811208*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"annot", "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], ":=", 
  RowBox[{"PSfrag", "[", 
   RowBox[{"s", ",", 
    RowBox[{"TeXCommand", "\[Rule]", 
     RowBox[{
     "\"\<\\\\fboxsep=1pt\\\\footnotesize\\\\colorbox{white}{\>\"", "<>", "t",
       "<>", "\"\<}\>\""}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"forcemaxPlot", "=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"vv", "=", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"20", "mm"}], ")"}], "3"]}], ",", 
      RowBox[{"Drange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
        "mm"}]}], ",", 
      RowBox[{"Rrange", "=", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"2", ",", "4", ",", "8"}], "}"}], "ohms"}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Show", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"dd", "/", "mm"}], ",", 
              RowBox[{"First", "@", 
               RowBox[{"ratiomax", "[", 
                RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"rr", ",", "Rrange"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Joined", "\[Rule]", "True"}], ",", 
          RowBox[{"PlotMarkers", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
          RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
          RowBox[{"Axes", "\[Rule]", "False"}], ",", 
          RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", 
         "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Graphics", "@", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Text", "[", 
              RowBox[{
               RowBox[{"annot", "[", 
                RowBox[{
                 RowBox[{"ToString", "[", "#", "]"}], ",", 
                 RowBox[{"\"\<\\\\SI{\>\"", "<>", 
                  RowBox[{"ToString", "[", "#", "]"}], "<>", 
                  "\"\<}{\\\\ohm}\>\""}]}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"1", ",", 
                 RowBox[{
                  RowBox[{"-", "6"}], "+", 
                  RowBox[{"First", "@", 
                   RowBox[{"ratiomax", "[", 
                    RowBox[{
                    RowBox[{"1", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}]}], 
                "}"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "]"}], "&"}], "@", 
            RowBox[{"Rrange", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Text", "[", 
              RowBox[{
               RowBox[{"annot", "[", 
                RowBox[{
                 RowBox[{"ToString", "[", "#", "]"}], ",", 
                 RowBox[{"\"\<\\\\SI{\>\"", "<>", 
                  RowBox[{"ToString", "[", "#", "]"}], "<>", 
                  "\"\<}{\\\\ohm}\>\""}]}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"1", ",", 
                 RowBox[{
                  RowBox[{"-", "6"}], "+", 
                  RowBox[{"First", "@", 
                   RowBox[{"ratiomax", "[", 
                    RowBox[{
                    RowBox[{"1", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}]}], 
                "}"}]}], "]"}], "&"}], "@", 
            RowBox[{"Rrange", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Text", "[", 
              RowBox[{
               RowBox[{"annot", "[", 
                RowBox[{
                 RowBox[{"ToString", "[", "#", "]"}], ",", 
                 RowBox[{"\"\<$\\\\resistanceCoil=\\\\SI{\>\"", "<>", 
                  RowBox[{"ToString", "[", "#", "]"}], "<>", 
                  "\"\<}{\\\\ohm}$\>\""}]}], "]"}], ",", 
               RowBox[{"{", 
                RowBox[{"1", ",", 
                 RowBox[{"5", "+", 
                  RowBox[{"First", "@", 
                   RowBox[{"ratiomax", "[", 
                    RowBox[{
                    RowBox[{"1", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}]}], 
                "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}]}], "]"}], "&"}], "@", 
            RowBox[{"Rrange", "[", 
             RowBox[{"[", "3", "]"}], "]"}]}]}], "}"}]}]}], 
       "\[IndentingNewLine]", "}"}], ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Wire diameter, mm\>\"", ",", "\"\<Maximum peak force, N\>\""}], 
        "}"}]}], ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"8", " ", 
        RowBox[{"72", "/", "2.54"}]}]}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.531789281132059*^9, 3.531789547217321*^9}, {
   3.531789597115576*^9, 3.531789621231968*^9}, {3.531792695681196*^9, 
   3.531792768902416*^9}, 3.5317928095537643`*^9, {3.531795252353503*^9, 
   3.531795348208302*^9}, {3.531795385182143*^9, 3.531795386251965*^9}, {
   3.531795423948827*^9, 3.53179545031741*^9}, {3.532035104085763*^9, 
   3.5320351059139013`*^9}}],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7Nnnuj6J2ajqYAwGj+25TetCrRVNHdLA4Jl9
yI2pbO+XWDnMmgkCL+2ZP3lq2192djh3BgTe2E/e7v3n+0Z3B4h5H+wP7ODh
8833gqr/aL96RfRiNW8fqPmf7Tuvvnd51+UL1f/F/n66uc2ELX5Q+77ZC7LX
d/V3BUDN+2GfW57GXBAZCDXvp/0um6e1MZcCoeb9tn+dLtbzqTAQat4f+1uP
3Cd5SAZCzftnXzE1c+51gUCoeQwOzuFXMoOdAh3OgtUzOATuTTqj5Q0zn9Gh
2ndaWrkKTD+Tw80SxxJPNph9zA6Hm0MufxCBmcfi0FfotvnCN5h7H9gv8wpw
rb1tCA9PocUXMh6etYWHp9mh7AMbNV3g4bly8rO5l7s84eEpf4j17cpPfvDw
nHffYs372AB4eO607xLUYg+Eh2fDzwd/9ygHwsPz0Z0TGSoGQfDwzE7tXxG0
NBgenv+i5P7wt4fAw3P3JoXVG5tD4OF57MQR/XkCIfDwPMp2vnZbeDA8PEs+
NondjA6Ghycj/6TJldXB8PBclm8lZ9kaDA9PnXnaC5ckBcPD08TO06/NJBge
nl6Wa17q6AfDw3PVyRWXL1oGw8Mzp3lHz93PJvDw/HR4U/eMFCd4eK4y/lD6
6Lo3PDynOsz250oJgIdnyHTvgLb7QfDw7O76deqeYjA8PG+JFey/UxiCSJ8z
dh6c8igEHp6zN9VGl/qEwsNTOTLSSO1qGDw8Fz64JZxVGA4PTx/2dWHHgsLh
4Vm4YFvsjL4weHhKVPQbLf0SCg/PZebRNss3hsLD80x9bvnzvaHw8PSw/HV5
9+RQeHi+Vbwn69oZCg/P67E+vxZ4hMLD8w+7+JoOw1B4eDKuN53ErBLqAAAS
1Kzl
    "], {{{}, {}, 
      {Hue[0.67, 0.6, 0.6], 
       LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
         19, 20, 21}]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], 
       LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
        37, 38, 39, 40, 41, 42}]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], 
       LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
        58, 59, 60, 61, 62, 63}]}}, {
      {Hue[0.67, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 2], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 3], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 4], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 5], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 7], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 8], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 9], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 10], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 11], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 12], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 13], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 14], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 15], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 16], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 17], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 18], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 19], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 20], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 21]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 22], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 23], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 24], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 25], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 26], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 27], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 28], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 29], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 30], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 31], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 32], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 33], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 34], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 35], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 36], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 37], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 38], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 39], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 40], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 41], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 42]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 43], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 44], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 45], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 46], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 47], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 48], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 49], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 50], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 51], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 52], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 53], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 54], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 55], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 56], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 57], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 58], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 59], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 60], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 61], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 62], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 63]}, {}}}], {InsetBox[
     FormBox["\<\"2\"\>",
      TraditionalForm], {1, 46.86953902836967}, {-1, 0}, Automatic, {1, 0}], 
    InsetBox[
     FormBox["\<\"4\"\>",
      TraditionalForm], {1, 59.467753463306934`}, {0, 0}, Automatic, {1, 0}], 
    InsetBox[
     FormBox["\<\"8\"\>",
      TraditionalForm], {1, 81.52922319857514}, {1, 0}, Automatic, {1, 0}]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{0.5, 20.},
  Frame->True,
  FrameLabel->{
    FormBox["\"Wire diameter, mm\"", TraditionalForm], 
    FormBox["\"Maximum peak force, N\"", TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->{{0.5, 2.5}, {10.846411832330444`, 93.77209946036102}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.531789313783682*^9, 3.531789331601789*^9}, {
   3.53178940614734*^9, 3.531789456806572*^9}, {3.531789532693316*^9, 
   3.531789547919001*^9}, 3.5317896105744534`*^9, {3.531792702839559*^9, 
   3.5317927104844007`*^9}, 3.531792770236305*^9, 3.531792810695086*^9, {
   3.5317952701572237`*^9, 3.531795320820176*^9}, 3.531795350935255*^9, 
   3.531795386894697*^9, {3.5317954323380327`*^9, 3.531795451226362*^9}, 
   3.532037491902046*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PSfragExport", "[", 
  RowBox[{"\"\<fig/magcoil-ratiomax\>\"", ",", "forcemaxPlot", ",", 
   RowBox[{"RenumberTags", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5317896256808167`*^9, 3.531789652837863*^9}, {
  3.5317928140828543`*^9, 3.5317928198251963`*^9}, {3.5317953529745703`*^9, 
  3.531795356164082*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"fig/magcoil-ratiomax\"\>", 
   ",", "\<\"fig/magcoil-ratiomax-psfrag.eps\"\>", 
   ",", "\<\"fig/magcoil-ratiomax-psfrag.tex\"\>"}], "}"}]], "Output",
 CellChangeTimes->{3.531789670549768*^9, 3.531792780760571*^9, 
  3.531792829250613*^9, 3.531795364063806*^9, 3.5317954023433*^9, 
  3.531795463339583*^9, 3.532037499694269*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"annot", "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], ":=", 
  RowBox[{"PSfrag", "[", 
   RowBox[{"s", ",", 
    RowBox[{"TeXCommand", "\[Rule]", 
     RowBox[{
     "\"\<\\\\fboxsep=1pt\\\\footnotesize\\\\colorbox{white}{\>\"", "<>", "t",
       "<>", "\"\<}\>\""}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"normMaxForcePlot", "=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"vv", "=", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"20", "mm"}], ")"}], "3"]}], ",", 
      RowBox[{"Drange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
        "mm"}]}], ",", 
      RowBox[{"Rrange", "=", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"2", ",", "4", ",", "8"}], "}"}], "ohms"}]}], ",", 
      RowBox[{"ll", "=", 
       RowBox[{"0.8", "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Show", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"dd", "/", "mm"}], ",", 
              RowBox[{
               RowBox[{"First", "@", 
                RowBox[{"ratiomax", "[", 
                 RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}], "/", 
               RowBox[{"CurrentFromDiam", "[", "dd", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"rr", ",", "Rrange"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Joined", "\[Rule]", "True"}], ",", 
          RowBox[{"PlotMarkers", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
          RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
          RowBox[{"Axes", "\[Rule]", "False"}], ",", 
          RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", 
         "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Graphics", "@", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Text", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"annot", "[", 
                RowBox[{
                 RowBox[{"ToString", "[", "#1", "]"}], ",", 
                 RowBox[{"\"\<\\\\SI{\>\"", "<>", 
                  RowBox[{"ToString", "[", "#1", "]"}], "<>", 
                  "\"\<}{\\\\ohm}\>\""}]}], "]"}], ",", "\[IndentingNewLine]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"#2", "/", "mm"}], ",", 
                 RowBox[{
                  RowBox[{"First", "@", 
                   RowBox[{"ratiomax", "[", 
                    RowBox[{"#2", ",", "#1", ",", "vv"}], "]"}]}], "/", 
                  RowBox[{"CurrentFromDiam", "[", "#2", "]"}]}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "1.5"}], "}"}]}], "]"}], "&"}], "[", 
            RowBox[{
             RowBox[{"Rrange", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"0.9", "mm"}]}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Text", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"annot", "[", 
                RowBox[{
                 RowBox[{"ToString", "[", "#1", "]"}], ",", 
                 RowBox[{"\"\<\\\\SI{\>\"", "<>", 
                  RowBox[{"ToString", "[", "#1", "]"}], "<>", 
                  "\"\<}{\\\\ohm}\>\""}]}], "]"}], ",", "\[IndentingNewLine]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"#2", "/", "mm"}], ",", 
                 RowBox[{
                  RowBox[{"First", "@", 
                   RowBox[{"ratiomax", "[", 
                    RowBox[{"#2", ",", "#1", ",", "vv"}], "]"}]}], "/", 
                  RowBox[{"CurrentFromDiam", "[", "#2", "]"}]}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"0", ",", "1.5"}], "}"}]}], "]"}], "&"}], "[", 
            RowBox[{
             RowBox[{"Rrange", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"0.9", "mm"}]}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Text", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"annot", "[", 
                RowBox[{
                 RowBox[{"ToString", "[", "#1", "]"}], ",", 
                 RowBox[{"\"\<$\\\\resistanceCoil=\\\\SI{\>\"", "<>", 
                  RowBox[{"ToString", "[", "#1", "]"}], "<>", 
                  "\"\<}{\\\\ohm}$\>\""}]}], "]"}], ",", 
               "\[IndentingNewLine]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"#2", "/", "mm"}], ",", 
                 RowBox[{
                  RowBox[{"First", "@", 
                   RowBox[{"ratiomax", "[", 
                    RowBox[{"#2", ",", "#1", ",", "vv"}], "]"}]}], "/", 
                  RowBox[{"CurrentFromDiam", "[", "#2", "]"}]}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", "1"}], ",", 
                 RowBox[{"-", "1"}]}], "}"}]}], "]"}], "&"}], "[", 
            RowBox[{
             RowBox[{"Rrange", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"0.9", "mm"}]}], "]"}]}], "}"}]}]}], 
       "\[IndentingNewLine]", "}"}], ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Wire diameter, mm\>\"", ",", "\"\<Force per amp., N/A\>\""}], 
        "}"}]}], ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"8", " ", 
        RowBox[{"72", "/", "2.54"}]}]}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.531792912972823*^9, 3.531792925140956*^9}, {
  3.5317936463163567`*^9, 3.53179370118922*^9}, {3.53179551940125*^9, 
  3.531795528561668*^9}, {3.5317955725146*^9, 3.53179577928441*^9}}],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJw90G1Ik1EYBuB375abjo21pdm0mgkFJZW4MtM6RyGFfkwdMY2s/Mqc+EVh
LTWRsJqxTFMLZ/ZHnCaVpJaaIlpKy88Kw6aoo6VgDHQ4dTqn5fbuueFw/jzP
fTiXT2KW9BpJEAT6f7ZvR/To+erFZ/VcNxxgjwGlm6syhX1cnGLPHMr7JGoJ
DXfF6qrtzKPMHux/qoGDR4a2Y0SvfdRRb3vZ2NG3iPwGM5TLQjY1b0Le/uEt
jxWuVP8SUvSfOHmvnkntm1HNZ3p1HOFCvbeCDgSED8xGMKg+C6ru1d5FRpLq
W0MVSkV2qRdJ9VmRO/uFx5ieoPo2EBk5Jd/1bgs5+jaR7mylxfDBhqgP4+Ql
z4QYsxUN2+cJPFV0oT8sbB05+mn49tAdjbrOQu2TWJqWOp5yehU53qNjlVmR
ks9aofoYeN90rTy4xYycnvGdquyQm+7g2fle97Or2R08TTsbUqUiAXgKCs77
fRcKwPOJOLpLIuKDZ/vetbpALQ88v05qxaUNXPB8KOMXX73BBk86+bG3jOcG
njGTgTEyDxZ4qjjSmVdJLuD55lwHd3mcAZ68UFFTq5wOniHFtlG9kYadnkNB
l1vzJgjs9BybaMs9XrIFnskDDzzHa23gGW+ajfSdsYJnUK5EzBWvg+cfQ7Sh
s9sCnrKFUe+enFXwtBD1h1hH9oBnh9BXafoiBM9yXX9ghlwIntZCc9tGkSd4
3sp4atJ4eYAnb5j5y+ulADw5ddM1bdf54MmYViX+beeBZ2F8Y2P3fS54Libt
T1NvssHzUVZsTmuZK3gWRDDiypuZ4NlEO6hd+rEDPMsuzecPxjLA83BYlMT4
mwRPfQXqU1bTsNMzeK4vQZNAYKfnyJXdrG8rm+C5cIZfwnGzgWeiIFt6TGYF
z6N+lelzg2voH7dgrHc=
    "], {{{}, {}, 
      {Hue[0.67, 0.6, 0.6], 
       LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
         19, 20, 21}]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], 
       LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
        37, 38, 39, 40, 41, 42}]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], 
       LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
        58, 59, 60, 61, 62, 63}]}}, {
      {Hue[0.67, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 2], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 3], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 4], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 5], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 7], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 8], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 9], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 10], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 11], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 12], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 13], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 14], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 15], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 16], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 17], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 18], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 19], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 20], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 21]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 22], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 23], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 24], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 25], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 26], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 27], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 28], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 29], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 30], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 31], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 32], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 33], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 34], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 35], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 36], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 37], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 38], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 39], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 40], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 41], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 42]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 43], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 44], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 45], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 46], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 47], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 48], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 49], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 50], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 51], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 52], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 53], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 54], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 55], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 56], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 57], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 58], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 59], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 60], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 61], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 62], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 63]}, {}}}], {InsetBox[
     FormBox["\<\"2\"\>",
      TraditionalForm], {0.9000000000000001, 3.469570760444375}, {0, 1.5}, 
     Automatic, {1, 0}], InsetBox[
     FormBox["\<\"4\"\>",
      TraditionalForm], {0.9000000000000001, 4.531550352531794}, {0, 1.5}, 
     Automatic, {1, 0}], InsetBox[
     FormBox["\<\"8\"\>",
      TraditionalForm], {0.9000000000000001, 5.5270040269491485`}, {-1, -1}, 
     Automatic, {1, 0}]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{0.5, 2.},
  Frame->True,
  FrameLabel->{
    FormBox["\"Wire diameter, mm\"", TraditionalForm], 
    FormBox["\"Force per amp., N/A\"", TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->{{0.5, 2.5}, {1.2937578321997374`, 6.695257308142282}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.5317929141721888`*^9, 3.531792927939824*^9}, {
   3.531793647155611*^9, 3.5317936586379557`*^9}, {3.5317936983888702`*^9, 
   3.5317937187759132`*^9}, 3.5317955319522877`*^9, {3.5317956346429787`*^9, 
   3.5317956615393667`*^9}, {3.531795706499606*^9, 3.531795782662627*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PSfragExport", "[", 
  RowBox[{"\"\<fig/magcoil-ratiomax-norm\>\"", ",", "normMaxForcePlot", ",", 
   RowBox[{"RenumberTags", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.531792938511148*^9, 3.5317929396924963`*^9}, {
  3.531795783507793*^9, 3.5317957851528463`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"fig/magcoil-ratiomax-norm\"\>", 
   ",", "\<\"fig/magcoil-ratiomax-norm-psfrag.eps\"\>", 
   ",", "\<\"fig/magcoil-ratiomax-norm-psfrag.tex\"\>"}], "}"}]], "Output",
 CellChangeTimes->{3.531792949172399*^9, 3.531793729192581*^9, 
  3.5317957939448833`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Optimal values for magnet and coil ratios\
\>", "Subsection",
 CellChangeTimes->{{3.5327675847633533`*^9, 3.532767591365954*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Vrange", "=", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], "3"]}], 
     ",", 
     RowBox[{"Drange", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
       "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Table", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"dd", "/", "mm"}], ",", 
           RowBox[{"a", "//.", 
            RowBox[{"Last", "@", 
             RowBox[{"ratiomax", "[", 
              RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"rr", ",", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"2", ",", "4", ",", "8"}], "}"}], "ohms"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", 
       RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "]"}], 
     "\[IndentingNewLine]", ",", 
     RowBox[{"{", 
      RowBox[{"vv", ",", "Vrange"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Rrange", "=", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2", ",", "4", ",", "8"}], "}"}], "ohms"}]}], ",", 
     RowBox[{"Drange", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
       "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Table", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"dd", "/", "mm"}], ",", 
           RowBox[{"a", "//.", 
            RowBox[{"Last", "@", 
             RowBox[{"ratiomax", "[", 
              RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"vv", ",", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], 
            "3"]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", 
       RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "]"}], 
     "\[IndentingNewLine]", ",", 
     RowBox[{"{", 
      RowBox[{"rr", ",", "Rrange"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
  "]"}]}], "Input",
 CellChangeTimes->{{3.531794214747499*^9, 3.531794247663385*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF0n8slHEcB/DHoRtiSQqJGvWXdaHsNPZ9ohkrnVImVFOdxvXTJJrUH4Q1
1qhujpaySrWiLfRjGkmYn/k5YRNHLrfc3fPc8zzncYR7ns+9t2fPP9/v+/P9
vvbdcfbKMakAwzC08q3+TZlAMzY62lJKIP+1TKGnf+8VNmIUSlrLDHp3beJb
VBqNFKWrUaHxAKq4qp5F3Z2rUSNWIrMNURmQqU+Dmm5onbPEBLdei/yVbomK
TXqun0BFt6tP1FlT3H4SOe7t/Zq63QI3zaPQg5bnosuXdFwfgxROt4b+JWC4
qc+AXqVnLF8N5ftY1JTyBFdnMlzfItpdNisPlpHc+ZeQT3b6iLOK4fow/IMx
tTetjkRda+sxvMTjTG2/Used1wLP7Vj3sbBew+0X4IM15UftBzTcPEu8zT2+
Niia5vqs8NjEuI2ERo94z8nc7OHDLiR4bpO5iT+36MFzNr9/8fcXFjxLL3iG
GqYJ8IxKcW/v3E+Ap1X6zVLFQQY8ZSLLn9fPL4DnxKhngCBuETwbhZOVrUr+
/hTqDfQ6mdFMgac6N1n7LJhEvKfv8a6CdnsdeDYMi3eNDunB0yncObuqnAVP
L9o2q+Y1AZ6umZvld1kCPP0+OTr4TmnBU3JqoO3tgg48C5TTlW/yKPD0EKiX
Y4rMntbFORWBFgKc96Re3icOzbHgaeft0nyxgwbPpbz88XmV2TPB7se863sK
PH9NGpUDUyR4unW3+vzJoMEzKLGvZGePATxthZKSrcUseAqTI/Y49OnBM3Kk
IQcTGcEzJfL02IaHDHiG10offY8wv88er9gDL8IwnPc03JFKPGIY8Kz0jG8c
zKTBU8zYbDGOmd+nX4cx3G5lHu851y23TgohwfNIgTcmr6bBk20fWR/2WAOe
0ftiziVU6NB/uujTXA==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 1.2000000000000002`},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {1.0383968831998789`, 2.250598196073272}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF0G1IU1EYB/Cr0wlhGlJMYZqbQ0Ikg/lhSnpuUllkiW9sUoZupKwEs6yW
RpISgi8RZokmmcs0U0eFpFimuSDwJUpScpU0C1GbbPfe7d4750upx2d/OJwv
z/M/h59EXZB6zpMgCPT/bNxb+YUqJlOSFl/MIvlmfiNVE5FaHMCg3M3MoZK6
+HzpOI8aGzaygLJj9D7D4U70aWwjFlTZs//ZqZFltNVnQ691oQfNdxk8T6EO
nepOZpcT9zPoZoi6VCKg8b4dDfjpkzIKafwei+QJRrZthcJ9PFoLbtsbSdtw
nxPpbrwsOyzkcZ8L/TVo+5UlDO5bQaksetecxeO+NXQpY1+er3X7fwR5OTtS
5JvCoPHNeYIci5japRKv434P0qyUStu0VrzvSYZ9jU9skDL4PQGJmq68GWi1
4T4v0v+E584hhxVte45JpoTH45bBU/FAUbzkZMHT3v1WHFK2Cp57xEcj4oo4
8FTdWmuyHGHBM/epMlqY7Pas9zPVTEfT4HnM4tX6rXAJPEMr1lfKFC7wTA4q
6no+7QDPxKuqlg/NDHguXDfvtkjt4Nme1qBp5Bzg2Rc4X0BXU+A5ki6ZCZdx
4Kk1jF7s+0yD55A9/MK9YAY8K754fRRqOPCsO2stvz/p9nzM5yxFedPgeS3d
J6o+0w6esX90gRMiJ3guRqaZ3msIctszJkZrNM6w4Kl/FNSV38GAZ07Bq/nb
B2jwrJ1YiK86YwXP87PD3nIBB561UVmGJ50u8KyaMcV2tjjA0z9OYPx+2gme
Kd3VCeZBCjzreucf7vjJgWeSsYbzXqTAUyxS2g7JWLen5uRcr2wVPHvU/WHW
Cbdnr4ef4YeaAk9WXy5TTPPgOSoThOalO8AzoNI0WU7x4DlYInapS5fRPzL3
zYs=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 0.6000000000000001},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.5840960970514723, 2.048709588683427}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF0G1IU1Ecx/GbDWpYYms2sySqacwwpRKsrHOzoZBRpr5ooI1MXVlDA4UU
SzMtyTUJJTVSfGJkKpnoNHEjqzEXplEU9ACtWs5s17nt7t65zVHq8d8PLvfN
Od8Dn+2ZeSnZfgRBoH/f4n95RmQZuV71ru4X2re0n+ispk1o0lMoZ2lTKIs6
GfmkwoYeNC7uN1Kfsuo5fSyaGF+cBbX6+6TcjAW03JtDl3X5o1FNDD5vQ+Hi
tX949xncd6DBXpUlXObA92n0JYBsV8TY8XsMWoiqL+YYbbjnQrOGOF3QpAP3
5tGdEyO7/S96cM+DpkUd1+JinLjnRX2l2vz6QB/u+ZCCENbImudxjyBTbnIs
e8pc6M3SeYJsy73CP9y00l9FViQobgvsDL7vR0p2Cd/Ge2n83mqSahRSs+lu
3OOQprsvykKLaLTi2WFOnvr+zAqezSJJ6MwaK3h+kv1I6pZ5wTN5ynHGvM0J
niGmb+6CQgY8kwK/FonVLHgqWmqFmmAWPF/1H7SL5U7wzCrmZEubXeBZPdlw
2p+iwbOyX4bGR23g6TEbHLmjPvDM4zZF1Gtt4Lm18pY8+LUdPAflIulnHg2e
RqWh5nwPDZ5CSeK5gEgveE5cPRSt1P33rNVWx+zczIJn+sO9huEbLHjG8uQR
5RkMeCrFQZrURCd4Hkm4YDywYR48S6bT0Fi5BzzJ8vXWkFgveOqVbr5ggAXP
gcK5p881DvBUmVQUl0+DZ4/A9+hjOwWe49FpckpMkCuewzP6lxqDBzwbesPG
6qK94Lk/q6RK8p4Fz1LBsUu6SRY8j1Z07YgfcoInV5p6PGyTGzy3FN/r7Bax
4PnB0jI8lOkCT442pyvw8QJ4qq2ydWadCzw7ptttKp4TPGs7CzZmBDPoL2WW
0z8=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 0.8},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.6744167962951031, 2.033655981189531}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.531794232627514*^9, 3.531794248720128*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF0H0s1HEcB/AfLiWUxGwNcd30JP6QVtvx+U1Wf2TkmjlFHpbMQ4Wk82yl
aUKtLCMyl25sDRmnFYZmeqCssLXWRl3m4c49/O5+9zvHKe53H+/tu+8/3+/7
89nLO+mGINmaIAj4fzZuc2Zgzk6jt0mmIGAzf6B58UHVIEHD1c3MQXvWzPD5
HD3U121kAX6doB+39hrhy9hG5GCMSN8ZsmAAc58Khm6rXQtPUux7NQTI9iXW
u+jYfgqqSzqipNto9r8W9hyfGMj2siLN82ioGXnpf/2ahu1joH5v0fRyLEGa
+wzQlitazzxt6TPCUFoTKc9j2L5V8Hs2XxuUrmX3N4Fvce4P1wWG7SPI7rXs
iRypFsY33xPkE8/4nu8yDbuvFXnvs+2bql4V+9+anOpsiHScVLHzbMgP7pd6
+Bf0bB+HFCZedKZUOrB4lk9Fhi12/kZPYQMhyHem0LOgJjiDO86gZ8Ip8fZh
HwN6VnT7tYZ/WkFPqciLP/toy7NNJKyOeWVAz2LPpBJvGw169u8Sh0VlacDi
GRDynpasqtHT5CHZ76tRgcVTVPj6Tqgtg55L7alvowso9BTQMNAUx6BndtSh
FAflCnreTPB1c4ik0HPsyLST0H0dPWejuVxJqhI9D0wGn63jUugJDbfe9beo
0HP3OWvHQZ0SPeV9xfe/1fxFz8v9Yp5sVIGeVxThxzrK1OgpjVCOcrr06Nls
b4q3i1tDz4yRzCH/Rho9fUJ3LDk/pdGzt1Mi90mh0PPnLvJFZeCW55p/bT5n
Zstz+SN/xPUrhZ4VYX1H7VON6Dl/uKWIH6hDz66SgcxaJxN6VhK8hynPDegp
uMuR+5Uy6ClOy3IJaqTQs+xMZbmbhkbPmIO8iZBVLXoq6niK5dgV9JRVDZd6
5GnhH9CCzc8=
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 0.6000000000000001},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.5840960970514723, 2.14084385941397}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF0n8s1GEcB/BzDquWldKOHRKzsTWR2tHs891SNG1UVq4lkx837KY2hDUb
dWSkLdV1JgmtJlp/WKEYi+l0xCpCK79/tLvpvt/vfb/n7qLw+Hhvz55/Ps/7
efba43kl42yyUCAQwP+1tm9kAqaU+SOnxSwcXs80uKW7Stt6jJCynjlYuP3F
OvnOApXqtSyCWu5xfHmWgQHtWnQQnSbRaEMY2Oj7A6LsPHVlmInMGyDd33Yo
K8lM+hmYGPc4KrxoJedZ6HSYquudYcl9HAwGe8lyPnCkzwQ6ZaqhNpQlfcsQ
ENNfotlJkz4LtI9IfcaHjaTPCnsinPNfVFlI3wp48dtvvG7YfJ+Acsndpyqy
MNC/Pi+gAlt3OwZMG0i/DRUV9/Vjk5km54VUycxs3ctijtxnS7kLdavny3nS
J6Ls7t2qCbYRUpueWs9h+1OhZvSUPpTm6Zc59GSb3kvcC/+ip7PkpF9oJo+e
sQUrVboTHHqmPLsQZB/FoKfKcezOaBCNnhE6Uf33a3r03F+8ai2UWtAzyiWz
sWHUiJ7h2bFPu58w6LmYO7lXd4BFz+fn1ImV/JZni3ghgy4zoGdfjOdPH28e
PVNffbraMkijZyfrk17hxqBn8ZCo1z6RR8/7l5duPvjGoGeNKUHvb0ej5/UY
B3+VjIVNz/r56LnJ1iX0rPaVuf12WELPUflUZKPcip7Rc0zsvIcRPV1nfpkz
s7Y8I3f9yA17w6NnWU2Fd7uYR8/u5hA6TGFEz6Q8UXJ8tQk9Sz8/OrNDz6Kn
slkO2i4DelrmNUxa1wp6Zmx77KfqMKCnRFmkEPfR6PlW4Rs/5sSi50S55m5i
E4ue3rLwBMeDVvQcyDl2qLyHQ8+KjtIjXi48el6qCtS0FWz9T6mTwq8wjoN/
iCDTCg==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 1.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.9201472476482957, 2.250598196073272}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxF0m1IU1EYB/CrU8SXzMyab/Tm/BAIjixQKZ+riWllUhMr1MKZmmb4pcBK
MNTIsqTUshcKdbGyqWVkirC0lqlpCisDl4qp6Hxjm9vuvfM6Sd3dsz8czpdz
/g/8ztktzjuVYU8QBKyt9d2ScaBeP9Afm2chZCOT4CrwVuT20ZC5kWlYvV06
qpll4dnT9cxCiut3jU8zBQP961kA1YR56vekASx9WvAd6A6ayae58zo4mKas
DBw0cf16cHFKqPSrYLn7BnDKjhO6K43cPArih+UlRLCZ62MgJ/7ciMcjhusz
QWxLxuNvcUauj4XBgDOR0hiCtPStgOlmRsKOJIbrWwXJzuTOoWs010eQoYwz
3zxigJ8b5wlyX5851nVtnqXfjpwfqHbMjDJw9+3JE3cERPU7mpvHI9neYbeY
F1quz4EUHUhKT6lZAqtn+FS+t5JvQs+5IJHqSzpBWj3DwrIVijEKPete+jTk
1uvRMy3vg/qWcAk9K5SzEWUpGvTMmfjqGMKj0bMiOLVJIrN5lo2pwmW1Ns/N
h3iKv8km9DzZeC/qX4cOPata1c9dRmn0PK64TzvO6cDq6c8/rY0UUOj5Iz1+
ulVgRs+P4vYAjXIJPVvt3JtGxDr0pOqKBaHDDHr2CXi7shKN6Ol5VzVUrGPQ
s+OGPysuXEbP8uhtctERI3pGxFwcD9tiwv9ZoE6EniLb/ySLNml8Q1fQs7t8
2YvfQqNny1Vtc6dcj57SKemis5cBPRv5q2/+SBbRs1+YeHkx2vp+FLTPdSvk
vSx6Pnkf2FMlXEHP/RcKSs/+snkW8g9f6hqk0TOyRLYnqs2Ins7nRUcDty+j
p9/1h/UNe2n0HFqoaW8TM+jp8DlT5vHWjJ6fNFluM10Mer5SS3RSTyN6VtZf
2ZrqTcF/VqzTTQ==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 1.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.9262498188625067, 2.048709588683427}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}]}], "}"}]], "Output",
 CellChangeTimes->{{3.531794232627514*^9, 3.531794249124813*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Vrange", "=", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], "3"]}], 
     ",", 
     RowBox[{"Drange", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
       "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Table", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"dd", "/", "mm"}], ",", 
           RowBox[{"b", "//.", 
            RowBox[{"Last", "@", 
             RowBox[{"ratiomax", "[", 
              RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"rr", ",", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"2", ",", "4", ",", "8"}], "}"}], "ohms"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", 
       RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "]"}], 
     "\[IndentingNewLine]", ",", 
     RowBox[{"{", 
      RowBox[{"vv", ",", "Vrange"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Rrange", "=", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2", ",", "4", ",", "8"}], "}"}], "ohms"}]}], ",", 
     RowBox[{"Drange", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
       "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Table", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"dd", "/", "mm"}], ",", 
           RowBox[{"b", "//.", 
            RowBox[{"Last", "@", 
             RowBox[{"ratiomax", "[", 
              RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"vv", ",", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], 
            "3"]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", 
       RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", "]"}], 
     "\[IndentingNewLine]", ",", 
     RowBox[{"{", 
      RowBox[{"rr", ",", "Rrange"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
  "]"}]}], "Input",
 CellChangeTimes->{{3.5317942433280487`*^9, 3.5317942543188*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7Jl3byg0OPrV3hgMHtvPVOf6nKv7xz4NDJ7Z
f5Ced0Eph8Nh1kwQeGmvu0l9seMDTodzZ0DgjX36V67GwBkcDhDzPthv3jl3
Uq8wTP1H+4Lqd9kl0zgcIOZ/tveOPKg1yZYdqv+L/W/Hy9wPjgk5QOz7Zq+1
/9Rt5WhhqHk/7DMlDtUu1pCAmvfTXlhrb6tnhhTUvN/2Ulm6miXOElDz/tiv
OHiaqUpMDGreP/v3/2suHAyTh5rH4BCW1Pr8eIOsw1mwegaHjVrTlFo2KkDN
Z3SoXeZrobhcEaqfyUEgUnlWc50S1D5mhwI3n/Mlz1Wh5rE42El8eLYuQ90B
Fp63rj/YlGbO5gALT3MHy3A7ZgYHWHg+d+J8vFCBEx6ey/8wWPzyZYeHZ1VU
U/aiZkR4Gr/b8NLpNSc8PJte5xtqVQnBw9NS5UZOgLc4PDxf23tzCKeIwcMz
hJEh1PW2DDw8+6Ypmq2OR4Rn5CWOC47vEOG5Z5ZVpcNpeXh4nvU4M9N4uzI8
PB/nTN50/IMiPDwXSVSLOSQqw8PTY4YTt9h8RXh4crwVr7G0U4aHJ+scx39z
tqvCw/PZidCJPt9U4OGpPC/8c/djQ3h4Xr9TPf1e+394+ryT93hx0WQOeHge
LGAQenoGEZ4q3r81279ywMMz1SHqlIi0EDw8v2WvKDgVKg4Pz0un4zw+1UnC
w1Mp2q9ppZkkPDxdjnzfmjtZBh6eFrXT7U/VKcLD8+6bh4kMaUrw8Jzq9e7x
kSJFeHiGP/FRbfmuDg/PJTqTw3QlVODhuWHO3dXFsZrw8BRZavjrVokKPDzP
srJp8F7WgIfnH9EVO1PUtOHhyWs4xbfOTw8enlWt9fECBwzh4Rl16mXI+UZ9
BwD2GJxA
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 5.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {1.3606419030217587`, 17.88885422600846}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7I+e3tFes/CBvTEYPLa/+VDrDYPqB/s0MHhm
3/jtLgOfK6fDrJkg8NLeqFhqrf6KX/bnzoDAG/uA58d7/vhwOEDM+2Aff+vD
HotkJqj6j/YbmvxO+X7lcICY/9n+0P2i8KCnbA4Q/V/se7I2qMWVsDtA7Ptm
HzKjyP1BJjvUvB/2y31UbztXwuz/aR//3WP3wQBhqHm/7Wd+TTkyz0AYat4f
+1qnt878GhJQ8/7Zp8e+Sq79JgU1j8GB43fM7BPNYg5nweoZHLb3yE0u7JCG
ms/ocPVIwz3XpZJQ/UwOHzikzXP5xaH2MTuc0mSo2ygoCTWPxWG27sKb0wUU
HGDhubdJsCLQ7BM8PMOqPjiI/PsOD897/prm36y44OHJKjNxQpI7hwMsPMP+
3dtdNAsRnjwNtZHJ29jh4Sktb237Yxk7PDwDvb5wJR9HhGfUldptHaf54eHJ
cTfOS+2LGDw8f7o/PCdpIwwPT6MUFR2DlaLw8HQPmLjv6ylJeHieDFFraOeU
gYenXf3RSXZeiPA0z+BpKHGXgYfnT8HMe2t2ycPDM7T4xm7ua8rw8PyzY0v3
dAMFeHiqLpy8rOiUIjw8z22b6xajqwoPz3lXY168/QFLP4/tje69Xr5ZjNsB
Fp6fN1QqRs7ngIfnjR8JllNy2eHhuVi6blIaHzs8PEO3NN/q28sGD08r7el5
1mFC8PC8I11kYZknBg9PHaGuGy7XReDhmcQc9PFJrBQ8PFd8tzOZfEgCHp49
vHX8n5lk4OGpl9D+be5iWXh4nvl/U3CFsCI8PN1tzxcyr1CBh+fWxDQ9T38l
eHjyaOX837pcGR6e1tfjMz/vUoOH57HVkzx7+1Xh4bkj9833uggdeHjWtQY/
ZN6g6wAAjjKhQQ==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.51971270092178, 14.843779603429308`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7L2/S9hw3LhhbwwGj+13pdbvYnr81D4NDJ7Z
F1jxb7MX/WA/ayYIvLTfPWVNhYHVT/tzZ0Dgjf3ne5se2WX+s4eY98F+5xkV
h+pNHA4Q9R/tV82b3OIazeEAMf+zvUtn8N7/yWwOEP1f7B80RySUqrI7QOz7
Zv+kqJR/82k2B4h5P+yjVh9U/HeCHWreT/tGltiHb+QEoeb9trd79FstW4sD
at4fe9G1h5ZKXhGAmvfPvv0xf9TsJ8JQ8xgcdl3Qe/TJQdzhLFg9g8MapaOx
0WrCUPMZHRK+FOYdXiUK1c/ksPg7t5gkkxjUPmaHBc2P1NNjJKHmsThsSLn8
7803CQdYeC6rfZToVv0RHp5LDrze/q/+Azw8b0Ve+9xd8Qsenrceaz1M8f4D
D89nlY3Vr8s4HGDh+S4s4OV7YU54eC743CF7TIITHp4Bd6cW6AVxwMPz0e6m
G1lXOeDh+fQTa+okOz54eHJnrlqmGs8BD8/s62vrZsSKwsOz4cHNXtFMYXh4
qnXwicyJEIKHZ9xEoSXz4yTh4Smhoc3dVy4FD89O60Z97n5ZeHiGlXnG8Z+S
hofnc/PDgoaCCvDw1EvZ4PojVBkengt6HN+kfFeAh6c+M1esheVneHg+WWZ+
/tNeTgdYeE6r2LRqFxOXAyw8PT++jjP7AUtfb+znKB6pN32KCM+nDx2bBBez
w8Mzxr23f2sbIn0uXaboKrockT4rVD4GuR8VhYenzaMM7xIDCXh4fpNY0H/x
vSg8PM17Je8vqZKEh+d9zpbDIS6y8PAMkXL9KHpJGh6eawM7fhmHKcDDs1Zv
esFTMSV4eN5eVvvibYoiPDz1Wr09X+grwcPT4PC8I8W8KvDwfKbhfKMsShUe
npJvqyanLVZ1AAANJqeE
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.388185557068813, 10.819141968494067`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}]}], "}"}]], "Output",
 CellChangeTimes->{3.5317942558554163`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7Jl3byg0OPrV3hgMHtvPVOf6nKv7xz4NDJ7Z
f5Ced0Eph8Nh1kwQeGmvu0l9seMDTodzZ0DgjX36V67GwBkcDhDzPthv3jl3
Uq8wTP1H+4Lqd9kl0zgcIOZ/tveOPKg1yZYdqv+L/W/Hy9wPjgk5QOz7Zq+1
/9Rt5WhhqHk/7DMlDtUu1pCAmvfTXlhrb6tnhhTUvN/2Ulm6miXOElDz/tiv
OHiaqUpMDGreP/v3/2suHAyTh5rH4BCW1Pr8eIOsw1mwegaHjVrTlFo2KkDN
Z3SoXeZrobhcEaqfyUEgUnlWc50S1D5mhwI3n/Mlz1Wh5rE42El8eLYuQ90B
Fp5HT+9or1n4AB6eNx9qvWFQ/QAPz8Zvdxn4XDnh4WlULLVWf8Uve1h4Bjw/
3vPHBxGe8bc+7LFIZoKH54Ymv1O+XxHheeh+UXjQUzZ4ePZkbVCLK2GHh2fI
jCL3B5ns8PBc7qN627mSEx6e8d89dh8MEIaH58yvKUfmGQjDw7PW6a0zPzD8
YeGZHvsqufabFDw8OX7HzD7RLAYPz+09cpMLO6Th4Xn1SMM916WS8PD8wCFt
nssvDg/PU5oMdRsFJeHhOVt34c3pAgrw8PT+LmHDceMGPDx3pdbvYnr8FB6e
BVb82+xFP9jDwnP3lDUVBlY/4eH5+d6mR3aZ/+xh4bnzjIpD9SZE+lw1b3KL
azQiPF06g/f+T0aE54PmiIRSVUR4Pikq5d98mg0enlGrDyr+O8EOD89GltiH
b+QE4eFp9+i3WrYWBzw8RdceWip5RQAenu2P+aNmPxGGh+euC3qPPjmIw8Nz
jdLR2Gg1YXh4JnwpzDu8ShQenou/c4tJMonBw3NB8yP19BhEeG5IufzvzTcJ
BwAl36V8
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 2.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {0.388185557068813, 11.704459397133288`}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7G9df7ApzZzNwRgMHtubO1iG2zEzOKSBwTP7
506cjxcqcDrMmgkCL+2X/2Gw+OXL7nDuDAi8sa+Kaspe1MzhADHvg73xuw0v
nV7D1H+0b3qdb6hVJQQ1/7O9pcqNnABvcaj+L/av7b05hFPEoPZ9sw9hZAh1
vS0DNe+Hfd80RbPV8RJQ837aR17iuOD4Tgpq3m/7PbOsKh1Oy0PN+2N/1uPM
TOPtylDz/tk/zpm86fgHRah5DA6LJKrFHBKVHc6C1TM4eMxw4habrwg1n9GB
4614jaUdTD+TA+scx39ztqtC7WN2eHYidKLPNxWoeSwOyvPCP3c/NnSAhefe
JsGKQLNP9rDwDKv64CDy77s9LDzv+Wuaf7Pigocnq8zECUnuHPDwDPt3b3fR
LER48jTURiZvY4eHp7S8te2PZezw8Az0+sKVfJwNHp5RV2q3dZzmh4cnx904
L7UvYvDw/On+8JykjTA8PI1SVHQMVorCw9M9YOK+r6ck4eF5MkStoZ1TBh6e
dvVHJ9l5ScHD0zyDp6HEXQYenj8FM++t2SUPD8/Q4hu7ua8hwvPPji3d0w0U
4OGpunDysqJTivDwPLdtrluMrio8PJfVPkp0q/4ID88lB15v/1f/AR6etyKv
fe6u+GUPC89bj7Uepnj/sYeF57PKxurXZYjwfBcW8PK9MCJ9LvjcIXtMghMe
ngF3pxboBXHAw/PR7qYbWVc54OH59BNr6iQ7Pnh4cmeuWqYazwEPz+zra+tm
xCLCs+HBzV7RTGF4eKp18InMiRCCh2fcRKEl8+Mk4eEpoaHN3VcuBQ/PTutG
fe5+WXh4hpV5xvGfkoaH53Pzw4KGgojw1EvZ4PojVBkengt6HN+kfFdwAADL
bZzc
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 5.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {1.0312487778101422`, 17.88885422600846}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}], ",", 
   GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7K/fqZ5+r/2/vTEYPLa/k/d4cdFkDoc0MHhm
f7CAQejpGU6HWTNB4KW9ivdvzfavHA7nzoDAG/tUh6hTItJCDhDzPth/y15R
cCpUHKr+o/2l03Een+okHSDmf7ZXivZrWmkmCdX/xd7lyPetuZNloPZ9s7eo
nW5/qk4Rat4P+7tvHiYypClBzftpP9Xr3eMjRYpQ837bhz/xUW35rg4174/9
Ep3JYboSKlDz/tlvmHN3dXGsJtQ8BgeRpYa/bpWoOJwFq2dwOMvKpsF7WQNq
PqPDH9EVO1PUtKH6mRx4Daf41vnpQe1jdqhqrY8XOGAINY8F6PuXIecb9R1g
4TnvasyLtz84HGDhaXTv9fLNYtzw8Py8oVIxcj4HPDxv/EiwnJLLDg/PxdJ1
k9L42OHhGbql+VbfXjZ4eFppT8+zDhOCh+cd6SILyzwxeHjqCHXdcLkuAg/P
JOagj09ipeDhueK7ncnkQxLw8OzhreP/zCQDD0+9hPZvcxfLwsPzzP+bgiuE
FeHh6W57vpB5hQo8PLcmpul5+ivBw5NHK+f/1uXK8PC0vh6f+XmXGjw8j62e
5NnbrwoPzx25b77XRejAw7OuNfgh8wZdeHjqM3PFWlh+hqfPJ8vMz3/aywkP
z2kVm1btYuKCh6fnx9dxZj/Y4OE5R/FIvelTDnh4Pn3o2CS4mB0enjHuvf1b
2zjg4bl0maKr6HI2eHhWqHwMcj8qCg9Pm0cZ3iUGEvDw/CaxoP/ie1F4eJr3
St5fUiUJD8/7nC2HQ1wQ4Rki5fpR9JI0PDzXBnb8Mg5TgIdnrd70gqdiiPC8
vaz2xdsURXh46rV6e77QV4KHp8HheUeKeVXg4flMw/lGWZQqPDwl31ZNTlus
6gAAJiairQ==
     "], {{{}, {}, 
       {Hue[0.67, 0.6, 0.6], 
        LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
         18, 19, 20, 21}]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], 
        LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
         37, 38, 39, 40, 41, 42}]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], 
        LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
         58, 59, 60, 61, 62, 63}]}}, {
       {Hue[0.67, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 1], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 2], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 3], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 4], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 5], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 6], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 7], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 8], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 9], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 10], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 11], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 12], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 13], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 14], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 15], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 16], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 17], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 18], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 19], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 20], InsetBox[
         StyleBox["\<\"\[FilledCircle]\"\>",
          StripOnInput->False,
          FontSize->6], 21]}, 
       {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 22], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 23], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 24], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 25], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 26], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 27], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 28], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 29], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 30], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 31], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 32], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 33], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 34], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 35], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 36], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 37], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 38], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 39], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 40], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 41], InsetBox[
         StyleBox["\<\"\[FilledSquare]\"\>",
          StripOnInput->False,
          FontSize->6], 42]}, 
       {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 43], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 44], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 45], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 46], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 47], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 48], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 49], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 50], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 51], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 52], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 53], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 54], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 55], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 56], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 57], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 58], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 59], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 60], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 61], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 62], InsetBox[
         StyleBox["\<\"\[FilledDiamond]\"\>",
          StripOnInput->False,
          FontSize->6], 63]}, {}}}],
    AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
    AxesOrigin->{0.5, 5.},
    Frame->True,
    PlotRange->{{0.5, 2.5}, {1.2014697679844166`, 17.75024983275714}},
    PlotRangeClipping->True,
    PlotRangePadding->{
      Scaled[0.02], 
      Scaled[0.02]}]}], "}"}]], "Output",
 CellChangeTimes->{3.531794256094636*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"annot", "[", 
   RowBox[{"s_", ",", "t_"}], "]"}], ":=", 
  RowBox[{"PSfrag", "[", 
   RowBox[{"s", ",", 
    RowBox[{"TeXCommand", "\[Rule]", 
     RowBox[{
     "\"\<\\\\fboxsep=1pt\\\\footnotesize\\\\colorbox{white}{\>\"", "<>", "t",
       "<>", "\"\<}\>\""}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"magRatioPlot", "=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"vv", "=", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"20", "mm"}], ")"}], "3"]}], ",", 
      RowBox[{"Drange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
        "mm"}]}], ",", 
      RowBox[{"Rrange", "=", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"2", ",", "4", ",", "8"}], "}"}], "ohms"}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Show", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"dd", "/", "mm"}], ",", 
              RowBox[{"a", "//.", 
               RowBox[{"Last", "@", 
                RowBox[{"ratiomax", "[", 
                 RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}]}], "}"}], 
            ",", 
            RowBox[{"{", 
             RowBox[{"rr", ",", "Rrange"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Joined", "\[Rule]", "True"}], ",", 
          RowBox[{"PlotMarkers", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
          RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
          RowBox[{"Axes", "\[Rule]", "False"}], ",", 
          RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", 
         "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{"annot", "[", 
              RowBox[{
               RowBox[{"ToString", "[", "#", "]"}], ",", 
               RowBox[{"\"\<$\\\\resistanceCoil=\\\\SI{\>\"", "<>", 
                RowBox[{"ToString", "[", "#", "]"}], "<>", 
                "\"\<}{\\\\ohm}$\>\""}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"0.9", ",", 
               RowBox[{"0.1", "+", 
                RowBox[{"(", 
                 RowBox[{"a", "//.", 
                  RowBox[{"Last", "@", 
                   RowBox[{"ratiomax", "[", 
                    RowBox[{
                    RowBox[{"0.9", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}], 
                 ")"}]}]}], "}"}]}], "]"}]}], "&"}], "[", "2", "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{"annot", "[", 
              RowBox[{
               RowBox[{"ToString", "[", "#", "]"}], ",", 
               RowBox[{"\"\<\\\\SI{\>\"", "<>", 
                RowBox[{"ToString", "[", "#", "]"}], "<>", 
                "\"\<}{\\\\ohm}\>\""}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"1.2", ",", 
               RowBox[{"(", 
                RowBox[{"a", "//.", 
                 RowBox[{"Last", "@", 
                  RowBox[{"ratiomax", "[", 
                   RowBox[{
                    RowBox[{"1.2", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}], 
                ")"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}]}], "]"}]}], "&"}], "[", "4", 
         "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{"annot", "[", 
              RowBox[{
               RowBox[{"ToString", "[", "#", "]"}], ",", 
               RowBox[{"\"\<\\\\SI{\>\"", "<>", 
                RowBox[{"ToString", "[", "#", "]"}], "<>", 
                "\"\<}{\\\\ohm}\>\""}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", 
               RowBox[{"(", 
                RowBox[{"a", "//.", 
                 RowBox[{"Last", "@", 
                  RowBox[{"ratiomax", "[", 
                   RowBox[{
                    RowBox[{"1", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}], 
                ")"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}]}], "]"}]}], "&"}], "[", "8", 
         "]"}]}], "\[IndentingNewLine]", "}"}], ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\"\<Wire diameter, mm\>\"", ",", 
         RowBox[{"PSfrag", "[", 
          RowBox[{"\"\<Magnet ratio\>\"", ",", 
           RowBox[{
           "TeXCommand", "->", "\"\<Magnet ratio $\\\\ratioMag$\>\""}]}], 
          "]"}]}], "}"}]}], ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"8", " ", 
        RowBox[{"72", "/", "2.54"}]}]}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"coilRatioPlot", "=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"vv", "=", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"20", "mm"}], ")"}], "3"]}], ",", 
      RowBox[{"Drange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
        "mm"}]}], ",", 
      RowBox[{"Rrange", "=", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"2", ",", "4", ",", "8"}], "}"}], "ohms"}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Show", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"dd", "/", "mm"}], ",", 
              RowBox[{"b", "//.", 
               RowBox[{"Last", "@", 
                RowBox[{"ratiomax", "[", 
                 RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}]}]}], "}"}], 
            ",", 
            RowBox[{"{", 
             RowBox[{"rr", ",", "Rrange"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"dd", ",", "Drange"}], "}"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Joined", "\[Rule]", "True"}], ",", 
          RowBox[{"PlotMarkers", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
          RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
          RowBox[{"Axes", "\[Rule]", "False"}], ",", 
          RowBox[{"Frame", "\[Rule]", "True"}]}], "\[IndentingNewLine]", 
         "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{"annot", "[", 
              RowBox[{
               RowBox[{"ToString", "[", "#", "]"}], ",", 
               RowBox[{"\"\<$\\\\resistanceCoil=\\\\SI{\>\"", "<>", 
                RowBox[{"ToString", "[", "#", "]"}], "<>", 
                "\"\<}{\\\\ohm}$\>\""}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"2", ",", 
               RowBox[{"(", 
                RowBox[{"b", "//.", 
                 RowBox[{"Last", "@", 
                  RowBox[{"ratiomax", "[", 
                   RowBox[{
                    RowBox[{"2", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}], 
                ")"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "1.5"}], "}"}]}], "]"}]}], "&"}], "[", "2", 
         "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{"annot", "[", 
              RowBox[{
               RowBox[{"ToString", "[", "#", "]"}], ",", 
               RowBox[{"\"\<\\\\SI{\>\"", "<>", 
                RowBox[{"ToString", "[", "#", "]"}], "<>", 
                "\"\<}{\\\\ohm}\>\""}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"2", ",", 
               RowBox[{"(", 
                RowBox[{"b", "//.", 
                 RowBox[{"Last", "@", 
                  RowBox[{"ratiomax", "[", 
                   RowBox[{
                    RowBox[{"2", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}], 
                ")"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", 
               RowBox[{"-", "1"}]}], "}"}]}], "]"}]}], "&"}], "[", "4", "]"}],
         ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"Text", "[", 
            RowBox[{
             RowBox[{"annot", "[", 
              RowBox[{
               RowBox[{"ToString", "[", "#", "]"}], ",", 
               RowBox[{"\"\<\\\\SI{\>\"", "<>", 
                RowBox[{"ToString", "[", "#", "]"}], "<>", 
                "\"\<}{\\\\ohm}\>\""}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"1.9", ",", 
               RowBox[{"(", 
                RowBox[{"b", "//.", 
                 RowBox[{"Last", "@", 
                  RowBox[{"ratiomax", "[", 
                   RowBox[{
                    RowBox[{"1.9", "mm"}], ",", "#", ",", "vv"}], "]"}]}]}], 
                ")"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", 
               RowBox[{"-", "1"}]}], "}"}]}], "]"}]}], "&"}], "[", "8", 
         "]"}]}], "\[IndentingNewLine]", "}"}], ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\"\<Wire diameter, mm\>\"", ",", 
         RowBox[{"PSfrag", "[", 
          RowBox[{"\"\<Coil ratio\>\"", ",", 
           RowBox[{
           "TeXCommand", "->", "\"\<Coil ratio $\\\\ratioCoil$\>\""}]}], 
          "]"}]}], "}"}]}], ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"8", " ", 
        RowBox[{"72", "/", "2.54"}]}]}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.531789281132059*^9, 3.531789547217321*^9}, {
   3.531789597115576*^9, 3.531789621231968*^9}, {3.531792695681196*^9, 
   3.531792768902416*^9}, 3.5317928095537643`*^9, {3.5317937774709377`*^9, 
   3.531793877350198*^9}, {3.531793907885622*^9, 3.5317941212349243`*^9}, {
   3.531795847341455*^9, 3.5317960542102823`*^9}, {3.531796091181736*^9, 
   3.5317961200889378`*^9}, {3.531796176250655*^9, 3.531796200443718*^9}, {
   3.531798671331854*^9, 3.5317986717848377`*^9}, 3.531798707716488*^9}],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxF0G1IU1EYB/Cr0wlhGlJMYZqbQ0Ikg/lhSnpuUllkiW9sUoZupKwEs6yW
RpISgi8RZokmmcs0U0eFpFimuSDwJUpScpU0C1GbbPfe7d4750upx2d/OJwv
z/M/h59EXZB6zpMgCPT/bNxb+YUqJlOSFl/MIvlmfiNVE5FaHMCg3M3MoZK6
+HzpOI8aGzaygLJj9D7D4U70aWwjFlTZs//ZqZFltNVnQ691oQfNdxk8T6EO
nepOZpcT9zPoZoi6VCKg8b4dDfjpkzIKafwei+QJRrZthcJ9PFoLbtsbSdtw
nxPpbrwsOyzkcZ8L/TVo+5UlDO5bQaksetecxeO+NXQpY1+er3X7fwR5OTtS
5JvCoPHNeYIci5japRKv434P0qyUStu0VrzvSYZ9jU9skDL4PQGJmq68GWi1
4T4v0v+E584hhxVte45JpoTH45bBU/FAUbzkZMHT3v1WHFK2Cp57xEcj4oo4
8FTdWmuyHGHBM/epMlqY7Pas9zPVTEfT4HnM4tX6rXAJPEMr1lfKFC7wTA4q
6no+7QDPxKuqlg/NDHguXDfvtkjt4Nme1qBp5Bzg2Rc4X0BXU+A5ki6ZCZdx
4Kk1jF7s+0yD55A9/MK9YAY8K754fRRqOPCsO2stvz/p9nzM5yxFedPgeS3d
J6o+0w6esX90gRMiJ3guRqaZ3msIctszJkZrNM6w4Kl/FNSV38GAZ07Bq/nb
B2jwrJ1YiK86YwXP87PD3nIBB561UVmGJ50u8KyaMcV2tjjA0z9OYPx+2gme
Kd3VCeZBCjzreucf7vjJgWeSsYbzXqTAUyxS2g7JWLen5uRcr2wVPHvU/WHW
Cbdnr4ef4YeaAk9WXy5TTPPgOSoThOalO8AzoNI0WU7x4DlYInapS5fRPzL3
zYs=
    "], {{{}, {}, 
      {Hue[0.67, 0.6, 0.6], 
       LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
         19, 20, 21}]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], 
       LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
        37, 38, 39, 40, 41, 42}]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], 
       LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
        58, 59, 60, 61, 62, 63}]}}, {
      {Hue[0.67, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 2], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 3], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 4], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 5], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 7], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 8], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 9], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 10], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 11], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 12], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 13], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 14], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 15], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 16], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 17], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 18], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 19], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 20], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 21]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 22], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 23], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 24], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 25], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 26], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 27], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 28], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 29], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 30], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 31], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 32], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 33], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 34], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 35], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 36], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 37], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 38], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 39], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 40], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 41], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 42]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 43], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 44], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 45], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 46], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 47], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 48], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 49], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 50], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 51], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 52], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 53], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 54], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 55], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 56], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 57], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 58], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 59], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 60], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 61], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 62], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 63]}, {}}}], InsetBox[
    FormBox["\<\"2\"\>",
     TraditionalForm], {0.9, 1.7743914193751005`}, {0, 0}, Automatic, {1, 0}],
    InsetBox[
    FormBox["\<\"4\"\>",
     TraditionalForm], {1.2, 0.9201472476482957}, {0, 1}, Automatic, {1, 0}], 
   InsetBox[
    FormBox["\<\"8\"\>",
     TraditionalForm], {1, 0.9800468663002724}, {0, 1}, Automatic, {1, 0}]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{0.5, 0.6000000000000001},
  Frame->True,
  FrameLabel->{
    FormBox["\"Wire diameter, mm\"", TraditionalForm], 
    FormBox[
     FormBox["\"Magnet ratio\"", TraditionalForm], TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->{{0.5, 2.5}, {0.5840960970514723, 2.048709588683427}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.531789313783682*^9, 3.531789331601789*^9}, {
   3.53178940614734*^9, 3.531789456806572*^9}, {3.531789532693316*^9, 
   3.531789547919001*^9}, 3.5317896105744534`*^9, {3.531792702839559*^9, 
   3.5317927104844007`*^9}, 3.531792770236305*^9, 3.531792810695086*^9, {
   3.531793781025792*^9, 3.531793878565318*^9}, {3.531793910855937*^9, 
   3.5317939499118967`*^9}, {3.531794005859108*^9, 3.531794063100502*^9}, {
   3.531794105474083*^9, 3.531794122035346*^9}, {3.531795849068141*^9, 
   3.5317959147526617`*^9}, {3.531795947553926*^9, 3.531795997962736*^9}, 
   3.531796055280178*^9, {3.531796093358783*^9, 3.5317961217826567`*^9}, {
   3.531796185502111*^9, 3.531796200885903*^9}, 3.53179721791033*^9, 
   3.531798672646532*^9, 3.531798708990356*^9}],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCwB2IQDQEP7I+e3tFes/CBvTEYPLa/+VDrDYPqB/s0MHhm
3/jtLgOfK6fDrJkg8NLeqFhqrf6KX/bnzoDAG/uA58d7/vhwOEDM+2Aff+vD
HotkJqj6j/YbmvxO+X7lcICY/9n+0P2i8KCnbA4Q/V/se7I2qMWVsDtA7Ptm
HzKjyP1BJjvUvB/2y31UbztXwuz/aR//3WP3wQBhqHm/7Wd+TTkyz0AYat4f
+1qnt878GhJQ8/7Zp8e+Sq79JgU1j8GB43fM7BPNYg5nweoZHLb3yE0u7JCG
ms/ocPVIwz3XpZJQ/UwOHzikzXP5xaH2MTuc0mSo2ygoCTWPxWG27sKb0wUU
HGDhubdJsCLQ7BM8PMOqPjiI/PsOD897/prm36y44OHJKjNxQpI7hwMsPMP+
3dtdNAsRnjwNtZHJ29jh4Sktb237Yxk7PDwDvb5wJR9HhGfUldptHaf54eHJ
cTfOS+2LGDw8f7o/PCdpIwwPT6MUFR2DlaLw8HQPmLjv6ylJeHieDFFraOeU
gYenXf3RSXZeiPA0z+BpKHGXgYfnT8HMe2t2ycPDM7T4xm7ua8rw8PyzY0v3
dAMFeHiqLpy8rOiUIjw8z22b6xajqwoPz3lXY168/QFLP4/tje69Xr5ZjNsB
Fp6fN1QqRs7ngIfnjR8JllNy2eHhuVi6blIaHzs8PEO3NN/q28sGD08r7el5
1mFC8PC8I11kYZknBg9PHaGuGy7XReDhmcQc9PFJrBQ8PFd8tzOZfEgCHp49
vHX8n5lk4OGpl9D+be5iWXh4nvl/U3CFsCI8PN1tzxcyr1CBh+fWxDQ9T38l
eHjyaOX837pcGR6e1tfjMz/vUoOH57HVkzx7+1Xh4bkj9833uggdeHjWtQY/
ZN6g6wAAjjKhQQ==
    "], {{{}, {}, 
      {Hue[0.67, 0.6, 0.6], 
       LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
         19, 20, 21}]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], 
       LineBox[{22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
        37, 38, 39, 40, 41, 42}]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], 
       LineBox[{43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 
        58, 59, 60, 61, 62, 63}]}}, {
      {Hue[0.67, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 2], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 3], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 4], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 5], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 7], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 8], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 9], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 10], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 11], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 12], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 13], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 14], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 15], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 16], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 17], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 18], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 19], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 20], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 21]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 22], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 23], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 24], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 25], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 26], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 27], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 28], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 29], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 30], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 31], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 32], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 33], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 34], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 35], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 36], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 37], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 38], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 39], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 40], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 41], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 42]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 43], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 44], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 45], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 46], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 47], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 48], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 49], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 50], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 51], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 52], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 53], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 54], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 55], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 56], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 57], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 58], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 59], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 60], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 61], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 62], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 63]}, {}}}], InsetBox[
    FormBox["\<\"2\"\>",
     TraditionalForm], {2, 5.628694942045293}, {0, 1.5}, Automatic, {1, 0}], 
   InsetBox[
    FormBox["\<\"4\"\>",
     TraditionalForm], {2, 7.069780350453633}, {1, -1}, Automatic, {1, 0}], 
   InsetBox[
    FormBox["\<\"8\"\>",
     TraditionalForm], {1.9, 10.32815127997186}, {0, -1}, Automatic, {1, 0}]},
  
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{0.5, 2.},
  Frame->True,
  FrameLabel->{
    FormBox["\"Wire diameter, mm\"", TraditionalForm], 
    FormBox[
     FormBox["\"Coil ratio\"", TraditionalForm], TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->{{0.5, 2.5}, {0.51971270092178, 14.843779603429308`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.531789313783682*^9, 3.531789331601789*^9}, {
   3.53178940614734*^9, 3.531789456806572*^9}, {3.531789532693316*^9, 
   3.531789547919001*^9}, 3.5317896105744534`*^9, {3.531792702839559*^9, 
   3.5317927104844007`*^9}, 3.531792770236305*^9, 3.531792810695086*^9, {
   3.531793781025792*^9, 3.531793878565318*^9}, {3.531793910855937*^9, 
   3.5317939499118967`*^9}, {3.531794005859108*^9, 3.531794063100502*^9}, {
   3.531794105474083*^9, 3.531794122035346*^9}, {3.531795849068141*^9, 
   3.5317959147526617`*^9}, {3.531795947553926*^9, 3.531795997962736*^9}, 
   3.531796055280178*^9, {3.531796093358783*^9, 3.5317961217826567`*^9}, {
   3.531796185502111*^9, 3.531796200885903*^9}, 3.53179721791033*^9, 
   3.531798672646532*^9, 3.531798709144979*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"PSfragExport", "[", 
   RowBox[{"\"\<fig/magcoil-ratiomax-a\>\"", ",", "magRatioPlot", ",", 
    RowBox[{"RenumberTags", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PSfragExport", "[", 
   RowBox[{"\"\<fig/magcoil-ratiomax-b\>\"", ",", "coilRatioPlot", ",", 
    RowBox[{"RenumberTags", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.531794136970467*^9, 3.531794151863718*^9}, {
  3.5317986758598537`*^9, 3.5317986760625887`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Effect of volume", "Subsection",
 CellChangeTimes->{{3.5327676141752777`*^9, 3.5327676220032682`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"FnFirst", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", "x"}], "\[IndentingNewLine]", 
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Vrange", "=", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"15", ",", "20", ",", "25"}], "}"}], "mm"}], ")"}], "3"]}], 
     ",", 
     RowBox[{"Rrange", "=", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "2", ",", "4", ",", "8", ",", "12", ",", "16", ",", "20", ",", "24", 
         ",", "28", ",", "32"}], "}"}], "ohms"}]}], ",", 
     RowBox[{"Drange", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"5", ",", "25", ",", "1"}], "]"}], "/", "10"}], " ", 
       "mm"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"x", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"rr", ",", 
         RowBox[{"Max", "[", 
          RowBox[{"FnFirst", "@@@", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Function", "[", 
              RowBox[{"dd", ",", 
               RowBox[{"ratiomax", "[", 
                RowBox[{"dd", ",", "rr", ",", "vv"}], "]"}]}], "]"}], "/@", 
             "Drange"}], ")"}]}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"vv", ",", "Vrange"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"rr", ",", "Rrange"}], "}"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"g1", "=", 
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{"x", ",", "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "6"}], "}"}]}], ",", 
       RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", 
       RowBox[{"Frame", "\[Rule]", "True"}], ",", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Coil resistance, Ohm\>\"", ",", 
          "\"\<Maximum peak force, N\>\""}], "}"}]}]}], "\[IndentingNewLine]",
       "]"}]}]}]}], "\[IndentingNewLine]", "]"}]}], "Input",
 CellChangeTimes->{{3.5319872031362267`*^9, 3.531987327523548*^9}, {
  3.531987361538599*^9, 3.531987416099774*^9}, {3.531987446603889*^9, 
  3.5319874602044287`*^9}, {3.531987513458009*^9, 3.5319876267254744`*^9}, {
  3.53198767137921*^9, 3.5319876846041594`*^9}, {3.531987773479803*^9, 
  3.531987821505211*^9}, {3.531988970301597*^9, 3.531988988641056*^9}, {
  3.5320376025703087`*^9, 3.532037604353537*^9}, {3.532037648681492*^9, 
  3.532037713771042*^9}, {3.532047265259912*^9, 3.532047269737534*^9}, {
  3.5320473025190353`*^9, 3.532047306328445*^9}, {3.532047339507326*^9, 
  3.5320475712132*^9}, {3.532047669390606*^9, 3.5320477082877483`*^9}, {
  3.532047780002222*^9, 3.5320478683220663`*^9}, {3.532047927906505*^9, 
  3.5320480161232157`*^9}, {3.532048310828573*^9, 3.5320485303597097`*^9}}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCQBmIQDQUOa9V5nkX8dnSAcAUcAnzcOxfVuUD5Cg58Wf+m
nJVyh/I1HMIrLaKvTPSA8g0crl5s+T+13RPKN3HYlimg2xbnBeVbOLSUfVsZ
KOEN5ds4KEn1W8+eDuM7OMSv4yj88QvGZ3DYZfO0NuZSINw9/6Lk/vC3h8Dd
s/DBLeGswnC4ezxOFy3/6h4Jd8+j3Y8ivtVHwd2zRCdq67rKaLh7pkgEK9a6
x8Ddc1PWVZHxJYzv4KB+w+Lp5eZYuHuOMFgp1r+C8QUcbt5ydtgQlgh3j8mr
PSmiailw93yKdppx6Voq3D1KdyLeLeBLh7vny+5J30/9gvEtHHZ2GaUmH8qA
u6ewMK9XNzsT7p5L5SkRf79kOgAAOHNw9Q==
   "], {{{}, {}, 
     {Hue[0.67, 0.6, 0.6], LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9}]}, 
     {Hue[0.9060679774997897, 0.6, 0.6], 
      LineBox[{10, 11, 12, 13, 14, 15, 16, 17, 18}]}, 
     {Hue[0.1421359549995791, 0.6, 0.6], 
      LineBox[{19, 20, 21, 22, 23, 24, 25, 26, 27}]}}, {
     {Hue[0.67, 0.6, 0.6], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 1], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 2], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 3], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 4], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 5], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 6], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 7], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 8], InsetBox[
       StyleBox["\<\"\[FilledCircle]\"\>",
        StripOnInput->False,
        FontSize->6], 9]}, 
     {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 10], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 11], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 12], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 13], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 14], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 15], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 16], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 17], InsetBox[
       StyleBox["\<\"\[FilledSquare]\"\>",
        StripOnInput->False,
        FontSize->6], 18]}, 
     {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 19], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 20], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 21], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 22], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 23], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 24], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 25], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 26], InsetBox[
       StyleBox["\<\"\[FilledDiamond]\"\>",
        StripOnInput->False,
        FontSize->6], 27]}, {}}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{5., 50.},
  Frame->True,
  FrameLabel->{
    FormBox["\"Coil resistance, Ohm\"", TraditionalForm], 
    FormBox["\"Maximum peak force, N\"", TraditionalForm]},
  PlotRange->{{2., 32.}, {35.9636504706717, 207.65592593786874`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.532048474826683*^9, 3.532048531009638*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"t1", "=", 
   RowBox[{"Graphics", "@", 
    RowBox[{"{", 
     RowBox[{"Text", "[", 
      RowBox[{
       RowBox[{"annot", "[", 
        RowBox[{
        "\"\<15\>\"", ",", "\"\<$\\\\volMag=(\\\\SI{15}{mm})^3$\>\""}], "]"}],
        ",", 
       RowBox[{"{", 
        RowBox[{"10", ",", "47"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"-", "1"}]}], "}"}]}], "]"}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"t2", "=", 
   RowBox[{"Graphics", "@", 
    RowBox[{"{", 
     RowBox[{"Text", "[", 
      RowBox[{
       RowBox[{"annot", "[", 
        RowBox[{"\"\<20\>\"", ",", "\"\<$(\\\\SI{20}{mm})^3$\>\""}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"10", ",", "98"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"-", "1"}]}], "}"}]}], "]"}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"t3", "=", 
   RowBox[{"Graphics", "@", 
    RowBox[{"{", 
     RowBox[{"Text", "[", 
      RowBox[{
       RowBox[{"annot", "[", 
        RowBox[{"\"\<25\>\"", ",", "\"\<$(\\\\SI{25}{mm})^3$\>\""}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"10", ",", "172"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"-", "1"}]}], "}"}]}], "]"}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"coilResistancePlot", "=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"g1", ",", "t1", ",", "t2", ",", "t3"}], "}"}], ",", 
    RowBox[{"ImageSize", "\[Rule]", 
     RowBox[{"8", " ", 
      RowBox[{"72", "/", "2.54"}]}]}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.5320485344771137`*^9, 3.532048569125773*^9}, {
  3.532048640613538*^9, 3.5320487566932793`*^9}, {3.532048797590313*^9, 
  3.532048836998288*^9}, {3.532048873352117*^9, 3.5320488734238663`*^9}}],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCQBmIQDQUOa9V5nkX8dnSAcAUcAnzcOxfVuUD5Cg58Wf+m
nJVyh/I1HMIrLaKvTPSA8g0crl5s+T+13RPKN3HYlimg2xbnBeVbOLSUfVsZ
KOEN5ds4KEn1W8+eDuM7OMSv4yj88QvGZ3DYZfO0NuZSINw9/6Lk/vC3h8Dd
s/DBLeGswnC4ezxOFy3/6h4Jd8+j3Y8ivtVHwd2zRCdq67rKaLh7pkgEK9a6
x8Ddc1PWVZHxJYzv4KB+w+Lp5eZYuHuOMFgp1r+C8QUcbt5ydtgQlgh3j8mr
PSmiailw93yKdppx6Voq3D1KdyLeLeBLh7vny+5J30/9gvEtHHZ2GaUmH8qA
u6ewMK9XNzsT7p5L5SkRf79kOgAAOHNw9Q==
    "], {{{}, {}, 
      {Hue[0.67, 0.6, 0.6], LineBox[{1, 2, 3, 4, 5, 6, 7, 8, 9}]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], 
       LineBox[{10, 11, 12, 13, 14, 15, 16, 17, 18}]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], 
       LineBox[{19, 20, 21, 22, 23, 24, 25, 26, 27}]}}, {
      {Hue[0.67, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 1], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 2], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 3], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 4], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 5], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 6], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 7], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 8], InsetBox[
        StyleBox["\<\"\[FilledCircle]\"\>",
         StripOnInput->False,
         FontSize->6], 9]}, 
      {Hue[0.9060679774997897, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 10], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 11], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 12], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 13], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 14], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 15], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 16], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 17], InsetBox[
        StyleBox["\<\"\[FilledSquare]\"\>",
         StripOnInput->False,
         FontSize->6], 18]}, 
      {Hue[0.1421359549995791, 0.6, 0.6], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 19], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 20], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 21], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 22], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 23], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 24], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 25], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 26], InsetBox[
        StyleBox["\<\"\[FilledDiamond]\"\>",
         StripOnInput->False,
         FontSize->6], 27]}, {}}}], InsetBox[
    FormBox["\<\"15\"\>",
     TraditionalForm], {10, 47}, {0, -1}, Automatic, {1, 0}], InsetBox[
    FormBox["\<\"20\"\>",
     TraditionalForm], {10, 98}, {0, -1}, Automatic, {1, 0}], InsetBox[
    FormBox["\<\"25\"\>",
     TraditionalForm], {10, 172}, {0, -1}, Automatic, {1, 0}]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{5., 50.},
  Frame->True,
  FrameLabel->{
    FormBox["\"Coil resistance, Ohm\"", TraditionalForm], 
    FormBox["\"Maximum peak force, N\"", TraditionalForm]},
  ImageSize->226.77165354330708`,
  PlotRange->{{2., 32.}, {35.9636504706717, 207.65592593786874`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.5320487099671583`*^9, 3.532048758088972*^9, {3.532048798647592*^9, 
   3.532048838473996*^9}, 3.532048873925396*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"PSfragExport", "[", 
   RowBox[{"\"\<fig/magcoil-resistance\>\"", ",", "coilResistancePlot", ",", 
    RowBox[{"RenumberTags", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.5320377089128447`*^9, 3.532037723476575*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1209, 756},
WindowMargins->{{Automatic, 90}, {116, Automatic}},
ShowSelection->True,
TaggingRules->{"GraphicsSelectionType" -> {}},
CellContext->Notebook,
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (November 13, \
2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 141, 3, 76, "Title"],
Cell[CellGroupData[{
Cell[745, 29, 24, 0, 67, "Section"],
Cell[CellGroupData[{
Cell[794, 33, 530, 11, 73, "Input"],
Cell[1327, 46, 584, 8, 27, "Output"]
}, Closed]],
Cell[1926, 57, 286, 5, 67, "Input"],
Cell[CellGroupData[{
Cell[2237, 66, 94, 1, 34, "Subsection"],
Cell[2334, 69, 1168, 26, 208, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3551, 101, 55, 0, 67, "Section"],
Cell[3609, 103, 173, 4, 41, "Text"],
Cell[3785, 109, 1424, 28, 238, "Input"],
Cell[CellGroupData[{
Cell[5234, 141, 1821, 39, 298, "Input"],
Cell[7058, 182, 263, 4, 20, "Print"],
Cell[7324, 188, 1191, 19, 27, "Output"],
Cell[8518, 209, 281, 4, 20, "Print"],
Cell[8802, 215, 1190, 19, 27, "Output"],
Cell[9995, 236, 271, 4, 20, "Print"],
Cell[10269, 242, 1189, 19, 27, "Output"],
Cell[11461, 263, 266, 4, 20, "Print"],
Cell[11730, 269, 1188, 19, 27, "Output"]
}, Open  ]],
Cell[12933, 291, 1756, 31, 328, "Input"],
Cell[CellGroupData[{
Cell[14714, 326, 1424, 29, 43, "Input"],
Cell[16141, 357, 1362, 19, 20, "Print"],
Cell[17506, 378, 1423, 22, 27, "Output"],
Cell[18932, 402, 1351, 19, 20, "Print"],
Cell[20286, 423, 1426, 22, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21749, 450, 1210, 33, 73, "Input"],
Cell[22962, 485, 130, 1, 20, "Print"],
Cell[23095, 488, 193, 4, 27, "Output"],
Cell[23291, 494, 127, 1, 20, "Print"],
Cell[23421, 497, 196, 4, 27, "Output"],
Cell[23620, 503, 138, 1, 20, "Print"],
Cell[23761, 506, 194, 4, 27, "Output"],
Cell[23958, 512, 127, 1, 20, "Print"],
Cell[24088, 515, 194, 4, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24319, 524, 472, 12, 27, "Input"],
Cell[24794, 538, 241, 7, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[25072, 550, 460, 12, 27, "Input"],
Cell[25535, 564, 218, 3, 20, "Print"],
Cell[25756, 569, 300, 6, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[26105, 581, 209, 3, 67, "Section"],
Cell[26317, 586, 1344, 26, 238, "Input"],
Cell[CellGroupData[{
Cell[27686, 616, 1593, 30, 283, "Input"],
Cell[29282, 648, 548, 7, 20, "Print"],
Cell[29833, 657, 786, 12, 27, "Output"],
Cell[30622, 671, 557, 7, 20, "Print"],
Cell[31182, 680, 784, 12, 27, "Output"],
Cell[31969, 694, 550, 7, 20, "Print"],
Cell[32522, 703, 784, 12, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[33343, 720, 2009, 39, 268, "Input"],
Cell[35355, 761, 2768, 58, 73, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38160, 824, 413, 8, 27, "Input"],
Cell[38576, 834, 11539, 388, 141, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50152, 1227, 570, 18, 27, "Input"],
Cell[50725, 1247, 167, 3, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50929, 1255, 2787, 73, 88, "Input"],
Cell[53719, 1330, 3771, 90, 171, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[57527, 1425, 5798, 146, 193, "Input"],
Cell[63328, 1573, 5259, 120, 156, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[68624, 1698, 290, 5, 27, "Input"],
Cell[68917, 1705, 480, 9, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[69446, 1720, 105, 1, 67, "Section"],
Cell[69554, 1723, 977, 20, 208, "Input"],
Cell[70534, 1745, 102, 1, 26, "Text"],
Cell[CellGroupData[{
Cell[70661, 1750, 1168, 31, 58, "Input"],
Cell[71832, 1783, 492, 8, 27, "Output"],
Cell[72327, 1793, 493, 8, 27, "Output"],
Cell[72823, 1803, 493, 8, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[73353, 1816, 3016, 85, 133, "Input"],
Cell[76372, 1903, 2837, 72, 58, "Output"],
Cell[79212, 1977, 4937, 134, 88, "Output"],
Cell[84152, 2113, 2855, 72, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87044, 2190, 12517, 332, 508, "Input"],
Cell[99564, 2524, 8442, 208, 164, "Output"]
}, Open  ]],
Cell[108021, 2735, 213, 5, 27, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[108271, 2745, 105, 1, 67, "Section"],
Cell[108379, 2748, 1609, 29, 298, "Input"],
Cell[CellGroupData[{
Cell[110013, 2781, 3720, 92, 223, "Input"],
Cell[113736, 2875, 1347, 24, 43, "Output"],
Cell[115086, 2901, 3489, 92, 73, "Output"],
Cell[118578, 2995, 2103, 51, 43, "Output"],
Cell[120684, 3048, 2062, 51, 43, "Output"],
Cell[122749, 3101, 3485, 92, 73, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[126271, 3198, 8981, 234, 343, "Input"],
Cell[135255, 3434, 7408, 172, 166, "Output"]
}, Open  ]],
Cell[142678, 3609, 392, 8, 27, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[143107, 3622, 158, 2, 67, "Section"],
Cell[CellGroupData[{
Cell[143290, 3628, 114, 1, 34, "Subsection"],
Cell[143407, 3631, 126, 3, 26, "Text"],
Cell[143536, 3636, 306, 8, 43, "Input"],
Cell[143845, 3646, 879, 16, 62, "Input"],
Cell[CellGroupData[{
Cell[144749, 3666, 1274, 37, 43, "Input"],
Cell[146026, 3705, 393, 9, 27, "Output"],
Cell[146422, 3716, 816, 25, 66, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[147275, 3746, 4464, 108, 379, "Input"],
Cell[151742, 3856, 3449, 86, 225, "Output"],
Cell[155194, 3944, 3363, 83, 207, "Output"],
Cell[158560, 4029, 1033, 19, 43, "Output"]
}, Open  ]],
Cell[159608, 4051, 146, 4, 43, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[159791, 4060, 165, 2, 34, "Subsection"],
Cell[159959, 4064, 256, 5, 41, "Text"],
Cell[160218, 4071, 2728, 61, 118, "Input"],
Cell[162949, 4134, 5277, 128, 208, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[168263, 4267, 107, 1, 34, "Subsection"],
Cell[168373, 4270, 326, 6, 26, "Text"],
Cell[168702, 4278, 210, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[168937, 4286, 1738, 44, 123, "Input"],
Cell[170678, 4332, 626, 15, 66, "Output"],
Cell[171307, 4349, 1476, 42, 79, "Output"]
}, Open  ]],
Cell[172798, 4394, 1023, 31, 62, "Input"],
Cell[173824, 4427, 205, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[174054, 4435, 622, 17, 46, "Input"],
Cell[174679, 4454, 593, 18, 62, "Output"]
}, Open  ]],
Cell[175287, 4475, 1641, 47, 115, "Input"],
Cell[CellGroupData[{
Cell[176953, 4526, 389, 9, 43, "Input"],
Cell[177345, 4537, 243, 4, 27, "Output"],
Cell[177591, 4543, 242, 4, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[177882, 4553, 209, 4, 35, "Subsection"],
Cell[178094, 4559, 2629, 59, 113, "Input"],
Cell[CellGroupData[{
Cell[180748, 4622, 4782, 110, 208, "Input"],
Cell[185533, 4734, 10014, 180, 175, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[195584, 4919, 459, 11, 43, "Input"],
Cell[196046, 4932, 684, 12, 27, "Output"]
}, Open  ]],
Cell[196745, 4947, 2152, 53, 113, "Input"],
Cell[CellGroupData[{
Cell[198922, 5004, 3991, 95, 223, "Input"],
Cell[202916, 5101, 5273, 100, 149, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[208226, 5206, 483, 11, 43, "Input"],
Cell[208712, 5219, 554, 10, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[209315, 5235, 119, 1, 34, "Subsection"],
Cell[209437, 5238, 135, 3, 26, "Text"],
Cell[209575, 5243, 1224, 21, 73, "Input"],
Cell[210802, 5266, 933, 16, 43, "Input"],
Cell[CellGroupData[{
Cell[211760, 5286, 1395, 34, 43, "Input"],
Cell[213158, 5322, 5549, 139, 164, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[218744, 5466, 410, 10, 43, "Input"],
Cell[219157, 5478, 420, 8, 27, "Output"]
}, Open  ]],
Cell[219592, 5489, 142, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[219759, 5496, 621, 15, 43, "Input"],
Cell[220383, 5513, 11174, 189, 251, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[231606, 5708, 179, 4, 34, "Subsection"],
Cell[231788, 5714, 243, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[232056, 5722, 573, 16, 27, "Input"],
Cell[232632, 5740, 447, 11, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[233116, 5756, 571, 16, 27, "Input"],
Cell[233690, 5774, 423, 11, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[234150, 5790, 611, 17, 27, "Input"],
Cell[234764, 5809, 748, 14, 48, "Message"],
Cell[235515, 5825, 423, 11, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[235975, 5841, 633, 17, 27, "Input"],
Cell[236611, 5860, 750, 14, 48, "Message"],
Cell[237364, 5876, 423, 11, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[237824, 5892, 734, 20, 43, "Input"],
Cell[238561, 5914, 546, 9, 27, "Output"],
Cell[239110, 5925, 547, 9, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[239694, 5939, 922, 21, 27, "Input"],
Cell[240619, 5962, 485, 10, 23, "Message"],
Cell[241107, 5974, 510, 13, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[241654, 5992, 715, 16, 27, "Input"],
Cell[242372, 6010, 787, 15, 40, "Message"],
Cell[243162, 6027, 785, 15, 40, "Message"],
Cell[243950, 6044, 785, 15, 40, "Message"],
Cell[244738, 6061, 639, 13, 23, "Message"],
Cell[245380, 6076, 627, 12, 23, "Message"],
Cell[246010, 6090, 629, 12, 23, "Message"],
Cell[246642, 6104, 627, 12, 23, "Message"],
Cell[247272, 6118, 638, 13, 23, "Message"],
Cell[247913, 6133, 781, 15, 40, "Message"],
Cell[248697, 6150, 450, 11, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[249184, 6166, 514, 14, 27, "Input"],
Cell[249701, 6182, 486, 10, 23, "Message"],
Cell[250190, 6194, 358, 10, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[250585, 6209, 724, 18, 27, "Input"],
Cell[251312, 6229, 408, 11, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[251757, 6245, 770, 18, 27, "Input"],
Cell[252530, 6265, 432, 11, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[252999, 6281, 824, 19, 27, "Input"],
Cell[253826, 6302, 686, 13, 40, "Message"],
Cell[254515, 6317, 684, 13, 40, "Message"],
Cell[255202, 6332, 683, 13, 40, "Message"],
Cell[255888, 6347, 538, 11, 23, "Message"],
Cell[256429, 6360, 526, 10, 23, "Message"],
Cell[256958, 6372, 454, 11, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257449, 6388, 972, 21, 27, "Input"],
Cell[258424, 6411, 535, 11, 23, "Message"],
Cell[258962, 6424, 527, 12, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[259538, 6442, 186, 4, 35, "Subsection"],
Cell[259727, 6448, 2344, 62, 169, "Input"],
Cell[262074, 6512, 143, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[262242, 6519, 2435, 63, 159, "Input"],
Cell[264680, 6584, 309, 8, 27, "Output"]
}, Open  ]],
Cell[265004, 6595, 163, 3, 26, "Text"],
Cell[265170, 6600, 4355, 92, 268, "Input"],
Cell[CellGroupData[{
Cell[269550, 6696, 10917, 280, 605, "Input"],
Cell[280470, 6978, 43264, 736, 187, "Output"],
Cell[323737, 7716, 43405, 739, 187, "Output"]
}, Open  ]],
Cell[367157, 8458, 500, 11, 43, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[367694, 8474, 235, 5, 34, "Subsection"],
Cell[367932, 8481, 192, 4, 26, "Text"],
Cell[368127, 8487, 898, 20, 103, "Input"],
Cell[369028, 8509, 151, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[369204, 8516, 1134, 32, 68, "Input"],
Cell[370341, 8550, 604, 14, 27, "Output"]
}, Open  ]],
Cell[370960, 8567, 2740, 64, 225, "Input"],
Cell[373703, 8633, 321, 5, 26, "Text"],
Cell[CellGroupData[{
Cell[374049, 8642, 1715, 39, 99, "Input"],
Cell[375767, 8683, 712, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[376504, 8700, 437, 6, 23, "Print"],
Cell[376944, 8708, 437, 6, 23, "Print"],
Cell[377384, 8716, 437, 6, 23, "Print"],
Cell[377824, 8724, 437, 6, 23, "Print"],
Cell[378264, 8732, 437, 6, 23, "Print"],
Cell[378704, 8740, 436, 6, 23, "Print"],
Cell[379143, 8748, 437, 6, 23, "Print"],
Cell[379583, 8756, 437, 6, 23, "Print"],
Cell[380023, 8764, 437, 6, 23, "Print"],
Cell[380463, 8772, 437, 6, 23, "Print"],
Cell[380903, 8780, 437, 6, 23, "Print"],
Cell[381343, 8788, 437, 6, 23, "Print"],
Cell[381783, 8796, 439, 6, 23, "Print"],
Cell[382225, 8804, 437, 6, 23, "Print"],
Cell[382665, 8812, 437, 6, 23, "Print"],
Cell[383105, 8820, 436, 6, 23, "Print"],
Cell[383544, 8828, 432, 6, 23, "Print"],
Cell[383979, 8836, 430, 6, 23, "Print"]
}, Open  ]],
Cell[384424, 8845, 918, 16, 40, "Message"],
Cell[CellGroupData[{
Cell[385367, 8865, 430, 6, 23, "Print"],
Cell[385800, 8873, 430, 6, 23, "Print"],
Cell[386233, 8881, 429, 6, 23, "Print"]
}, Open  ]],
Cell[386677, 8890, 1745, 42, 43, "Output"],
Cell[388425, 8934, 430, 6, 23, "Print"],
Cell[388858, 8942, 720, 13, 23, "Message"],
Cell[389581, 8957, 429, 6, 23, "Print"],
Cell[390013, 8965, 720, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[390758, 8982, 430, 6, 23, "Print"],
Cell[391191, 8990, 432, 6, 23, "Print"],
Cell[391626, 8998, 430, 6, 23, "Print"],
Cell[392059, 9006, 431, 6, 23, "Print"],
Cell[392493, 9014, 430, 6, 23, "Print"]
}, Open  ]],
Cell[392938, 9023, 722, 13, 23, "Message"],
Cell[393663, 9038, 769, 14, 23, "Message"],
Cell[CellGroupData[{
Cell[394457, 9056, 430, 6, 23, "Print"],
Cell[394890, 9064, 432, 6, 23, "Print"],
Cell[395325, 9072, 430, 6, 23, "Print"],
Cell[395758, 9080, 432, 6, 23, "Print"],
Cell[396193, 9088, 432, 6, 23, "Print"],
Cell[396628, 9096, 430, 6, 23, "Print"],
Cell[397061, 9104, 430, 6, 23, "Print"],
Cell[397494, 9112, 430, 6, 23, "Print"],
Cell[397927, 9120, 429, 6, 23, "Print"],
Cell[398359, 9128, 430, 6, 23, "Print"],
Cell[398792, 9136, 430, 6, 23, "Print"],
Cell[399225, 9144, 430, 6, 23, "Print"],
Cell[399658, 9152, 430, 6, 23, "Print"],
Cell[400091, 9160, 430, 6, 23, "Print"]
}, Open  ]],
Cell[400536, 9169, 5281, 144, 103, "Output"],
Cell[CellGroupData[{
Cell[405842, 9317, 432, 6, 23, "Print"],
Cell[406277, 9325, 430, 6, 23, "Print"]
}, Open  ]],
Cell[406722, 9334, 720, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[407467, 9351, 430, 6, 23, "Print"],
Cell[407900, 9359, 430, 6, 23, "Print"],
Cell[408333, 9367, 432, 6, 23, "Print"]
}, Open  ]],
Cell[408780, 9376, 719, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[409524, 9393, 429, 6, 23, "Print"],
Cell[409956, 9401, 430, 6, 23, "Print"]
}, Open  ]],
Cell[410401, 9410, 722, 13, 23, "Message"],
Cell[411126, 9425, 767, 14, 23, "Message"],
Cell[CellGroupData[{
Cell[411918, 9443, 429, 6, 23, "Print"],
Cell[412350, 9451, 432, 6, 23, "Print"],
Cell[412785, 9459, 432, 6, 23, "Print"],
Cell[413220, 9467, 430, 6, 23, "Print"],
Cell[413653, 9475, 432, 6, 23, "Print"],
Cell[414088, 9483, 430, 6, 23, "Print"],
Cell[414521, 9491, 432, 6, 23, "Print"],
Cell[414956, 9499, 430, 6, 23, "Print"],
Cell[415389, 9507, 429, 6, 23, "Print"],
Cell[415821, 9515, 430, 6, 23, "Print"],
Cell[416254, 9523, 430, 6, 23, "Print"],
Cell[416687, 9531, 430, 6, 23, "Print"],
Cell[417120, 9539, 430, 6, 23, "Print"],
Cell[417553, 9547, 432, 6, 23, "Print"]
}, Open  ]],
Cell[418000, 9556, 5284, 145, 103, "Output"],
Cell[CellGroupData[{
Cell[423309, 9705, 432, 6, 23, "Print"],
Cell[423744, 9713, 430, 6, 23, "Print"],
Cell[424177, 9721, 430, 6, 23, "Print"],
Cell[424610, 9729, 429, 6, 23, "Print"],
Cell[425042, 9737, 430, 6, 23, "Print"],
Cell[425475, 9745, 428, 6, 23, "Print"]
}, Open  ]],
Cell[425918, 9754, 720, 13, 23, "Message"],
Cell[426641, 9769, 430, 6, 23, "Print"],
Cell[427074, 9777, 720, 13, 23, "Message"],
Cell[427797, 9792, 430, 6, 23, "Print"],
Cell[428230, 9800, 722, 13, 23, "Message"],
Cell[428955, 9815, 767, 14, 23, "Message"],
Cell[CellGroupData[{
Cell[429747, 9833, 429, 6, 23, "Print"],
Cell[430179, 9841, 430, 6, 23, "Print"],
Cell[430612, 9849, 430, 6, 23, "Print"],
Cell[431045, 9857, 430, 6, 23, "Print"],
Cell[431478, 9865, 430, 6, 23, "Print"],
Cell[431911, 9873, 430, 6, 23, "Print"],
Cell[432344, 9881, 429, 6, 23, "Print"],
Cell[432776, 9889, 429, 6, 23, "Print"],
Cell[433208, 9897, 430, 6, 23, "Print"],
Cell[433641, 9905, 432, 6, 23, "Print"],
Cell[434076, 9913, 432, 6, 23, "Print"],
Cell[434511, 9921, 430, 6, 23, "Print"],
Cell[434944, 9929, 430, 6, 23, "Print"]
}, Open  ]],
Cell[435389, 9938, 5266, 143, 103, "Output"],
Cell[CellGroupData[{
Cell[440680, 10085, 430, 6, 23, "Print"],
Cell[441113, 10093, 430, 6, 23, "Print"],
Cell[441546, 10101, 432, 6, 23, "Print"],
Cell[441981, 10109, 432, 6, 23, "Print"],
Cell[442416, 10117, 430, 6, 23, "Print"],
Cell[442849, 10125, 429, 6, 23, "Print"],
Cell[443281, 10133, 430, 6, 23, "Print"],
Cell[443714, 10141, 430, 6, 23, "Print"],
Cell[444147, 10149, 432, 6, 23, "Print"],
Cell[444582, 10157, 432, 6, 23, "Print"],
Cell[445017, 10165, 432, 6, 23, "Print"],
Cell[445452, 10173, 430, 6, 23, "Print"],
Cell[445885, 10181, 430, 6, 23, "Print"],
Cell[446318, 10189, 430, 6, 23, "Print"],
Cell[446751, 10197, 430, 6, 23, "Print"],
Cell[447184, 10205, 431, 6, 23, "Print"],
Cell[447618, 10213, 430, 6, 23, "Print"],
Cell[448051, 10221, 430, 6, 23, "Print"],
Cell[448484, 10229, 430, 6, 23, "Print"],
Cell[448917, 10237, 430, 6, 23, "Print"],
Cell[449350, 10245, 432, 6, 23, "Print"]
}, Open  ]],
Cell[449797, 10254, 5286, 145, 103, "Output"],
Cell[455086, 10401, 432, 6, 23, "Print"],
Cell[455521, 10409, 719, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[456265, 10426, 430, 6, 23, "Print"],
Cell[456698, 10434, 432, 6, 23, "Print"],
Cell[457133, 10442, 429, 6, 23, "Print"],
Cell[457565, 10450, 432, 6, 23, "Print"],
Cell[458000, 10458, 428, 6, 23, "Print"],
Cell[458431, 10466, 429, 6, 23, "Print"],
Cell[458863, 10474, 432, 6, 23, "Print"],
Cell[459298, 10482, 432, 6, 23, "Print"],
Cell[459733, 10490, 430, 6, 23, "Print"],
Cell[460166, 10498, 430, 6, 23, "Print"],
Cell[460599, 10506, 430, 6, 23, "Print"],
Cell[461032, 10514, 432, 6, 23, "Print"],
Cell[461467, 10522, 430, 6, 23, "Print"],
Cell[461900, 10530, 430, 6, 23, "Print"],
Cell[462333, 10538, 429, 6, 23, "Print"]
}, Open  ]],
Cell[462777, 10547, 720, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[463522, 10564, 430, 6, 23, "Print"],
Cell[463955, 10572, 430, 6, 23, "Print"]
}, Open  ]],
Cell[464400, 10581, 720, 13, 23, "Message"],
Cell[465123, 10596, 769, 14, 23, "Message"],
Cell[CellGroupData[{
Cell[465917, 10614, 432, 6, 23, "Print"],
Cell[466352, 10622, 430, 6, 23, "Print"],
Cell[466785, 10630, 430, 6, 23, "Print"]
}, Open  ]],
Cell[467230, 10639, 5272, 143, 103, "Output"],
Cell[CellGroupData[{
Cell[472527, 10786, 432, 6, 23, "Print"],
Cell[472962, 10794, 430, 6, 23, "Print"],
Cell[473395, 10802, 430, 6, 23, "Print"]
}, Open  ]],
Cell[473840, 10811, 722, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[474587, 10828, 432, 6, 23, "Print"],
Cell[475022, 10836, 430, 6, 23, "Print"],
Cell[475455, 10844, 431, 6, 23, "Print"],
Cell[475889, 10852, 430, 6, 23, "Print"]
}, Open  ]],
Cell[476334, 10861, 720, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[477079, 10878, 430, 6, 23, "Print"],
Cell[477512, 10886, 430, 6, 23, "Print"],
Cell[477945, 10894, 430, 6, 23, "Print"],
Cell[478378, 10902, 430, 6, 23, "Print"],
Cell[478811, 10910, 430, 6, 23, "Print"],
Cell[479244, 10918, 429, 6, 23, "Print"],
Cell[479676, 10926, 430, 6, 23, "Print"],
Cell[480109, 10934, 429, 6, 23, "Print"],
Cell[480541, 10942, 429, 6, 23, "Print"],
Cell[480973, 10950, 430, 6, 23, "Print"]
}, Open  ]],
Cell[481418, 10959, 720, 13, 23, "Message"],
Cell[482141, 10974, 766, 14, 23, "Message"],
Cell[CellGroupData[{
Cell[482932, 10992, 430, 6, 23, "Print"],
Cell[483365, 11000, 430, 6, 23, "Print"],
Cell[483798, 11008, 430, 6, 23, "Print"],
Cell[484231, 11016, 432, 6, 23, "Print"]
}, Open  ]],
Cell[484678, 11025, 5300, 149, 103, "Output"],
Cell[CellGroupData[{
Cell[490003, 11178, 430, 6, 23, "Print"],
Cell[490436, 11186, 430, 6, 23, "Print"]
}, Open  ]],
Cell[490881, 11195, 722, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[491628, 11212, 430, 6, 23, "Print"],
Cell[492061, 11220, 432, 6, 23, "Print"],
Cell[492496, 11228, 430, 6, 23, "Print"],
Cell[492929, 11236, 429, 6, 23, "Print"],
Cell[493361, 11244, 430, 6, 23, "Print"],
Cell[493794, 11252, 430, 6, 23, "Print"],
Cell[494227, 11260, 432, 6, 23, "Print"],
Cell[494662, 11268, 430, 6, 23, "Print"],
Cell[495095, 11276, 430, 6, 23, "Print"],
Cell[495528, 11284, 430, 6, 23, "Print"]
}, Open  ]],
Cell[495973, 11293, 720, 13, 23, "Message"],
Cell[496696, 11308, 430, 6, 23, "Print"],
Cell[497129, 11316, 720, 13, 23, "Message"],
Cell[497852, 11331, 766, 14, 23, "Message"],
Cell[CellGroupData[{
Cell[498643, 11349, 430, 6, 23, "Print"],
Cell[499076, 11357, 430, 6, 23, "Print"],
Cell[499509, 11365, 429, 6, 23, "Print"],
Cell[499941, 11373, 432, 6, 23, "Print"],
Cell[500376, 11381, 430, 6, 23, "Print"],
Cell[500809, 11389, 430, 6, 23, "Print"],
Cell[501242, 11397, 430, 6, 23, "Print"],
Cell[501675, 11405, 432, 6, 23, "Print"]
}, Open  ]],
Cell[502122, 11414, 5284, 145, 103, "Output"],
Cell[507409, 11561, 429, 6, 23, "Print"],
Cell[507841, 11569, 720, 13, 23, "Message"],
Cell[508564, 11584, 432, 6, 23, "Print"],
Cell[508999, 11592, 720, 13, 23, "Message"],
Cell[CellGroupData[{
Cell[509744, 11609, 432, 6, 23, "Print"],
Cell[510179, 11617, 430, 6, 23, "Print"]
}, Open  ]],
Cell[510624, 11626, 720, 13, 23, "Message"],
Cell[511347, 11641, 767, 14, 23, "Message"],
Cell[CellGroupData[{
Cell[512139, 11659, 430, 6, 23, "Print"],
Cell[512572, 11667, 429, 6, 23, "Print"],
Cell[513004, 11675, 430, 6, 23, "Print"],
Cell[513437, 11683, 430, 6, 23, "Print"],
Cell[513870, 11691, 430, 6, 23, "Print"],
Cell[514303, 11699, 430, 6, 23, "Print"],
Cell[514736, 11707, 429, 6, 23, "Print"],
Cell[515168, 11715, 430, 6, 23, "Print"],
Cell[515601, 11723, 432, 6, 23, "Print"],
Cell[516036, 11731, 429, 6, 23, "Print"],
Cell[516468, 11739, 430, 6, 23, "Print"],
Cell[516901, 11747, 428, 6, 23, "Print"],
Cell[517332, 11755, 432, 6, 23, "Print"],
Cell[517767, 11763, 430, 6, 23, "Print"],
Cell[518200, 11771, 430, 6, 23, "Print"],
Cell[518633, 11779, 430, 6, 23, "Print"],
Cell[519066, 11787, 430, 6, 23, "Print"]
}, Open  ]],
Cell[519511, 11796, 5292, 146, 103, "Output"]
}, Closed]],
Cell[524818, 11945, 142, 3, 23, "Text"],
Cell[CellGroupData[{
Cell[524985, 11952, 3952, 105, 298, "Input"],
Cell[528940, 12059, 54202, 1326, 133, "Output"],
Cell[583145, 13387, 64433, 1524, 271, "Output"]
}, Open  ]],
Cell[647593, 14914, 155, 3, 26, "Text"],
Cell[CellGroupData[{
Cell[647773, 14921, 4132, 109, 298, "Input"],
Cell[651908, 15032, 54223, 1326, 136, "Output"],
Cell[706134, 16360, 55149, 1368, 278, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[761332, 17734, 112, 1, 34, "Subsection"],
Cell[CellGroupData[{
Cell[761469, 17739, 5698, 142, 218, "Input"],
Cell[767170, 17883, 9890, 246, 165, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[777097, 18134, 353, 6, 27, "Input"],
Cell[777453, 18142, 372, 7, 27, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[777862, 18154, 6242, 150, 308, "Input"],
Cell[784107, 18306, 9806, 244, 168, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[793950, 18555, 309, 5, 27, "Input"],
Cell[794262, 18562, 300, 6, 27, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[794611, 18574, 139, 3, 35, "Subsection"],
Cell[CellGroupData[{
Cell[794775, 18581, 3580, 99, 298, "Input"],
Cell[798358, 18682, 27506, 684, 132, "Output"],
Cell[825867, 19368, 27494, 684, 131, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[853398, 20057, 3580, 99, 298, "Input"],
Cell[856981, 20158, 27467, 684, 133, "Output"],
Cell[884451, 20844, 27446, 684, 133, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[911934, 21533, 10841, 277, 378, "Input"],
Cell[922778, 21812, 10237, 251, 163, "Output"],
Cell[933018, 22065, 10232, 252, 166, "Output"]
}, Open  ]],
Cell[943265, 22320, 203, 4, 27, "Input"],
Cell[943471, 22326, 323, 6, 27, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[943831, 22337, 108, 1, 34, "Subsection"],
Cell[CellGroupData[{
Cell[943964, 22342, 3025, 73, 158, "Input"],
Cell[946992, 22417, 4280, 110, 246, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[951309, 22532, 1881, 57, 73, "Input"],
Cell[953193, 22591, 4776, 119, 164, "Output"]
}, Open  ]],
Cell[957984, 22713, 277, 5, 27, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
